/*  
 *  - - - - - - - -  
 *   g a l _ d a t  
 *  - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  For a given UTC date, calculate delta(AT) = TAI-UTC.  
 *  
 *     :------------------------------------------:  
 *     :                                          :  
 *     :                 IMPORTANT                :  
 *     :                                          :  
 *     :  A new version of this routine must be   :  
 *     :  produced whenever a new leap second is  :  
 *     :  announced.  There are five items to     :  
 *     :  change on each such occasion:           :  
 *     :                                          :  
 *     :  1) The parameter NDAT must be           :  
 *     :     increased by 1.                      :  
 *     :                                          :  
 *     :  2) A new line must be added to the      :  
 *     :     arrays IDATE and DATS.               :  
 *     :                                          :  
 *     :  3) The parameter IYV must be set to     :  
 *     :     the current year.                    :  
 *     :                                          :  
 *     :  4) The "Latest leap second" comment     :  
 *     :     below must be set to the new leap    :  
 *     :     second date.                         :  
 *     :                                          :  
 *     :  5) The "This revision" comment, later,  :  
 *     :     must be set to the current date.     :  
 *     :                                          :  
 *     :  Change (3) must also be carried out     :  
 *     :  whenever the routine is re-issued,      :  
 *     :  even if no leap seconds have been       :  
 *     :  added.                                  :  
 *     :                                          :  
 *     :  Latest leap second:  2006 January 1     :  
 *     :                                          :  
 *     :__________________________________________:  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     iy                  i        UTC:  year (Notes 1 and 2)  
 *     im                  i        month (Note 2)  
 *     id                  i        day (Notes 2 and 3)  
 *     fd                  d        fraction of day (Note 4)  
 *  
 *  Returned:
 *  
 *     *deltat             d        TAI minus UTC, seconds  
 *     gal_dat             i        status (Note 5):  
 *                                    1 = dubious year (Note 1)  
 *                                    0 = OK  
 *                                   -1 = bad year  
 *                                   -2 = bad month  
 *                                   -3 = bad day (Note 3)  
 *                                   -4 = bad fraction (Note 4)  
 *  
 *  Notes:  
 *  
 *  1) UTC began at 1960 January 1.0 (JD 2436934.5) and it is improper  
 *     to call the routine with an earlier epoch.  If this is attempted,  
 *     zero is returned together with a warning status.  
 *  
 *     Because leap seconds cannot, in principle, be predicted in  
 *     advance, a reliable check for dates beyond the valid range is  
 *     impossible.  To guard against gross errors, a year five or more  
 *     after the release year of the present routine (see parameter IYV)  
 *     is considered dubious.  In this case a warning status is returned  
 *     but the result is computed in the normal way.  
 *  
 *     For both too-early and too-late years, the warning status is j=+1.  
 *     This is distinct from the error status j=-1, which signifies a  
 *     year so early that JD could not be computed.  
 *  
 *  2) If the specified date is for a day which ends with a leap second,  
 *     the UTC-TAI value returned is for the period leading up to the  
 *     leap second.  If the date is for a day which begins as a leap  
 *     second ends, the UTC-TAI returned is for the period following the  
 *     leap second.  
 *  
 *  3) The day number must be in the normal calendar range, for example  
 *     1 through 30 for April.  The "almanac" convention of allowing  
 *     such dates as January 0 and December 32 is not supported in this  
 *     routine, in order to avoid confusion near leap seconds.  
 *  
 *  4) The fraction of day is used only for dates before the introduction  
 *     of leap seconds, the first of which occurred at the end of 1971.  
 *     It is tested for validity (zero to less than 1 is the valid range)  
 *     even if not used;  if invalid, zero is used and status j=-4 is  
 *     returned.  For many applications, setting FD to zero is  
 *     acceptable;  the resulting error is always less than 3 ms (and  
 *     occurs only pre-1972).  
 *  
 *  5) The status value returned in the case where there are multiple  
 *     errors refers to the first error detected.  For example, if the  
 *     month and day are 13 and 32 respectively, j=-2 (bad month)  
 *     will be returned.  
 *  
 *  6) In cases where a valid result is not available, zero is returned.  
 *  
 *  References:  
 *  
 *  1) For epochs from 1961 January 1 onwards, the expressions from the  
 *     file ftp://maia.usno.navy.mil/ser7/tai-utc.dat are used.  
 *  
 *  2) The 5ms timestep at 1961 January 1 is taken from 2.58.1 (p87) of  
 *     the 1992 Explanatory Supplement.  
 *  
 *  Called:
 *  
 *     gal_cal2jd         Gregorian calendar to Julian Day number  
 *  
 *  This revision:
 *  
 *     2008 December 29  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_dat.h"  
#include "gal_cal2jd.h"  
  
int  
gal_dat  
 (  
    int iy,  
    int im,  
    int id,  
    double fd,  
    double *deltat
 )  

{  
  
/*  
 * Release year for this version of gal_DAT.  
 */  
  
#define IYV 2009  
  
/*  
 * Number of Delta(AT) changes (increase by 1 for each new leap second).  
 */  
  
#define NDAT 39  
  
/*  
 * Number of Delta(AT) expressions before leap seconds were introduced.  
 */  
  
#define NERA1 14  
  
/*  
 * Dates (year, month) on which new Delta(AT) came into force.  
 */  
  
    static const int IDATE[NDAT][2] = {  
        { 1960,  1,  } ,  
        { 1961,  1,  } ,  
        { 1961,  8,  } ,  
        { 1962,  1,  } ,  
        { 1963, 11,  } ,  
        { 1964,  1,  } ,  
        { 1964,  4,  } ,  
        { 1964,  9,  } ,  
        { 1965,  1,  } ,  
        { 1965,  3,  } ,  
        { 1965,  7,  } ,  
        { 1965,  9,  } ,  
        { 1966,  1,  } ,  
        { 1968,  2,  } ,  
        { 1972,  1,  } ,  
        { 1972,  7,  } ,  
        { 1973,  1,  } ,  
        { 1974,  1,  } ,  
        { 1975,  1,  } ,  
        { 1976,  1,  } ,  
        { 1977,  1,  } ,  
        { 1978,  1,  } ,  
        { 1979,  1,  } ,  
        { 1980,  1,  } ,  
        { 1981,  7,  } ,  
        { 1982,  7,  } ,  
        { 1983,  7,  } ,  
        { 1985,  7,  } ,  
        { 1988,  1,  } ,  
        { 1990,  1,  } ,  
        { 1991,  1,  } ,  
        { 1992,  7,  } ,  
        { 1993,  7,  } ,  
        { 1994,  7,  } ,  
        { 1996,  1,  } ,  
        { 1997,  7,  } ,  
        { 1999,  1,  } ,  
        { 2006,  1,  } ,  
        { 2009,  1,  } ,
    } ;  
  
/*  
 * New Delta(AT) which came into force on the given dates.  
 */  
  
    static const double DATS[NDAT] = {  
        1.4178180,  
        1.4228180,  
        1.3728180,  
        1.8458580,  
        1.9458580,  
        3.2401300,  
        3.3401300,  
        3.4401300,  
        3.5401300,  
        3.6401300,  
        3.7401300,  
        3.8401300,  
        4.3131700,  
        4.2131700,  
        10.0,  
        11.0,  
        12.0,  
        13.0,  
        14.0,  
        15.0,  
        16.0,  
        17.0,  
        18.0,  
        19.0,  
        20.0,  
        21.0,  
        22.0,  
        23.0,  
        24.0,  
        25.0,  
        26.0,  
        27.0,  
        28.0,  
        29.0,  
        30.0,  
        31.0,  
        32.0,  
        33.0,  
        34.0,
    } ;  
  
/*  
 * Reference dates (MJD) and drift rates (s/day), pre leap seconds.  
 */  
  
    static const double DRIFT[NERA1][2] = {  
  
      { 37300.0, 0.001296 , } ,  
      { 37300.0, 0.001296 , } ,  
      { 37300.0, 0.001296 , } ,  
      { 37665.0, 0.0011232, } ,  
      { 37665.0, 0.0011232, } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 38761.0, 0.001296 , } ,  
      { 39126.0, 0.002592 , } ,  
      { 39126.0, 0.002592 , } ,  
  
    } ;  
  
/*  
 * Miscellaneous local variables.  
 */  
  
    int js, i, m, is ;  
    double dat, djm0, djm ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Initialize the result to zero and the status to OK.  
 */  
  
    dat = 0.0 ;  
    js = 0 ;  
  
/*  
 * If invalid fraction of a day, set error status and give up.  
 */  
  
    if ( fd < 0.0 || fd >= 1.0 ) {  
        js = -4 ;  
    }  
    else {  
  
/*  
 * Convert the date into an MJD.  
 */  
  
        js = gal_cal2jd ( iy, im, id, &djm0, &djm ) ;  
  
/*  
 * If invalid year, month, or day, give up.  
 */  
  
        if ( js >= 0 ) {  
  
/*  
 * If pre-UTC year, set warning status and give up.  
 */  
  
            if ( iy < IDATE[0][0]) {  
                js = 1 ;  
            }  
            else {  
  
/*  
 * If suspiciously late year, set warning status but proceed.  
 */  
  
                if ( iy > IYV + 5 ) {  
                    js = 1 ;  
                }  
  
/*  
 * Combine year and month.  
 */  
  
                m = 12 * iy + im ;  
  
/*  
 * Find the most recent table entry.  
 */  
  
                for ( i = NDAT - 1; 0 <= i; i-- ) {  
                    is = i ;  
                    if ( m >= (12 * IDATE[i][0] + IDATE[i][1] ) ) {  
                        break ;  
                    }  
                }  
  
/*  
 * Get the Delta(AT).  
 */  
  
                dat = DATS[is] ;  
  
/*  
 * If pre-1972, adjust for drift.  
 */  
  
                if ( is <= NERA1 ) {  
                    dat += ( djm + fd - DRIFT[is][0] ) * DRIFT[is][1] ;  
                }  
  
            }  
        }  
    }  
  
/*  
 * Return the Delta(AT) value and the status.  
 */  
  
    *deltat = dat ;  
    return js ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
