/*  
 *  - - - - - - - - -  
 *   g a l _ d t d b  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  An approximation to TDB-TT, the difference between barycentric  
 *  dynamical time and terrestrial time, for an observer on the Earth.  
 *  
 *  The different time scales - proper, coordinate and realized - are  
 *  related to each other:  
 *  
 *            TAI             <-  physically realized  
 *             :  
 *          offset            <-  observed (nominally +32.184s)  
 *             :  
 *            TT              <-  terrestrial time  
 *             :  
 *    rate adjustment (L_G)   <-  definition of TT  
 *             :  
 *            TCG             <-  time scale for GCRS  
 *             :  
 *      "periodic" terms      <-  gal_dtdb is an implementation  
 *             :  
 *    rate adjustment (L_C)   <-  function of solar-system ephemeris  
 *             :  
 *            TCB             <-  time scale for BCRS  
 *             :  
 *    rate adjustment (-L_B)  <-  definition of TDB  
 *             :  
 *            TDB             <-  TCB scaled to track TT  
 *             :  
 *      "periodic" terms      <-  -gal_dtdb is an approximation  
 *             :  
 *            TT              <-  terrestrial time  
 *  
 *  Adopted values for the various constants can be found in the IERS  
 *  Conventions (McCarthy & Petit 2003).  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        date, TDB (Notes 1-3)  
 *     ut                  d        universal time (UT1, fraction of one day)  
 *     elong               d        longitude (east positive, radians)  
 *     u                   d        distance from Earth spin axis (km)  
 *     v                   d        distance north of equatorial plane (km)  
 *  
 *  Returned:
 *  
 *     gal_dtdb            d        TDB-TT (seconds)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *     Although the epoch is, formally, barycentric dynamical time (TDB),  
 *     the terrestrial dynamical time (TT) can be used with no practical  
 *     effect on the accuracy of the prediction.  
 *  
 *  2) TT can be regarded as a coordinate time that is realized as an  
 *     offset of 32.184s from International Atomic Time,  TAI.  TT is a  
 *     specific linear transformation of geocentric coordinate time TCG,  
 *     which is the time scale for the Geocentric Celestial Reference  
 *     System, GCRS.  
 *  
 *  3) TDB is a coordinate time, and is a specific linear transformation  
 *     of barycentric coordinate time TCB, which is the time scale for  
 *     the Barycentric Celestial Reference System, BCRS.  
 *  
 *  4) The difference TCG-TCB depends on the masses and positions of the  
 *     bodies of the solar system and the velocity of the Earth.  It is  
 *     dominated by a rate difference, the residual being of a periodic  
 *     character.  The latter, which is modeled by the present routine,  
 *     comprises a main (annual) sinusoidal term of amplitude  
 *     approximately 0.00166 seconds, plus planetary terms up to about  
 *     20 microseconds, and lunar and diurnal terms up to 2 microseconds.  
 *     These effects come from the changing transverse Doppler effect  
 *     and gravitational red-shift as the observer (on the Earth's  
 *     surface) experiences variations in speed (with respect to the  
 *     BCRS) and gravitational potential.  
 *  
 *  5) TDB can be regarded as the same as TCB but with a rate adjustment  
 *     to keep it close to TT, which is convenient for many applications.  
 *     The history of successive attempts to define TDB is set out in  
 *     Resolution 3 adopted by the IAU General Assembly in 2006, which  
 *     defines a fixed TDB(TCB) transformation that is consistent with  
 *     contemporary solar-system ephemerides.  Future ephemerides will  
 *     imply slightly changed transformations between TCG and TCB, which  
 *     could introduce a linear drift between TDB and TT;  however, any  
 *     such drift is unlikely to exceed 1 nanosecond per century.  
 *  
 *  6) The geocentric TDB-TT model used in the present routine is that of  
 *     Fairhead & Bretagnon (1990), in its full form.  It was originally  
 *     supplied by Fairhead (private communications with P.T.Wallace,  
 *     1990) as a Fortran subroutine.  The present routine contains an  
 *     adaptation of the Fairhead code.  The numerical results are  
 *     essentially unaffected by the changes, the differences with  
 *     respect to the Fairhead & Bretagnon original being at the 1e-20 s  
 *     level.  
 *  
 *     The topocentric part of the model is from Moyer (1981) and  
 *     Murray (1983), with fundamental arguments adapted from  
 *     Simon et al. 1994.  It is an approximation to the expression  
 *     ( v / c ) . ( r / c ), where v is the barycentric velocity of  
 *     the Earth, r is the geocentric position of the observer and  
 *     c is the speed of light.  
 *  
 *     By supplying zeroes for u and v, the topocentric part of the  
 *     model can be nullified, and the routine will return the Fairhead  
 *     & Bretagnon result alone.  
 *  
 *  7) During the interval 1950-2050, the absolute accuracy is better  
 *     than +/- 3 nanoseconds relative to time ephemerides obtained by  
 *     direct numerical integrations based on the JPL DE405 solar system  
 *     ephemeris.  
 *  
 *  8) It must be stressed that the present routine is merely a model,  
 *     and that numerical integration of solar-system ephemerides is the  
 *     definitive method for predicting the relationship between TCG and  
 *     TCB and hence between TT and TDB.  
 *  
 *  References:  
 *  
 *     Fairhead, L., & Bretagnon, P., Astron.Astrophys., 229, 240-247  
 *     (1990).  
 *  
 *     IAU 2006 Resolution 3.  
 *  
 *     McCarthy, D. D., Petit, G. (eds.), IERS Conventions (2003),  
 *     IERS Technical Note No. 32, BKG (2004)  
 *  
 *     Moyer, T.D., Cel.Mech., 23, 33 (1981).  
 *  
 *     Murray, C.A., Vectorial Astrometry, Adam Hilger (1983).  
 *  
 *     Seidelmann, P.K. et al., Explanatory Supplement to the  
 *     Astronomical Almanac, Chapter 2, University Science Books (1992).  
 *  
 *     Simon, J.L., Bretagnon, P., Chapront, J., Chapront-Touze, M.,  
 *     Francou, G. & Laskar, J., Astron.Astrophys., 282, 663-683 (1994).  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_dtdb.h"  
  
double  
gal_dtdb  
 (  
    double date1,  
    double date2,  
    double ut,  
    double elong,  
    double u,  
    double v  
 )  
{  
  
    double t, tsol, w, elsun, emsun, d, elj, els, wt, w0, w1, w2, w3, w4, wf, wj ;  
  
/*  
 *  =====================  
 *  Fairhead et al. model  
 *  =====================  
 *  
 *  787 sets of three coefficients.  
 *  
 *  Each set is amplitude (microseconds)  
 *              frequency (radians per Julian millennium since J2000),  
 *              phase (radians).  
 *  
 *  Sets   1-474 are the T**0 terms,  
 *   "   475-679  "   "  T**1   "  
 *   "   680-764  "   "  T**2   "  
 *   "   765-784  "   "  T**3   "  
 *   "   785-787  "   "  T**4   "  .  
 */  
  
 int j ;  
 static const double FAIRHD[787][3] = {  
  
     { 1656.674564e-6,    6283.075849991e0, 6.240054195e0 } ,  
     {   22.417471e-6,    5753.384884897e0, 4.296977442e0 } ,  
     {   13.839792e-6,   12566.151699983e0, 6.196904410e0 } ,  
     {    4.770086e-6,     529.690965095e0, 0.444401603e0 } ,  
     {    4.676740e-6,    6069.776754553e0, 4.021195093e0 } ,  
     {    2.256707e-6,     213.299095438e0, 5.543113262e0 } ,  
     {    1.694205e-6,      -3.523118349e0, 5.025132748e0 } ,  
     {    1.554905e-6,   77713.771467920e0, 5.198467090e0 } ,  
     {    1.276839e-6,    7860.419392439e0, 5.988822341e0 } ,  
     {    1.193379e-6,    5223.693919802e0, 3.649823730e0 } ,  
     {    1.115322e-6,    3930.209696220e0, 1.422745069e0 } ,  
     {    0.794185e-6,   11506.769769794e0, 2.322313077e0 } ,  
     {    0.447061e-6,      26.298319800e0, 3.615796498e0 } ,  
     {    0.435206e-6,    -398.149003408e0, 4.349338347e0 } ,  
     {    0.600309e-6,    1577.343542448e0, 2.678271909e0 } ,  
     {    0.496817e-6,    6208.294251424e0, 5.696701824e0 } ,  
     {    0.486306e-6,    5884.926846583e0, 0.520007179e0 } ,  
     {    0.432392e-6,      74.781598567e0, 2.435898309e0 } ,  
     {    0.468597e-6,    6244.942814354e0, 5.866398759e0 } ,  
     {    0.375510e-6,    5507.553238667e0, 4.103476804e0 } ,  
     {    0.243085e-6,    -775.522611324e0, 3.651837925e0 } ,  
     {    0.173435e-6,   18849.227549974e0, 6.153743485e0 } ,  
     {    0.230685e-6,    5856.477659115e0, 4.773852582e0 } ,  
     {    0.203747e-6,   12036.460734888e0, 4.333987818e0 } ,  
     {    0.143935e-6,    -796.298006816e0, 5.957517795e0 } ,  
     {    0.159080e-6,   10977.078804699e0, 1.890075226e0 } ,  
     {    0.119979e-6,      38.133035638e0, 4.551585768e0 } ,  
     {    0.118971e-6,    5486.777843175e0, 1.914547226e0 } ,  
     {    0.116120e-6,    1059.381930189e0, 0.873504123e0 } ,  
     {    0.137927e-6,   11790.629088659e0, 1.135934669e0 } ,  
     {    0.098358e-6,    2544.314419883e0, 0.092793886e0 } ,  
     {    0.101868e-6,   -5573.142801634e0, 5.984503847e0 } ,  
     {    0.080164e-6,     206.185548437e0, 2.095377709e0 } ,  
     {    0.079645e-6,    4694.002954708e0, 2.949233637e0 } ,  
     {    0.062617e-6,      20.775395492e0, 2.654394814e0 } ,  
     {    0.075019e-6,    2942.463423292e0, 4.980931759e0 } ,  
     {    0.064397e-6,    5746.271337896e0, 1.280308748e0 } ,  
     {    0.063814e-6,    5760.498431898e0, 4.167901731e0 } ,  
     {    0.048042e-6,    2146.165416475e0, 1.495846011e0 } ,  
     {    0.048373e-6,     155.420399434e0, 2.251573730e0 } ,  
     {    0.058844e-6,     426.598190876e0, 4.839650148e0 } ,  
     {    0.046551e-6,      -0.980321068e0, 0.921573539e0 } ,  
     {    0.054139e-6,   17260.154654690e0, 3.411091093e0 } ,  
     {    0.042411e-6,    6275.962302991e0, 2.869567043e0 } ,  
     {    0.040184e-6,      -7.113547001e0, 3.565975565e0 } ,  
     {    0.036564e-6,    5088.628839767e0, 3.324679049e0 } ,  
     {    0.040759e-6,   12352.852604545e0, 3.981496998e0 } ,  
     {    0.036507e-6,     801.820931124e0, 6.248866009e0 } ,  
     {    0.036955e-6,    3154.687084896e0, 5.071801441e0 } ,  
     {    0.042732e-6,     632.783739313e0, 5.720622217e0 } ,  
     {    0.042560e-6,  161000.685737473e0, 1.270837679e0 } ,  
     {    0.040480e-6,   15720.838784878e0, 2.546610123e0 } ,  
     {    0.028244e-6,   -6286.598968340e0, 5.069663519e0 } ,  
     {    0.033477e-6,    6062.663207553e0, 4.144987272e0 } ,  
     {    0.034867e-6,     522.577418094e0, 5.210064075e0 } ,  
     {    0.032438e-6,    6076.890301554e0, 0.749317412e0 } ,  
     {    0.030215e-6,    7084.896781115e0, 3.389610345e0 } ,  
     {    0.029247e-6,  -71430.695617928e0, 4.183178762e0 } ,  
     {    0.033529e-6,    9437.762934887e0, 2.404714239e0 } ,  
     {    0.032423e-6,    8827.390269875e0, 5.541473556e0 } ,  
     {    0.027567e-6,    6279.552731642e0, 5.040846034e0 } ,  
     {    0.029862e-6,   12139.553509107e0, 1.770181024e0 } ,  
     {    0.022509e-6,   10447.387839604e0, 1.460726241e0 } ,  
     {    0.020937e-6,    8429.241266467e0, 0.652303414e0 } ,  
     {    0.020322e-6,     419.484643875e0, 3.735430632e0 } ,  
     {    0.024816e-6,   -1194.447010225e0, 1.087136918e0 } ,  
     {    0.025196e-6,    1748.016413067e0, 2.901883301e0 } ,  
     {    0.021691e-6,   14143.495242431e0, 5.952658009e0 } ,  
     {    0.017673e-6,    6812.766815086e0, 3.186129845e0 } ,  
     {    0.022567e-6,    6133.512652857e0, 3.307984806e0 } ,  
     {    0.016155e-6,   10213.285546211e0, 1.331103168e0 } ,  
     {    0.014751e-6,    1349.867409659e0, 4.308933301e0 } ,  
     {    0.015949e-6,    -220.412642439e0, 4.005298270e0 } ,  
     {    0.015974e-6,   -2352.866153772e0, 6.145309371e0 } ,  
     {    0.014223e-6,   17789.845619785e0, 2.104551349e0 } ,  
     {    0.017806e-6,      73.297125859e0, 3.475975097e0 } ,  
     {    0.013671e-6,    -536.804512095e0, 5.971672571e0 } ,  
     {    0.011942e-6,    8031.092263058e0, 2.053414715e0 } ,  
     {    0.014318e-6,   16730.463689596e0, 3.016058075e0 } ,  
     {    0.012462e-6,     103.092774219e0, 1.737438797e0 } ,  
     {    0.010962e-6,       3.590428652e0, 2.196567739e0 } ,  
     {    0.015078e-6,   19651.048481098e0, 3.969480770e0 } ,  
     {    0.010396e-6,     951.718406251e0, 5.717799605e0 } ,  
     {    0.011707e-6,   -4705.732307544e0, 2.654125618e0 } ,  
     {    0.010453e-6,    5863.591206116e0, 1.913704550e0 } ,  
     {    0.012420e-6,    4690.479836359e0, 4.734090399e0 } ,  
     {    0.011847e-6,    5643.178563677e0, 5.489005403e0 } ,  
     {    0.008610e-6,    3340.612426700e0, 3.661698944e0 } ,  
     {    0.011622e-6,    5120.601145584e0, 4.863931876e0 } ,  
     {    0.010825e-6,     553.569402842e0, 0.842715011e0 } ,  
     {    0.008666e-6,    -135.065080035e0, 3.293406547e0 } ,  
     {    0.009963e-6,     149.563197135e0, 4.870690598e0 } ,  
     {    0.009858e-6,    6309.374169791e0, 1.061816410e0 } ,  
     {    0.007959e-6,     316.391869657e0, 2.465042647e0 } ,  
     {    0.010099e-6,     283.859318865e0, 1.942176992e0 } ,  
     {    0.007147e-6,    -242.728603974e0, 3.661486981e0 } ,  
     {    0.007505e-6,    5230.807466803e0, 4.920937029e0 } ,  
     {    0.008323e-6,   11769.853693166e0, 1.229392026e0 } ,  
     {    0.007490e-6,   -6256.777530192e0, 3.658444681e0 } ,  
     {    0.009370e-6,  149854.400134205e0, 0.673880395e0 } ,  
     {    0.007117e-6,      38.027672636e0, 5.294249518e0 } ,  
     {    0.007857e-6,   12168.002696575e0, 0.525733528e0 } ,  
     {    0.007019e-6,    6206.809778716e0, 0.837688810e0 } ,  
     {    0.006056e-6,     955.599741609e0, 4.194535082e0 } ,  
     {    0.008107e-6,   13367.972631107e0, 3.793235253e0 } ,  
     {    0.006731e-6,    5650.292110678e0, 5.639906583e0 } ,  
     {    0.007332e-6,      36.648562930e0, 0.114858677e0 } ,  
     {    0.006366e-6,    4164.311989613e0, 2.262081818e0 } ,  
     {    0.006858e-6,    5216.580372801e0, 0.642063318e0 } ,  
     {    0.006919e-6,    6681.224853400e0, 6.018501522e0 } ,  
     {    0.006826e-6,    7632.943259650e0, 3.458654112e0 } ,  
     {    0.005308e-6,   -1592.596013633e0, 2.500382359e0 } ,  
     {    0.005096e-6,   11371.704689758e0, 2.547107806e0 } ,  
     {    0.004841e-6,    5333.900241022e0, 0.437078094e0 } ,  
     {    0.005582e-6,    5966.683980335e0, 2.246174308e0 } ,  
     {    0.006304e-6,   11926.254413669e0, 2.512929171e0 } ,  
     {    0.006603e-6,   23581.258177318e0, 5.393136889e0 } ,  
     {    0.005123e-6,      -1.484472708e0, 2.999641028e0 } ,  
     {    0.004648e-6,    1589.072895284e0, 1.275847090e0 } ,  
     {    0.005119e-6,    6438.496249426e0, 1.486539246e0 } ,  
     {    0.004521e-6,    4292.330832950e0, 6.140635794e0 } ,  
     {    0.005680e-6,   23013.539539587e0, 4.557814849e0 } ,  
     {    0.005488e-6,      -3.455808046e0, 0.090675389e0 } ,  
     {    0.004193e-6,    7234.794256242e0, 4.869091389e0 } ,  
     {    0.003742e-6,    7238.675591600e0, 4.691976180e0 } ,  
     {    0.004148e-6,    -110.206321219e0, 3.016173439e0 } ,  
     {    0.004553e-6,   11499.656222793e0, 5.554998314e0 } ,  
     {    0.004892e-6,    5436.993015240e0, 1.475415597e0 } ,  
     {    0.004044e-6,    4732.030627343e0, 1.398784824e0 } ,  
     {    0.004164e-6,   12491.370101415e0, 5.650931916e0 } ,  
     {    0.004349e-6,   11513.883316794e0, 2.181745369e0 } ,  
     {    0.003919e-6,   12528.018664345e0, 5.823319737e0 } ,  
     {    0.003129e-6,    6836.645252834e0, 0.003844094e0 } ,  
     {    0.004080e-6,   -7058.598461315e0, 3.690360123e0 } ,  
     {    0.003270e-6,      76.266071276e0, 1.517189902e0 } ,  
     {    0.002954e-6,    6283.143160294e0, 4.447203799e0 } ,  
     {    0.002872e-6,      28.449187468e0, 1.158692983e0 } ,  
     {    0.002881e-6,     735.876513532e0, 0.349250250e0 } ,  
     {    0.003279e-6,    5849.364112115e0, 4.893384368e0 } ,  
     {    0.003625e-6,    6209.778724132e0, 1.473760578e0 } ,  
     {    0.003074e-6,     949.175608970e0, 5.185878737e0 } ,  
     {    0.002775e-6,    9917.696874510e0, 1.030026325e0 } ,  
     {    0.002646e-6,   10973.555686350e0, 3.918259169e0 } ,  
     {    0.002575e-6,   25132.303399966e0, 6.109659023e0 } ,  
     {    0.003500e-6,     263.083923373e0, 1.892100742e0 } ,  
     {    0.002740e-6,   18319.536584880e0, 4.320519510e0 } ,  
     {    0.002464e-6,     202.253395174e0, 4.698203059e0 } ,  
     {    0.002409e-6,       2.542797281e0, 5.325009315e0 } ,  
     {    0.003354e-6,  -90955.551694697e0, 1.942656623e0 } ,  
     {    0.002296e-6,    6496.374945429e0, 5.061810696e0 } ,  
     {    0.003002e-6,    6172.869528772e0, 2.797822767e0 } ,  
     {    0.003202e-6,   27511.467873537e0, 0.531673101e0 } ,  
     {    0.002954e-6,   -6283.008539689e0, 4.533471191e0 } ,  
     {    0.002353e-6,     639.897286314e0, 3.734548088e0 } ,  
     {    0.002401e-6,   16200.772724501e0, 2.605547070e0 } ,  
     {    0.003053e-6,  233141.314403759e0, 3.029030662e0 } ,  
     {    0.003024e-6,   83286.914269554e0, 2.355556099e0 } ,  
     {    0.002863e-6,   17298.182327326e0, 5.240963796e0 } ,  
     {    0.002103e-6,   -7079.373856808e0, 5.756641637e0 } ,  
     {    0.002303e-6,   83996.847317911e0, 2.013686814e0 } ,  
     {    0.002303e-6,   18073.704938650e0, 1.089100410e0 } ,  
     {    0.002381e-6,      63.735898303e0, 0.759188178e0 } ,  
     {    0.002493e-6,    6386.168624210e0, 0.645026535e0 } ,  
     {    0.002366e-6,       3.932153263e0, 6.215885448e0 } ,  
     {    0.002169e-6,   11015.106477335e0, 4.845297676e0 } ,  
     {    0.002397e-6,    6243.458341645e0, 3.809290043e0 } ,  
     {    0.002183e-6,    1162.474704408e0, 6.179611691e0 } ,  
     {    0.002353e-6,    6246.427287062e0, 4.781719760e0 } ,  
     {    0.002199e-6,    -245.831646229e0, 5.956152284e0 } ,  
     {    0.001729e-6,    3894.181829542e0, 1.264976635e0 } ,  
     {    0.001896e-6,   -3128.388765096e0, 4.914231596e0 } ,  
     {    0.002085e-6,      35.164090221e0, 1.405158503e0 } ,  
     {    0.002024e-6,   14712.317116458e0, 2.752035928e0 } ,  
     {    0.001737e-6,    6290.189396992e0, 5.280820144e0 } ,  
     {    0.002229e-6,     491.557929457e0, 1.571007057e0 } ,  
     {    0.001602e-6,   14314.168113050e0, 4.203664806e0 } ,  
     {    0.002186e-6,     454.909366527e0, 1.402101526e0 } ,  
     {    0.001897e-6,   22483.848574493e0, 4.167932508e0 } ,  
     {    0.001825e-6,   -3738.761430108e0, 0.545828785e0 } ,  
     {    0.001894e-6,    1052.268383188e0, 5.817167450e0 } ,  
     {    0.001421e-6,      20.355319399e0, 2.419886601e0 } ,  
     {    0.001408e-6,   10984.192351700e0, 2.732084787e0 } ,  
     {    0.001847e-6,   10873.986030480e0, 2.903477885e0 } ,  
     {    0.001391e-6,   -8635.942003763e0, 0.593891500e0 } ,  
     {    0.001388e-6,      -7.046236698e0, 1.166145902e0 } ,  
     {    0.001810e-6,  -88860.057071188e0, 0.487355242e0 } ,  
     {    0.001288e-6,   -1990.745017041e0, 3.913022880e0 } ,  
     {    0.001297e-6,   23543.230504682e0, 3.063805171e0 } ,  
     {    0.001335e-6,    -266.607041722e0, 3.995764039e0 } ,  
     {    0.001376e-6,   10969.965257698e0, 5.152914309e0 } ,  
     {    0.001745e-6,  244287.600007027e0, 3.626395673e0 } ,  
     {    0.001649e-6,   31441.677569757e0, 1.952049260e0 } ,  
     {    0.001416e-6,    9225.539273283e0, 4.996408389e0 } ,  
     {    0.001238e-6,    4804.209275927e0, 5.503379738e0 } ,  
     {    0.001472e-6,    4590.910180489e0, 4.164913291e0 } ,  
     {    0.001169e-6,    6040.347246017e0, 5.841719038e0 } ,  
     {    0.001039e-6,    5540.085789459e0, 2.769753519e0 } ,  
     {    0.001004e-6,    -170.672870619e0, 0.755008103e0 } ,  
     {    0.001284e-6,   10575.406682942e0, 5.306538209e0 } ,  
     {    0.001278e-6,      71.812653151e0, 4.713486491e0 } ,  
     {    0.001321e-6,   18209.330263660e0, 2.624866359e0 } ,  
     {    0.001297e-6,   21228.392023546e0, 0.382603541e0 } ,  
     {    0.000954e-6,    6282.095528923e0, 0.882213514e0 } ,  
     {    0.001145e-6,    6058.731054289e0, 1.169483931e0 } ,  
     {    0.000979e-6,    5547.199336460e0, 5.448375984e0 } ,  
     {    0.000987e-6,   -6262.300454499e0, 2.656486959e0 } ,  
     {    0.001070e-6, -154717.609887482e0, 1.827624012e0 } ,  
     {    0.000991e-6,    4701.116501708e0, 4.387001801e0 } ,  
     {    0.001155e-6,     -14.227094002e0, 3.042700750e0 } ,  
     {    0.001176e-6,     277.034993741e0, 3.335519004e0 } ,  
     {    0.000890e-6,   13916.019109642e0, 5.601498297e0 } ,  
     {    0.000884e-6,   -1551.045222648e0, 1.088831705e0 } ,  
     {    0.000876e-6,    5017.508371365e0, 3.969902609e0 } ,  
     {    0.000806e-6,   15110.466119866e0, 5.142876744e0 } ,  
     {    0.000773e-6,   -4136.910433516e0, 0.022067765e0 } ,  
     {    0.001077e-6,     175.166059800e0, 1.844913056e0 } ,  
     {    0.000954e-6,   -6284.056171060e0, 0.968480906e0 } ,  
     {    0.000737e-6,    5326.786694021e0, 4.923831588e0 } ,  
     {    0.000845e-6,    -433.711737877e0, 4.749245231e0 } ,  
     {    0.000819e-6,    8662.240323563e0, 5.991247817e0 } ,  
     {    0.000852e-6,     199.072001436e0, 2.189604979e0 } ,  
     {    0.000723e-6,   17256.631536341e0, 6.068719637e0 } ,  
     {    0.000940e-6,    6037.244203762e0, 6.197428148e0 } ,  
     {    0.000885e-6,   11712.955318231e0, 3.280414875e0 } ,  
     {    0.000706e-6,   12559.038152982e0, 2.824848947e0 } ,  
     {    0.000732e-6,    2379.164473572e0, 2.501813417e0 } ,  
     {    0.000764e-6,   -6127.655450557e0, 2.236346329e0 } ,  
     {    0.000908e-6,     131.541961686e0, 2.521257490e0 } ,  
     {    0.000907e-6,   35371.887265976e0, 3.370195967e0 } ,  
     {    0.000673e-6,    1066.495477190e0, 3.876512374e0 } ,  
     {    0.000814e-6,   17654.780539750e0, 4.627122566e0 } ,  
     {    0.000630e-6,      36.027866677e0, 0.156368499e0 } ,  
     {    0.000798e-6,     515.463871093e0, 5.151962502e0 } ,  
     {    0.000798e-6,     148.078724426e0, 5.909225055e0 } ,  
     {    0.000806e-6,     309.278322656e0, 6.054064447e0 } ,  
     {    0.000607e-6,     -39.617508346e0, 2.839021623e0 } ,  
     {    0.000601e-6,     412.371096874e0, 3.984225404e0 } ,  
     {    0.000646e-6,   11403.676995575e0, 3.852959484e0 } ,  
     {    0.000704e-6,   13521.751441591e0, 2.300991267e0 } ,  
     {    0.000603e-6,  -65147.619767937e0, 4.140083146e0 } ,  
     {    0.000609e-6,   10177.257679534e0, 0.437122327e0 } ,  
     {    0.000631e-6,    5767.611978898e0, 4.026532329e0 } ,  
     {    0.000576e-6,   11087.285125918e0, 4.760293101e0 } ,  
     {    0.000674e-6,   14945.316173554e0, 6.270510511e0 } ,  
     {    0.000726e-6,    5429.879468239e0, 6.039606892e0 } ,  
     {    0.000710e-6,   28766.924424484e0, 5.672617711e0 } ,  
     {    0.000647e-6,   11856.218651625e0, 3.397132627e0 } ,  
     {    0.000678e-6,   -5481.254918868e0, 6.249666675e0 } ,  
     {    0.000618e-6,   22003.914634870e0, 2.466427018e0 } ,  
     {    0.000738e-6,    6134.997125565e0, 2.242668890e0 } ,  
     {    0.000660e-6,     625.670192312e0, 5.864091907e0 } ,  
     {    0.000694e-6,    3496.032826134e0, 2.668309141e0 } ,  
     {    0.000531e-6,    6489.261398429e0, 1.681888780e0 } ,  
     {    0.000611e-6, -143571.324284214e0, 2.424978312e0 } ,  
     {    0.000575e-6,   12043.574281889e0, 4.216492400e0 } ,  
     {    0.000553e-6,   12416.588502848e0, 4.772158039e0 } ,  
     {    0.000689e-6,    4686.889407707e0, 6.224271088e0 } ,  
     {    0.000495e-6,    7342.457780181e0, 3.817285811e0 } ,  
     {    0.000567e-6,    3634.621024518e0, 1.649264690e0 } ,  
     {    0.000515e-6,   18635.928454536e0, 3.945345892e0 } ,  
     {    0.000486e-6,    -323.505416657e0, 4.061673868e0 } ,  
     {    0.000662e-6,   25158.601719765e0, 1.794058369e0 } ,  
     {    0.000509e-6,     846.082834751e0, 3.053874588e0 } ,  
     {    0.000472e-6,  -12569.674818332e0, 5.112133338e0 } ,  
     {    0.000461e-6,    6179.983075773e0, 0.513669325e0 } ,  
     {    0.000641e-6,   83467.156352816e0, 3.210727723e0 } ,  
     {    0.000520e-6,   10344.295065386e0, 2.445597761e0 } ,  
     {    0.000493e-6,   18422.629359098e0, 1.676939306e0 } ,  
     {    0.000478e-6,    1265.567478626e0, 5.487314569e0 } ,  
     {    0.000472e-6,     -18.159247265e0, 1.999707589e0 } ,  
     {    0.000559e-6,   11190.377900137e0, 5.783236356e0 } ,  
     {    0.000494e-6,    9623.688276691e0, 3.022645053e0 } ,  
     {    0.000463e-6,    5739.157790895e0, 1.411223013e0 } ,  
     {    0.000432e-6,   16858.482532933e0, 1.179256434e0 } ,  
     {    0.000574e-6,   72140.628666286e0, 1.758191830e0 } ,  
     {    0.000484e-6,   17267.268201691e0, 3.290589143e0 } ,  
     {    0.000550e-6,    4907.302050146e0, 0.864024298e0 } ,  
     {    0.000399e-6,      14.977853527e0, 2.094441910e0 } ,  
     {    0.000491e-6,     224.344795702e0, 0.878372791e0 } ,  
     {    0.000432e-6,   20426.571092422e0, 6.003829241e0 } ,  
     {    0.000481e-6,    5749.452731634e0, 4.309591964e0 } ,  
     {    0.000480e-6,    5757.317038160e0, 1.142348571e0 } ,  
     {    0.000485e-6,    6702.560493867e0, 0.210580917e0 } ,  
     {    0.000426e-6,    6055.549660552e0, 4.274476529e0 } ,  
     {    0.000480e-6,    5959.570433334e0, 5.031351030e0 } ,  
     {    0.000466e-6,   12562.628581634e0, 4.959581597e0 } ,  
     {    0.000520e-6,   39302.096962196e0, 4.788002889e0 } ,  
     {    0.000458e-6,   12132.439962106e0, 1.880103788e0 } ,  
     {    0.000470e-6,   12029.347187887e0, 1.405611197e0 } ,  
     {    0.000416e-6,   -7477.522860216e0, 1.082356330e0 } ,  
     {    0.000449e-6,   11609.862544012e0, 4.179989585e0 } ,  
     {    0.000465e-6,   17253.041107690e0, 0.353496295e0 } ,  
     {    0.000362e-6,   -4535.059436924e0, 1.583849576e0 } ,  
     {    0.000383e-6,   21954.157609398e0, 3.747376371e0 } ,  
     {    0.000389e-6,      17.252277143e0, 1.395753179e0 } ,  
     {    0.000331e-6,   18052.929543158e0, 0.566790582e0 } ,  
     {    0.000430e-6,   13517.870106233e0, 0.685827538e0 } ,  
     {    0.000368e-6,   -5756.908003246e0, 0.731374317e0 } ,  
     {    0.000330e-6,   10557.594160824e0, 3.710043680e0 } ,  
     {    0.000332e-6,   20199.094959633e0, 1.652901407e0 } ,  
     {    0.000384e-6,   11933.367960670e0, 5.827781531e0 } ,  
     {    0.000387e-6,   10454.501386605e0, 2.541182564e0 } ,  
     {    0.000325e-6,   15671.081759407e0, 2.178850542e0 } ,  
     {    0.000318e-6,     138.517496871e0, 2.253253037e0 } ,  
     {    0.000305e-6,    9388.005909415e0, 0.578340206e0 } ,  
     {    0.000352e-6,    5749.861766548e0, 3.000297967e0 } ,  
     {    0.000311e-6,    6915.859589305e0, 1.693574249e0 } ,  
     {    0.000297e-6,   24072.921469776e0, 1.997249392e0 } ,  
     {    0.000363e-6,    -640.877607382e0, 5.071820966e0 } ,  
     {    0.000323e-6,   12592.450019783e0, 1.072262823e0 } ,  
     {    0.000341e-6,   12146.667056108e0, 4.700657997e0 } ,  
     {    0.000290e-6,    9779.108676125e0, 1.812320441e0 } ,  
     {    0.000342e-6,    6132.028180148e0, 4.322238614e0 } ,  
     {    0.000329e-6,    6268.848755990e0, 3.033827743e0 } ,  
     {    0.000374e-6,   17996.031168222e0, 3.388716544e0 } ,  
     {    0.000285e-6,    -533.214083444e0, 4.687313233e0 } ,  
     {    0.000338e-6,    6065.844601290e0, 0.877776108e0 } ,  
     {    0.000276e-6,      24.298513841e0, 0.770299429e0 } ,  
     {    0.000336e-6,   -2388.894020449e0, 5.353796034e0 } ,  
     {    0.000290e-6,    3097.883822726e0, 4.075291557e0 } ,  
     {    0.000318e-6,     709.933048357e0, 5.941207518e0 } ,  
     {    0.000271e-6,   13095.842665077e0, 3.208912203e0 } ,  
     {    0.000331e-6,    6073.708907816e0, 4.007881169e0 } ,  
     {    0.000292e-6,     742.990060533e0, 2.714333592e0 } ,  
     {    0.000362e-6,   29088.811415985e0, 3.215977013e0 } ,  
     {    0.000280e-6,   12359.966151546e0, 0.710872502e0 } ,  
     {    0.000267e-6,   10440.274292604e0, 4.730108488e0 } ,  
     {    0.000262e-6,     838.969287750e0, 1.327720272e0 } ,  
     {    0.000250e-6,   16496.361396202e0, 0.898769761e0 } ,  
     {    0.000325e-6,   20597.243963041e0, 0.180044365e0 } ,  
     {    0.000268e-6,    6148.010769956e0, 5.152666276e0 } ,  
     {    0.000284e-6,    5636.065016677e0, 5.655385808e0 } ,  
     {    0.000301e-6,    6080.822454817e0, 2.135396205e0 } ,  
     {    0.000294e-6,    -377.373607916e0, 3.708784168e0 } ,  
     {    0.000236e-6,    2118.763860378e0, 1.733578756e0 } ,  
     {    0.000234e-6,    5867.523359379e0, 5.575209112e0 } ,  
     {    0.000268e-6, -226858.238553767e0, 0.069432392e0 } ,  
     {    0.000265e-6,  167283.761587465e0, 4.369302826e0 } ,  
     {    0.000280e-6,   28237.233459389e0, 5.304829118e0 } ,  
     {    0.000292e-6,   12345.739057544e0, 4.096094132e0 } ,  
     {    0.000223e-6,   19800.945956225e0, 3.069327406e0 } ,  
     {    0.000301e-6,   43232.306658416e0, 6.205311188e0 } ,  
     {    0.000264e-6,   18875.525869774e0, 1.417263408e0 } ,  
     {    0.000304e-6,   -1823.175188677e0, 3.409035232e0 } ,  
     {    0.000301e-6,     109.945688789e0, 0.510922054e0 } ,  
     {    0.000260e-6,     813.550283960e0, 2.389438934e0 } ,  
     {    0.000299e-6,  316428.228673312e0, 5.384595078e0 } ,  
     {    0.000211e-6,    5756.566278634e0, 3.789392838e0 } ,  
     {    0.000209e-6,    5750.203491159e0, 1.661943545e0 } ,  
     {    0.000240e-6,   12489.885628707e0, 5.684549045e0 } ,  
     {    0.000216e-6,    6303.851245484e0, 3.862942261e0 } ,  
     {    0.000203e-6,    1581.959348283e0, 5.549853589e0 } ,  
     {    0.000200e-6,    5642.198242609e0, 1.016115785e0 } ,  
     {    0.000197e-6,     -70.849445304e0, 4.690702525e0 } ,  
     {    0.000227e-6,    6287.008003254e0, 2.911891613e0 } ,  
     {    0.000197e-6,     533.623118358e0, 1.048982898e0 } ,  
     {    0.000205e-6,   -6279.485421340e0, 1.829362730e0 } ,  
     {    0.000209e-6,  -10988.808157535e0, 2.636140084e0 } ,  
     {    0.000208e-6,    -227.526189440e0, 4.127883842e0 } ,  
     {    0.000191e-6,     415.552490612e0, 4.401165650e0 } ,  
     {    0.000190e-6,   29296.615389579e0, 4.175658539e0 } ,  
     {    0.000264e-6,   66567.485864652e0, 4.601102551e0 } ,  
     {    0.000256e-6,   -3646.350377354e0, 0.506364778e0 } ,  
     {    0.000188e-6,   13119.721102825e0, 2.032195842e0 } ,  
     {    0.000185e-6,    -209.366942175e0, 4.694756586e0 } ,  
     {    0.000198e-6,   25934.124331089e0, 3.832703118e0 } ,  
     {    0.000195e-6,    4061.219215394e0, 3.308463427e0 } ,  
     {    0.000234e-6,    5113.487598583e0, 1.716090661e0 } ,  
     {    0.000188e-6,    1478.866574064e0, 5.686865780e0 } ,  
     {    0.000222e-6,   11823.161639450e0, 1.942386641e0 } ,  
     {    0.000181e-6,   10770.893256262e0, 1.999482059e0 } ,  
     {    0.000171e-6,    6546.159773364e0, 1.182807992e0 } ,  
     {    0.000206e-6,      70.328180442e0, 5.934076062e0 } ,  
     {    0.000169e-6,   20995.392966449e0, 2.169080622e0 } ,  
     {    0.000191e-6,   10660.686935042e0, 5.405515999e0 } ,  
     {    0.000228e-6,   33019.021112205e0, 4.656985514e0 } ,  
     {    0.000184e-6,   -4933.208440333e0, 3.327476868e0 } ,  
     {    0.000220e-6,    -135.625325010e0, 1.765430262e0 } ,  
     {    0.000166e-6,   23141.558382925e0, 3.454132746e0 } ,  
     {    0.000191e-6,    6144.558353121e0, 5.020393445e0 } ,  
     {    0.000180e-6,    6084.003848555e0, 0.602182191e0 } ,  
     {    0.000163e-6,   17782.732072784e0, 4.960593133e0 } ,  
     {    0.000225e-6,   16460.333529525e0, 2.596451817e0 } ,  
     {    0.000222e-6,    5905.702242076e0, 3.731990323e0 } ,  
     {    0.000204e-6,     227.476132789e0, 5.636192701e0 } ,  
     {    0.000159e-6,   16737.577236597e0, 3.600691544e0 } ,  
     {    0.000200e-6,    6805.653268085e0, 0.868220961e0 } ,  
     {    0.000187e-6,   11919.140866668e0, 2.629456641e0 } ,  
     {    0.000161e-6,     127.471796607e0, 2.862574720e0 } ,  
     {    0.000205e-6,    6286.666278643e0, 1.742882331e0 } ,  
     {    0.000189e-6,     153.778810485e0, 4.812372643e0 } ,  
     {    0.000168e-6,   16723.350142595e0, 0.027860588e0 } ,  
     {    0.000149e-6,   11720.068865232e0, 0.659721876e0 } ,  
     {    0.000189e-6,    5237.921013804e0, 5.245313000e0 } ,  
     {    0.000143e-6,    6709.674040867e0, 4.317625647e0 } ,  
     {    0.000146e-6,    4487.817406270e0, 4.815297007e0 } ,  
     {    0.000144e-6,    -664.756045130e0, 5.381366880e0 } ,  
     {    0.000175e-6,    5127.714692584e0, 4.728443327e0 } ,  
     {    0.000162e-6,    6254.626662524e0, 1.435132069e0 } ,  
     {    0.000187e-6,   47162.516354635e0, 1.354371923e0 } ,  
     {    0.000146e-6,   11080.171578918e0, 3.369695406e0 } ,  
     {    0.000180e-6,    -348.924420448e0, 2.490902145e0 } ,  
     {    0.000148e-6,     151.047669843e0, 3.799109588e0 } ,  
     {    0.000157e-6,    6197.248551160e0, 1.284375887e0 } ,  
     {    0.000167e-6,     146.594251718e0, 0.759969109e0 } ,  
     {    0.000133e-6,   -5331.357443741e0, 5.409701889e0 } ,  
     {    0.000154e-6,      95.979227218e0, 3.366890614e0 } ,  
     {    0.000148e-6,   -6418.140930027e0, 3.384104996e0 } ,  
     {    0.000128e-6,   -6525.804453965e0, 3.803419985e0 } ,  
     {    0.000130e-6,   11293.470674356e0, 0.939039445e0 } ,  
     {    0.000152e-6,   -5729.506447149e0, 0.734117523e0 } ,  
     {    0.000138e-6,     210.117701700e0, 2.564216078e0 } ,  
     {    0.000123e-6,    6066.595360816e0, 4.517099537e0 } ,  
     {    0.000140e-6,   18451.078546566e0, 0.642049130e0 } ,  
     {    0.000126e-6,   11300.584221356e0, 3.485280663e0 } ,  
     {    0.000119e-6,   10027.903195729e0, 3.217431161e0 } ,  
     {    0.000151e-6,    4274.518310832e0, 4.404359108e0 } ,  
     {    0.000117e-6,    6072.958148291e0, 0.366324650e0 } ,  
     {    0.000165e-6,   -7668.637425143e0, 4.298212528e0 } ,  
     {    0.000117e-6,   -6245.048177356e0, 5.379518958e0 } ,  
     {    0.000130e-6,   -5888.449964932e0, 4.527681115e0 } ,  
     {    0.000121e-6,    -543.918059096e0, 6.109429504e0 } ,  
     {    0.000162e-6,    9683.594581116e0, 5.720092446e0 } ,  
     {    0.000141e-6,    6219.339951688e0, 0.679068671e0 } ,  
     {    0.000118e-6,   22743.409379516e0, 4.881123092e0 } ,  
     {    0.000129e-6,    1692.165669502e0, 0.351407289e0 } ,  
     {    0.000126e-6,    5657.405657679e0, 5.146592349e0 } ,  
     {    0.000114e-6,     728.762966531e0, 0.520791814e0 } ,  
     {    0.000120e-6,      52.596639600e0, 0.948516300e0 } ,  
     {    0.000115e-6,      65.220371012e0, 3.504914846e0 } ,  
     {    0.000126e-6,    5881.403728234e0, 5.577502482e0 } ,  
     {    0.000158e-6,  163096.180360983e0, 2.957128968e0 } ,  
     {    0.000134e-6,   12341.806904281e0, 2.598576764e0 } ,  
     {    0.000151e-6,   16627.370915377e0, 3.985702050e0 } ,  
     {    0.000109e-6,    1368.660252845e0, 0.014730471e0 } ,  
     {    0.000131e-6,    6211.263196841e0, 0.085077024e0 } ,  
     {    0.000146e-6,    5792.741760812e0, 0.708426604e0 } ,  
     {    0.000146e-6,     -77.750543984e0, 3.121576600e0 } ,  
     {    0.000107e-6,    5341.013788022e0, 0.288231904e0 } ,  
     {    0.000138e-6,    6281.591377283e0, 2.797450317e0 } ,  
     {    0.000113e-6,   -6277.552925684e0, 2.788904128e0 } ,  
     {    0.000115e-6,    -525.758811831e0, 5.895222200e0 } ,  
     {    0.000138e-6,    6016.468808270e0, 6.096188999e0 } ,  
     {    0.000139e-6,   23539.707386333e0, 2.028195445e0 } ,  
     {    0.000146e-6,   -4176.041342449e0, 4.660008502e0 } ,  
     {    0.000107e-6,   16062.184526117e0, 4.066520001e0 } ,  
     {    0.000142e-6,   83783.548222473e0, 2.936315115e0 } ,  
     {    0.000128e-6,    9380.959672717e0, 3.223844306e0 } ,  
     {    0.000135e-6,    6205.325306007e0, 1.638054048e0 } ,  
     {    0.000101e-6,    2699.734819318e0, 5.481603249e0 } ,  
     {    0.000104e-6,    -568.821874027e0, 2.205734493e0 } ,  
     {    0.000103e-6,    6321.103522627e0, 2.440421099e0 } ,  
     {    0.000119e-6,    6321.208885629e0, 2.547496264e0 } ,  
     {    0.000138e-6,    1975.492545856e0, 2.314608466e0 } ,  
     {    0.000121e-6,     137.033024162e0, 4.539108237e0 } ,  
     {    0.000123e-6,   19402.796952817e0, 4.538074405e0 } ,  
     {    0.000119e-6,   22805.735565994e0, 2.869040566e0 } ,  
     {    0.000133e-6,   64471.991241142e0, 6.056405489e0 } ,  
     {    0.000129e-6,     -85.827298831e0, 2.540635083e0 } ,  
     {    0.000131e-6,   13613.804277336e0, 4.005732868e0 } ,  
     {    0.000104e-6,    9814.604100291e0, 1.959967212e0 } ,  
     {    0.000112e-6,   16097.679950283e0, 3.589026260e0 } ,  
     {    0.000123e-6,    2107.034507542e0, 1.728627253e0 } ,  
     {    0.000121e-6,   36949.230808424e0, 6.072332087e0 } ,  
     {    0.000108e-6,  -12539.853380183e0, 3.716133846e0 } ,  
     {    0.000113e-6,   -7875.671863624e0, 2.725771122e0 } ,  
     {    0.000109e-6,    4171.425536614e0, 4.033338079e0 } ,  
     {    0.000101e-6,    6247.911759770e0, 3.441347021e0 } ,  
     {    0.000113e-6,    7330.728427345e0, 0.656372122e0 } ,  
     {    0.000113e-6,   51092.726050855e0, 2.791483066e0 } ,  
     {    0.000106e-6,    5621.842923210e0, 1.815323326e0 } ,  
     {    0.000101e-6,     111.430161497e0, 5.711033677e0 } ,  
     {    0.000103e-6,     909.818733055e0, 2.812745443e0 } ,  
     {    0.000101e-6,    1790.642637886e0, 1.965746028e0 } ,  
  
/*  
 * T  
 */  
  
     {  102.156724e-6,    6283.075849991e0, 4.249032005e0 } ,  
     {    1.706807e-6,   12566.151699983e0, 4.205904248e0 } ,  
     {    0.269668e-6,     213.299095438e0, 3.400290479e0 } ,  
     {    0.265919e-6,     529.690965095e0, 5.836047367e0 } ,  
     {    0.210568e-6,      -3.523118349e0, 6.262738348e0 } ,  
     {    0.077996e-6,    5223.693919802e0, 4.670344204e0 } ,  
     {    0.054764e-6,    1577.343542448e0, 4.534800170e0 } ,  
     {    0.059146e-6,      26.298319800e0, 1.083044735e0 } ,  
     {    0.034420e-6,    -398.149003408e0, 5.980077351e0 } ,  
     {    0.032088e-6,   18849.227549974e0, 4.162913471e0 } ,  
     {    0.033595e-6,    5507.553238667e0, 5.980162321e0 } ,  
     {    0.029198e-6,    5856.477659115e0, 0.623811863e0 } ,  
     {    0.027764e-6,     155.420399434e0, 3.745318113e0 } ,  
     {    0.025190e-6,    5746.271337896e0, 2.980330535e0 } ,  
     {    0.022997e-6,    -796.298006816e0, 1.174411803e0 } ,  
     {    0.024976e-6,    5760.498431898e0, 2.467913690e0 } ,  
     {    0.021774e-6,     206.185548437e0, 3.854787540e0 } ,  
     {    0.017925e-6,    -775.522611324e0, 1.092065955e0 } ,  
     {    0.013794e-6,     426.598190876e0, 2.699831988e0 } ,  
     {    0.013276e-6,    6062.663207553e0, 5.845801920e0 } ,  
     {    0.011774e-6,   12036.460734888e0, 2.292832062e0 } ,  
     {    0.012869e-6,    6076.890301554e0, 5.333425680e0 } ,  
     {    0.012152e-6,    1059.381930189e0, 6.222874454e0 } ,  
     {    0.011081e-6,      -7.113547001e0, 5.154724984e0 } ,  
     {    0.010143e-6,    4694.002954708e0, 4.044013795e0 } ,  
     {    0.009357e-6,    5486.777843175e0, 3.416081409e0 } ,  
     {    0.010084e-6,     522.577418094e0, 0.749320262e0 } ,  
     {    0.008587e-6,   10977.078804699e0, 2.777152598e0 } ,  
     {    0.008628e-6,    6275.962302991e0, 4.562060226e0 } ,  
     {    0.008158e-6,    -220.412642439e0, 5.806891533e0 } ,  
     {    0.007746e-6,    2544.314419883e0, 1.603197066e0 } ,  
     {    0.007670e-6,    2146.165416475e0, 3.000200440e0 } ,  
     {    0.007098e-6,      74.781598567e0, 0.443725817e0 } ,  
     {    0.006180e-6,    -536.804512095e0, 1.302642751e0 } ,  
     {    0.005818e-6,    5088.628839767e0, 4.827723531e0 } ,  
     {    0.004945e-6,   -6286.598968340e0, 0.268305170e0 } ,  
     {    0.004774e-6,    1349.867409659e0, 5.808636673e0 } ,  
     {    0.004687e-6,    -242.728603974e0, 5.154890570e0 } ,  
     {    0.006089e-6,    1748.016413067e0, 4.403765209e0 } ,  
     {    0.005975e-6,   -1194.447010225e0, 2.583472591e0 } ,  
     {    0.004229e-6,     951.718406251e0, 0.931172179e0 } ,  
     {    0.005264e-6,     553.569402842e0, 2.336107252e0 } ,  
     {    0.003049e-6,    5643.178563677e0, 1.362634430e0 } ,  
     {    0.002974e-6,    6812.766815086e0, 1.583012668e0 } ,  
     {    0.003403e-6,   -2352.866153772e0, 2.552189886e0 } ,  
     {    0.003030e-6,     419.484643875e0, 5.286473844e0 } ,  
     {    0.003210e-6,      -7.046236698e0, 1.863796539e0 } ,  
     {    0.003058e-6,    9437.762934887e0, 4.226420633e0 } ,  
     {    0.002589e-6,   12352.852604545e0, 1.991935820e0 } ,  
     {    0.002927e-6,    5216.580372801e0, 2.319951253e0 } ,  
     {    0.002425e-6,    5230.807466803e0, 3.084752833e0 } ,  
     {    0.002656e-6,    3154.687084896e0, 2.487447866e0 } ,  
     {    0.002445e-6,   10447.387839604e0, 2.347139160e0 } ,  
     {    0.002990e-6,    4690.479836359e0, 6.235872050e0 } ,  
     {    0.002890e-6,    5863.591206116e0, 0.095197563e0 } ,  
     {    0.002498e-6,    6438.496249426e0, 2.994779800e0 } ,  
     {    0.001889e-6,    8031.092263058e0, 3.569003717e0 } ,  
     {    0.002567e-6,     801.820931124e0, 3.425611498e0 } ,  
     {    0.001803e-6,  -71430.695617928e0, 2.192295512e0 } ,  
     {    0.001782e-6,       3.932153263e0, 5.180433689e0 } ,  
     {    0.001694e-6,   -4705.732307544e0, 4.641779174e0 } ,  
     {    0.001704e-6,   -1592.596013633e0, 3.997097652e0 } ,  
     {    0.001735e-6,    5849.364112115e0, 0.417558428e0 } ,  
     {    0.001643e-6,    8429.241266467e0, 2.180619584e0 } ,  
     {    0.001680e-6,      38.133035638e0, 4.164529426e0 } ,  
     {    0.002045e-6,    7084.896781115e0, 0.526323854e0 } ,  
     {    0.001458e-6,    4292.330832950e0, 1.356098141e0 } ,  
     {    0.001437e-6,      20.355319399e0, 3.895439360e0 } ,  
     {    0.001738e-6,    6279.552731642e0, 0.087484036e0 } ,  
     {    0.001367e-6,   14143.495242431e0, 3.987576591e0 } ,  
     {    0.001344e-6,    7234.794256242e0, 0.090454338e0 } ,  
     {    0.001438e-6,   11499.656222793e0, 0.974387904e0 } ,  
     {    0.001257e-6,    6836.645252834e0, 1.509069366e0 } ,  
     {    0.001358e-6,   11513.883316794e0, 0.495572260e0 } ,  
     {    0.001628e-6,    7632.943259650e0, 4.968445721e0 } ,  
     {    0.001169e-6,     103.092774219e0, 2.838496795e0 } ,  
     {    0.001162e-6,    4164.311989613e0, 3.408387778e0 } ,  
     {    0.001092e-6,    6069.776754553e0, 3.617942651e0 } ,  
     {    0.001008e-6,   17789.845619785e0, 0.286350174e0 } ,  
     {    0.001008e-6,     639.897286314e0, 1.610762073e0 } ,  
     {    0.000918e-6,   10213.285546211e0, 5.532798067e0 } ,  
     {    0.001011e-6,   -6256.777530192e0, 0.661826484e0 } ,  
     {    0.000753e-6,   16730.463689596e0, 3.905030235e0 } ,  
     {    0.000737e-6,   11926.254413669e0, 4.641956361e0 } ,  
     {    0.000694e-6,    3340.612426700e0, 2.111120332e0 } ,  
     {    0.000701e-6,    3894.181829542e0, 2.760823491e0 } ,  
     {    0.000689e-6,    -135.065080035e0, 4.768800780e0 } ,  
     {    0.000700e-6,   13367.972631107e0, 5.760439898e0 } ,  
     {    0.000664e-6,    6040.347246017e0, 1.051215840e0 } ,  
     {    0.000654e-6,    5650.292110678e0, 4.911332503e0 } ,  
     {    0.000788e-6,    6681.224853400e0, 4.699648011e0 } ,  
     {    0.000628e-6,    5333.900241022e0, 5.024608847e0 } ,  
     {    0.000755e-6,    -110.206321219e0, 4.370971253e0 } ,  
     {    0.000628e-6,    6290.189396992e0, 3.660478857e0 } ,  
     {    0.000635e-6,   25132.303399966e0, 4.121051532e0 } ,  
     {    0.000534e-6,    5966.683980335e0, 1.173284524e0 } ,  
     {    0.000543e-6,    -433.711737877e0, 0.345585464e0 } ,  
     {    0.000517e-6,   -1990.745017041e0, 5.414571768e0 } ,  
     {    0.000504e-6,    5767.611978898e0, 2.328281115e0 } ,  
     {    0.000485e-6,    5753.384884897e0, 1.685874771e0 } ,  
     {    0.000463e-6,    7860.419392439e0, 5.297703006e0 } ,  
     {    0.000604e-6,     515.463871093e0, 0.591998446e0 } ,  
     {    0.000443e-6,   12168.002696575e0, 4.830881244e0 } ,  
     {    0.000570e-6,     199.072001436e0, 3.899190272e0 } ,  
     {    0.000465e-6,   10969.965257698e0, 0.476681802e0 } ,  
     {    0.000424e-6,   -7079.373856808e0, 1.112242763e0 } ,  
     {    0.000427e-6,     735.876513532e0, 1.994214480e0 } ,  
     {    0.000478e-6,   -6127.655450557e0, 3.778025483e0 } ,  
     {    0.000414e-6,   10973.555686350e0, 5.441088327e0 } ,  
     {    0.000512e-6,    1589.072895284e0, 0.107123853e0 } ,  
     {    0.000378e-6,   10984.192351700e0, 0.915087231e0 } ,  
     {    0.000402e-6,   11371.704689758e0, 4.107281715e0 } ,  
     {    0.000453e-6,    9917.696874510e0, 1.917490952e0 } ,  
     {    0.000395e-6,     149.563197135e0, 2.763124165e0 } ,  
     {    0.000371e-6,    5739.157790895e0, 3.112111866e0 } ,  
     {    0.000350e-6,   11790.629088659e0, 0.440639857e0 } ,  
     {    0.000356e-6,    6133.512652857e0, 5.444568842e0 } ,  
     {    0.000344e-6,     412.371096874e0, 5.676832684e0 } ,  
     {    0.000383e-6,     955.599741609e0, 5.559734846e0 } ,  
     {    0.000333e-6,    6496.374945429e0, 0.261537984e0 } ,  
     {    0.000340e-6,    6055.549660552e0, 5.975534987e0 } ,  
     {    0.000334e-6,    1066.495477190e0, 2.335063907e0 } ,  
     {    0.000399e-6,   11506.769769794e0, 5.321230910e0 } ,  
     {    0.000314e-6,   18319.536584880e0, 2.313312404e0 } ,  
     {    0.000424e-6,    1052.268383188e0, 1.211961766e0 } ,  
     {    0.000307e-6,      63.735898303e0, 3.169551388e0 } ,  
     {    0.000329e-6,      29.821438149e0, 6.106912080e0 } ,  
     {    0.000357e-6,    6309.374169791e0, 4.223760346e0 } ,  
     {    0.000312e-6,   -3738.761430108e0, 2.180556645e0 } ,  
     {    0.000301e-6,     309.278322656e0, 1.499984572e0 } ,  
     {    0.000268e-6,   12043.574281889e0, 2.447520648e0 } ,  
     {    0.000257e-6,   12491.370101415e0, 3.662331761e0 } ,  
     {    0.000290e-6,     625.670192312e0, 1.272834584e0 } ,  
     {    0.000256e-6,    5429.879468239e0, 1.913426912e0 } ,  
     {    0.000339e-6,    3496.032826134e0, 4.165930011e0 } ,  
     {    0.000283e-6,    3930.209696220e0, 4.325565754e0 } ,  
     {    0.000241e-6,   12528.018664345e0, 3.832324536e0 } ,  
     {    0.000304e-6,    4686.889407707e0, 1.612348468e0 } ,  
     {    0.000259e-6,   16200.772724501e0, 3.470173146e0 } ,  
     {    0.000238e-6,   12139.553509107e0, 1.147977842e0 } ,  
     {    0.000236e-6,    6172.869528772e0, 3.776271728e0 } ,  
     {    0.000296e-6,   -7058.598461315e0, 0.460368852e0 } ,  
     {    0.000306e-6,   10575.406682942e0, 0.554749016e0 } ,  
     {    0.000251e-6,   17298.182327326e0, 0.834332510e0 } ,  
     {    0.000290e-6,    4732.030627343e0, 4.759564091e0 } ,  
     {    0.000261e-6,    5884.926846583e0, 0.298259862e0 } ,  
     {    0.000249e-6,    5547.199336460e0, 3.749366406e0 } ,  
     {    0.000213e-6,   11712.955318231e0, 5.415666119e0 } ,  
     {    0.000223e-6,    4701.116501708e0, 2.703203558e0 } ,  
     {    0.000268e-6,    -640.877607382e0, 0.283670793e0 } ,  
     {    0.000209e-6,    5636.065016677e0, 1.238477199e0 } ,  
     {    0.000193e-6,   10177.257679534e0, 1.943251340e0 } ,  
     {    0.000182e-6,    6283.143160294e0, 2.456157599e0 } ,  
     {    0.000184e-6,    -227.526189440e0, 5.888038582e0 } ,  
     {    0.000182e-6,   -6283.008539689e0, 0.241332086e0 } ,  
     {    0.000228e-6,   -6284.056171060e0, 2.657323816e0 } ,  
     {    0.000166e-6,    7238.675591600e0, 5.930629110e0 } ,  
     {    0.000167e-6,    3097.883822726e0, 5.570955333e0 } ,  
     {    0.000159e-6,    -323.505416657e0, 5.786670700e0 } ,  
     {    0.000154e-6,   -4136.910433516e0, 1.517805532e0 } ,  
     {    0.000176e-6,   12029.347187887e0, 3.139266834e0 } ,  
     {    0.000167e-6,   12132.439962106e0, 3.556352289e0 } ,  
     {    0.000153e-6,     202.253395174e0, 1.463313961e0 } ,  
     {    0.000157e-6,   17267.268201691e0, 1.586837396e0 } ,  
     {    0.000142e-6,   83996.847317911e0, 0.022670115e0 } ,  
     {    0.000152e-6,   17260.154654690e0, 0.708528947e0 } ,  
     {    0.000144e-6,    6084.003848555e0, 5.187075177e0 } ,  
     {    0.000135e-6,    5756.566278634e0, 1.993229262e0 } ,  
     {    0.000134e-6,    5750.203491159e0, 3.457197134e0 } ,  
     {    0.000144e-6,    5326.786694021e0, 6.066193291e0 } ,  
     {    0.000160e-6,   11015.106477335e0, 1.710431974e0 } ,  
     {    0.000133e-6,    3634.621024518e0, 2.836451652e0 } ,  
     {    0.000134e-6,   18073.704938650e0, 5.453106665e0 } ,  
     {    0.000134e-6,    1162.474704408e0, 5.326898811e0 } ,  
     {    0.000128e-6,    5642.198242609e0, 2.511652591e0 } ,  
     {    0.000160e-6,     632.783739313e0, 5.628785365e0 } ,  
     {    0.000132e-6,   13916.019109642e0, 0.819294053e0 } ,  
     {    0.000122e-6,   14314.168113050e0, 5.677408071e0 } ,  
     {    0.000125e-6,   12359.966151546e0, 5.251984735e0 } ,  
     {    0.000121e-6,    5749.452731634e0, 2.210924603e0 } ,  
     {    0.000136e-6,    -245.831646229e0, 1.646502367e0 } ,  
     {    0.000120e-6,    5757.317038160e0, 3.240883049e0 } ,  
     {    0.000134e-6,   12146.667056108e0, 3.059480037e0 } ,  
     {    0.000137e-6,    6206.809778716e0, 1.867105418e0 } ,  
     {    0.000141e-6,   17253.041107690e0, 2.069217456e0 } ,  
     {    0.000129e-6,   -7477.522860216e0, 2.781469314e0 } ,  
     {    0.000116e-6,    5540.085789459e0, 4.281176991e0 } ,  
     {    0.000116e-6,    9779.108676125e0, 3.320925381e0 } ,  
     {    0.000129e-6,    5237.921013804e0, 3.497704076e0 } ,  
     {    0.000113e-6,    5959.570433334e0, 0.983210840e0 } ,  
     {    0.000122e-6,    6282.095528923e0, 2.674938860e0 } ,  
     {    0.000140e-6,     -11.045700264e0, 4.957936982e0 } ,  
     {    0.000108e-6,   23543.230504682e0, 1.390113589e0 } ,  
     {    0.000106e-6,  -12569.674818332e0, 0.429631317e0 } ,  
     {    0.000110e-6,    -266.607041722e0, 5.501340197e0 } ,  
     {    0.000115e-6,   12559.038152982e0, 4.691456618e0 } ,  
     {    0.000134e-6,   -2388.894020449e0, 0.577313584e0 } ,  
     {    0.000109e-6,   10440.274292604e0, 6.218148717e0 } ,  
     {    0.000102e-6,    -543.918059096e0, 1.477842615e0 } ,  
     {    0.000108e-6,   21228.392023546e0, 2.237753948e0 } ,  
     {    0.000101e-6,   -4535.059436924e0, 3.100492232e0 } ,  
     {    0.000103e-6,      76.266071276e0, 5.594294322e0 } ,  
     {    0.000104e-6,     949.175608970e0, 5.674287810e0 } ,  
     {    0.000101e-6,   13517.870106233e0, 2.196632348e0 } ,  
     {    0.000100e-6,   11933.367960670e0, 4.056084160e0 } ,  
  
/*  
 * T^2  
 */  
  
     {    4.322990e-6,    6283.075849991e0, 2.642893748e0 } ,  
     {    0.406495e-6,       0.000000000e0, 4.712388980e0 } ,  
     {    0.122605e-6,   12566.151699983e0, 2.438140634e0 } ,  
     {    0.019476e-6,     213.299095438e0, 1.642186981e0 } ,  
     {    0.016916e-6,     529.690965095e0, 4.510959344e0 } ,  
     {    0.013374e-6,      -3.523118349e0, 1.502210314e0 } ,  
     {    0.008042e-6,      26.298319800e0, 0.478549024e0 } ,  
     {    0.007824e-6,     155.420399434e0, 5.254710405e0 } ,  
     {    0.004894e-6,    5746.271337896e0, 4.683210850e0 } ,  
     {    0.004875e-6,    5760.498431898e0, 0.759507698e0 } ,  
     {    0.004416e-6,    5223.693919802e0, 6.028853166e0 } ,  
     {    0.004088e-6,      -7.113547001e0, 0.060926389e0 } ,  
     {    0.004433e-6,   77713.771467920e0, 3.627734103e0 } ,  
     {    0.003277e-6,   18849.227549974e0, 2.327912542e0 } ,  
     {    0.002703e-6,    6062.663207553e0, 1.271941729e0 } ,  
     {    0.003435e-6,    -775.522611324e0, 0.747446224e0 } ,  
     {    0.002618e-6,    6076.890301554e0, 3.633715689e0 } ,  
     {    0.003146e-6,     206.185548437e0, 5.647874613e0 } ,  
     {    0.002544e-6,    1577.343542448e0, 6.232904270e0 } ,  
     {    0.002218e-6,    -220.412642439e0, 1.309509946e0 } ,  
     {    0.002197e-6,    5856.477659115e0, 2.407212349e0 } ,  
     {    0.002897e-6,    5753.384884897e0, 5.863842246e0 } ,  
     {    0.001766e-6,     426.598190876e0, 0.754113147e0 } ,  
     {    0.001738e-6,    -796.298006816e0, 2.714942671e0 } ,  
     {    0.001695e-6,     522.577418094e0, 2.629369842e0 } ,  
     {    0.001584e-6,    5507.553238667e0, 1.341138229e0 } ,  
     {    0.001503e-6,    -242.728603974e0, 0.377699736e0 } ,  
     {    0.001552e-6,    -536.804512095e0, 2.904684667e0 } ,  
     {    0.001370e-6,    -398.149003408e0, 1.265599125e0 } ,  
     {    0.001889e-6,   -5573.142801634e0, 4.413514859e0 } ,  
     {    0.001722e-6,    6069.776754553e0, 2.445966339e0 } ,  
     {    0.001124e-6,    1059.381930189e0, 5.041799657e0 } ,  
     {    0.001258e-6,     553.569402842e0, 3.849557278e0 } ,  
     {    0.000831e-6,     951.718406251e0, 2.471094709e0 } ,  
     {    0.000767e-6,    4694.002954708e0, 5.363125422e0 } ,  
     {    0.000756e-6,    1349.867409659e0, 1.046195744e0 } ,  
     {    0.000775e-6,     -11.045700264e0, 0.245548001e0 } ,  
     {    0.000597e-6,    2146.165416475e0, 4.543268798e0 } ,  
     {    0.000568e-6,    5216.580372801e0, 4.178853144e0 } ,  
     {    0.000711e-6,    1748.016413067e0, 5.934271972e0 } ,  
     {    0.000499e-6,   12036.460734888e0, 0.624434410e0 } ,  
     {    0.000671e-6,   -1194.447010225e0, 4.136047594e0 } ,  
     {    0.000488e-6,    5849.364112115e0, 2.209679987e0 } ,  
     {    0.000621e-6,    6438.496249426e0, 4.518860804e0 } ,  
     {    0.000495e-6,   -6286.598968340e0, 1.868201275e0 } ,  
     {    0.000456e-6,    5230.807466803e0, 1.271231591e0 } ,  
     {    0.000451e-6,    5088.628839767e0, 0.084060889e0 } ,  
     {    0.000435e-6,    5643.178563677e0, 3.324456609e0 } ,  
     {    0.000387e-6,   10977.078804699e0, 4.052488477e0 } ,  
     {    0.000547e-6,  161000.685737473e0, 2.841633844e0 } ,  
     {    0.000522e-6,    3154.687084896e0, 2.171979966e0 } ,  
     {    0.000375e-6,    5486.777843175e0, 4.983027306e0 } ,  
     {    0.000421e-6,    5863.591206116e0, 4.546432249e0 } ,  
     {    0.000439e-6,    7084.896781115e0, 0.522967921e0 } ,  
     {    0.000309e-6,    2544.314419883e0, 3.172606705e0 } ,  
     {    0.000347e-6,    4690.479836359e0, 1.479586566e0 } ,  
     {    0.000317e-6,     801.820931124e0, 3.553088096e0 } ,  
     {    0.000262e-6,     419.484643875e0, 0.606635550e0 } ,  
     {    0.000248e-6,    6836.645252834e0, 3.014082064e0 } ,  
     {    0.000245e-6,   -1592.596013633e0, 5.519526220e0 } ,  
     {    0.000225e-6,    4292.330832950e0, 2.877956536e0 } ,  
     {    0.000214e-6,    7234.794256242e0, 1.605227587e0 } ,  
     {    0.000205e-6,    5767.611978898e0, 0.625804796e0 } ,  
     {    0.000180e-6,   10447.387839604e0, 3.499954526e0 } ,  
     {    0.000229e-6,     199.072001436e0, 5.632304604e0 } ,  
     {    0.000214e-6,     639.897286314e0, 5.960227667e0 } ,  
     {    0.000175e-6,    -433.711737877e0, 2.162417992e0 } ,  
     {    0.000209e-6,     515.463871093e0, 2.322150893e0 } ,  
     {    0.000173e-6,    6040.347246017e0, 2.556183691e0 } ,  
     {    0.000184e-6,    6309.374169791e0, 4.732296790e0 } ,  
     {    0.000227e-6,  149854.400134205e0, 5.385812217e0 } ,  
     {    0.000154e-6,    8031.092263058e0, 5.120720920e0 } ,  
     {    0.000151e-6,    5739.157790895e0, 4.815000443e0 } ,  
     {    0.000197e-6,    7632.943259650e0, 0.222827271e0 } ,  
     {    0.000197e-6,      74.781598567e0, 3.910456770e0 } ,  
     {    0.000138e-6,    6055.549660552e0, 1.397484253e0 } ,  
     {    0.000149e-6,   -6127.655450557e0, 5.333727496e0 } ,  
     {    0.000137e-6,    3894.181829542e0, 4.281749907e0 } ,  
     {    0.000135e-6,    9437.762934887e0, 5.979971885e0 } ,  
     {    0.000139e-6,   -2352.866153772e0, 4.715630782e0 } ,  
     {    0.000142e-6,    6812.766815086e0, 0.513330157e0 } ,  
     {    0.000120e-6,   -4705.732307544e0, 0.194160689e0 } ,  
     {    0.000131e-6,  -71430.695617928e0, 0.000379226e0 } ,  
     {    0.000124e-6,    6279.552731642e0, 2.122264908e0 } ,  
     {    0.000108e-6,   -6256.777530192e0, 0.883445696e0 } ,  
  
/*  
 * T^3  
 */  
  
     {    0.143388e-6,    6283.075849991e0, 1.131453581e0 } ,  
     {    0.006671e-6,   12566.151699983e0, 0.775148887e0 } ,  
     {    0.001480e-6,     155.420399434e0, 0.480016880e0 } ,  
     {    0.000934e-6,     213.299095438e0, 6.144453084e0 } ,  
     {    0.000795e-6,     529.690965095e0, 2.941595619e0 } ,  
     {    0.000673e-6,    5746.271337896e0, 0.120415406e0 } ,  
     {    0.000672e-6,    5760.498431898e0, 5.317009738e0 } ,  
     {    0.000389e-6,    -220.412642439e0, 3.090323467e0 } ,  
     {    0.000373e-6,    6062.663207553e0, 3.003551964e0 } ,  
     {    0.000360e-6,    6076.890301554e0, 1.918913041e0 } ,  
     {    0.000316e-6,     -21.340641002e0, 5.545798121e0 } ,  
     {    0.000315e-6,    -242.728603974e0, 1.884932563e0 } ,  
     {    0.000278e-6,     206.185548437e0, 1.266254859e0 } ,  
     {    0.000238e-6,    -536.804512095e0, 4.532664830e0 } ,  
     {    0.000185e-6,     522.577418094e0, 4.578313856e0 } ,  
     {    0.000245e-6,   18849.227549974e0, 0.587467082e0 } ,  
     {    0.000180e-6,     426.598190876e0, 5.151178553e0 } ,  
     {    0.000200e-6,     553.569402842e0, 5.355983739e0 } ,  
     {    0.000141e-6,    5223.693919802e0, 1.336556009e0 } ,  
     {    0.000104e-6,    5856.477659115e0, 4.239842759e0 } ,  
  
/*  
 * T^4  
 */  
  
     {    0.003826e-6,    6283.075849991e0, 5.705257275e0 } ,  
     {    0.000303e-6,   12566.151699983e0, 5.407132842e0 } ,  
     {    0.000209e-6,     155.420399434e0, 1.989815753e0 } ,  
} ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Time since J2000.0 in Julian millennia.  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJM ;  
  
/*  
 * =================  
 * Topocentric terms  
 * =================  
 */  
  
/*  
 * Convert UT to local solar time in radians.  
 */  
  
    tsol = fmod ( ut, 1.0 ) * GAL_2PI + elong ;  
  
/*  
 * FUNDAMENTAL ARGUMENTS:  Simon et al. 1994.  
 */  
  
/*  
 * Combine time argument (millennia) with deg/arcsec factor.  
 */  
  
    w = t / 3600.0 ;  
  
/*  
 * Sun Mean Longitude.  
 */  
  
    elsun = fmod ( 280.46645683 + 1296027711.03429 * w, 360.0 ) * GAL_D2R ;  
  
/*  
 * Sun Mean Anomaly.  
 */  
  
    emsun = fmod ( 357.52910918 + 1295965810.481 * w, 360.0 ) * GAL_D2R ;  
  
/*  
 * Mean Elongation of Moon from Sun.  
 */  
  
    d = fmod ( 297.85019547 + 16029616012.090 * w, 360.0 ) * GAL_D2R ;  
  
/*  
 * Mean Longitude of Jupiter.  
 */  
  
    elj = fmod ( 34.35151874 + 109306899.89453 * w, 360.0 ) * GAL_D2R ;  
  
/*  
 * Mean Longitude of Saturn.  
 */  
  
    els = fmod ( 50.07744430 + 44046398.47038 * w, 360.0 ) * GAL_D2R ;  
  
/*  
 * TOPOCENTRIC TERMS:  Moyer 1981 and Murray 1983.  
 */  
  
    wt =  + 0.00029e-10 * u * sin ( tsol + elsun - els )  
          + 0.00100e-10 * u * sin ( tsol - 2.0 * emsun )  
          + 0.00133e-10 * u * sin ( tsol - d )  
          + 0.00133e-10 * u * sin ( tsol + elsun - elj )  
          - 0.00229e-10 * u * sin ( tsol + 2.0 * elsun + emsun )  
          - 0.0220e-10  * v * cos ( elsun + emsun )  
          + 0.05312e-10 * u * sin ( tsol - emsun )  
          - 0.13677e-10 * u * sin ( tsol + 2.0 * elsun )  
          - 1.3184e-10  * v * cos ( elsun )  
          + 3.17679e-10 * u * sin ( tsol ) ;  
  
/*  
 * =====================  
 * Fairhead et al. model  
 * =====================  
 */  
  
/*  
 * T**0  
 */  
  
    w0 = 0.0 ;  
    for (j=473;j>=0;j--) {  
        w0 += FAIRHD[j][0] * sin ( FAIRHD[j][1] * t + FAIRHD[j][2] ) ;  
    }  
  
/*  
 * T**1  
 */  
  
    w1 = 0.0 ;  
    for (j=678;j>=474;j--) {  
        w1 += FAIRHD[j][0] * sin ( FAIRHD[j][1] * t + FAIRHD[j][2] ) ;  
    }  
  
/*  
 * T**2  
 */  
  
    w2 = 0.0 ;  
    for (j=763;j>=679;j--) {  
        w2 += FAIRHD[j][0] * sin ( FAIRHD[j][1] * t + FAIRHD[j][2] ) ;  
    }  
  
/*  
 * T**3  
 */  
  
    w3 = 0.0 ;  
    for (j=783;j>=764;j--) {  
        w3 += FAIRHD[j][0] * sin ( FAIRHD[j][1] * t + FAIRHD[j][2] ) ;  
    }  
  
/*  
 * T**4  
 */  
  
    w4 = 0.0 ;  
    for (j=786;j>=784;j--) {  
        w4 += FAIRHD[j][0] * sin ( FAIRHD[j][1] * t + FAIRHD[j][2] ) ;  
    }  
  
/*  
 * Multiply by powers of T and combine.  
 */  
  
    wf = t * ( t * ( t * ( t * w4 + w3 ) + w2 ) + w1 ) + w0 ;  
  
/*  
 * Adjustments to use JPL planetary masses instead of IAU.  
 */  
  
    wj =   0.00065e-6 * sin ( 6069.776754e0 * t + 4.021194e0 ) +  
           0.00033e-6 * sin (  213.299095e0 * t + 5.543132e0 ) +  
        ( -0.00196e-6 * sin ( 6208.294251e0 * t + 5.696701e0 ) ) +  
        ( -0.00173e-6 * sin (   74.781599e0 * t + 2.435900e0 ) ) +  
           0.03638e-6 * t * t ;  
  
/*  
 * ============  
 * Final result  
 * ============  
 */  
  
/*  
 * TDB-TT in seconds.  
 */  
  
    return wt + wf + wj ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
