/*  
 *  - - - - - - - - - - -  
 *   g a l _ j d 2 c a l  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Julian Date to Gregorian year, month, day, and fraction of a day.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     dj1,dj2             d        Julian Date (Notes 1, 2)  
 *  
 *  Returned:
 *  
 *     *iy                 i        year  
 *     *im                 i        month  
 *     *id                 i        day  
 *     *fd                 d        fraction of day  
 *     gal_jd2cal          i        status:  
 *                                    0 = OK  
 *                                   -1 = unacceptable date (Note 3)  
 *  
 *  Notes:  
 *  
 *  1) The earliest valid date is -68569.5 (-4900 March 1).  The  
 *     largest value accepted is 10^9.  
 *  
 *  2) The Julian Date is apportioned in any convenient way between  
 *     the arguments dj1 and dj2.  For example, JD=2450123.7 could  
 *     be expressed in any of these ways, among others:  
 *  
 *             dj1            dj2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *  3) In early eras the conversion is from the "Proleptic Gregorian  
 *     Calendar";  no account is taken of the date(s) of adoption of  
 *     the Gregorian Calendar, nor is the AD/BC numbering convention  
 *     observed.  
 *  
 *  References:  
 *  
 *     Explanatory Supplement to the Astronomical Almanac,  
 *     P. Kenneth Seidelmann (ed), University Science Books (1992),  
 *     Section 12.92 (p604).  
 *  
 *  This revision:
 *  
 *     2001 September 16 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_jd2cal.h"  
  
int  
gal_jd2cal  
 (  
    double dj1,  
    double dj2,  
    int *iy,  
    int *im,  
    int *id,  
    double *fd
 )  
{  
  
/*  
 * Minimum and maximum allowed JD  
 */  
  
    const double GAL_DJMIN = -68569.5 ;  
    const double GAL_DJMAX = 1e9 ;  
  
    int jd, l, n, i, j ;  
    double dj, d1, d2, f1, f2, f, d ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Check if date is acceptable.  
 */  
  
    dj = dj1 + dj2 ;  
    if ( dj < GAL_DJMIN || dj > GAL_DJMAX ) {  
        j = -1 ;  
    }  
    else {  
        j = 0 ;  
  
/*  
 * Copy the date, big then small, and re-align to midnight.  
 */  
  
        if ( dj1 >= dj2 ) {  
            d1 = dj1 ;  
            d2 = dj2 ;  
        }  
        else {  
            d1 = dj2 ;  
            d2 = dj1 ;  
        }  
        d2 = d2 - 0.5 ;  
  
/*  
 * Separate day and fraction.  
 */  
  
        f1 = fmod ( d1, 1.0 ) ;  
        f2 = fmod ( d2, 1.0 ) ;  
        f  = fmod( f1 + f2, 1.0 ) ;  
        if ( f < 0.0 ) {  
            f += 1.0 ;  
        }  
        d = floor ( d1 - f1 ) + floor ( d2 - f2 ) + floor ( f1 + f2 - f ) ;  
        jd = floor ( d ) + 1.0 ;  
  
/*  
 * Express day in Gregorian calendar.  
 */  
  
        l = jd + 68569 ;  
        n = ( 4*l ) / 146097 ;  
        l = l - ( 146097*n + 3 ) / 4 ;  
        i = ( 4000 * (l+1) ) / 1461001 ;  
        l = l - ( 1461*i ) / 4 + 31 ;  
        j = ( 80*l ) / 2447 ;  
        *id = l - ( 2447 * (j) ) / 80 ;  
        l = (j) / 11 ;  
        *im = (j) + 2 - 12*l ;  
        *iy = 100 * ( n-49 ) + i + l ;  
        *fd = f ;  
        j = 0 ;  
    }  

    return j ;
      
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
