/*  
 *  - - - - - - - - - - -  
 *   g a l _ j d c a l f  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Julian Date to Gregorian Calendar, expressed in a form convenient  
 *  for formatting messages:  rounded to a specified precision, and with  
 *  the fields stored in a single array.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     ndp                 i        number of decimal places of days in fraction  
 *     dj1,dj2             d        dj1+dj2 = Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     iymdf              i[4]      year, month, day, fraction in Gregorian  
 *     calendar  
 *     gal_jdcalf          i        status:  
 *                                  -1 = date out of range  
 *                                   0 = OK  
 *                                  +1 = NDP not 0-9 (interpreted as 0)  
 *  
 *  Notes:  
 *  
 *  1) The Julian Date is apportioned in any convenient way between  
 *     the arguments dj1 and dj2.  For example, JD=2450123.7 could  
 *     be expressed in any of these ways, among others:  
 *  
 *               dj1          dj2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *  2) In early eras the conversion is from the "Proleptic Gregorian  
 *     Calendar";  no account is taken of the date(s) of adoption of  
 *     the Gregorian Calendar, nor is the AD/BC numbering convention  
 *     observed.  
 *  
 *  3) Refer to the routine gal_jd2cal.  
 *  
 *  4) ndp should be 4 or less if internal overflows are to be  
 *     avoided on machines which use 16-bit integers.  
 *  
 *  Called:
 *  
 *     gal_jd2cal         JD to Gregorian calendar  
 *  
 *  References:  
 *  
 *     Explanatory Supplement to the Astronomical Almanac,  
 *     P. Kenneth Seidelmann (ed), University Science Books (1992),  
 *     Section 12.92 (p604).  
 *  
 *  This revision:
 *  
 *     2007 August 21 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_jdcalf.h"  
#include "gal_jd2cal.h"
  
int  
gal_jdcalf  
 (  
    int ndp,  
    double dj1,  
    double dj2,  
    int iymdf[4]
 )  
{  
  
    int js, j ;  
    double denom, d1, d2, f1, f2, f ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Denominator of fraction (e.g. 100 for 2 decimal places).  
 */  
  
    if ( ndp >= 0 && ndp <= 9 ) {  
        j = 0 ;  
        denom = pow ( 10, ndp ) ;  
    }  
    else {  
        j = 1 ;  
        denom = 1.0 ;  
    }  
  
/*  
 * Copy the date, big then small, and realign to midnight.  
 */  
  
    if ( dj1 >= dj2 ) {  
        d1 = dj1 ;  
        d2 = dj2 ;  
    }  
    else {  
        d1 = dj2 ;  
        d2 = dj1 ;  
    }  
    d2 = d2 - 0.5 ;  
  
/*  
 * Separate days and fractions.  
 */  
  
    f1 = fmod ( d1, 1.0 ) ;  
    f2 = fmod ( d2, 1.0 ) ;  
    d1 = floor ( d1 - f1 ) ;  
    d2 = floor ( d2 - f2 ) ;  
  
/*  
 * Round the total fraction to the specified number of places.  
 */  
  
    f = floor (( f1+f2 ) * denom) / denom ;  
  
/*  
 * Re-assemble the rounded date and re-align to noon.  
 */  
  
    d2 = d2 + f + 0.5 ;  
  
/*  
 * Convert to Gregorian Calendar.  
 */  
  
    js = gal_jd2cal ( d1, d2, &iymdf[0], &iymdf[1], &iymdf[2], &f ) ;  
    if ( js == 0 ) {  
        iymdf[3] = floor( f * denom ) ;  
    }  
    else {  
        j = js ;  
    }  

    return j ;
      
/*  
 *  Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
