/*  
 *  - - - - - - - - - - - -
 *   g a l _ m a s u d a t  
 *  - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine calculates a number of Sun data values
 *     for the planet Mars. 
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     mtc1               d        MTC date part 1 ( see Note 1 )  
 *     mtc2               d        MTC date part 2 ( see Note 1 ) 
 *     m                  d        Mars mean anomaly ( radians )
 *     ls                 d        Aerocentric solar longitude ( radians )
 *     eot                d        Equation of time ( radians )
 *     lon                d        Longitude of local observer ( radians ) ( see Note 3 )
 *     lat                d        Latitude of local observer ( radians )
 *  
 *  Returned:
 *
 *     *ssl               d        Subsolar longitude ( radians )
 *     *sdec              d        Solar declination ( planetographic ) ( radians )
 *     *mhd               d        Mars heliocentric distance ( AU )
 *     *mhlon             d        Mars heliocentric longitude ( radians )
 *     *mhlat             d        Mars heliocentric latitude ( radians )
 *     *sel               d        Local solar elevation ( radians )
 *     *saz               d        Local solar azimuth ( radians ) 
 *  
 *  Notes:
 *
 *  1) mtc1 contains the sol number, and mtc2 the fractional part of the 
 *     sol. A Mars "day" is called a "sol".
 *
 *  2) As defined, consistent with the terrestrial convention for Mean
 *     Solar Time, JD 2451549.5 (2000 January 6 00:00:00) corresponds to
 *     a near coincidence of the terrestrial Greenwich mean solar
 *     midnight and the Martian mean solar (prime meridian) midnight. The
 *     addition of the integer number 44796 assures a positive result
 *     for any date since JD 2405522 (1873 December 29.5).
 *
 *  3) Longitude is measured westwards from the prime meridian (0-2pi).
 *   
 *  Called:
 *
 *     gal_anp            Normalise angle in range 0 <= a < 2pi
 *
 *  References:
 *
 *     A post-Pathfinder evaluation of areocentric solar coordinates with 
 *     improved timing recipes for Mars seasonal/diurnal climate studies 
 *     by Michael Allison, Megan McEwen, 
 *     Planetary and Space Science 48 (2000) 215-235 
 *
 *     Mars24 URL: http://www.giss.nasa.gov/tools/mars24/help/algorithm.html
 *     The referenced URL contains corrections to the referenced article.
 *
 *  This revision:
 *  
 *     2009 January 6  
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>
#include "gal_masudat.h"
#include "gal_const.h"
#include "gal_anp.h"
  
void 
gal_masudat  
 (  
   double mtc1,  
   double mtc2,   
   double m,    
   double ls,       
   double eot,             
   double lon,              
   double lat,              
   double *ssl,           
   double *sdec,             
   double *mhd,             
   double *mhlon,           
   double *mhlat,           
   double *sel,              
   double *saz               
  ) 
  
{

  const double j2000_mtc = 44791.6194406688737217 ;
  
  double mtc, dt, p0, p1, p2, p3, p4, p5, p6, p7, p8, z, h, sls ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  
/*
 * Calculate date & delta time
 */

  mtc = mtc1 + mtc2 ;
  dt = ( mtc1 - j2000_mtc ) + mtc2 ;
  
/*
 * Determine subsolar longitude
 */
 
  *ssl = gal_anp ( mtc * GAL_2PI + eot + GAL_PI )  ;
 
/*
 * Determine solar declination (planetographic). (A1997, eq. 5)
 */

  p0 = GAL_D2R * 0.25 ;
  sls = sin ( ls ) ;
  
  *sdec = asin ( 0.42565 * sls ) + p0 * sls ;

/*
 * Determine heliocentric distance. (AM2000, eq. 25, corrected)
 */
 
  *mhd = 1.523679 * ( 1.00436 - 0.09309 * cos ( m ) - 0.004336 * cos ( 2.0 * m ) - 0.00031 * cos ( 3.0 * m ) - 0.00003 * cos ( 4.0 * m ) ) ;

/*
 * Determine heliocentric longitude. (AM2000, eq. 26)
 */

  p1 = GAL_D2R * 85.061 ;
  p2 = GAL_D2R * 0.015 ;
  p3 = GAL_D2R * 71.0 ;
  p4 = GAL_D2R * 5.5e-6 ; 
  
  *mhlon = gal_anp ( ls + p1 - p2 * sin ( p3 + 2.0 * ls ) - p4 * dt ) ;

/*
 * Determine heliocentric latitude.
 */

  p5 = GAL_D2R * 1.8497 ;
  p6 = GAL_D2R * 2.23e-5 ;
  p7 = GAL_D2R * 144.50 ;
  p8 = GAL_D2R * 2.57e-6 ;
  
  *mhlat = -( p5 - p6 * dt ) * sin ( ls - p7 + p8 * dt ) ;

/*
 * Determine local solar elevation
 * For any given point on Mars's surface, we want to determine the angle of the sun. The zenith angle is:
 * where lat is the planetographic latitude, lon is the planetographic longitude, and H the hour angle, lon - ssl.
 */

  h = lon - (*ssl) ; 
  z = acos ( sin ( *sdec ) * sin ( lat ) + cos ( *sdec ) * cos ( lat ) * cos ( h ) ) ;
  *sel = GAL_PI / 2.0 - z ;

/*
 * Determine local solar azimuth
 * The second element of the sun's location as seen from a point on Mars' surface is its azimuth, 
 * i.e., compass angle relative to due north.
 */
 
  *saz = gal_anp ( atan2 ( sin ( h ), cos ( lat ) * tan ( *sdec ) - sin ( lat ) * cos ( h ) ) ) ;

/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

