/*  
 *  - - - - - - - - -  
 *   g a l _ b p 0 0  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Frame bias and precession, IAU 2000.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date,date2          d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     rb               d[3][3]     frame bias matrix (Note 2)  
 *     rp               d[3][3]     precession matrix (Note 3)  
 *     rbp              d[3][3]     bias-precession matrix (Note 4)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The matrix rb transforms vectors from GCRS to mean J2000 by  
 *     applying frame bias.  
 *  
 *  3) The matrix rp transforms vectors from J2000 mean equator and  
 *     equinox to mean equator and equinox of date by applying  
 *     precession.  
 *  
 *  4) The matrix rbp transforms vectors from GCRS to mean equator and  
 *     equinox of date by applying frame bias then precession. It is the  
 *     product rp x rb.  
 *  
 *  Called:
 *  
 *     gal_bi00           frame bias components, IAU 2000  
 *     gal_pr00           IAU 2000 precession adjustments  
 *     gal_ir             initialize r-matrix to identity  
 *     gal_rx             rotate around X-axis  
 *     gal_ry             rotate around Y-axis  
 *     gal_rz             rotate around Z-axis  
 *     gal_rxr            product of two r-matrices  
 *  
 *  References:  
 *  
 *     Capitaine, N., Chapront, J., Lambert, S. and Wallace, P.,  
 *     "Expressions for the Celestial Intermediate Pole and Celestial  
 *     Ephemeris Origin consistent with the IAU 2000A precession-nutation  
 *     model", Astronomy & Astrophysics, 400, 1145-1154 (2003)  
 *  
 *     n.b. The celestial ephemeris origin (CEO) was renamed "celestial  
 *     intermediate origin" (CIO) by IAU 2006 Resolution 2.  
 *  
 *  This revision:
 *  
 *     2007 April 11 ( c version 2008 January 18 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_bp00.h"  
#include "gal_bi00.h"  
#include "gal_pr00.h"  
#include "gal_ir.h"  
#include "gal_rx.h"  
#include "gal_ry.h"  
#include "gal_rz.h"  
#include "gal_rxr.h"  
  
void  
gal_bp00  
 (  
    double date1,  
    double date2,  
    double rb[3][3],  
    double rp[3][3],  
    double rbp[3][3]  
 )  
{  
  
/*  
 * J2000 obliquity (Lieske et al. 1977)  
 */  
  
    double const EPS0 = 84381.448 * GAL_AS2R ;  
  
    double  t, dpsibi, depsbi, dra0 ;  
    double  psia77, oma77, chia, dpsipr, depspr, psia, oma ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental epoch J2000.0 and current date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 * Frame bias.  
 */  
  
    gal_bi00 ( &dpsibi, &depsbi, &dra0 ) ;  
  
/*  
 * Precession angles (Lieske et al. 1977)  
 */  
  
    psia77  =           ( 5038.7784  +  
                        (   -1.07259 +  
                        (   -0.001147 ) * t ) * t ) * t * GAL_AS2R ;  
  
    oma77   = EPS0 + (  (    0.05127 +  
                        (   -0.007726 ) * t ) * t ) * t * GAL_AS2R ;  
  
    chia    =           (   10.5526  +  
                        (   -2.38064 +  
                        (   -0.001125 ) * t ) * t ) * t * GAL_AS2R ;  
  
/*  
 * Apply IAU 2000 precession corrections.  
 */  
  
    gal_pr00 ( date1, date2, &dpsipr, &depspr ) ;  
    psia = psia77 + dpsipr ;  
    oma  = oma77  + depspr ;  
  
/*  
 * Frame bias matrix: GCRS to J2000.  
 */  
  
    gal_ir ( rb ) ;  
    gal_rz ( dra0, rb ) ;  
    gal_ry ( dpsibi * sin ( EPS0 ), rb ) ;  
    gal_rx ( -depsbi, rb ) ;  
  
/*  
 * Precession matrix: J2000 to mean of date.  
 */  
  
    gal_ir ( rp ) ;  
    gal_rx ( EPS0, rp ) ;  
    gal_rz ( -psia, rp ) ;  
    gal_rx ( -oma, rp ) ;  
    gal_rz ( chia, rp ) ;  
  
/*  
 * Bias-precession matrix: GCRS to mean of date.  
 */  
  
    gal_rxr ( rp, rb, rbp ) ;  
  
/*  
 *  Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
