/*  
 *  - - - - - - - - - - -  
 *   g a l _ c 2 i x y s  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Form the celestial to intermediate-frame-of-date matrix given the CIP  
 *  x,y and the CIO locator s.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     x,y                 d        Celestial Intermediate Pole (Note 1)  
 *     s                   d        the CIO locator s (Note 2)  
 *  
 *  Returned:
 *  
 *     rc2i             d[3][3]     celestial-to-intermediate matrix (Note 3)  
 *  
 *  Notes:  
 *  
 *  1) The Celestial Intermediate Pole coordinates are the x,y components  
 *     of the unit vector in the Geocentric Celestial Reference System.  
 *  
 *  2) The CIO locator s (in radians) positions the Celestial  
 *     Intermediate Origin on the equator of the CIP.  
 *  
 *  3) The matrix rc2i is the first stage in the transformation from  
 *     celestial to terrestrial coordinates:  
 *  
 *        [TRS]  =  rpom * R_3(era) * rc2i * [CRS]  
 *  
 *               =  rc2t * [CRS]  
 *  
 *     where [CRS] is a vector in the Geocentric Celestial Reference  
 *     System and [TRS] is a vector in the International Terrestrial  
 *     Reference System (see IERS Conventions 2003), era is the Earth  
 *     Rotation Angle and rpom is the polar motion matrix.  
 *  
 *  Called:
 *  
 *     gal_ir             initialize r-matrix to identity  
 *     gal_rz             rotate around Z-axis  
 *     gal_ry             rotate around Y-axis  
 *  
 *  References:  
 *  
 *     McCarthy, D. D., Petit, G. (eds.), IERS Conventions (2003),  
 *     IERS Technical Note No. 32, BKG (2004)  
 *  
 *  This revision:
 *  
 *     2006 October 10 ( c version 2008 January 18 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_c2ixys.h"  
#include "gal_ir.h"  
#include "gal_rz.h"  
#include "gal_ry.h"  
  
void  
gal_c2ixys  
 (  
    double x,  
    double y,  
    double s,  
    double rc2i[3][3]  
 )  
{  
  
    double r2, e, d ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Obtain the spherical angles E and d.  
 */  
  
    r2 = x * x + y * y ;  
    if ( r2 != 0.0 ) {  
        e = atan2 ( y, x ) ;  
    }  
    else {  
        e = 0.0 ;  
    }  
    d = atan ( sqrt ( r2 / ( 1.0 - r2 ) ) ) ;  
  
/*  
 * Form the matrix.  
 */  
  
    gal_ir ( rc2i ) ;  
    gal_rz ( e, rc2i ) ;  
    gal_ry ( d, rc2i ) ;  
    gal_rz ( -( e + s ), rc2i ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
