/*  
 *  - - - - - - - - - - -  
 *   g a l _ e e c t 0 0  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Equation of the equinoxes complementary terms, consistent with  
 *  IAU 2000 resolutions.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     gal_eect00          d        complementary terms (Note 2)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The "complementary terms" are part of the equation of the  
 *     equinoxes (EE), classically the difference between apparent and  
 *     mean Sidereal Time:  
 *  
 *        GAST = GMST + EE  
 *  
 *     with:  
 *  
 *        EE = dpsi * cos(eps)  
 *  
 *     where dpsi is the nutation in longitude and eps is the obliquity  
 *     of date.  However, if the rotation of the Earth were constant in  
 *     an inertial frame the classical formulation would lead to apparent  
 *     irregularities in the UT1 timescale traceable to side-effects of  
 *     precession-nutation.  In order to eliminate these effects from  
 *     UT1, "complementary terms" were introduced in 1994 (IAU, 1994) and  
 *     took effect from 1997 (Capitaine and Gontier, 1993):  
 *  
 *        GAST = GMST + CT + EE  
 *  
 *     By convention, the complementary terms are included as part of the  
 *     equation of the equinoxes rather than as part of the mean Sidereal  
 *     Time.  This slightly compromises the "geometrical" interpretation  
 *     of mean sidereal time but is otherwise inconsequential.  
 *  
 *     The present routine computes CT in the above expression,  
 *     compatible with IAU 2000 resolutions (Capitaine et al., 2002, and  
 *     IERS Conventions 2003).  
 *  
 *  Called:
 *  
 *     gal_fal03          mean anomaly of the Moon  
 *     gal_falp03         mean anomaly of the Sun  
 *     gal_faf03          mean argument of the latitude of the Moon  
 *     gal_fad03          mean elongation of the Moon from the Sun  
 *     gal_faom03         mean longitude of the Moon's ascending node  
 *     gal_fave03         mean longitude of Venus  
 *     gal_fae03          mean longitude of Earth  
 *     gal_fapa03         general accumulated precession in longitude  
 *  
 *  References:  
 *  
 *     Capitaine, N. & Gontier, A.-M., Astron. Astrophys., 275,  
 *     645-650 (1993)  
 *  
 *     Capitaine, N., Wallace, P.T. and McCarthy, D.D., "Expressions to  
 *     implement the IAU 2000 definition of UT1", Astronomy &  
 *     Astrophysics, 406, 1135-1149 (2003)  
 *  
 *     IAU Resolution C7, Recommendation 3 (1994)  
 *  
 *     McCarthy, D. D., Petit, G. (eds.), IERS Conventions (2003),  
 *     IERS Technical Note No. 32, BKG (2004)  
 *  
 *  This revision:
 *  
 *     2007 February 9 ( c version 2008 March 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_eect00.h"  
#include "gal_fal03.h"  
#include "gal_falp03.h"  
#include "gal_faf03.h"  
#include "gal_fad03.h"  
#include "gal_faom03.h"  
#include "gal_fave03.h"  
#include "gal_fae03.h"  
#include "gal_fapa03.h"  
#include "gal_const.h"  
  
double  
gal_eect00  
 (  
    double date1,  
    double date2  
 )  
{  
  
/*  
 * Time since J2000, in Julian centuries  
 */  
  
    double t ;  
  
/*  
 * Miscellaneous  
 */  
  
    int i, j ;  
    double a, s0, s1 ;  
  
/*  
 * Fundamental arguments  
 */  
  
    double fa[14] ;  
  
/*  
 * -----------------------------------------  
 * The series for the EE complementary terms  
 *  -----------------------------------------  
 */  
  
/*  
 * Number of terms in the series  
 */  
  
#define NE0 33  
#define NE1 1  
  
/*  
 * Coefficients of l,l',F,D,Om,LVe,LE,pA  
 */  
  
    int ke0[ NE0 ][8] = {  
  
/*  
 * Argument coefficients for t^0  
 */  
  
     {  0,  0,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  3,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  2,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  3,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  3,  0,  0,  0, } ,  
     {  0,  1,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  1,  0,  0, -1,  0,  0,  0, } ,  
     {  1,  0,  0,  0, -1,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  1,  2, -2,  3,  0,  0,  0, } ,  
     {  0,  1,  2, -2,  1,  0,  0,  0, } ,  
     {  0,  0,  4, -4,  4,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1, -8, 12,  0, } ,  
     {  0,  0,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  3,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  0,  0,  0,  0, } ,  
     {  0,  1, -2,  2, -3,  0,  0,  0, } ,  
     {  0,  1, -2,  2, -1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  8,-13, -1, } ,  
     {  0,  0,  0,  2,  0,  0,  0,  0, } ,  
     {  2,  0, -2,  0, -1,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  1,  0,  0,  0, } ,  
     {  0,  1,  2, -2,  2,  0,  0,  0, } ,  
     {  1,  0,  0, -2, -1,  0,  0,  0, } ,  
     {  0,  0,  4, -2,  4,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  4,  0,  0,  0, } ,  
     {  1,  0, -2,  0, -3,  0,  0,  0, } ,  
     {  1,  0, -2,  0, -1,  0,  0,  0, } ,  
  
     } ;  
  
/*  
 * Argument coefficients for t^1  
 */  
  
    int ke1[ NE1 ][8] = {  
  
       { 0,  0,  0,  0,  1,  0,  0,  0, } ,  
  
    } ;  
  
/*  
 * Sine and cosine coefficients  
 */  
  
    double se0[ NE0 ][2] = {  
  
/*  
 * Sine and cosine coefficients for t^0  
 */  
  
              {   +2640.96e-6,          -0.39e-6, } ,  
              {     +63.52e-6,          -0.02e-6, } ,  
              {     +11.75e-6,          +0.01e-6, } ,  
              {     +11.21e-6,          +0.01e-6, } ,  
              {      -4.55e-6,          +0.00e-6, } ,  
              {      +2.02e-6,          +0.00e-6, } ,  
              {      +1.98e-6,          +0.00e-6, } ,  
              {      -1.72e-6,          +0.00e-6, } ,  
              {      -1.41e-6,          -0.01e-6, } ,  
              {      -1.26e-6,          -0.01e-6, } ,  
              {      -0.63e-6,          +0.00e-6, } ,  
              {      -0.63e-6,          +0.00e-6, } ,  
              {      +0.46e-6,          +0.00e-6, } ,  
              {      +0.45e-6,          +0.00e-6, } ,  
              {      +0.36e-6,          +0.00e-6, } ,  
              {      -0.24e-6,          -0.12e-6, } ,  
              {      +0.32e-6,          +0.00e-6, } ,  
              {      +0.28e-6,          +0.00e-6, } ,  
              {      +0.27e-6,          +0.00e-6, } ,  
              {      +0.26e-6,          +0.00e-6, } ,  
              {      -0.21e-6,          +0.00e-6, } ,  
              {      +0.19e-6,          +0.00e-6, } ,  
              {      +0.18e-6,          +0.00e-6, } ,  
              {      -0.10e-6,          +0.05e-6, } ,  
              {      +0.15e-6,          +0.00e-6, } ,  
              {      -0.14e-6,          +0.00e-6, } ,  
              {      +0.14e-6,          +0.00e-6, } ,  
              {      -0.14e-6,          +0.00e-6, } ,  
              {      +0.14e-6,          +0.00e-6, } ,  
              {      +0.13e-6,          +0.00e-6, } ,  
              {      -0.11e-6,          +0.00e-6, } ,  
              {      +0.11e-6,          +0.00e-6, } ,  
              {      +0.11e-6,          +0.00e-6, } ,  
  
    } ;  
  
    double se1[ NE1 ][2] = {  
  
/*  
 * Sine and cosine coefficients for t^1  
 */  
  
              {      -0.87e-6,          +0.00e-6, } ,  
  
    } ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental epoch J2000.0 and current date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 * Fundamental Arguments (from IERS Conventions 2003)  
 */  
  
/*  
 * Mean anomaly of the Moon.  
 */  
  
    fa[0] = gal_fal03 ( t ) ;  
  
/*  
 * Mean anomaly of the Sun.  
 */  
  
    fa[1] = gal_falp03 ( t ) ;  
  
/*  
 * Mean longitude of the Moon minus that of the ascending node.  
 */  
  
    fa[2] = gal_faf03 ( t ) ;  
  
/*  
 * Mean elongation of the Moon from the Sun.  
 */  
  
    fa[3] = gal_fad03 ( t ) ;  
  
/*  
 * Mean longitude of the ascending node of the Moon.  
 */  
  
    fa[4] = gal_faom03 ( t ) ;  
  
/*  
 * Mean longitude of Venus.  
 */  
  
    fa[5] = gal_fave03 ( t ) ;  
  
/*  
 * Mean longitude of Earth.  
 */  
  
    fa[6] = gal_fae03 ( t ) ;  
  
/*  
 * General precession in longitude.  
 */  
  
    fa[7] = gal_fapa03 ( t ) ;  
  
/*  
 * Evaluate the EE complementary terms.  
 */  
  
    s0 = 0.0 ;  
    s1 = 0.0 ;  
  
    for ( i = NE0 - 1; i >= 0; i-- ) {  
        a = 0.0 ;  
        for (j = 0; j < 8; j++ ) {  
            a += ( double ) ke0[i][j] * fa[j] ;  
        }  
        s0 += ( se0[i][0] * sin ( a ) + se0[i][1] * cos ( a ) ) ;  
    }  
    for ( i = NE1 - 1; i >= 0; i-- ) {  
        a = 0.0 ;  
        for ( j = 0; j < 8; j++ ) {  
            a += ( double ) ke1[i][j] * fa[j] ;  
        }  
        s1 += ( se1[i][0] * sin ( a ) + se1[i][1] * cos ( a ) ) ;  
    }  
    return ( s0 + s1 * t ) * GAL_AS2R ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
