/*  
 *  - - - - - - - - - - -  
 *   g a l _ g m s t 8 2  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Universal Time to Greenwich Mean Sidereal Time (IAU 1982 model).  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     dj1,dj2             d        UT1 Julian Date (see note)  
 *  
 *  Returned:
 *  
 *     gal_gmst82          d        The result is the Greenwich Mean Sidereal  
 *                                  Time (radians), in the range 0 to 2pi.  
 *  
 *  Notes:  
 *  
 *  1) The UT1 epoch dj1+dj2 is a Julian Date, apportioned in any  
 *     convenient way between the arguments dj1 and dj2.  For example,  
 *     JD(UT1)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *             dj1            dj2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 and MJD methods are good compromises  
 *     between resolution and convenience.  The date & time method is  
 *     best matched to the algorithm used:  maximum accuracy (or, at  
 *     least, minimum noise) is delivered when the dj1 argument is for  
 *     0hrs UT1 on the day in question and the dj2 argument lies in the  
 *     range 0 to 1, or vice versa.  
 *  
 *  2) The algorithm is based on the IAU 1982 expression.  This is always  
 *     described as giving the GMST at 0 hours UT1.  In fact, it gives the  
 *     difference between the GMST and the UT, the steady 4-minutes-per-day  
 *     drawing-ahead of ST with respect to UT.  When whole days are ignored,  
 *     the expression happens to equal the GMST at 0 hours UT1 each day.  
 *  
 *  3) In this routine, the entire UT1 (the sum of the two arguments dj1  
 *     and dj2) is used directly as the argument for the standard formula,  
 *     the constant term of which is adjusted by 12 hours to take account  
 *     of the noon phasing of Julian Date.  The UT1 is then added, but  
 *     omitting whole days to conserve accuracy.  
 *  
 *  Called:
 *  
 *     gal_anp            normalize angle into range 0 to 2pi  
 *  
 *  References:  
 *  
 *     Transactions of the International Astronomical Union,  
 *     XVIII B, 67 (1983).  
 *  
 *     Aoki et al., Astron. Astrophys. 105, 359-361 (1982).  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_gmst82.h"  
#include "gal_anp.h"  
  
double  
gal_gmst82  
 (  
    double dj1,  
    double dj2  
 )  
{  
  
    double d1, d2, t, f ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Julian centuries since fundamental epoch.  
 */  
  
    if ( dj1 < dj2 ) {  
        d1 = dj1 ;  
        d2 = dj2 ;  
    }  
    else {  
        d1 = dj2 ;  
        d2 = dj1 ;  
    }  
    t = ( d1 + ( d2 - GAL_J2000 ) ) / GAL_DJC ;  
  
/*  
 * Fractional part of JD(UT1), in seconds.  
 */  
  
    f = GAL_D2S * ( fmod ( d1, 1.0 ) + fmod ( d2, 1.0 ) ) ;  
  
/*  
 * GMST at this UT1.  
 * Coefficients of IAU 1982 GMST-UT1 model  
 * Note: the first constant, A, has to be adjusted by 12 hours because  
 * the UT1 is supplied as a Julian date, which begins at noon.  
 */  
  
    return gal_anp ( GAL_S2R * ( ( (24110.54841 - GAL_D2S / 2.0) +  
                                             ( 8640184.812866 +  
                                                   ( 0.093104 +  
                                                      -6.2e-6 * t ) * t ) * t ) + f ) ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
