/*  
 *  - - - - - - - - - -  
 *   g a l _ n u m a t  
 *  - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Form the matrix of nutation.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     epsa                d        mean obliquity of date (Note 1)  
 *     dpsi,deps           d        nutation (Note 2)  
 *  
 *  Returned:
 *  
 *     rmatn            d[3][3]     nutation matrix (Note 3)  
 *  
 *  Notes:  
 *  
 *  
 *  1) The supplied mean obliquity epsa, must be consistent with the  
 *     precession-nutation models from which dpsi and deps were obtained.  
 *  
 *  2) The caller is responsible for providing the nutation components;  
 *     they are in longitude and obliquity, in radians and are with  
 *     respect to the equinox and ecliptic of date.  
 *  
 *  3) The matrix operates in the sense V(true) = rmatn * V(mean),  
 *     where the p-vector V(true) is with respect to the true  
 *     equatorial triad of date and the p-vector V(mean) is with  
 *     respect to the mean equatorial triad of date.  
 *  
 *  Called:
 *  
 *     gal_ir             initialize r-matrix to identity  
 *     gal_rx             rotate around X-axis  
 *     gal_rz             rotate around Z-axis  
 *  
 *  References:  
 *  
 *     Explanatory Supplement to the Astronomical Almanac,  
 *     P. Kenneth Seidelmann (ed), University Science Books (1992),  
 *     Section 3.222-3 (p114).  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 February 4 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_numat.h"  
#include "gal_ir.h"  
#include "gal_rx.h"  
#include "gal_rz.h"  
  
void  
gal_numat  
 (  
    double epsa,  
    double dpsi,  
    double deps,  
    double rmatn[3][3]  
 )  
{  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Build the rotation matrix.  
 */  
  
    gal_ir ( rmatn ) ;  
    gal_rx ( epsa, rmatn ) ;  
    gal_rz ( -dpsi, rmatn ) ;  
    gal_rx ( -( epsa + deps ), rmatn ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
