/*  
 *  - - - - - - - - - - -  
 *   g a l _ n u t 0 0 b  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Nutation, IAU 2000B model.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     *dpsi,*deps         d        nutation, luni-solar + planetary (Note 2)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The nutation components in longitude and obliquity are in radians  
 *     and with respect to the equinox and ecliptic of date.  The  
 *     obliquity at J2000 is assumed to be the Lieske et al. (1977) value  
 *     of 84381.448 arcsec.  (The errors that result from using this  
 *     routine with the IAU 2006 value of 84381.406 arcsec can be  
 *     neglected.)  
 *  
 *     The nutation model consists only of luni-solar terms, but includes  
 *     also a fixed offset which compensates for certain long-period  
 *     planetary terms (Note 7).  
 *  
 *  3) This routine is an implementation of the IAU 2000B abridged  
 *     nutation model formally adopted by the IAU General Assembly in  
 *     2000.  The routine computes the MHB_2000_SHORT luni-solar nutation  
 *     series (Luzum 2001), but without the associated corrections for  
 *     the precession rate adjustments and the offset between the GCRS  
 *     and J2000 mean poles.  
 *  
 *  4) The full IAU 2000A (MHB2000) nutation model contains nearly 1400  
 *     terms.  The IAU 2000B model (McCarthy & Luzum 2003) contains only  
 *     77 terms, plus additional simplifications, yet still delivers  
 *     results of 1 mas accuracy at present epochs.  This combination of  
 *     accuracy and size makes the IAU 2000B abridged nutation model  
 *     suitable for most practical applications.  
 *  
 *     The routine delivers a pole accurate to 1 mas from 1900 to 2100  
 *     (usually better than 1 mas, very occasionally just outside 1 mas).  
 *     The full IAU 2000A model, which is implemented in the routine  
 *     gal_nut00a (q.v.), delivers considerably greater accuracy at  
 *     current epochs;  however, to realize this improved accuracy,  
 *     corrections for the essentially unpredictable free-core-nutation  
 *     (FCN) must also be included.  
 *  
 *  5) The present routine provides classical nutation.  The  
 *     MHB_2000_SHORT algorithm, from which it is adapted, deals also  
 *     with (i) the offsets between the GCRS and mean poles and (ii) the  
 *     adjustments in longitude and obliquity due to the changed  
 *     precession rates.  These additional functions, namely frame bias  
 *     and precession adjustments, are supported by the SOFA routines  
 *     gal_bi00 and gal_pr00.  
 *  
 *  6) The MHB_2000_SHORT algorithm also provides "total" nutations,  
 *     comprising the arithmetic sum of the frame bias, precession  
 *     adjustments, and nutation (luni-solar + planetary).  These total  
 *     nutations can be used in combination with an existing IAU 1976  
 *     precession implementation, such as gal_pmat76, to deliver GCRS-to-  
 *     true predictions of mas accuracy at current epochs.  However, for  
 *     symmetry with the gal_nut00a routine (q.v. for the reasons), the  
 *     SOFA routines do not generate the "total nutations" directly.  
 *     Should they be required, they could of course easily be generated  
 *     by calling gal_bi00, gal_pr00 and the present routine and adding  
 *     the results.  
 *  
 *  7) The IAU 2000B model includes "planetary bias" terms that are fixed  
 *     in size but compensate for long-period nutations.  The amplitudes  
 *     quoted in McCarthy & Luzum (2003), namely Dpsi = -1.5835 mas and  
 *     Depsilon = +1.6339 mas, are optimized for the "total nutations"  
 *     method described in Note 6.  The Luzum (2001) values used in this  
 *     SOFA implementation, namely -0.135 mas and +0.388 mas, are  
 *     optimized for the "rigorous" method, where frame bias, precession  
 *     and nutation are applied separately and in that order.  During the  
 *     interval 1995-2050, the SOFA implementation delivers a maximum  
 *     error of 1.001 mas (not including FCN).  
 *  
 *  References:  
 *  
 *     Lieske, J.H., Lederle, T., Fricke, W., Morando, B., "Expressions  
 *     for the precession quantities based upon the IAU /1976/ system of  
 *     astronomical constants", Astron.Astrophys. 58, 1-2, 1-16. (1977)  
 *  
 *     Luzum, B., private communication, 2001 (Fortran code  
 *     MHB_2000_SHORT)  
 *  
 *     McCarthy, D.D. & Luzum, B.J., "An abridged model of the  
 *     precession-nutation of the celestial pole", Cel.Mech.Dyn.Astron.  
 *     85, 37-49 (2003)  
 *  
 *     Simon, J.-L., Bretagnon, P., Chapront, J., Chapront-Touze, M.,  
 *     Francou, G., Laskar, J., Astron.Astrophys. 282, 663-683 (1994)  
 *  
 *  This revision:
 *  
 *     2007 June 26 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_nut00b.h"  
  
void  
gal_nut00b  
 (  
    double date1,  
    double date2,  
    double *dpsi,  
    double *deps  
 )  
{  
  
/*  
 * Miscellaneous  
 */  
  
    double t, el, elp, f, d, om, arg, dp, de, sarg, carg,  
           dpsils, depsls, dpsipl, depspl ;  
  
    int i ;  
  
/*  ----------------------------------------  
 *  Tables of argument and term coefficients  
 *  ----------------------------------------  
 */  
  
/*  -------------------------  
 *  Luni-Solar nutation model  
 *  -------------------------  
 */  
  
/*  
 * Number of terms in the luni-solar nutation model  
 */  
  
#define NLS 77  
  
/*  
 * Coefficients for fundamental arguments  
 */  
  
    static const int NALS[ NLS ][5] = {  
  
/*  
 *  Luni-Solar argument multipliers:  
 *  
 *              L     L'    F     D     Om  
 */  
  
     {          0,    0,    0,    0,    1, } ,  
     {          0,    0,    2,   -2,    2, } ,  
     {          0,    0,    2,    0,    2, } ,  
     {          0,    0,    0,    0,    2, } ,  
     {          0,    1,    0,    0,    0, } ,  
     {          0,    1,    2,   -2,    2, } ,  
     {          1,    0,    0,    0,    0, } ,  
     {          0,    0,    2,    0,    1, } ,  
     {          1,    0,    2,    0,    2, } ,  
     {          0,   -1,    2,   -2,    2, } ,  
     {          0,    0,    2,   -2,    1, } ,  
     {         -1,    0,    2,    0,    2, } ,  
     {         -1,    0,    0,    2,    0, } ,  
     {          1,    0,    0,    0,    1, } ,  
     {         -1,    0,    0,    0,    1, } ,  
     {         -1,    0,    2,    2,    2, } ,  
     {          1,    0,    2,    0,    1, } ,  
     {         -2,    0,    2,    0,    1, } ,  
     {          0,    0,    0,    2,    0, } ,  
     {          0,    0,    2,    2,    2, } ,  
     {          0,   -2,    2,   -2,    2, } ,  
     {         -2,    0,    0,    2,    0, } ,  
     {          2,    0,    2,    0,    2, } ,  
     {          1,    0,    2,   -2,    2, } ,  
     {         -1,    0,    2,    0,    1, } ,  
     {          2,    0,    0,    0,    0, } ,  
     {          0,    0,    2,    0,    0, } ,  
     {          0,    1,    0,    0,    1, } ,  
     {         -1,    0,    0,    2,    1, } ,  
     {          0,    2,    2,   -2,    2, } ,  
     {          0,    0,   -2,    2,    0, } ,  
     {          1,    0,    0,   -2,    1, } ,  
     {          0,   -1,    0,    0,    1, } ,  
     {         -1,    0,    2,    2,    1, } ,  
     {          0,    2,    0,    0,    0, } ,  
     {          1,    0,    2,    2,    2, } ,  
     {         -2,    0,    2,    0,    0, } ,  
     {          0,    1,    2,    0,    2, } ,  
     {          0,    0,    2,    2,    1, } ,  
     {          0,   -1,    2,    0,    2, } ,  
     {          0,    0,    0,    2,    1, } ,  
     {          1,    0,    2,   -2,    1, } ,  
     {          2,    0,    2,   -2,    2, } ,  
     {         -2,    0,    0,    2,    1, } ,  
     {          2,    0,    2,    0,    1, } ,  
     {          0,   -1,    2,   -2,    1, } ,  
     {          0,    0,    0,   -2,    1, } ,  
     {         -1,   -1,    0,    2,    0, } ,  
     {          2,    0,    0,   -2,    1, } ,  
     {          1,    0,    0,    2,    0, } ,  
     {          0,    1,    2,   -2,    1, } ,  
     {          1,   -1,    0,    0,    0, } ,  
     {         -2,    0,    2,    0,    2, } ,  
     {          3,    0,    2,    0,    2, } ,  
     {          0,   -1,    0,    2,    0, } ,  
     {          1,   -1,    2,    0,    2, } ,  
     {          0,    0,    0,    1,    0, } ,  
     {         -1,   -1,    2,    2,    2, } ,  
     {         -1,    0,    2,    0,    0, } ,  
     {          0,   -1,    2,    2,    2, } ,  
     {         -2,    0,    0,    0,    1, } ,  
     {          1,    1,    2,    0,    2, } ,  
     {          2,    0,    0,    0,    1, } ,  
     {         -1,    1,    0,    1,    0, } ,  
     {          1,    1,    0,    0,    0, } ,  
     {          1,    0,    2,    0,    0, } ,  
     {         -1,    0,    2,   -2,    1, } ,  
     {          1,    0,    0,    0,    2, } ,  
     {         -1,    0,    0,    1,    0, } ,  
     {          0,    0,    2,    1,    2, } ,  
     {         -1,    0,    2,    4,    2, } ,  
     {         -1,    1,    0,    1,    1, } ,  
     {          0,   -2,    2,   -2,    1, } ,  
     {          1,    0,    2,    2,    1, } ,  
     {         -2,    0,    2,    2,    2, } ,  
     {         -1,    0,    0,    0,    2, } ,  
     {          1,    1,    2,   -2,    2, } ,  
  
    } ;  
  
/*  
 * Longitude and obliquity coefficients  
 */  
  
    static const double CLS[ NLS ][6] = {  
  
/*  
 *  Luni-Solar nutation coefficients, unit 1e-7 arcsec:  
 *  longitude (sin, t*sin, cos), obliquity (cos, t*cos, sin)  
 */  
  
     { -172064161e0, -174666e0,  33386e0, 92052331e0,  9086e0, 15377e0, } ,  
     {  -13170906e0,   -1675e0, -13696e0,  5730336e0, -3015e0, -4587e0, } ,  
     {   -2276413e0,    -234e0,   2796e0,   978459e0,  -485e0,  1374e0, } ,  
     {    2074554e0,     207e0,   -698e0,  -897492e0,   470e0,  -291e0, } ,  
     {    1475877e0,   -3633e0,  11817e0,    73871e0,  -184e0, -1924e0, } ,  
     {    -516821e0,    1226e0,   -524e0,   224386e0,  -677e0,  -174e0, } ,  
     {     711159e0,      73e0,   -872e0,    -6750e0,     0e0,   358e0, } ,  
     {    -387298e0,    -367e0,    380e0,   200728e0,    18e0,   318e0, } ,  
     {    -301461e0,     -36e0,    816e0,   129025e0,   -63e0,   367e0, } ,  
     {     215829e0,    -494e0,    111e0,   -95929e0,   299e0,   132e0, } ,  
     {     128227e0,     137e0,    181e0,   -68982e0,    -9e0,    39e0, } ,  
     {     123457e0,      11e0,     19e0,   -53311e0,    32e0,    -4e0, } ,  
     {     156994e0,      10e0,   -168e0,    -1235e0,     0e0,    82e0, } ,  
     {      63110e0,      63e0,     27e0,   -33228e0,     0e0,    -9e0, } ,  
     {     -57976e0,     -63e0,   -189e0,    31429e0,     0e0,   -75e0, } ,  
     {     -59641e0,     -11e0,    149e0,    25543e0,   -11e0,    66e0, } ,  
     {     -51613e0,     -42e0,    129e0,    26366e0,     0e0,    78e0, } ,  
     {      45893e0,      50e0,     31e0,   -24236e0,   -10e0,    20e0, } ,  
     {      63384e0,      11e0,   -150e0,    -1220e0,     0e0,    29e0, } ,  
     {     -38571e0,      -1e0,    158e0,    16452e0,   -11e0,    68e0, } ,  
     {      32481e0,       0e0,      0e0,   -13870e0,     0e0,     0e0, } ,  
     {     -47722e0,       0e0,    -18e0,      477e0,     0e0,   -25e0, } ,  
     {     -31046e0,      -1e0,    131e0,    13238e0,   -11e0,    59e0, } ,  
     {      28593e0,       0e0,     -1e0,   -12338e0,    10e0,    -3e0, } ,  
     {      20441e0,      21e0,     10e0,   -10758e0,     0e0,    -3e0, } ,  
     {      29243e0,       0e0,    -74e0,     -609e0,     0e0,    13e0, } ,  
     {      25887e0,       0e0,    -66e0,     -550e0,     0e0,    11e0, } ,  
     {     -14053e0,     -25e0,     79e0,     8551e0,    -2e0,   -45e0, } ,  
     {      15164e0,      10e0,     11e0,    -8001e0,     0e0,    -1e0, } ,  
     {     -15794e0,      72e0,    -16e0,     6850e0,   -42e0,    -5e0, } ,  
     {      21783e0,       0e0,     13e0,     -167e0,     0e0,    13e0, } ,  
     {     -12873e0,     -10e0,    -37e0,     6953e0,     0e0,   -14e0, } ,  
     {     -12654e0,      11e0,     63e0,     6415e0,     0e0,    26e0, } ,  
     {     -10204e0,       0e0,     25e0,     5222e0,     0e0,    15e0, } ,  
     {      16707e0,     -85e0,    -10e0,      168e0,    -1e0,    10e0, } ,  
     {      -7691e0,       0e0,     44e0,     3268e0,     0e0,    19e0, } ,  
     {     -11024e0,       0e0,    -14e0,      104e0,     0e0,     2e0, } ,  
     {       7566e0,     -21e0,    -11e0,    -3250e0,     0e0,    -5e0, } ,  
     {      -6637e0,     -11e0,     25e0,     3353e0,     0e0,    14e0, } ,  
     {      -7141e0,      21e0,      8e0,     3070e0,     0e0,     4e0, } ,  
     {      -6302e0,     -11e0,      2e0,     3272e0,     0e0,     4e0, } ,  
     {       5800e0,      10e0,      2e0,    -3045e0,     0e0,    -1e0, } ,  
     {       6443e0,       0e0,     -7e0,    -2768e0,     0e0,    -4e0, } ,  
     {      -5774e0,     -11e0,    -15e0,     3041e0,     0e0,    -5e0, } ,  
     {      -5350e0,       0e0,     21e0,     2695e0,     0e0,    12e0, } ,  
     {      -4752e0,     -11e0,     -3e0,     2719e0,     0e0,    -3e0, } ,  
     {      -4940e0,     -11e0,    -21e0,     2720e0,     0e0,    -9e0, } ,  
     {       7350e0,       0e0,     -8e0,      -51e0,     0e0,     4e0, } ,  
     {       4065e0,       0e0,      6e0,    -2206e0,     0e0,     1e0, } ,  
     {       6579e0,       0e0,    -24e0,     -199e0,     0e0,     2e0, } ,  
     {       3579e0,       0e0,      5e0,    -1900e0,     0e0,     1e0, } ,  
     {       4725e0,       0e0,     -6e0,      -41e0,     0e0,     3e0, } ,  
     {      -3075e0,       0e0,     -2e0,     1313e0,     0e0,    -1e0, } ,  
     {      -2904e0,       0e0,     15e0,     1233e0,     0e0,     7e0, } ,  
     {       4348e0,       0e0,    -10e0,      -81e0,     0e0,     2e0, } ,  
     {      -2878e0,       0e0,      8e0,     1232e0,     0e0,     4e0, } ,  
     {      -4230e0,       0e0,      5e0,      -20e0,     0e0,    -2e0, } ,  
     {      -2819e0,       0e0,      7e0,     1207e0,     0e0,     3e0, } ,  
     {      -4056e0,       0e0,      5e0,       40e0,     0e0,    -2e0, } ,  
     {      -2647e0,       0e0,     11e0,     1129e0,     0e0,     5e0, } ,  
     {      -2294e0,       0e0,    -10e0,     1266e0,     0e0,    -4e0, } ,  
     {       2481e0,       0e0,     -7e0,    -1062e0,     0e0,    -3e0, } ,  
     {       2179e0,       0e0,     -2e0,    -1129e0,     0e0,    -2e0, } ,  
     {       3276e0,       0e0,      1e0,       -9e0,     0e0,     0e0, } ,  
     {      -3389e0,       0e0,      5e0,       35e0,     0e0,    -2e0, } ,  
     {       3339e0,       0e0,    -13e0,     -107e0,     0e0,     1e0, } ,  
     {      -1987e0,       0e0,     -6e0,     1073e0,     0e0,    -2e0, } ,  
     {      -1981e0,       0e0,      0e0,      854e0,     0e0,     0e0, } ,  
     {       4026e0,       0e0,   -353e0,     -553e0,     0e0,  -139e0, } ,  
     {       1660e0,       0e0,     -5e0,     -710e0,     0e0,    -2e0, } ,  
     {      -1521e0,       0e0,      9e0,      647e0,     0e0,     4e0, } ,  
     {       1314e0,       0e0,      0e0,     -700e0,     0e0,     0e0, } ,  
     {      -1283e0,       0e0,      0e0,      672e0,     0e0,     0e0, } ,  
     {      -1331e0,       0e0,      8e0,      663e0,     0e0,     4e0, } ,  
     {       1383e0,       0e0,     -2e0,     -594e0,     0e0,    -2e0, } ,  
     {       1405e0,       0e0,      4e0,     -610e0,     0e0,     2e0, } ,  
     {       1290e0,       0e0,      0e0,     -556e0,     0e0,     0e0, } ,  
  
    } ;  
  
/*  ---------------------------------------  
 *  Fixed offset in lieu of planetary terms (radians)  
 *  ---------------------------------------  
 */  
  
    const double DPPLAN = -0.135 * GAL_MAS2R ;  
    const double DEPLAN = +0.388 * GAL_MAS2R ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental epoch J2000.0 and given date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 *  -------------------  
 *  LUNI-SOLAR NUTATION  
 *  -------------------  
 */  
  
/*  
 * Fundamental (Delaunay) arguments from Simon et al. (1994)  
 */  
  
/*  
 * Mean anomaly of the Moon.  
 */  
  
    el  = fmod (     485868.249036 +  
               ( + 1717915923.2178 ) * t, GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean anomaly of the Sun.  
 */  
  
    elp = fmod (     1287104.79305 +  
               (  + 129596581.0481 ) * t, GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean argument of the latitude of the Moon.  
 */  
  
    f   = fmod (     335779.526232 +  
               ( + 1739527262.8478 ) * t, GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean elongation of the Moon from the Sun.  
 */  
  
    d   = fmod (     1072260.70369 +  
               ( + 1602961601.2090 ) * t, GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean longitude of the ascending node of the Moon.  
 */  
  
    om  = fmod (     450160.398036 +  
               (    - 6962890.5431 ) * t, GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Initialize the nutation values.  
 */  
  
    dp = 0.0 ;  
    de = 0.0 ;  
  
/*  
 * Summation of luni-solar nutation series (in reverse order).  
 */  
  
    for ( i = NLS - 1; i >= 0; i-- ) {  
  
/*  
 * Argument and functions.  
 */  
  
        arg = fmod ( ( double ) NALS[i][0] * el  +  
                     ( double ) NALS[i][1] * elp +  
                     ( double ) NALS[i][2] * f   +  
                      ( double ) NALS[i][3] * d   +  
                    ( double ) NALS[i][4] * om, GAL_2PI ) ;  
  
        sarg = sin ( arg ) ;  
        carg = cos ( arg ) ;  
  
/*  
 * Term.  
 */  
  
        dp += ( CLS[i][0] + CLS[i][1] * t ) * sarg + CLS[i][2] * carg ;  
        de += ( CLS[i][3] + CLS[i][4] * t ) * carg + CLS[i][5] * sarg ;  
  
    }  
  
/*  
 * Convert from 0.1 microarcsec units to radians.  
 */  
  
    dpsils = dp * GAL_U2R ;  
    depsls = de * GAL_U2R ;  
  
/*  
 *  -----------------------------  
 *  IN LIEU OF PLANETARY NUTATION  
 *  -----------------------------  
 */  
  
/*  
 * Fixed offset to correct for missing terms in truncated series.  
 */  
  
    dpsipl = DPPLAN ;  
    depspl = DEPLAN ;  
  
/*  
 *  -------  
 *  RESULTS  
 *  -------  
 */  
  
/*  
 * Add luni-solar and planetary components.  
 */  
  
    *dpsi = dpsils + dpsipl ;  
    *deps = depsls + depspl ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
