/*  
 *  - - - - - - - - - -  
 *   g a l _ n u t 8 0  
 *  - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Nutation, IAU 1980 model.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     *dpsi               d        nutation in longitude (radians)  
 *     *deps               d        nutation in obliquity (radians)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The nutation components are with respect to the ecliptic of  
 *     date.  
 *  
 *  Called:
 *  
 *     gal_anpm           normalize angle into range +/- pi  
 *  
 *  References:  
 *  
 *     Explanatory Supplement to the Astronomical Almanac,  
 *     P. Kenneth Seidelmann (ed), University Science Books (1992),  
 *     Section 3.222 (p111).  
 *  
 *  This revision:
 *  
 *     2007 April 10 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_nut80.h"  
#include "gal_anpm.h"  
  
void  
gal_nut80  
 (  
    double date1,  
    double date2,  
    double *dpsi,  
    double *deps  
 )  
{  
  
    const double U2R = GAL_AS2R / 1e4 ;        /*  Units of 0.1 milliarcsecond to radians */  
  
    double t, el, elp, f, d, om, dp, de, arg, s, c ;  
  
    int j ;  
  
/*  
 *  ------------------------------------------------  
 *  Table of multiples of arguments and coefficients  
 *  ------------------------------------------------  
 *  
 *  The coefficient values are in 0.1 mas units and the rates of change  
 *  are in mas per Julian millennium.  
 */  
  
    static const double x[106][9] = {  
  
/*  
 *               Multiple of             Longitude           Obliquity  
 *        L    L'     F     D Omega   coeff. of sin        coeff. of cos  
 */  
  
     {  0.0,  0.0,  0.0,  0.0,  1.0, -171996.0, -1742.0,  92025.0,  89.0, } ,  
     {  0.0,  0.0,  0.0,  0.0,  2.0,    2062.0,     2.0,   -895.0,   5.0, } ,  
     { -2.0,  0.0,  2.0,  0.0,  1.0,      46.0,     0.0,    -24.0,   0.0, } ,  
     {  2.0,  0.0, -2.0,  0.0,  0.0,      11.0,     0.0,      0.0,   0.0, } ,  
     { -2.0,  0.0,  2.0,  0.0,  2.0,      -3.0,     0.0,      1.0,   0.0, } ,  
     {  1.0, -1.0,  0.0, -1.0,  0.0,      -3.0,     0.0,      0.0,   0.0, } ,  
     {  0.0, -2.0,  2.0, -2.0,  1.0,      -2.0,     0.0,      1.0,   0.0, } ,  
     {  2.0,  0.0, -2.0,  0.0,  1.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  2.0, -2.0,  2.0,  -13187.0,   -16.0,   5736.0, -31.0, } ,  
     {  0.0,  1.0,  0.0,  0.0,  0.0,    1426.0,   -34.0,     54.0,  -1.0, } ,  
     {  0.0,  1.0,  2.0, -2.0,  2.0,    -517.0,    12.0,    224.0,  -6.0, } ,  
     {  0.0, -1.0,  2.0, -2.0,  2.0,     217.0,    -5.0,    -95.0,   3.0, } ,  
     {  0.0,  0.0,  2.0, -2.0,  1.0,     129.0,     1.0,    -70.0,   0.0, } ,  
     {  2.0,  0.0,  0.0, -2.0,  0.0,      48.0,     0.0,      1.0,   0.0, } ,  
     {  0.0,  0.0,  2.0, -2.0,  0.0,     -22.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  2.0,  0.0,  0.0,  0.0,      17.0,    -1.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  0.0,  0.0,  1.0,     -15.0,     0.0,      9.0,   0.0, } ,  
     {  0.0,  2.0,  2.0, -2.0,  2.0,     -16.0,     1.0,      7.0,   0.0, } ,  
     {  0.0, -1.0,  0.0,  0.0,  1.0,     -12.0,     0.0,      6.0,   0.0, } ,  
     { -2.0,  0.0,  0.0,  2.0,  1.0,      -6.0,     0.0,      3.0,   0.0, } ,  
     {  0.0, -1.0,  2.0, -2.0,  1.0,      -5.0,     0.0,      3.0,   0.0, } ,  
     {  2.0,  0.0,  0.0, -2.0,  1.0,       4.0,     0.0,     -2.0,   0.0, } ,  
     {  0.0,  1.0,  2.0, -2.0,  1.0,       4.0,     0.0,     -2.0,   0.0, } ,  
     {  1.0,  0.0,  0.0, -1.0,  0.0,      -4.0,     0.0,      0.0,   0.0, } ,  
     {  2.0,  1.0,  0.0, -2.0,  0.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0, -2.0,  2.0,  1.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0, -2.0,  2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  0.0,  0.0,  2.0,       1.0,     0.0,      0.0,   0.0, } ,  
     { -1.0,  0.0,  0.0,  1.0,  1.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  2.0, -2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  0.0,  2.0,   -2274.0,    -2.0,    977.0,  -5.0, } ,  
     {  1.0,  0.0,  0.0,  0.0,  0.0,     712.0,     1.0,     -7.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  0.0,  1.0,    -386.0,    -4.0,    200.0,   0.0, } ,  
     {  1.0,  0.0,  2.0,  0.0,  2.0,    -301.0,     0.0,    129.0,  -1.0, } ,  
     {  1.0,  0.0,  0.0, -2.0,  0.0,    -158.0,     0.0,     -1.0,   0.0, } ,  
     { -1.0,  0.0,  2.0,  0.0,  2.0,     123.0,     0.0,    -53.0,   0.0, } ,  
     {  0.0,  0.0,  0.0,  2.0,  0.0,      63.0,     0.0,     -2.0,   0.0, } ,  
     {  1.0,  0.0,  0.0,  0.0,  1.0,      63.0,     1.0,    -33.0,   0.0, } ,  
     { -1.0,  0.0,  0.0,  0.0,  1.0,     -58.0,    -1.0,     32.0,   0.0, } ,  
     { -1.0,  0.0,  2.0,  2.0,  2.0,     -59.0,     0.0,     26.0,   0.0, } ,  
     {  1.0,  0.0,  2.0,  0.0,  1.0,     -51.0,     0.0,     27.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  2.0,  2.0,     -38.0,     0.0,     16.0,   0.0, } ,  
     {  2.0,  0.0,  0.0,  0.0,  0.0,      29.0,     0.0,     -1.0,   0.0, } ,  
     {  1.0,  0.0,  2.0, -2.0,  2.0,      29.0,     0.0,    -12.0,   0.0, } ,  
     {  2.0,  0.0,  2.0,  0.0,  2.0,     -31.0,     0.0,     13.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  0.0,  0.0,      26.0,     0.0,     -1.0,   0.0, } ,  
     { -1.0,  0.0,  2.0,  0.0,  1.0,      21.0,     0.0,    -10.0,   0.0, } ,  
     { -1.0,  0.0,  0.0,  2.0,  1.0,      16.0,     0.0,     -8.0,   0.0, } ,  
     {  1.0,  0.0,  0.0, -2.0,  1.0,     -13.0,     0.0,      7.0,   0.0, } ,  
     { -1.0,  0.0,  2.0,  2.0,  1.0,     -10.0,     0.0,      5.0,   0.0, } ,  
     {  1.0,  1.0,  0.0, -2.0,  0.0,      -7.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  2.0,  0.0,  2.0,       7.0,     0.0,     -3.0,   0.0, } ,  
     {  0.0, -1.0,  2.0,  0.0,  2.0,      -7.0,     0.0,      3.0,   0.0, } ,  
     {  1.0,  0.0,  2.0,  2.0,  2.0,      -8.0,     0.0,      3.0,   0.0, } ,  
     {  1.0,  0.0,  0.0,  2.0,  0.0,       6.0,     0.0,      0.0,   0.0, } ,  
     {  2.0,  0.0,  2.0, -2.0,  2.0,       6.0,     0.0,     -3.0,   0.0, } ,  
     {  0.0,  0.0,  0.0,  2.0,  1.0,      -6.0,     0.0,      3.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  2.0,  1.0,      -7.0,     0.0,      3.0,   0.0, } ,  
     {  1.0,  0.0,  2.0, -2.0,  1.0,       6.0,     0.0,     -3.0,   0.0, } ,  
     {  0.0,  0.0,  0.0, -2.0,  1.0,      -5.0,     0.0,      3.0,   0.0, } ,  
     {  1.0, -1.0,  0.0,  0.0,  0.0,       5.0,     0.0,      0.0,   0.0, } ,  
     {  2.0,  0.0,  2.0,  0.0,  1.0,      -5.0,     0.0,      3.0,   0.0, } ,  
     {  0.0,  1.0,  0.0, -2.0,  0.0,      -4.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0, -2.0,  0.0,  0.0,       4.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  0.0,  1.0,  0.0,      -4.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  1.0,  0.0,  0.0,  0.0,      -3.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0,  2.0,  0.0,  0.0,       3.0,     0.0,      0.0,   0.0, } ,  
     {  1.0, -1.0,  2.0,  0.0,  2.0,      -3.0,     0.0,      1.0,   0.0, } ,  
     { -1.0, -1.0,  2.0,  2.0,  2.0,      -3.0,     0.0,      1.0,   0.0, } ,  
     { -2.0,  0.0,  0.0,  0.0,  1.0,      -2.0,     0.0,      1.0,   0.0, } ,  
     {  3.0,  0.0,  2.0,  0.0,  2.0,      -3.0,     0.0,      1.0,   0.0, } ,  
     {  0.0, -1.0,  2.0,  2.0,  2.0,      -3.0,     0.0,      1.0,   0.0, } ,  
     {  1.0,  1.0,  2.0,  0.0,  2.0,       2.0,     0.0,     -1.0,   0.0, } ,  
     { -1.0,  0.0,  2.0, -2.0,  1.0,      -2.0,     0.0,      1.0,   0.0, } ,  
     {  2.0,  0.0,  0.0,  0.0,  1.0,       2.0,     0.0,     -1.0,   0.0, } ,  
     {  1.0,  0.0,  0.0,  0.0,  2.0,      -2.0,     0.0,      1.0,   0.0, } ,  
     {  3.0,  0.0,  0.0,  0.0,  0.0,       2.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  1.0,  2.0,       2.0,     0.0,     -1.0,   0.0, } ,  
     { -1.0,  0.0,  0.0,  0.0,  2.0,       1.0,     0.0,     -1.0,   0.0, } ,  
     {  1.0,  0.0,  0.0, -4.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     { -2.0,  0.0,  2.0,  2.0,  2.0,       1.0,     0.0,     -1.0,   0.0, } ,  
     { -1.0,  0.0,  2.0,  4.0,  2.0,      -2.0,     0.0,      1.0,   0.0, } ,  
     {  2.0,  0.0,  0.0, -4.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  1.0,  2.0, -2.0,  2.0,       1.0,     0.0,     -1.0,   0.0, } ,  
     {  1.0,  0.0,  2.0,  2.0,  1.0,      -1.0,     0.0,      1.0,   0.0, } ,  
     { -2.0,  0.0,  2.0,  4.0,  2.0,      -1.0,     0.0,      1.0,   0.0, } ,  
     { -1.0,  0.0,  4.0,  0.0,  2.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0, -1.0,  0.0, -2.0,  0.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  2.0,  0.0,  2.0, -2.0,  1.0,       1.0,     0.0,     -1.0,   0.0, } ,  
     {  2.0,  0.0,  2.0,  2.0,  2.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0,  0.0,  2.0,  1.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  4.0, -2.0,  2.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  3.0,  0.0,  2.0, -2.0,  2.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0,  2.0, -2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  2.0,  0.0,  1.0,       1.0,     0.0,      0.0,   0.0, } ,  
     { -1.0, -1.0,  0.0,  2.0,  1.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0, -2.0,  0.0,  1.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  2.0, -1.0,  2.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  0.0,  2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0, -2.0, -2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0, -1.0,  2.0,  0.0,  1.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  1.0,  0.0, -2.0,  1.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  1.0,  0.0, -2.0,  2.0,  0.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  2.0,  0.0,  0.0,  2.0,  0.0,       1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  0.0,  2.0,  4.0,  2.0,      -1.0,     0.0,      0.0,   0.0, } ,  
     {  0.0,  1.0,  0.0,  1.0,  0.0,       1.0,     0.0,      0.0,   0.0, } ,  
    } ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental epoch J2000.0 and given date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 * FUNDAMENTAL ARGUMENTS in the FK5 reference system  
 */  
  
/*  
 * Mean longitude of the Moon minus mean longitude of the Moon's  
 * perigee.  
 */  
  
    el = gal_anpm ( ( 485866.733 + ( 715922.633 +  
                         ( 31.310 + 0.064 * t ) * t ) * t ) * GAL_AS2R  
                       + fmod (1325 * t, 1.0) * GAL_2PI ) ;  
  
/*  
 * Mean longitude of the Sun minus mean longitude of the Sun's perigee.  
 */  
  
    elp = gal_anpm ( ( 1287099.804 + ( 1292581.224 +  
                         ( -0.577 -0.012 * t ) * t ) * t ) * GAL_AS2R  
                       + fmod (99.0 * t, 1.0) * GAL_2PI ) ;  
  
/*  
 * Mean longitude of the Moon minus mean longitude of the Moon's node.  
 */  
  
    f = gal_anpm ( ( 335778.877 + ( 295263.137 +  
                       ( -13.257 + 0.011 * t ) * t ) * t ) * GAL_AS2R  
                     + fmod (1342.0 * t, 1.0) * GAL_2PI ) ;  
  
/*  
 * Mean elongation of the Moon from the Sun.  
 */  
  
    d = gal_anpm ( ( 1072261.307 + ( 1105601.328 +  
                       ( -6.891 + 0.019 * t ) * t ) * t ) * GAL_AS2R  
                    + fmod (1236.0 * t, 1.0) * GAL_2PI ) ;  
  
/*  
 * Longitude of the mean ascending node of the lunar orbit on the  
 * ecliptic, measured from the mean equinox of date.  
 */  
  
    om = gal_anpm ( ( 450160.280 + ( -482890.539 +  
                      ( 7.455 + 0.008 * t ) * t ) * t ) * GAL_AS2R  
                    + fmod ( -5.0 * t, 1.0) * GAL_2PI ) ;  
  
/*  
 * ---------------  
 * Nutation series  
 * ---------------  
 */  
  
/*  
 * Change time argument from centuries to millennia.  
 */  
  
    t /= 10.0 ;  
  
/*  
 * Initialize nutation components.  
 */  
  
    dp = 0.0 ;  
    de = 0.0 ;  
  
/*  
 * Sum the nutation terms, ending with the biggest.  
 */  
  
    for (j=105; j>=0; j--) {  
  
/*  
 * Form argument for current term.  
 */  
  
        arg = ( double ) x[j][0] * el  
            + ( double ) x[j][1] * elp  
            + ( double ) x[j][2] * f  
            + ( double ) x[j][3] * d  
            + ( double ) x[j][4] * om ;  
  
/*  
 * Accumulate current nutation term.  
 */  
  
        s = ( double ) x[j][5] + ( double ) x[j][6] * t ;  
        c = ( double ) x[j][7] + ( double ) x[j][8] * t ;  
        if ( s != 0.0 ) {  
            dp += s * sin ( arg ) ;  
        }  
        if ( c != 0.0 ) {  
            de += c * cos ( arg ) ;  
        }  
  
/*  
 * Next term.  
 */  
  
    }  
  
/*  
 * Convert results from 0.1 mas units to radians.  
 */  
  
    *dpsi = dp * U2R ;  
    *deps = de * U2R ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
