/*  
 *  - - - - - - - - - - - - - - 
 *   g a l _ e m d e t a i l s  
 *  - - - - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine returns the full details of the requested ellipsoid model.
 *  
 *  Status: 
 *  
 *     Canonical model.  
 *  
 *  Given:  
 *
 *     em               i           Ellipsoid Model Identifier (Note 1)
 *  
 *  Returned:  
 *  
 *     *body            i           Solar System Body Identifier
 *     *name          c[40]         Ellipsoid Model name
 *     *sma             d           Semi-major axis ( m )
 *     *inf             d           Inverse flattening factor 
 *     gal_emdetails    i           0 = success
 *                                  1 = invalid Ellipsoid Model Identifier 
 *  
 *  Notes:  
 *  
 *  1) The header file gal_ellipsoids.h defines the following constants for the valid
 *     values of em:
 *
 *     Identifier                 Ellisoid Model
 *     ----------                 --------------
 *
 *	   GAL_EMEA_DEL1800	          Delambre 1800
 *	   GAL_EMEA_AIRY1830          Airy 1830
 *	   GAL_EMEA_EVER1830          Everest 1830
 *	   GAL_EMEA_EVER1830BA        Everest 1830 Boni Alt
 *	   GAL_EMEA_BESL1841          Bessel 1841
 *	   GAL_EMEA_CL1866            Clarke 1866
 *	   GAL_EMEA_CL1880	          Clarke 1880
 *	   GAL_EMEA_CLA1880M          Clarke 1880 Modified
 *	   GAL_EMEA_HEL1906	          Helmert 1906
 *	   GAL_EMEA_INTL1909          International 1909
 *	   GAL_EMEA_KRSV	          Krassovsky
 *	   GAL_EMEA_MERC1960	      Mercury 1960
 *	   GAL_EMEA_WGS1960	          World Geodetic System 1960
 *	   GAL_EMEA_IAU1964	          IAU 1964
 *	   GAL_EMEA_AUSNAT1965	      Australian National 1965
 *	   GAL_EMEA_WGS1966	          World Geodetic System 1966
 *	   GAL_EMEA_MERC1968M	      Modified Mercury 1968
 *	   GAL_EMEA_SA1969		      South American 1969
 *	   GAL_EMEA_GRS1967	          Geodetic Reference System 1967
 *	   GAL_EMEA_WGS1972	          World Geodetic System 1972
 *	   GAL_EMEA_IAG1975	          IAG 1975
 *	   GAL_EMEA_IAU1976           IAU 1976
 *	   GAL_EMEA_GRS1980	          Geodetic Reference System 1980
 *	   GAL_EMEA_MERIT1983	      MERIT 1983
 *	   GAL_EMEA_WGS1984	          World Geodetic System 1984
 *	   GAL_EMEA_IERS1989          IERS 1989
 *	   GAL_EMEA_IERS2000          IERS 2000
 *     GAL_EMME_IAU1991           IAU/IAG/COSPAR 1991 Mercury    
 *     GAL_EMVE_IAU1991           IAU/IAG/COSPAR 1991 Venus      
 *     GAL_EMEA_IAU1991           IAU/IAG/COSPAR 1991 Earth      
 *     GAL_EMMA_IAU1991           IAU/IAG/COSPAR 1991 Mars       
 *     GAL_EMJU_IAU1991           IAU/IAG/COSPAR 1991 Jupiter    
 *     GAL_EMSA_IAU1991           IAU/IAG/COSPAR 1991 Saturn     
 *     GAL_EMUR_IAU1991           IAU/IAG/COSPAR 1991 Uranus     
 *     GAL_EMNE_IAU1991           IAU/IAG/COSPAR 1991 Neptune    
 *     GAL_EMPL_IAU1991           IAU/IAG/COSPAR 1991 Pluto      
 *     GAL_EMSU_IAU1991           IAU/IAG/COSPAR 1991 Sun        
 *
 *  2) Where differences in values were found between references 
 *     Seidelmann was selected. IAU/IAG/COSPAR 1991 values are taken
 *     from Davis et al. 1991
 *  
 *  References:
 *  
 *     Explanatory Supplement to the Astronomical Almanac
 *     Edited by P. Kenneth Seidelmann, 1992
 *     Page 220
 *
 *     Map Projection Transformations
 *     by Qihe Yang, John P. Snyder and Waldo R. Tobler
 *     Page 14
 *
 *     McCarthy, D.D., IERS Conventions 2000, Chapter 4 (2002).
 *
 *     Report of the IAU/IAG/COSPAR Working Group on
 *     Cartographic Coordinates and Rotational Elements of
 *     the Planets and Satellites: 1991
 *     M. E. Davis et al.
 *  
 *  This revision:  
 * 
 *     2009 January 6 
 *  
 *  Copyright (C) 2008, 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdio.h>
#include <strings.h>
#include "gal_emdetails.h"
#include "gal_const.h"

int
gal_emdetails  
  (
    const int em,
    int *body,
    char *name,
    double *sma,
    double *inf
  ) 

{

/*
 * ----------------
 * Ellipsoid Models
 * ----------------
 */

#define EMMAX 37  /* Number of Ellipsoid Models defined */
 
  static const struct {
  
    int    body  ;  /* Solar System Body Identifier */
    char   *name ;  /* Ellipsoid Model Name         */
    double sma   ;  /* Semi-Major Axis              */
    double inf   ;  /* Inverse Flattening           */
    
  } *emp, ELLIPSOIDS[EMMAX] = {
  
              
    { GAL_SSB_EA, "Delambre 1800",                  6375653.0,   334.0          } ,
    { GAL_SSB_EA, "Airy 1830",                      6377563.396, 299.324964     } ,
    { GAL_SSB_EA, "Everest 1830",                   6377276.345, 300.8017       } ,
    { GAL_SSB_EA, "Everest 1830 (Boni alternate)",  6377301.243, 300.8017       } ,
    { GAL_SSB_EA, "Bessel 1841",                    6377397.155, 299.152813     } ,
    { GAL_SSB_EA, "Clarke 1866",                    6378206.4,   294.978698     } ,
    { GAL_SSB_EA, "Clarke 1880",                    6378249.145, 293.465        } ,
    { GAL_SSB_EA, "Clarke 1880 (modified)",         6378249.145, 293.4663       } ,
    { GAL_SSB_EA, "Helmert 1906",                   6378200.0,   298.3          } ,
    { GAL_SSB_EA, "International 1909",             6378388.0,   297.0          } ,
    { GAL_SSB_EA, "Krassovsky",                     6378245.0,   298.3          } ,
    { GAL_SSB_EA, "Mercury 1960",                   6378166.0,   298.3          } ,
    { GAL_SSB_EA, "WGS60",                          6378165.0,   298.3          } ,
    { GAL_SSB_EA, "IAU 1964",                       6378160.0,   298.25         } ,
    { GAL_SSB_EA, "Australian National 1965",       6378160.0,   298.25         } ,
    { GAL_SSB_EA, "WGS66",                          6378145.0,   298.25         } ,
    { GAL_SSB_EA, "Modified Mercury 1968",          6378150.0,   298.3          } ,
    { GAL_SSB_EA, "South American 1969",            6378160.0,   298.25         } ,
    { GAL_SSB_EA, "Geodetic Reference System 1967", 6378160.0,   298.2471674273 } ,
    { GAL_SSB_EA, "WGS72",                          6378135.0,   298.26         } ,
    { GAL_SSB_EA, "IAG 1975",                       6378140.0,   298.256        } ,
    { GAL_SSB_EA, "IAU 1976",                       6378140.0,   298.257        } ,
    { GAL_SSB_EA, "Geodetic Reference System 1980", 6378137.0,   298.257222101  } ,
    { GAL_SSB_EA, "MERIT 1983",                     6378137.0,   298.257        } , 
    { GAL_SSB_EA, "WGS84",                          6378137.0,   298.257223563  } ,
    { GAL_SSB_EA, "IERS 1989",                      6378136.0,   298.257        } ,
    { GAL_SSB_EA, "IERS 2000",                      6378136.6,   298.25642      } ,
    { GAL_SSB_ME, "IAU/IAG/COSPAR 1991 Mercury",    2439700.0,     0.0          } ,
    { GAL_SSB_VE, "IAU/IAG/COSPAR 1991 Venus",      6501900.0,     0.0          } ,
    { GAL_SSB_EA, "IAU/IAG/COSPAR 1991 Earth",      6378140.0,   298.257        } ,
    { GAL_SSB_MA, "IAU/IAG/COSPAR 1991 Mars",       3397000.0,   153.846        } ,
    { GAL_SSB_JU, "IAU/IAG/COSPAR 1991 Jupiter",   71492000.0,    15.4154       } ,
    { GAL_SSB_SA, "IAU/IAG/COSPAR 1991 Saturn",    60268000.0,    10.2082       } ,
    { GAL_SSB_UR, "IAU/IAG/COSPAR 1991 Uranus",    25559000.0,    43.611        } ,
    { GAL_SSB_NE, "IAU/IAG/COSPAR 1991 Neptune",   24764000.0,    58.4795       } ,
    { GAL_SSB_PL, "IAU/IAG/COSPAR 1991 Pluto",      1151000.0,     0.0          } ,
    { GAL_SSB_SU, "IAU/IAG/COSPAR 1991 Sun",           6.96e8,     0.0          } ,
  
  } ;

/*
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
  if ( em < 0 || em >= EMMAX ) {
    *body = -1   ;
    name[0]  = '\0' ;
    *sma  = 0.0  ;
    *inf  = 0.0  ;
    return 1 ;
  }
  else {
    emp = &ELLIPSOIDS[em] ;
    *body = emp->body ;
    strcpy ( name, emp->name ) ;
    *sma  = emp->sma  ;
    *inf  = emp->inf  ;
    return 0 ;
  }
	
/*
 * Finished.
 */
 
}
 
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008, 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

 

