/*  
 *  - - - - - - - - - - -  
 *   g a l _ p l a n 9 4  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Approximate heliocentric position and velocity of a nominated major  
 *  planet:  Mercury, Venus, EMB, Mars, Jupiter, Saturn, Uranus or  
 *  Neptune (but not the Earth itself).  
 *  
 *  Given:
 *  
 *     date1               d        TDB date part A (Note 1)  
 *     date2               d        TDB date part B (Note 1)  
 *     np                  i        planet (1=Mercury, 2=Venus, 3=EMB ... 8=Neptune)  
 *  
 *  Returned:
 *  
 *     pv               d[2][3]     planet pos,vel (heliocentric, J2000, AU, AU/d)  
 *     gal_plan94          i        status: -1 = illegal NP (outside 1-8)  
 *                                   0 = OK  
 *                                  +1 = warning: date outside 1000-3000 AD  
 *                                  +2 = warning: solution failed to converge  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *     The limited accuracy of the present algorithm is such that any  
 *     of the methods is satisfactory.  
 *  
 *  2) If an np value outside the range 1-8 is supplied, an error  
 *     status (j = -1) is returned and the pv vector set to zeroes.  
 *  
 *  3) For np=3 the result is for the Earth-Moon Barycenter.  To  
 *     obtain the heliocentric position and velocity of the Earth,  
 *     use instead the SOFA routine gal_epv00.  
 *  
 *  4) On successful return, the array PV contains the following:  
 *  
 *        pv[0][0]  x       }  
 *        pv[0][1]  y       } heliocentric position, AU  
 *        pv[0][2]  z       }  
 *  
 *        pv[1][0]  xdot    }  
 *        pv[1][1]  ydot    } heliocentric velocity, AU/d  
 *        pv[1][2]  zdot    }  
 *  
 *     The reference frame is equatorial and is with respect to the  
 *     mean equator and equinox of epoch J2000.  
 *  
 *  5) The algorithm is due to J.L. Simon, P. Bretagnon, J. Chapront,  
 *     M. Chapront-Touze, G. Francou and J. Laskar (Bureau des  
 *     Longitudes, Paris, France).  From comparisons with JPL  
 *     ephemeris DE102, they quote the following maximum errors  
 *     over the interval 1800-2050:  
 *  
 *                     L (arcsec)    B (arcsec)      R (km)  
 *  
 *        Mercury          4             1             300  
 *        Venus            5             1             800  
 *        EMB              6             1            1000  
 *        Mars            17             1            7700  
 *        Jupiter         71             5           76000  
 *        Saturn          81            13          267000  
 *        Uranus          86             7          712000  
 *        Neptune         11             1          253000  
 *  
 *     Over the interval 1000-3000, they report that the accuracy is no  
 *     worse than 1.5 times that over 1800-2050.  Outside 1000-3000 the  
 *     accuracy declines.  
 *  
 *     Comparisons of the present routine with the JPL DE200 ephemeris  
 *     give the following RMS errors over the interval 1960-2025:  
 *  
 *                      position (km)     velocity (m/s)  
 *  
 *        Mercury            334               0.437  
 *        Venus             1060               0.855  
 *        EMB               2010               0.815  
 *        Mars              7690               1.98  
 *        Jupiter          71700               7.70  
 *        Saturn          199000              19.4  
 *        Uranus          564000              16.4  
 *        Neptune         158000              14.4  
 *  
 *     Comparisons against DE200 over the interval 1800-2100 gave the  
 *     following maximum absolute differences.  (The results using  
 *     DE406 were essentially the same.)  
 *  
 *                   L (arcsec)   B (arcsec)     R (km)   Rdot (m/s)  
 *  
 *        Mercury        7            1            500       0.7  
 *        Venus          7            1           1100       0.9  
 *        EMB            9            1           1300       1.0  
 *        Mars          26            1           9000       2.5  
 *        Jupiter       78            6          82000       8.2  
 *        Saturn        87           14         263000      24.6  
 *        Uranus        86            7         661000      27.4  
 *        Neptune       11            2         248000      21.4  
 *  
 *  6) The present SOFA re-implementation of the original Simon et al.  
 *     Fortran code differs from the original in the following respects:  
 *  
 *       *  The date is supplied in two parts.  
 *  
 *       *  The result is returned only in equatorial Cartesian form;  
 *          the ecliptic longitude, latitude and radius vector are not  
 *          returned.  
 *  
 *       *  The result is in the J2000 equatorial frame, not ecliptic.  
 *  
 *       *  More is done in-line:  there are fewer calls to other  
 *          routines.  
 *  
 *       *  Different error/warning status values are used.  
 *  
 *       *  A different Kepler's-equation-solver is used (avoiding  
 *          use of COMPLEX*16).  
 *  
 *       *  Polynomials in T are nested to minimize rounding errors.  
 *  
 *       *  Explicit double-precision constants are used to avoid  
 *          mixed-mode expressions.  
 *  
 *       *  There are other, cosmetic, changes to comply with SOFA  
 *          style conventions.  
 *  
 *     None of the above changes affects the result significantly.  
 *  
 *  7) The returned status, j, indicates the most serious condition  
 *     encountered during execution of the routine.  Illegal np is  
 *     considered the most serious, overriding failure to converge,  
 *     which in turn takes precedence over the remote epoch warning.  
 *  
 *  Called:
 *  
 *     gal_anp         normalize angle into range 0 to 2pi  
 *  
 *  References:  Simon, J.L, Bretagnon, P., Chapront, J.,  
 *               Chapront-Touze, M., Francou, G., and Laskar, J.,  
 *               Astron. Astrophys. 282, 663 (1994).  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_plan94.h"  
#include "gal_anpm.h"  
  
int  
gal_plan94  
 (  
    double date1,  
    double date2,  
    int np,  
    double pv[2][3] )  
{  
  
/*  
 * Maximum number of iterations allowed to solve Kepler's equation  
 */  
  
    const int KMAX = 10 ;  
  
/*  
 * Sin and cos of J2000 mean obliquity (IAU 1976)  
 */  
  
    const double SINEPS = 0.3977771559319137 ;  
    const double COSEPS = 0.9174820620691818 ;  
  
/*  
 * Gaussian constant  
 */  
  
    const double GK = 0.017202098950 ;  
  
    int jstat, k, i ;  
  
    double t, da, dl, de, dp, di, dom, dmu, arga, argl, am,  
           ae, dae, ae2, at, r, v, si2, xq, xp, tl, xsw,  
           xcw, xm2, xf, ci2, xms, xmc, xpxq2, x, y, z ;  
  
/*  
 * Planetary inverse masses  
 */  
  
    static const double AMAS[8] = {  
        6023600.0,  
        408523.5,  
        328900.5,  
        3098710.0,  
        1047.355,  
        3498.5,  
        22869.0,  
        19314.0  
    } ;  
  
/*  
 *  Tables giving the mean Keplerian elements, limited to T**2 terms:  
 *  
 *         A       semi-major axis (AU)  
 *         DLM     mean longitude (degree and arcsecond)  
 *         E       eccentricity  
 *         PI      longitude of the perihelion (degree and arcsecond)  
 *         DINC    inclination (degree and arcsecond)  
 *         OMEGA   longitude of the ascending node (degree and arcsecond)  
 */  
  
    static const double A[8][3] = {  
  
     {  0.3870983098e0,             0e0,      0e0, } ,  
     {  0.7233298200e0,             0e0,      0e0, } ,  
     {  1.0000010178e0,             0e0,      0e0, } ,  
     {  1.5236793419e0,           3e-10,      0e0, } ,  
     {  5.2026032092e0,       19132e-10,  -39e-10, } ,  
     {  9.5549091915e0, -0.0000213896e0,  444e-10, } ,  
     { 19.2184460618e0,       -3716e-10,  979e-10, } ,  
     { 30.1103868694e0,      -16635e-10,  686e-10, } ,  
  
    } ;  
  
    static const double DLM[8][3] = {  
  
     { 252.25090552e0, 5381016286.88982e0,  -1.92789e0, } ,  
     { 181.97980085e0, 2106641364.33548e0,   0.59381e0, } ,  
     { 100.46645683e0, 1295977422.83429e0,  -2.04411e0, } ,  
     { 355.43299958e0,  689050774.93988e0,   0.94264e0, } ,  
     {  34.35151874e0,  109256603.77991e0, -30.60378e0, } ,  
     {  50.07744430e0,   43996098.55732e0,  75.61614e0, } ,  
     { 314.05500511e0,   15424811.93933e0,  -1.75083e0, } ,  
     { 304.34866548e0,    7865503.20744e0,   0.21103e0, } ,  
  
    } ;  
  
    static const double E[8][3] = {  
  
     { 0.2056317526e0,  0.0002040653e0,      -28349e-10, } ,  
     { 0.0067719164e0, -0.0004776521e0,       98127e-10, } ,  
     { 0.0167086342e0, -0.0004203654e0, -0.0000126734e0, } ,  
     { 0.0934006477e0,  0.0009048438e0,      -80641e-10, } ,  
     { 0.0484979255e0,  0.0016322542e0, -0.0000471366e0, } ,  
     { 0.0555481426e0, -0.0034664062e0, -0.0000643639e0, } ,  
     { 0.0463812221e0, -0.0002729293e0,  0.0000078913e0, } ,  
     { 0.0094557470e0,  0.0000603263e0,             0e0, } ,  
  
    } ;  
  
    static const double PI[8][3] = {  
  
     {  77.45611904e0,  5719.11590e0,   -4.83016e0, } ,  
     { 131.56370300e0,   175.48640e0, -498.48184e0, } ,  
     { 102.93734808e0, 11612.35290e0,   53.27577e0, } ,  
     { 336.06023395e0, 15980.45908e0,  -62.32800e0, } ,  
     {  14.33120687e0,  7758.75163e0,  259.95938e0, } ,  
     {  93.05723748e0, 20395.49439e0,  190.25952e0, } ,  
     { 173.00529106e0,  3215.56238e0,  -34.09288e0, } ,  
     {  48.12027554e0,  1050.71912e0,   27.39717e0, } ,  
  
    } ;  
  
    static const double DINC[8][3] = {  
  
     { 7.00498625e0, -214.25629e0,   0.28977e0, } ,  
     { 3.39466189e0,  -30.84437e0, -11.67836e0, } ,  
     {          0e0,  469.97289e0,  -3.35053e0, } ,  
     { 1.84972648e0, -293.31722e0,  -8.11830e0, } ,  
     { 1.30326698e0,  -71.55890e0,  11.95297e0, } ,  
     { 2.48887878e0,   91.85195e0, -17.66225e0, } ,  
     { 0.77319689e0,  -60.72723e0,   1.25759e0, } ,  
     { 1.76995259e0,    8.12333e0,   0.08135e0, } ,  
  
    } ;  
  
    static const double OMEGA[8][3] = {  
  
     {  48.33089304e0,  -4515.21727e0,  -31.79892e0, } ,  
     {  76.67992019e0, -10008.48154e0,  -51.32614e0, } ,  
     { 174.87317577e0,  -8679.27034e0,   15.34191e0, } ,  
     {  49.55809321e0, -10620.90088e0, -230.57416e0, } ,  
     { 100.46440702e0,   6362.03561e0,  326.52178e0, } ,  
     { 113.66550252e0,  -9240.19942e0,  -66.23743e0, } ,  
     {  74.00595701e0,   2669.15033e0,  145.93964e0, } ,  
     { 131.78405702e0,   -221.94322e0,   -0.78728e0, } ,  
  
    } ;  
  
/*  
 *  Tables for trigonometric terms to be added to the mean elements  
 *  of the semi-major axes.  
 */  
  
    static const double KP[8][9] = {  
  
     { 69613.0, 75645.0, 88306.0, 59899.0, 15746.0, 71087.0, 142173.0,  3086.0,    0.0, } ,  
     { 21863.0, 32794.0, 26934.0, 10931.0, 26250.0, 43725.0,  53867.0, 28939.0,    0.0, } ,  
     { 16002.0, 21863.0, 32004.0, 10931.0, 14529.0, 16368.0,  15318.0, 32794.0,    0.0, } ,  
     {  6345.0,  7818.0, 15636.0,  7077.0,  8184.0, 14163.0,   1107.0,  4872.0,    0.0, } ,  
     {  1760.0,  1454.0,  1167.0,   880.0,   287.0,  2640.0,     19.0,  2047.0, 1454.0, } ,  
     {   574.0,     0.0,   880.0,   287.0,    19.0,  1760.0,   1167.0,   306.0,  574.0, } ,  
     {   204.0,     0.0,   177.0,  1265.0,     4.0,   385.0,    200.0,   208.0,  204.0, } ,  
     {     0.0,   102.0,   106.0,     4.0,    98.0,  1367.0,    487.0,   204.0,    0.0, } ,  
  
    } ;  
  
    static const double CA[8][9] = {  
  
     {       4.0,     -13.0,     11.0,    -9.0,     -9.0,    -3.0,    -1.0,     4.0,     0.0, } ,  
     {    -156.0,      59.0,    -42.0,     6.0,     19.0,   -20.0,   -10.0,   -12.0,     0.0, } ,  
     {      64.0,    -152.0,     62.0,    -8.0,     32.0,   -41.0,    19.0,   -11.0,     0.0, } ,  
     {     124.0,     621.0,   -145.0,   208.0,     54.0,   -57.0,    30.0,    15.0,     0.0, } ,  
     {  -23437.0,   -2634.0,   6601.0,  6259.0,  -1507.0, -1821.0,  2620.0, -2115.0, -1489.0, } ,  
     {   62911.0, -119919.0,  79336.0, 17814.0, -24241.0, 12068.0,  8306.0, -4893.0,  8902.0, } ,  
     {  389061.0, -262125.0, -44088.0,  8387.0, -22976.0, -2093.0,  -615.0, -9720.0,  6633.0, } ,  
     { -412235.0, -157046.0, -31430.0, 37817.0,  -9740.0,   -13.0, -7449.0,  9644.0,     0.0, } ,  
  
    } ;  
  
    static const double SA[8][9] = {  
  
     {     -29.0,     -1.0,      9.0,      6.0,     -6.0,      5.0,      4.0,      0.0,     0.0, } ,  
     {     -48.0,   -125.0,    -26.0,    -37.0,     18.0,    -13.0,    -20.0,     -2.0,     0.0, } ,  
     {    -150.0,    -46.0,     68.0,     54.0,     14.0,     24.0,    -28.0,     22.0,     0.0, } ,  
     {    -621.0,    532.0,   -694.0,    -20.0,    192.0,    -94.0,     71.0,    -73.0,     0.0, } ,  
     {  -14614.0, -19828.0,  -5869.0,   1881.0,  -4372.0,  -2255.0,    782.0,    930.0,   913.0, } ,  
     {  139737.0,      0.0,  24667.0,  51123.0,  -5102.0,   7429.0,  -4095.0,  -1976.0, -9566.0, } ,  
     { -138081.0,      0.0,  37205.0, -49039.0, -41901.0, -33872.0, -27037.0, -12474.0, 18797.0, } ,  
     {       0.0,  28492.0, 133236.0,  69654.0,  52322.0, -49577.0, -26430.0,  -3593.0,     0.0, } ,  
  
    } ;  
  
/*  
 *  Tables giving the trigonometric terms to be added to the mean  
 *  elements of the mean longitudes.  
 */  
  
    static const double KQ[8][10] = {  
  
     {  3086.0, 15746.0, 69613.0, 59899.0, 75645.0, 88306.0, 12661.0, 2658.0,  0.0,    0.0, } ,  
     { 21863.0, 32794.0, 10931.0,    73.0,  4387.0, 26934.0,  1473.0, 2157.0,  0.0,    0.0, } ,  
     {    10.0, 16002.0, 21863.0, 10931.0,  1473.0, 32004.0,  4387.0,   73.0,  0.0,    0.0, } ,  
     {    10.0,  6345.0,  7818.0,  1107.0, 15636.0,  7077.0,  8184.0,  532.0, 10.0,    0.0, } ,  
     {    19.0,  1760.0,  1454.0,   287.0,  1167.0,   880.0,   574.0, 2640.0, 19.0, 1454.0, } ,  
     {    19.0,   574.0,   287.0,   306.0,  1760.0,    12.0,    31.0,   38.0, 19.0,  574.0, } ,  
     {     4.0,   204.0,   177.0,     8.0,    31.0,   200.0,  1265.0,  102.0,  4.0,  204.0, } ,  
     {     4.0,   102.0,   106.0,     8.0,    98.0,  1367.0,   487.0,  204.0,  4.0,  102.0, } ,  
  
    } ;  
  
    static const double CL[8][10] = {  
  
     {      21.0,    -95.0,  -157.0,    41.0,    -5.0,    42.0,   23.0,   30.0,      0.0,     0.0, } ,  
     {    -160.0,   -313.0,  -235.0,    60.0,   -74.0,   -76.0,  -27.0,   34.0,      0.0,     0.0, } ,  
     {    -325.0,   -322.0,   -79.0,   232.0,   -52.0,    97.0,   55.0,  -41.0,      0.0,     0.0, } ,  
     {    2268.0,   -979.0,   802.0,   602.0,  -668.0,   -33.0,  345.0,  201.0,    -55.0,     0.0, } ,  
     {    7610.0,  -4997.0, -7689.0, -5841.0, -2617.0,  1115.0, -748.0, -607.0,   6074.0,   354.0, } ,  
     {  -18549.0,  30125.0, 20012.0,  -730.0,   824.0,    23.0, 1289.0, -352.0, -14767.0, -2062.0, } ,  
     { -135245.0, -14594.0,  4197.0, -4030.0, -5630.0, -2898.0, 2540.0, -306.0,   2939.0,  1986.0, } ,  
     {   89948.0,   2103.0,  8963.0,  2695.0,  3682.0,  1648.0,  866.0, -154.0,  -1963.0,  -283.0, } ,  
  
    } ;  
  
    static const double SL[8][10] = {  
  
     {   -342.0,    136.0,   -23.0,    62.0,    66.0,   -52.0,  -33.0,    17.0,     0.0,     0.0, } ,  
     {    524.0,   -149.0,   -35.0,   117.0,   151.0,   122.0,  -71.0,   -62.0,     0.0,     0.0, } ,  
     {   -105.0,   -137.0,   258.0,    35.0,  -116.0,   -88.0, -112.0,   -80.0,     0.0,     0.0, } ,  
     {    854.0,   -205.0,  -936.0,  -240.0,   140.0,  -341.0,  -97.0,  -232.0,   536.0,     0.0, } ,  
     { -56980.0,   8016.0,  1012.0,  1448.0, -3024.0, -3710.0,  318.0,   503.0,  3767.0,   577.0, } ,  
     { 138606.0, -13478.0, -4964.0,  1441.0, -1319.0, -1482.0,  427.0,  1236.0, -9167.0, -1918.0, } ,  
     {  71234.0, -41116.0,  5334.0, -4935.0, -1848.0,    66.0,  434.0, -1748.0,  3780.0,  -701.0, } ,  
     { -47645.0,  11647.0,  2166.0,  3194.0,   679.0,     0.0, -244.0,  -419.0, -2531.0,    48.0, } ,  
  
    } ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Validate the planet number.  
 */  
  
    if ( np < 1 || np > 8 ) {  
  
        jstat = -1 ;  
  
/*  
 *    Reset the result in case of failure.  
 */  
  
        for ( k = 0; k < 2; k++ ) {  
            for ( i = 0; i < 3; i++ ) {  
                pv[k][i] = 0.0 ;  
            }  
        }  
  
    }  
    else {  
  
/*  
 *    Time: Julian millennia since J2000.  
 */  
  
        t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJM ;  
  
/*  
 *    OK status unless remote epoch.  
 */  
  
        if ( fabs ( t ) <= 1.0 ) {  
            jstat = 0 ;  
        }  
        else {  
            jstat = 1 ;  
        }  
  
/*  
 * Adjust Planet number for C indicies starting at 0 not 1  
 */  
  
        np-- ;  
  
/*  
 *    Compute the mean elements.  
 */  
  
        da = A[np][0] +  
           ( A[np][1] +  
             A[np][2] * t ) * t ;  
  
        dl = ( 3600.0 * DLM[np][0] +  
                      ( DLM[np][1] +  
                        DLM[np][2] * t ) * t ) * GAL_AS2R ;  
  
        de = E[np][0] +  
           ( E[np][1] +  
             E[np][2] * t ) * t ;  
  
        dp = gal_anpm ( ( 3600.0 * PI[np][0] +  
                                     ( PI[np][1] +  
                                       PI[np][2] * t ) * t ) * GAL_AS2R ) ;  
  
        di = ( 3600.0 * DINC[np][0] +  
                      ( DINC[np][1] +  
                        DINC[np][2] * t ) * t ) * GAL_AS2R ;  
  
        dom = gal_anpm ( ( 3600.0 * OMEGA[np][0]  
                                    + ( OMEGA[np][1]  
                                      + OMEGA[np][2] * t ) * t ) * GAL_AS2R ) ;  
  
/*  
 *    Apply the trigonometric terms.  
 */  
  
        dmu = 0.35953620 * t ;  
  
        for (k=0; k<8; k++) {  
  
            arga = KP[np][k] * dmu ;  
  
            argl = KQ[np][k] * dmu ;  
  
            da += ( CA[np][k] * cos ( arga ) +  
                    SA[np][k] * sin ( arga ) ) * 1e-7 ;  
  
            dl += ( CL[np][k] * cos ( argl ) +  
                    SL[np][k] * sin ( argl ) ) * 1e-7 ;  
  
        }  
  
        arga = KP[np][8] * dmu ;  
  
        da += t * ( CA[np][8] * cos ( arga ) +  
                    SA[np][8] * sin ( arga ) ) * 1e-7 ;  
  
        for (k=8; k<10; k++) {  
  
            argl = KQ[np][k] * dmu ;  
  
            dl += t * ( CL[np][k] * cos ( argl ) +  
                        SL[np][k] * sin ( argl ) ) * 1e-7 ;  
  
        }  
  
        dl = fmod ( dl, GAL_2PI ) ;  
  
/*  
 *    Iterative solution of Kepler's equation to get eccentric anomaly.  
 */  
  
        am = dl - dp ;  
        ae = am + de * sin ( am ) ;  
  
        k = 0 ;  
        do {  
            dae = ( am - ae + de * sin ( ae ) ) / ( 1.0 - de * cos ( ae ) ) ;  
            ae += dae ;  
            k++ ;  
            if ( k >= KMAX ) {  
                jstat = 2 ;  
            }  
        } while ( k < KMAX && fabs ( dae ) > 1e-12 ) ;  
  
/*  
 *    True anomaly.  
 */  
  
        ae2 = ae / 2.0 ;  
        at = 2.0 * atan2 ( sqrt ( ( 1.0 + de ) / ( 1.0 - de ) ) * sin ( ae2 ), cos ( ae2 ) ) ;  
  
/*  
 *    Distance (AU) and speed (radians per day).  
 */  
  
        r = da * ( 1.0 - de * cos ( ae ) ) ;  
        v = GK * sqrt ( ( 1.0 + 1.0 / AMAS[np] ) / ( da * da * da ) ) ;  
  
        si2 = sin ( di / 2.0 ) ;  
        xq = si2 * cos ( dom ) ;  
        xp = si2 * sin ( dom ) ;  
        tl = at + dp ;  
        xsw = sin ( tl ) ;  
        xcw = cos ( tl ) ;  
        xm2 = 2.0 * ( xp * xcw - xq * xsw ) ;  
        xf = da / sqrt ( 1.0 - de * de ) ;  
        ci2 = cos( di / 2.0 ) ;  
        xms = ( de * sin ( dp ) + xsw ) * xf ;  
        xmc = ( de * cos ( dp ) + xcw ) * xf ;  
        xpxq2 = 2.0 * xp * xq ;  
  
/*  
 *    Position (J2000 ecliptic x,y,z in AU).  
 */  
  
        x = r * ( xcw - xm2 * xp ) ;  
        y = r * ( xsw + xm2 * xq ) ;  
        z = r * ( -xm2 * ci2 ) ;  
  
/*  
 *    Rotate to equatorial.  
 */  
  
        pv[0][0] = x  ;  
        pv[0][1] = y * COSEPS - z * SINEPS ;  
        pv[0][2] = y * SINEPS + z * COSEPS ;  
  
/*  
 *    Velocity (J2000 ecliptic xdot,ydot,zdot in AU/d).  
 */  
  
        x = v * ( ( -1.0 + 2.0 * xp * xp ) * xms + xpxq2 * xmc ) ;  
        y = v * ( ( 1.0 - 2.0 * xq * xq ) * xmc - xpxq2 * xms ) ;  
        z = v * ( 2.0 * ci2 * ( xp * xms + xq * xmc ) ) ;  
  
/*  
 *    Rotate to equatorial.  
 */  
  
        pv[1][0] = x ;  
        pv[1][1] = y * COSEPS - z * SINEPS ;  
        pv[1][2] = y * SINEPS + z * COSEPS ;  
  
    }  
  
/*  
 * Return the status.  
 */  
  
    return jstat ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
