/*  
 *  - - - - - - - - - - - -
 *   g a l _ a c c d r a g  
 *  - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     Computes the perturbational acceleration due to atmospheric 
 *     drag
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     pv            d[2][3]        Position and velocity of satellite ( m, ms^-1 )
 *     area             d           Satellite surface area ( m^2 )
 *     mass             d           Satellite mass ( kg )
 *     cd               d           Drag coefficient
 *     p                d           Atmospheric density ( kg m^-3 )
 *     omega            d           Planet rotation rate ( rad s^-1 )
 *  
 *  Returned:  
 *  
 *     a               d[3]         Acceleration (a=d^2r/dt^2)
 *
 *  Called:
 *
 *     gal_sxp         scalar p-vector product
 *     gal_pmp         p-vector subtraction
 *     gal_pn          p-vector unit vector and modulus 
 *
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 83-85
 *  
 *  This revision:  
 * 
 *     2009 January 4 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include "gal_accdrag.h"
#include "gal_sxp.h"
#include "gal_pmp.h"
#include "gal_pn.h"

void
gal_accdrag
  (
    double pv[2][3],
    double area,
    double mass,
    double cd,
    double p,
    double omega,
    double a[3] 
  ) 
  
{

  double vr[3], ev[3], or[3], vrm ;
    
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate intermediates
 */
 
  gal_sxp ( omega, pv[0], or ) ;
  gal_pmp ( pv[1], or, vr ) ;
  gal_pn ( vr, &vrm, ev ) ;

/*
 * Acceleration 
 */

  gal_sxp ( -0.5 * cd * area / mass * p * vrm * vrm, ev, a ) ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
