/*  
 *  - - - - - - - - - 
 *   g a l _ a c c h  
 *  - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     Computes the body fixed acceleration due to the harmonic 
 *     gravity field of the central body.
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     p               d[3]         Position vector in body fixed frame
 *     gm            *gal_gm_t      Gravity Model
 *     max_n            i           Maximum degree to use 
 *     max_m            i           Maximum order to use
 *  
 *  Returned:  
 *  
 *     abf             d[3]         Body Fixed Acceleration
 *     gal_acch         i           Status
 *                                    0 = Success
 *                                    1 = Failed to allocate memory
 *                                    2 = Maximum degree or order exceeds limits of gravity model
 *  Called:
 *
 *     gal_pdp                      p-vector inner (=scalar=dot) product.
 *     gal_stget                    gets spherical terms C & S of degree n and order m
 *                                  from the given gravity model
 *
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 61-68
 *  
 *  This revision:  
 * 
 *     2008 May 25 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include <stdlib.h>
#include "gal_const.h"
#include "gal_acch.h"
#include "gal_pdp.h"
#include "gal_stget.h"

int
gal_acch
  (
    double pbf[3],
    gal_gm_t *gm,
    int max_n, 
    int max_m,
    double abf[3] 
  ) 
  
{

  int    n, m ;                         /* Loop counters              */
  double r_sqr, rho, fac ;              /* Auxiliary quantities       */
  double x0, y0, z0 ;                   /* Normalized coordinates     */
  double ax, ay, az ;                   /* Acceleration vector        */
  double c, s ;                         /* Gravitational coefficients */
  double a ;                            /* Acceleration               */
  double *v ;                           /* Harmonic functions         */
  double *w ;                           /* work array                 */
       
#define V(n, m) *( v + (n) * ( max_n + 2 ) + (m) ) 
#define W(n, m) *( w + (n) * ( max_n + 2 ) + (m) ) 
     
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  if ( max_n > gm->max_degree || max_m > gm->max_order ) {
    return 2 ;
  }
  
  if ( ( v = malloc ( ( max_n + 2 ) * ( max_n + 2 ) * sizeof ( double ) ) ) == NULL ) {
    return 1 ;
  } ;
  
  if ( ( w = malloc ( ( max_n + 2 ) * ( max_n + 2 ) * sizeof ( double ) ) ) == NULL ) {
    free (v) ;
    return 1 ;
  } ;
  
/*
 * Auxiliary quantities
 */

 /* 
  * Square of distance 
  */
    
  r_sqr = gal_pdp ( pbf, pbf ) ;       
  rho   =  gm->sma * gm->sma / r_sqr ;

/* 
 * Normalized coordinates 
 */ 
    
  x0 = gm->sma * pbf[0] / r_sqr ;        
  y0 = gm->sma * pbf[1] / r_sqr ;          
  z0 = gm->sma * pbf[2] / r_sqr ;
  
/*
 * Evaluate harmonic functions 
 * V_nm = (R_ref/r)^(n+1) * P_nm(sin(phi)) * cos(m*lambda)
 * and 
 * W_nm = (R_ref/r)^(n+1) * P_nm(sin(phi)) * sin(m*lambda)
 * up to degree and order n_max+1
 */
  
/*
 * Calculate zonal terms V(n,0); set W(n,0)=0.0
 */
  
  V(0,0) = gm->sma / sqrt ( r_sqr ) ;
  W(0,0) = 0.0 ;
        
  V(1,0) = z0 * V(0,0) ;
  W(1,0) = 0.0 ;
  
  for ( n = 2; n <= max_n + 1; n++ ) {
    V(n,0) = ( ( 2 * n - 1 ) * z0 * V(n-1,0) - ( n - 1 ) * rho * V(n-2,0) ) / n ;
    W(n,0) = 0.0 ;
  } 
  
/*
 * Calculate tesseral and sectorial terms 
 */
  
  for ( m = 1; m <= max_m + 1; m++ ) {
      
/*
 * Calculate V(m,m) .. V(n_max+1,m)
 */
  
    V(m,m) = ( 2 * m - 1 ) * ( x0 * V(m-1,m-1) - y0 * W(m-1,m-1) ) ;
    W(m,m) = ( 2 * m - 1 ) * ( x0 * W(m-1,m-1) + y0 * V(m-1,m-1) ) ;
  
    if  (m <= max_n ) {
      V(m+1,m) = ( 2 * m + 1 ) * z0 * V(m,m) ;
      W(m+1,m) = ( 2 * m + 1 ) * z0 * W(m,m) ;
    } 
  
    for ( n = m + 2; n <= max_n + 1; n++ ) {
      V(n,m) = ( ( 2 * n - 1 ) * z0 * V(n-1,m) - ( n + m - 1 ) * rho * V(n-2,m) ) / ( n - m ) ;
      W(n,m) = ( ( 2 * n - 1 ) * z0 * W(n-1,m) - ( n + m - 1 ) * rho * W(n-2,m) ) / ( n - m ) ;
    } 
  
  } 
  
/*
 * Calculate accelerations ax,ay,az
 */
  
  ax = ay = az = 0.0 ;
  
  for ( m = 0; m <= max_m; m++ ) {
    for ( n = m; n <= max_n ; n++ ) {
      gal_stget ( n, m, gm, &c, &s ) ;
      if ( m == 0) {
        ax -=             c * V(n+1,1) ;
        ay -=             c * W(n+1,1) ;
        az -= ( n + 1 ) * c * V(n+1,0) ;
      }
      else { 
        fac = 0.5 * ( n - m + 1 ) * ( n - m + 2 ) ;
        ax +=         + 0.5 * ( -c * V(n+1,m+1) - s * W(n+1,m+1) ) + fac * (  c * V(n+1,m-1) + s * W(n+1,m-1) ) ;
        ay +=         + 0.5 * ( -c * W(n+1,m+1) + s * V(n+1,m+1) ) + fac * ( -c * W(n+1,m-1) + s * V(n+1,m-1) ) ;
        az += ( n - m + 1 ) * ( -c * V(n+1,m)   - s * W(n+1,m)   ) ;
      } 
    }
  }
  
/*
 * Body-fixed acceleration
 */
 
  a = gm->gm / ( gm->sma * gm->sma ) ;
  
  abf[0] = a * ax ;
  abf[1] = a * ay ;
  abf[2] = a * az ;
  
  free ( w ) ;
  free ( v ) ;
  
  return GAL_SUCCESS ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
