/*  
 *  - - - - - - - - - -
 *   g a l _ a c c p m  
 *  - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     Computes the perturbational acceleration due to a point mass
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     ps              d[3]         Position vector of satellite
 *     ppm             d[3]         Position vector of point mass
 *     gm               d           Gravitational coefficient of point mass
 *  
 *  Returned:  
 *  
 *     a               d[3]         Acceleration (a=d^2r/dt^2)
 *
 *  Called:
 *
 *     gal_pmp                      p-vector minus p-vector
 *     gal_pm                       modulus of a p-vector
 *     gal_sxp                      Scale p-vector
 *     gal_ppsp                     p-vector plus scled p-vector
 *
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 69-70
 *  
 *  This revision:  
 * 
 *     2008 June 8 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_accpm.h"
#include "gal_pmp.h"
#include "gal_pm.h"
#include "gal_sxp.h"
#include "gal_ppsp.h"

void
gal_accpm
  (
    double ps[3],
    double ppm[3],
    double gm,
    double a[3] 
  ) 
  
{

  double d[3], md, mppm, t[3] ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Relative position vector of satellite w.r.t. point mass 
 */
  
  gal_pmp ( ps, ppm, d ) ; 
  
/*
 * Acceleration 
 */
 
  md = gal_pm ( d ) ;
  mppm = gal_pm ( ppm ) ;
  gal_sxp ( pow ( md, -3.0 ), d, t ) ;
  gal_ppsp ( t, pow ( mppm, -3.0 ), ppm, t ) ;
  gal_sxp ( -gm, t, a ) ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
