/*  
 *  - - - - - - - - - - -
 *   g a l _ a c c s r p   
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     Computes the perturbational acceleration due to solar 
 *     radiation pressure
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     psat            d[3]         Position vector of satellite
 *     psun            d[3]         Position vector of the Sun
 *     p                d           Solar radiation pressure at one AU ( Newtons meters^-1 )
 *     au               d           Length of Astronomical Unit  ( meters )
 *     area             d           Satellite surface area ( meters^2 )
 *     mass             d           Satellite mass ( kg )
 *     eps              d           Reflectivity ( see Note 1 )
 *     v                d           Shadow function ( 0 = eclipse, 1 = full sunlight )
 *     n               d[3]         Normal unit vector for the satellite's surface
 *  
 *  Returned:  
 *  
 *     a                d[3]         Acceleration (a=d^2r/dt^2)
 *
 *  Notes:
 *
 *  1) Reflectivity is usually in the range 0.2 to 0.9.
 *
 *     Solar Panel                       0.21
 *     High Gain Antenna                 0.30
 *     Aluminium coated mylar solar sail 0.88
 *
 *  Called:
 *
 *     gal_pmp          p-vector minus p-vector
 *     gal_pn           convert p-vector into modulus and unit vector
 *     gal_sxp          p-vector scalar product
 *     gal_ppp          p-vector addition
 *     gal_pdp          p-vector dot product
 * 
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 77-79
 *  
 *  This revision:  
 * 
 *     2009 January 3 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include "gal_accsrp.h"
#include "gal_pmp.h"
#include "gal_pn.h"
#include "gal_sxp.h"
#include "gal_ppp.h"
#include "gal_pdp.h"

void
gal_accsrp
  (
    double psat[3],
    double psun[3],
    double p,
    double au,
    double area,
    double mass,
    double eps,
    double v,
    double n[3],
    double a[3] 
  ) 

{
  
  double d[3], md, k, e[3], se[3], sn[3]  ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Relative position vector of sun w.r.t. satellite  
 */
  
  gal_pmp ( psun, psat, d ) ; 

/*
 * Calculate unit vector pointing to sun
 */
 
  gal_pn ( d, &md, e ) ;

/*
 * Calculate intermediates
 */
 
  gal_sxp ( 1.0 - eps, e, se ) ;
  gal_sxp ( 2.0 * eps * gal_pdp ( n, e ), n, sn ) ;
  gal_ppp ( se, sn, d ) ;

/*
 * Acceleration 
 */

  k = -p * v * au * au / ( md * md ) * area / mass ;
  
  gal_sxp ( k, d, a ) ;   
  
/*  
 * Finished.  
 */  
 
}
  
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
