/*  
 *  - - - - - - - - - - -
 *   g a l _ e a a d h p  
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     Computes the Earth's Atmospheric Density using the
 *     Harris-Priester Density Model
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     pv              d[3]         Position vector of satellite ( m, ms^-1 )
 *                                  True-of-date inertial reference frame
 *     height           d           Height of the satellite above mean sea level ( m )
 *     alpha            d           Right ascension of the Sun ( radians )
 *     delta            d           Declination of the Sun ( radians )
 *  
 *  Returned:  
 *  
 *     ad               d           Atmospheric density ( kg m^-3 )
 *     gal_eaadhp       i           0 = success
 *                                  1 = height out of range of model
 *
 *  Called:
 *
 *     gal_pdp          p-vector dot product
 *     gal_pm           modulus of p-vector
 *
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 89-91
 *  
 *  This revision:  
 * 
 *     2009 January 4 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_eaadhp.h"
#include "gal_pdp.h"
#include "gal_pm.h"

int
gal_eaadhp
  (
    double p[3],
    double height,
    double alpha,
    double delta,
    double *ad
  ) 
  
{

  const double ralag = 0.523599 ; /* Right ascension lag ( radians ) */
  const int    nprm  = 3 ;        /* Harris-Priester parameter 2(6) low(high) inclination */

/*
 * Harris-Priester atmospheric density coefficients valid for mean
 * solar activity (Long et al. 1989) g km^-3
 */
 
#define NUM_ADC 50

  const double adc[NUM_ADC][3] = {
  
   {  100.0, 497400.000000, 497400.000000, } ,  
   {  120.0,  24900.000000,  24900.000000, } ,  
   {  130.0,   8377.000000,   8710.000000, } ,  
   {  140.0,   3899.000000,   4059.000000, } ,  
   {  150.0,   2122.000000,   2215.000000, } ,  
   {  160.0,   1263.000000,   1344.000000, } ,  
   {  170.0,    800.800000,    875.800000, } ,  
   {  180.0,    528.300000,    601.000000, } ,  
   {  190.0,    361.700000,    429.700000, } ,  
   {  200.0,    255.700000,    316.200000, } ,  
   {  210.0,    183.900000,    239.600000, } ,  
   {  220.0,    134.100000,    185.300000, } ,  
   {  230.0,     99.490000,    145.500000, } ,  
   {  240.0,     74.880000,    115.700000, } ,  
   {  250.0,     57.090000,     93.080000, } ,  
   {  260.0,     44.030000,     75.550000, } ,  
   {  270.0,     34.300000,     61.820000, } ,  
   {  280.0,     26.970000,     50.950000, } ,  
   {  290.0,     21.390000,     42.260000, } ,  
   {  300.0,     17.080000,     35.260000, } ,  
   {  320.0,     10.990000,     25.110000, } ,  
   {  340.0,      7.214000,     18.190000, } ,  
   {  360.0,      4.824000,     13.370000, } ,  
   {  380.0,      3.274000,      9.955000, } ,  
   {  400.0,      2.249000,      7.492000, } ,  
   {  420.0,      1.558000,      5.684000, } ,  
   {  440.0,      1.091000,      4.355000, } ,  
   {  460.0,      0.770100,      3.362000, } ,  
   {  480.0,      0.547400,      2.612000, } ,  
   {  500.0,      0.391600,      2.042000, } ,  
   {  520.0,      0.281900,      1.605000, } ,  
   {  540.0,      0.204200,      1.267000, } ,  
   {  560.0,      0.148800,      1.005000, } ,  
   {  580.0,      0.109200,      0.799700, } ,  
   {  600.0,      0.080700,      0.639000, } ,  
   {  620.0,      0.060120,      0.512300, } ,  
   {  640.0,      0.045190,      0.412100, } ,  
   {  660.0,      0.034300,      0.332500, } ,  
   {  680.0,      0.026320,      0.269100, } ,  
   {  700.0,      0.020430,      0.218500, } ,  
   {  720.0,      0.016070,      0.177900, } ,  
   {  740.0,      0.012810,      0.145200, } ,  
   {  760.0,      0.010360,      0.119000, } ,  
   {  780.0,      0.008496,      0.097760, } ,  
   {  800.0,      0.007069,      0.080590, } ,  
   {  840.0,      0.004680,      0.057410, } ,  
   {  880.0,      0.003200,      0.042100, } ,  
   {  920.0,      0.002210,      0.031300, } ,  
   {  960.0,      0.001560,      0.023600, } ,  
   { 1000.0,      0.001150,      0.018100, } ,  
  } ;

  double hmin, hmax, dmin, dmax, u[3], cpsi2, cdec ;
  int ih ;
      
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Convert height to km
 */
 
  height /= 1000.0 ;

/*
 * Check height within model limits, if not return with a 
 * value of zero for atmospheric density  
 */
 
  if ( height <= adc[0][0] || height >= adc[NUM_ADC-1][0] ) {
    *ad = 0.0 ;
    return 1 ;
  }

/*  
 * Unit vector u towards the apex of the diurnal bulge
 * in inertial geocentric coordinates
 */
 
  cdec = cos ( delta ) ;
  u[0] = cdec * cos ( alpha + ralag ) ;
  u[1] = cdec * sin ( alpha + ralag ) ;
  u[2] = sin ( delta ) ;

/*
 * Cosine of half angle between satellite position vector and
 * apex of diurnal bulge
 */
 
  cpsi2 = 0.5 + 0.5 * gal_pdp ( p, u ) / gal_pm ( p ) ;

/*
 * Find correct height index
 */
 
  ih = 0 ;
  while ( ih < NUM_ADC - 1 && ( height < adc[ih][0] || height >= adc[ih + 1][0] ) ) {  
    ih++ ;
  }

/*
 * Exponential density interpolation 
 */
  
  hmin = ( adc[ih][0] - adc[ih + 1][0] ) / log ( adc[ih + 1][1] / adc[ih][1] ) ;
  hmax = ( adc[ih][0] - adc[ih + 1][0] ) / log ( adc[ih + 1][2] / adc[ih][2] ) ;

  dmin = adc[ih][1] * exp ( ( adc[ih][0] - height ) / hmin ) ;
  dmax = adc[ih][2] * exp ( ( adc[ih][0] - height ) / hmax ) ;

/*
 * Calculate Density
 */
 
  *ad = ( dmin + ( dmax - dmin) * pow ( cpsi2, nprm ) ) * 1.0e-12 ; /* kg m^-3 */

  return 0 ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  

