/*  
 *  - - - - - - - - - - - 
 *   g a l _ g m n o r m
 *  - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine normalizes a gravity model's coefficients
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     *gm1          gal_gm_t       Source gravity model
 *  
 *  Returned:  
 *  
 *     *gm2          gal_gm_t       Target gravity model
 *     *gal_gmnorm   gal_gm_t       *gm2
 *  
 *  Called:  
 *  
 *     gal_stnf                     Compute spherical terms normalization factor
 *     gal_stget                    Get spherical terms from gracity model
 *     gal_stset                    Set spherical terms in gravity model
 *     gal_facexp_alloc             Allocate factorial exponent lookup table
 *     gal_facexp_free              Deallocate factorial exponent lookup table
 *  
 *  This revision:  
 * 
 *     2008 May 30 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include "gal_gmnorm.h"
#include "gal_stnf.h"
#include "gal_stget.h"
#include "gal_stset.h"
#include "gal_facexp_alloc.h"
#include "gal_facexp_free.h"

gal_gm_t *
gal_gmnorm
  (
    gal_gm_t *gm1,
    gal_gm_t *gm2
  ) 
  
{

  int n, m, i, j, k ;
  double c, s, nf ;
  
  gal_facexp_t *facexp ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  j = ( gm1->max_degree < gm2->max_degree ) ? gm1->max_degree : gm2->max_degree ;
  k = ( gm1->max_order  < gm2->max_order  ) ? gm1->max_order  : gm2->max_order  ;
  
  if ( gm1->normalized ) {
    for ( n = 0; n <= j; n++ ) {
      i = ( n < k ) ? n : k ;
      for ( m = 0; m <= i; m++ ) {
        gal_stget ( n, m, gm1, &c, &s ) ;
        gal_stset ( n, m, c, s, gm2 ) ;
      }
    }
  }
  else {
    facexp = gal_facexp_alloc ( j + k ) ;
    for ( n = 0; n <= j; n++ ) {
      i = ( n < k ) ? n : k ;
      for ( m = 0; m <= i; m++ ) {
        gal_stget ( n, m, gm1, &c, &s ) ;
        nf = gal_stnf ( facexp, n, m ) ;
        gal_stset ( n, m, c * nf, s * nf, gm2 ) ;
      }
    }
    gal_facexp_free ( facexp ) ;
  } 

  return gm2 ;
    
/*  
 * Finished.  
 */  

}
 
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
