/*  
 *  - - - - - - - - - - 
 *   g a l _ s t s e t    
 *  - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine sets spherical terms C & S of degree N and order M
 *     in the given gravity model 
 *  
 *  Status: 
 *  
 *     support routine.  
 *  
 *  Given:  
 *
 *     n                i           required degree
 *     m                i           required order
 *     c                d           C coefficient
 *     s                d           S coefficient
 *     *gm           gal_gm_t       pointer to gravity model structure     
 *  
 *  Returned:  
 *  
 *     gal_stset        i           0 = success 
 *                                  1 = degree or order out of range
 *  
 *  Notes:  
 *  
 *  1) Gravitational coefficients C, S are efficiently stored in a single array CS. The lower 
 *     triangle matrix CS holds the non-sectorial C coefficients C[n][m] ( n != m ). Sectorial C 
 *     coefficients C[n][n] are the diagonal elements of CS and the upper triangular matrix 
 *     stores the S[n][m] ( m != 0 ) coefficients in columns, for the same degree n. Mapping of 
 *     CS to C, S is achieved through C[n][m] = CS[n][m], S[n][m] = CS[m-1][n].
 *  
 *  This revision:  
 * 
 *     2008 March 22 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include "gal_stset.h"
#include "gal_const.h"

int
gal_stset
  (
    const int n,
    const int m,
    const double c,
    const double s,
    gal_gm_t *gm
  ) 
  
{

  double *ptr ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  if ( n > gm->max_degree || m > gm->max_order ) {
    return GAL_FAILURE ;
  }
  else {
    ptr = gm->terms + n * ( gm->max_degree + 1 ) + m ;
    *ptr = c ;
    if ( m != 0 ) {
      ptr = gm->terms + ( m - 1 ) * ( gm->max_degree + 1 ) + n ;
      *ptr = s ;
    }
    return GAL_SUCCESS ; 
  }
  
/*  
 * Finished.  
 */  

}
 
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
