/*  
 *  - - - - - - - - - - 
 *   g a l _ e a 2 t a   
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates the true anomaly from eccentric anomaly.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     ea             double        Eccentric anomaly ( radians )
 *     ecc            double        Eccentricity
 *  
 *  Returned:  
 *  
 *     gal_ea2ta        d           True anomaly ( radians )
 *
 *  Notes:
 *
 *  1) This routine is valid for elliptical orbits only.
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Page 85
 *
 *  This revision:  
 * 
 *     2009 January 2 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_ea2ta.h"

double
gal_ea2ta
  (
    double ea,
    double ecc
  ) 
  
{

  double sinv, cosv, sea, cea, x ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  sea = sin ( ea ) ;
  cea = cos ( ea ) ;
  x = 1.0 - ecc * cea ;
  
  sinv = ( sea * sqrt ( 1.0 - ecc * ecc ) ) / x ;
  cosv = ( cea - ecc ) / x ;
  
  return atan2 ( sinv, cosv ) ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
