/*  
 *  - - - - - - - - - - - 
 *   g a l _ k e p 2 p v  
 *  - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine computes position and velocity from the classical orbital elements.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     gm             double        Gravitational parameter
 *     ecc              d           Eccentricity
 *     raan             d           Longitude of the ascending mode (radians) 
 *     argp             d           Argument of Pericentre (radians) 
 *     inc              d           Inclination (radians)
 *     p                d           Semi-Latus Rectum (m)
 *     v                d           True Anomaly (radians)
 *     truelon          d           True Longitude (radians)
 *     u                d           Argument of Latitude (radians)
 *     lonper           d           True Longitude of Periapsis (radians)
 *  
 *  Returned:  
 *  
 *     pv             d[2][3]       Position & Velocity Vectors
 * 
 *
 *  Notes:
 *
 *  1) If a parameter is undefined or unknown then set the value to GAL_UNDEFINED is returned. GAL_UNDEFINED is defined in gal_const.h .
 *
 *  Called:
 *
 *     gal_pqw2ijk                 Convert pv-vector from PQW to IJK frame 
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 118-122
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 28-32
 *
 *     Methods of Orbit Determination for the Micro Computer
 *     Boulet, Dan
 *     Willmann-Bell 1991
 *     Pages 149-157
 *
 *  This revision:  
 * 
 *     2008 September 20 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_kep2pv.h"
#include "gal_const.h"
#include "gal_pqw2ijk.h"

void
gal_kep2pv
  (
    double gm,
    double ecc, 
    double raan,
    double argp,
    double inc,
    double p,
    double v,
    double truelon,
    double u,
    double lonper,
    double pv[2][3]
  ) 
  
{

#define SMALL ( 0.00000001 )

  int circular, inclined ;

  double x, cosv, sinv, pqw[2][3] ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Set orbit type flags
 */
 
  if ( ecc < SMALL ) {
    circular = 1 ;
  }
  else {
    circular = 0 ;
  }  

  if  ( ( fabs ( inc ) < SMALL ) || ( fabs ( inc - GAL_PI ) < SMALL ) ) {
    inclined = 0 ;
  }
  else {
    inclined = 1 ;
  }

/*
 * Setup angles for special cases
 */
 
  if ( circular == 1 ) {
    if ( inclined == 0 ) {
      argp = 0.0 ;
      raan = 0.0 ;
      v = truelon ;
    }
    else {
      argp = 0.0 ;
      v = u ;
    }
  }
  else if ( inclined == 0 ) {
    argp = lonper ;
    raan = 0.0 ;
  }
   
/*
 * Form pqw position and velocity vectors
 */
 
  cosv = cos ( v ) ;
  sinv = sin ( v ) ;
  x = p / ( 1.0 + ecc * cosv ) ;
  pqw[0][0] = x * cosv ;
  pqw[0][1] = x * sinv ;
  pqw[0][2] = 0.0 ;
  
  if ( fabs ( p ) < 0.0001 ) {
    p = 0.0001 ;
  }

  x = sqrt ( gm ) / sqrt ( p ) ;  
  pqw[1][0] = -sinv * x ;
  pqw[1][1] = ( ecc + cosv ) * x ;
  pqw[1][2] = 0.0 ;

/*
 * Rotate to IJK 
 */
  
  gal_pqw2ijk ( pqw, raan, argp, inc, pv ) ; 
        
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

