/*  
 *  - - - - - - - - - - 
 *   g a l _ m a 2 e a   
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates the eccentric anomaly from mean anomaly.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     ma             double        Mean anomaly ( radians )
 *     ecc            double        Eccentricity
 *  
 *  Returned:  
 *  
 *     gal_ma2ea        d           Eccentric anomaly ( radians )
 *
 *  Notes:
 *
 *  1) This routine is valid for elliptical orbits only.
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 72-75
 *
 *  This revision:  
 * 
 *     2009 January 2 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include <float.h>
#include "gal_ma2ea.h"
#include "gal_const.h"

double
gal_ma2ea
  (
    double ma,
    double ecc
  ) 
  
{

  double ea, ean ;
  double tol = DBL_EPSILON * 10.0 ; 
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  if ( -GAL_PI < ma || ma > GAL_PI ) {
    ea = ma - ecc ;
  }
  else {
    ea = ma + ecc ;
  }
  
  do {
    ean = ea ;
    ea = ean + ( ma - ean + ecc * sin ( ean ) ) / ( 1.0 - ecc * cos ( ean ) ) ;
  } while ( fabs ( ean - ea ) > tol ) ;
  
  return ea ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
