/*  
 *  - - - - - - - - - - 
 *   g a l _ m a 2 h a   
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates the hyperbolic anomaly from mean anomaly.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     ma             double        Mean anomaly ( radians )
 *     ecc            double        Eccentricity
 *  
 *  Returned:  
 *  
 *     gal_ma2ha        d           Hyperbolic anomaly ( radians )
 *
 *  Notes:
 *
 *  1) This routine is valid for hyperbolic orbits only.
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 78-80
 *
 *  This revision:  
 * 
 *     2009 January 2 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include <float.h>
#include "gal_const.h"
#include "gal_ma2ha.h"

double
gal_ma2ha
  (
    double ma,
    double ecc
  ) 
  
{

  double ha, han ;
  double tol = DBL_EPSILON * 10.0 ; 
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  if ( ecc < 1.6 ) {
    if ( ( -GAL_PI < ma && ma < 0.0 ) || ma > GAL_PI ) {
      ha = ma - ecc ;
    }
    else {
      ha = ma + ecc ;
    }
  }
  else {
    if ( ecc < 3.6 && fabs ( ma ) > GAL_PI ) {
      ha = ma - GAL_SIGN ( 1.0, ma ) * ecc ;
    }
    else {
      ha = ma / ( ecc - 1.0 ) ;
    }
  }
  
  do {
    han = ha ;
    ha = han + ( ma - ecc * sinh ( han ) + han ) / ( ecc * cosh ( han ) - 1.0 ) ;
  } while ( fabs ( han - ha ) > tol ) ;
  
  return ha ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
