/*  
 *  - - - - - - - - - - - 
 *   g a l _ p v 2 k e p  
 *  - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine computes the classical orbital elements from position and velocity.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     gm             double        Gravitational parameter
 *     pv             d[2][3]       Position & Velocity Vectors
 *  
 *  Returned:  
 *  
 *     *sma             d           Semi-Major Axis (m)
 *     *ecc             d           Eccentricity
 *     *raan            d           Longitude of the ascending mode (radians) 
 *     *argp            d           Argument of Pericentre (radians) 
 *     *ma              d           Mean Anomaly (radians)
 *     *inc             d           Inclination (radians)
 *     *p               d           Semi-Latus Rectum (m)
 *     *v               d           True Anomaly (radians)
 *     *truelon         d           True Longitude (radians)
 *     *u               d           Argument of Latitude (radians)
 *     *lonper          d           True Longitude of Periapsis (radians)
 * 
 *
 *  Notes:
 *
 *  1) If a parameter is undefined the value GAL_UNDEFINED is returned. The user should check the return values
 *     for undefined cases. GAL_UNDEFINED is defined in gal_const.h .
 *
 *  Called:
 *
 *     gal_pxp                      cross product of two p vectors
 *     gal_pm                       modulus of a p vector
 *     gal_pdp                      p vector dot product
 *     gal_anp                      Normalize angle into the range 0 <= a < 2pi
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 118-122
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Pages 28-32
 *
 *     Methods of Orbit Determination for the Micro Computer
 *     Boulet, Dan
 *     Willmann-Bell 1991
 *     Pages 149-157
 *
 *  This revision:  
 * 
 *     2008 September 20 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include <float.h>
#include "gal_pv2kep.h"
#include "gal_pxp.h"
#include "gal_pm.h"
#include "gal_pdp.h"
#include "gal_anp.h"
#include "gal_const.h"

void
gal_pv2kep
  (
    double gm,
    double pv[2][3],
    double *sma,
    double *ecc, 
    double *raan,
    double *argp,
    double *ma,
    double *inc,
    double *p,
    double *v,
    double *truelon,
    double *u,
    double *lonper
  ) 
  
{

#define SMALL ( 0.00000001 )

  double h[3], hmag, rmag, vdotv, rdotv, W[3], vmag, E, x, e[3], y, emag, gmr, h2, ecosv, esinv, n[3], nmag ;
  int circular, inclined ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Set default values to undefined
 */
 
  *sma     = GAL_UNDEFINED ;
  *ecc     = GAL_UNDEFINED ;
  *raan    = GAL_UNDEFINED ;
  *argp    = GAL_UNDEFINED ;
  *ma      = GAL_UNDEFINED ;
  *inc     = GAL_UNDEFINED ;
  *p       = GAL_UNDEFINED ;
  *v       = GAL_UNDEFINED ;
  *truelon = GAL_UNDEFINED ;
  *u       = GAL_UNDEFINED ;
  *lonper  = GAL_UNDEFINED ;
   
/*
 * Calculate interim results
 */
 
  rmag  = gal_pm ( pv[0] ) ;
  vmag  = gal_pm ( pv[1] ) ;
  vdotv = gal_pdp ( &pv[1][0], &pv[1][0] ) ;
  rdotv = gal_pdp ( &pv[0][0], &pv[1][0] ) ;
  gmr   = gm * rmag ;

/*
 * Calculate Eccentricity Vector
 */

  x = vdotv / gm - 1.0 / rmag ;
  y = rdotv / gm ;
 
  e[0] = x * pv[0][0] - y * pv[1][0] ;
  e[1] = x * pv[0][1] - y * pv[1][1] ;
  e[2] = x * pv[0][2] - y * pv[1][2] ;
  emag = gal_pm ( e ) ;

/*
 * Calculate Eccentricity
 */

  *ecc = gal_pm ( e ) ; 

  if ( *ecc < SMALL ) {
    circular = 1 ;
  }
  else {
    circular = 0 ;
  }  

/*
 * Find angular momentum vector ( areal velocity vector )
 */

  gal_pxp ( &pv[0][0], &pv[1][0], h ) ;
  hmag  = gal_pm ( h ) ;   
  h2    = hmag * hmag ;
  ecosv = h2 / gmr - 1.0 ;
  esinv = rdotv * hmag / gmr ;

/*
 * Build n vector
 */
 
 n[0] = -h[1] ;
 n[1] =  h[0] ;
 n[2] =   0.0 ;
 nmag = gal_pm ( n ) ;
  
/*
 * Calculate semi-major axis for Elliptical and Hyperbolic Orbits
 * Parabolic orbits sma is infinite, and is set to zero here
 */
 
  *sma = 2.0 / rmag - vmag * vmag / gm  ;
  if ( *sma != 0.0 ) {
    *sma = 1.0 / *sma ;
  }
      
/*
 * Calculate semi-latus rectum
 */
 
  *p = h2 / gm ;
  
/*
 * Calculate Inclination
 * Formula from GTDS Math Theory page 3-62
 */

  *inc = atan2 ( sqrt ( h[0] * h[0] + h[1] * h[1] ) / hmag, h[2] / hmag ) ; 
  
  if  ( ( fabs ( *inc ) < SMALL ) || ( fabs ( *inc - GAL_PI ) < SMALL ) ) {
    inclined = 0 ;
  }
  else {
    inclined = 1 ;
  }
   
/*
 * Calculate longitude of ascending node
 */

  W[0] =  h[0] / hmag ;
  W[1] =  h[1] / hmag ;
  W[2] =  h[2] / hmag ;

  if ( nmag > SMALL ) {
    *raan = gal_anp ( atan2 ( W[0], -W[1] ) ) ;
  }  

/*
 * Calculate True Anomaly
 */

  *v = gal_anp ( atan2 ( esinv, ecosv ) ) ;

  if ( inclined == 1 ) {

 /*
  * Calculate Argument of Latitude
  */
  
    *u = gal_anp ( atan2 ( pv[0][2], -pv[0][0] * W[1] + pv[0][1] * W[0] ) ) ;

/* 
 * Calculate Argument of Perigee
 */
 
    *argp = gal_anp ( (*u) - (*v) ) ;
      
  }
 
/*
 * Calculate Longitude of Periapsis
 */

  if ( circular == 0 && inclined == 0 ) {

    x = e[0] / *ecc ;
    if ( fabs ( x ) > 1.0 ) {
      x = GAL_SIGN ( 1.0, x ) ;
    }
    *lonper= acos ( x ) ;
    if ( e[1] < 0.0 ) {
      *lonper = GAL_2PI - (*lonper) ;
    }
    if ( *inc > GAL_PI * 0.5 ) {
      *lonper = GAL_2PI - (*lonper) ;
    }

  }
  
/*
 * Calculate Mean Anomaly
 */
 
  if ( esinv == 0.0 ) {
    *ma = (*v) ;
  }

/* 
 * Elliptical trajectory. 
 */

  else if ( *ecc < ( 1.0 - SMALL ) ) {

/*
 * Calculate Eccentric Anomaly
 */
 
    E = 2.0 * atan ( sqrt ( ( 1.0 - (*ecc) ) / ( 1.0 + (*ecc) ) ) * tan ( 0.5 * (*v) ) ) ;
    
/*
 * Calculate Mean Anomaly
 */
 
    *ma = gal_anp ( E - (*ecc) * sin ( E ) ) ; 

    if ( circular == 1 ) {
    
      if ( inclined == 0 ) {

        x = pv[0][0] / rmag ;  
        if ( fabs ( x ) > 1.0  ) {
          x = GAL_SIGN (1.0, x ) ;
        }
        *truelon = acos ( x ) ;
        if ( pv[0][1] < 0.0  ) {
           *truelon = GAL_2PI - *truelon ;
        }
        if ( *inc > GAL_PI / 2.0 ) {
           *truelon = GAL_2PI - *truelon ;
        }
        *ma = *truelon ;
        
      }
      else {

/* 
 * Calculate Mean Anomaly
 */
 
        *ma = gal_anp ( (*argp) + (*ma) ) ;
        *argp = GAL_UNDEFINED ;

      }

      *v = GAL_UNDEFINED ;
      
    }
    
  }
    
/* 
 * Hyperbolic Trajectory
 */

  else if (  *ecc > 1.0 + SMALL ){
  
    if ( fabs ( *v ) + 0.00001 < GAL_PI - acos ( 1.0 / (*ecc) ) ) {
      x = ( *ecc + cos ( *v ) ) / ( 1.0 + ecosv ) ;
      E = log ( x + sqrt ( x * x  - 1.0 ) ) ;
      *ma = (*ecc) * sinh ( E ) - E ;
    }
  
  }

/* 
 * Parabolic Trajectory
 */

  else {
   
/*
 * Calculate Mean Anomaly
 */

    if ( fabs ( *v ) < 168.0 * GAL_D2R ) {
      E = rdotv / gm ;
      *ma = (*p) * E + ( E * E * E ) / 3.0 ;
    }

  }

  if ( circular == 0 || inclined == 0 ) {
    *u = GAL_UNDEFINED ;
  }
    
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
