/*  
 *  - - - - - - - - - - - 
 *   g a l _ p v t 2 p v  
 *  - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates position and velocity from starting position and 
 *     velocity at given time using Universal variables. This routine is valid
 *     for all orbit types.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     gm               d           Gravitational coefficient
 *     pv0            d[2][3]       Epoch Position & Velocity Vectors (m, m/s)
 *     ed0, ed1         d           Epoch date (TT)
 *     tt0, tt1         d           Required date (TT)              
 *  
 *  Returned:  
 *  
 *     pv             d[2][3]       Position & velocity vectors for required date (m, m/s )
 * 
 *  Notes:  
 *  
 *  1) The TT dates are Julian Dates, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *               ed0          ed1  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *
 *  2) The iteration method is the Laguerre's method described in Chobotov page 58.
 *     It was selected as it converged faster than the Newton-Raphson method 
 *     described by Vallado, and the choice of initial value is simpler.
 *
 *  3) The calculations of sn and cn are from Vallado as they are simpler
 *      to implement.
 *
 *  Called:
 *
 *     gal_pm             Modulus of p-vector
 *     gal_pdp            p-vector dot product
 *     gal_sxp            product of p-vector and scalar
 *     gal_ppp            p-vector addition
 *
 *
 *  References:
 *
 *     Orbital Mechanics Third Edition
 *     AiAA Education Series
 *     Chobotov, Vladimir A. Ed.
 *     Pages 55-61
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 101-103
 *
 *  This revision:  
 * 
 *     2008 September 13
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_pvt2pv.h"
#include "gal_pm.h"
#include "gal_const.h"
#include "gal_pdp.h"
#include "gal_sxp.h"
#include "gal_ppp.h"

void
gal_pvt2pv
  (
    double gm,
    double pv0[2][3],
    double ed0,
    double ed1, 
    double tt0,
    double tt1,
    double pv[2][3]
  ) 
  
{

  double dt, r0, v0, a, sgm, p, xn, rdotv, xn2, xn3, rzn, cn, sn, zn ;
  double f, g, fdot, gdot, fr[3], gv[3], snzn,r, alpha, cnzn, ra ;
  double fx, fxdd, dn, dxn ;
    
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate vector magnitudes, and derive inverse-alpha
 */
 
  r0 = gal_pm ( pv0[0] ) ;
  v0 = gal_pm ( pv0[1] ) ;

  alpha = 2.0 / r0 - v0 * v0 / gm ;
  
  sgm = sqrt ( gm ) ;
  
/*
 * Calculate delta Time
 */
 
  dt = ( ( tt0 - ed0 ) + ( tt1 - ed1 ) ) * GAL_D2S ;
  
  if ( alpha > 0.0 ) {

    a = 1.0 / alpha ;  
    p = GAL_2PI * pow ( a, 3.0 / 2.0 ) / sgm ;
    dt -= GAL_SIGN ( floor ( fabs ( dt ) / p ) * p, dt ) ;
    
  }
  
/*
 * Calculate Universal variable first estimate 
 */  
  
  xn = sgm * dt * fabs ( alpha ) ;

/*
 * Calculate constants
 */
 
  rdotv = gal_pdp ( pv0[0], pv0[1] ) / sgm ;

/*
 * Solve
 */
 
  do {

    xn2 = xn  * xn ;
    xn3 = xn2 * xn ;
    zn  = xn2 * alpha ;
    
/*
 * Calculate Cn and Sn
 */

    if ( zn > 1e-6 ) {
      rzn = sqrt ( zn ) ;
      cn = ( 1.0 - cos ( rzn ) ) / zn ;
      sn = ( rzn - sin ( rzn ) ) / sqrt ( zn * zn * zn ) ;
    }
    else if ( zn < -1e-6 ) {
      rzn = sqrt ( -zn ) ;
      cn = ( 1.0 - cosh ( rzn ) ) / zn ;
      sn = ( sinh ( rzn ) - rzn ) / sqrt ( (-zn) * (-zn) * (-zn) ) ;
    }
    else {
      cn = 1.0 / 2.0 ;
      sn = 1.0 / 6.0 ;
    }

/*
 * Calculate Universal variable
 */
 
    snzn = xn * ( 1.0 - sn * zn ) ;
    cnzn = ( 1.0 - zn * cn ) ;
    ra   = ( 1.0 - r0 * alpha ) ;
    fx   = ra * sn * xn3 + rdotv * cn * xn2 + r0 * xn - sgm * dt ;
    r    = xn2 * cn + rdotv * snzn + r0 * cnzn ;
    fxdd = ra * snzn + rdotv * cnzn ;
    dn   = 2.0 * sqrt ( 4.0 * r * r - 5.0 * fx * fxdd ) ;
    dxn  = ( 5.0 * fx ) / ( r + GAL_SIGN ( dn, r ) ) ;
    xn  -= dxn ;
    
  } while ( fabs ( dxn * dxn * alpha ) >= 1e-8 ) ; 
    
/*
 * Calculate f and g
 */

  xn2 = xn * xn ; 
  f   = 1.0 - xn2 / r0 * cn ;
  g   = dt - xn2 * xn / sgm * sn ;
    
/* 
 * Calculate f dot and g dot
 */

  fdot = sgm / ( r * r0 ) * ( sn * zn - 1.0 ) * xn ;
  gdot = 1.0 - xn2 / r * cn ;
    
/*
 * Calculate new position and velocity vectors
 */
 
  gal_sxp ( f, pv0[0], fr ) ;
  gal_sxp ( g, pv0[1], gv ) ;
  gal_ppp ( fr, gv, pv[0] ) ;
    
  gal_sxp ( fdot, pv0[0], fr ) ;
  gal_sxp ( gdot, pv0[1], gv ) ;
  gal_ppp ( fr, gv, pv[1] ) ;
  
/*  
 * Finished.  
 */
   
}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
 
