/*  
 *  - - - - - - - - - 
 *   g a l _ t 2 p a   
 *  - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates the parabolic anomaly from time dt.
 *  
 *  Status: 
 *  
 *     Classical Model.  
 *  
 *  Given:  
 *
 *     gm             double        Gravitional parameter
 *     dt             double        Time since periapsis ( seconds )
 *     p              double        Semi parameter ( meters )
 *  
 *  Returned:  
 *  
 *     gal_t2pa        d           Parabolic anomaly ( radians )
 *
 *  Notes:
 *
 *  1) This routine is valid for parabolic trajectories only.
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 75-78
 *
 *  This revision:  
 * 
 *     2009 January 2 
 *  
 *  Copyright (C) 2009 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_t2pa.h"
#include "gal_const.h"
#include <float.h>
  
__BEGIN_DECLS  

double
gal_t2pa
  (
    double gm,
    double dt,
    double p
  ) 
  
{

  double tol = 10.0 * DBL_EPSILON ;
  double x, pa, pan, p2 ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

 pa = 0.0 ;
 
 x = 2.0 * sqrt ( gm / ( p * p * p ) ) * dt ;
 
 do {
   pan = pa ;
   p2 = pan * pan ;
   pa = pan - ( p2 * pan / 3.0 + pan - x ) / ( p2 + 1.0 ) ;
 } while ( fabs ( pan - pa ) > tol ) ;
 
 return pa ;
 
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2009 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
