/*  
 *  - - - - - - - - - - - - - - - - -
 *   g a l _ f a c e x p _ a l l o c
 *  - - - - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine computes a factorial exponent lookup table
 *     required by the gal_factorial2 routine.
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     max_factorial            i           Maximum factorial required
 *  
 *  Returned:  
 * 
 *     *gal_facexp_alloc    gal_facexp_t    Pointer to lookup table if success
 *                                          NULL if memory allocation fails
 *
 *  References:
 *
 *     Calculation of Factorials
 *     M. L. Charnow and Jesse L. Maury, Jr.
 *     NASA TM X-55733 GSFC X-542-66-460
 *     Steptember 1966
 * 
 *  This revision:  
 * 
 *     2008 May 30 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include "gal_facexp_alloc.h"
#include <stdlib.h>

gal_facexp_t *
gal_facexp_alloc
  (
     int max_factorial
   )

{

#define SIEVE( n ) *( sieve + ( n ) )

  gal_facexp_t *facexp ;

  char *sieve ;
  int n, s, fact, m, p, r, prime ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Allocate facexp structure
 */

  if ( ( facexp = malloc ( sizeof(gal_facexp_t) ) ) == NULL ) {
    return NULL ;
  }

  facexp->max_factorial = max_factorial ;

/*
 * Allocate sieve
 */

  facexp->num_prime = 0 ; 
  if ( ( sieve = calloc( max_factorial+1, sizeof(char) ) ) == NULL ) {
    free ( facexp ) ;
    return NULL ;
  } 

/*
 * Sieve of Eratosthenes
 */

  for ( n = 2; n <= max_factorial; n++ ) {
    if ( SIEVE(n) == 0 ) {
       facexp->num_prime++ ;
       for ( s = n + n; s <= max_factorial; s += n ) {
         SIEVE(s) = 1 ;
       }
    }
  }

/*
 * Allocate storage for primes
 */

  if ( ( facexp->primes = malloc( facexp->num_prime * sizeof(int) ) ) == NULL ) {
    free ( facexp ) ;
    free ( sieve ) ;
    return NULL ;
  }

/*
 * Store primes found by sieve
 */

  s = 0 ;
  for ( n = 2; n <= max_factorial; n++ ) {
    if ( SIEVE(n) == 0 ) {
      PRIME(s++) = n ;
    } 
  }

  free ( sieve ) ; 

/*
 * Allocate storage for exponents, init to zero
 */

  if ( ( facexp->exponents = calloc( max_factorial * ( facexp->num_prime + 1 ), sizeof ( int ) ) ) == NULL ) {
    free ( facexp->primes ) ;
    free ( facexp ) ;
    return NULL ;
  } ;

/*
 * Find prime factors and exponents in colums [1] - [num_prime]
 * Store index of highest prime with non-zero exponent in column [0]
 */

  for ( fact = 2; fact <= max_factorial; fact++ ) {

    for ( n = 2; n <= fact; n++ ) {
  
      m = n ;
      p = 0 ;

      while ( m != 1 ) {
        prime = PRIME( p ) ;
        r = m % prime ;    
        while ( m != 1 && r == 0 ) { 
          EXPONENT( fact, p + 1 ) = EXPONENT( fact, p + 1 ) + 1 ;
          if ( p >= EXPONENT( fact, 0 ) ) {
             EXPONENT( fact, 0 ) = p + 1 ;
          }
          m /= prime ;
          r = m % prime ;    
        }

        p++ ;

      }
    }
  }

/*
 * Return pointer to exponent lookup table
 */

  return facexp ;
  
/*  
 * Finished.  
 */  
 
}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
