/*  
 *  - - - - - - - - - - - - - - -
 *   g a l _ f a c t o r i a l 2 
 *  - - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine computes the factorial n!, or the value of n! / m!, or 
 *     n! * m!.
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     *facexp     gal_facexp_t     Pointer to factorial exponent lookup table
 *     n                i           n >= 0
 *     m                i           m >= 0
 *     s                i           When  0 result is n!
 *                                  When -1 result is n! / m!
 *                                  When +1 result is n! * m!
 *  
 *  Returned:  
 * 
 *     *f              ld           result  
 *     gal_factorial    i           status
 *                                  0 = Success
 *                                  1 = Factorial requested is beyond the range of the lookup table
 *                                  2 = Factorial requested is greater than x!
 *
 *  Notes
 *
 *  1) When s=0 the maximum factorial that can be represented by IEEE double precision
 *     real is 170!
 *
 *  References:
 *
 *     Calculation of Factorials
 *     M. L. Charnow and Jesse L. Maury, Jr.
 *     NASA TM X-55733 GSFC X-542-66-460
 *     Steptember 1966
 * 
 *  This revision:  
 * 
 *     2008 June 14 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_factorial2.h"
#include "gal_facexpdef.h"

int
gal_factorial2
  (
    gal_facexp_t *facexp,
    int n,
    int m,
    int s,
    long double *f 
  ) 
  
{

  int se2, max ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Check that requested factorials are within the range of the exponent
 * lookup table 
 */


  if ( n > facexp->max_factorial || m > facexp->max_factorial || n < 0 || m < 0 ) {
    return 1 ;
  }

/*
 * The largest factorial that can be represented by an IEEE long double precision
 * real is x! <- This needs adjusting!
 */

  if ( s == 0 && n > 1000 ) {
    return 2 ;
  }

/*
 * Shortcut - anything divided by itself is 1.0
 */

  if ( s == -1 && n == m ) {
    *f = 1.0 ;
    return 0 ;
  }

/*
 * The first entry in the exponent lookup table is 1!, but
 * 0! = 1! = 1.0 by definition
 */

  if ( n == 0 ) {
    n = 1 ;
  }

  if ( s != 0 && m == 0 ) {
    m = 1 ;
  }

/*
 * Find the index of the largest prime number with a non-zero exponent
 * in either of the two requested factorials
 */

  max = EXPONENT( n, 0 ) ;
  if ( s != 0 ) {
    se2 = EXPONENT( m, 0 ) ;
    if ( se2 > max ) {
      max = se2 ;
    }
  }
 
/*
 * Calculate the result
 *
 * When s =  0 then n!      is returned
 * When s = -1 then n! / m! is returned
 * When s = +1 then n! * m! is returned
 *
 * The calculation starts with the largest prime, lowest exponent
 * and moves down to the smallest prime with largest exponent. This
 * has the effect of adding the numerically smaller numbers together
 * first.
 */

  *f = 1.0 ;
  while ( max > 0 ) {
    if ( s != 0 ) {
      *f *= pow ( PRIME( max - 1 ), EXPONENT( n, max ) + s * EXPONENT( m, max ) ) ;
    }
    else {
      *f *= pow ( PRIME( max - 1 ), EXPONENT( n, max ) ) ;
    }
    max-- ;
  }

  return 0 ;
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
