/*  
 *  - - - - - - - - 
 *   g a l _ r k f  
 *  - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine integrates an ordinary deferential equation 
 *     using the Runge-Kutte-Fehlberg method.
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     ystart          d[]          Starting y values
 *     nvar             i           Number of equations to integrate
 *     x1               d           Starting x value
 *     x2               d           Ending x value
 *     eps              d           Accuracy
 *     h1               d           First guess step-size
 *     hmin             d           Minimum step-size
 *     *derivs        func          User defined function for calculating the right hand side derivatives
 *     *rkfs          func          Required Runge-Kutte-Fehlberg stepper routine
 *     *derivsp         i           Pointer to parameters structure for derivs routine
 *
 *  Returned:  
 *  
 *     ystart          d[]          Ending y values
 *     gal_rkf          i           Status
 *                                    0 = Success
 *                                    1 = Failed to allocate workspace memory
 *                                    2 = Stepsize underflow
 *                                    3 = Maximum steps exceeded
 *                                    4 = Stepsize too small
 *  
 *  References:
 *
 *     NASA Technical Report TR R-352
 *     Some Experimental Results Concerning The Error Propagation in
 *     Runge-Kutte type integration formulas
 *     by Erwin Fehlberg
 *     October 1970
 *
 *  This revision:  
 * 
 *     2008 May 18
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdlib.h>
#include <math.h>
#include "gal_const.h"
#include "gal_rkf.h"
#include "gal_rkfqs.h"

int gal_rkf
  (
    double ystart[],
    int nvar,
    double x1,
    double x2,
    double eps,
    double h1,
    double hmin,
    void ( *derivs ) ( double, double [], double [], int * ),
    int ( *rkfs ) ( double [], double [], int, double, double, double [], double [], void ( * ) ( double, double [], double [], int * ), int * ) ,
    int *derivsp
  ) 
{

#define RKF_MAXSTEP  1000000
#define RKF_TINY     1.0e-30
   
  int nstp, i, jstat ;
  double x, hnext, hdid, h, *yscal, *y, *dydx ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Allocate Workspace
 */
 
  yscal = malloc ( sizeof ( double ) * nvar ) ;
  if ( yscal == NULL ) {
    return 1 ;
  }
  y = malloc ( sizeof ( double ) * nvar ) ;
  if ( y == NULL ) {
    free ( yscal ) ;
    return 1 ;
  }
  dydx =  malloc ( sizeof ( double ) * nvar ) ;
  if ( dydx == NULL ) {
    free ( y ) ;
    free ( yscal ) ;
    return 1 ;
  }

/*
 * Do the stuff
 */
 	
  x = x1 ;
  h = GAL_SIGN ( h1, x2-x1 ) ;
  
  for ( i = 0 ; i < nvar ; i++ ) {
    y[i] = ystart[i] ;
  }
  
  for ( nstp = 0; nstp < RKF_MAXSTEP; nstp++ ) {
  
    ( *derivs ) ( x, y, dydx, derivsp ) ;

    for ( i = 0; i < nvar; i++ ) {
      yscal[i] = fabs ( dydx[i] * h ) + RKF_TINY ;
    }
    
    if ( ( x + h - x2 ) * ( x + h - x1 ) > 0.0) {
      h = x2 - x ;
    }
    
    if ( ( jstat = gal_rkfqs ( y, dydx, nvar, &x, h, eps, yscal, &hdid, &hnext, derivs, rkfs, derivsp ) ) != GAL_SUCCESS ) {
      free ( dydx ) ;
      free ( y ) ;
      free ( yscal ) ;
      return jstat ;
    } ;
    
    if ( ( x - x2 ) * ( x2 - x1 ) >= 0.0 ) {
    
      for ( i = 0; i < nvar; i++ ) {
        ystart[i] = y[i] ;
      }
      
      free ( dydx ) ;
      free ( y ) ;
      free ( yscal ) ;
      return GAL_SUCCESS ;
      
    }

    if ( fabs ( hnext ) <= hmin ) {
      free ( dydx ) ;
      free ( y ) ;
      free ( yscal ) ;
      return 4 ;
    }
    
    h = hnext ;

  }

/*
 * De-allocate workspace memory
 */
   
  free ( dydx ) ;
  free ( y ) ;
  free ( yscal ) ;
  return 3 ;
  
/*  
 * Finished.  
 */  

}  
 
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
