/*  
 *  - - - - - - - - - -
 *   g a l _ r k f q s
 *  - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine takes one "quality-controlled" Runge-Kutta-Fehlberg step
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     y               d[]			dependent variable vector
 *     n                i           Number of equations to integrate
 *     *x               d           Independent variable value
 *     htry             d           Stepsize to attempt
 *     eps              d           Accuracy
 *     *derivs        func          User defined function for calculating the right hand side derivatives
 *     *rkfs          func          Required Runge-Kutte-Fehlberg stepper routine
 *     *derivsp         i           Pointer to parameters structure for derivs routine
 *
 * Outputs:
 *
 *     dydx            d[]          derivative of dependent variable vector
 *     yscal           d[]          Used for error scaling
 *     *hdid            d           Stepsize accomplished
 *     *hnext           d           Estimated next stepsize
 *     gal_rkfqs        i           Status
 *                                    0 = Success
 *                                    1 = Unable to allocate workspace memory
 *                                    2 = Stepsize underflow
 *
 *  References:
 *
 *     NASA Technical Report TR R-352
 *     Some Experimental Results Concerning The Error Propagation in
 *     Runge-Kutte type integration formulas
 *     by Erwin Fehlberg
 *     October 1970
 *
 *  This revision:  
 * 
 *     2008 May 18 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdlib.h>
#include <math.h>
#include "gal_const.h"
#include "gal_rkfqs.h"

int gal_rkfqs
  (
    double y[],
    double dydx[],
    int n,
    double *x,
    double htry,
    double eps,
    double yscal[],
    double *hdid,
    double *hnext,
    void ( *derivs ) ( double, double [], double [], int * ),
    int ( *rkfs ) ( double [], double [], int, double, double, double [], double [], void ( * ) ( double, double [], double [], int * ), int * ),
    int *derivsp
  ) 
  
{

  int i, jstat ;
  double errmax, h, htemp, xnew, *yerr, *ytemp ;

#define SAFETY  0.9
#define PGROW   (-1.0 / 8.0) 
#define PSHRINK (-1.0 / 7.0) 
#define ERRCON  2.56578451395034701e-8 /* ERRCON equals (5/SAFETY) raised to the power (1/PGROW) */
	
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Allocate workspace memory
 */
 
  yerr  = malloc ( sizeof ( double ) * n ) ;
  if ( yerr == NULL ) {
    return 1 ;
  }
  ytemp = malloc ( sizeof ( double ) * n ) ;
  if ( ytemp == NULL ) {
    free ( yerr ) ;
    return 1 ;
  }

/*
 * Do it!
 */
 
  h = htry ;
  
  for ( ; ; ) {
  
    if ( ( jstat = ( *rkfs ) ( y, dydx, n, *x, h, ytemp, yerr, derivs, derivsp ) ) != GAL_SUCCESS ) {
      free ( ytemp ) ;
      free ( yerr ) ;
      return jstat ;
    } 
    
    errmax = 0.0 ;
    for ( i = 0; i < n; i++ ) {
      errmax = GAL_MAX ( errmax, fabs ( yerr[i] / yscal[i] ) ) ;
    }
    errmax /= eps ;
    if ( errmax <= 1.0 ) {
      break ;
    }
    
    htemp = SAFETY * h * pow ( errmax, PSHRINK ) ;
    h = ( h >= 0.0 ? GAL_MAX ( htemp, 0.1 * h ) : GAL_MIN ( htemp, 0.1 * h ) ) ;
    xnew = *x + h ;
    
/*
 * Test for Underflow
 */
 
    if ( xnew == *x ) {
      return 2 ;
    }
    
  }
  
  if ( errmax > ERRCON ) {
    *hnext = SAFETY * h * pow ( errmax, PGROW ) ;
  }
  else {
    *hnext = 5.0 * h ;
  }
  
  *x += *hdid = h ;
  
  for ( i = 0; i < n; i++ ) {
    y[i] = ytemp[i] ;
  }
    
/*
 * De-allocate workspace memory
 */
 
    free ( ytemp ) ;
    free ( yerr ) ;
    
    return GAL_SUCCESS ;	
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
