/*  
 *  - - - - - - - - - - -
 *   g a l _ r k f s 5 6 
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine takes a Runge-Kutte-Fehlberg 5(6) step
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     y               d[]          dependent variable vector
 *     dydx            d[]          derivative of dependent variable vector
 *     n                i           Number of equations to integrate
 *     x                d           Independent variable value
 *     h                d           Stepsize
 *     *derivs         func         User defined function for calculating the right hand side derivatives
 *     *derivsp         i           Pointer to parameters structure for derivs routine
 *
 *  Returned:  
 *  
 *     yout            d[]          Ending y values
 *     yerr            d[]          Errors
 *     gal_rkfs45       i           Status
 *                                    0 = Success
 *                                    1 = Failed to allocate memory
 *  
 *  References:
 *
 *     NASA Technical Report TR R-352
 *     Some Experimental Results Concerning The Error Propagation in
 *     Runge-Kutte type integration formulas
 *     by Erwin Fehlberg
 *     October 1970
 *
 *  This revision:  
 * 
 *     2008 May 18 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdlib.h>
#include "gal_const.h"
#include "gal_rkfs45.h"

int gal_rkfs56
  (
    double y[],
    double dydx[],
    int n,
    double x,
    double h,
    double yout[],
    double yerr[],
    void ( *derivs ) ( double, double [], double [], int * ),
    int *derivsp
  ) 
  
{

struct {
  double a[8] ;
  double b[8][7] ;
  double c[8] ;
  double chat[8] ;
} param = {

 { 
          0.0,
    1.0 / 6.0,
   4.0 / 15.0,
    2.0 / 3.0,
    4.0 / 5.0,
          1.0,
          0.0,
          1.0, 
 } ,

 {
  {           0.0,          0.0,           0.0,           0.0,          0.0, 0.0, 0.0 } ,
  {     1.0 / 6.0,          0.0,           0.0,           0.0,          0.0, 0.0, 0.0 } ,
  {    4.0 / 75.0,  16.0 / 75.0,           0.0,           0.0,          0.0, 0.0, 0.0 } ,
  {     5.0 / 6.0,   -8.0 / 3.0,     5.0 / 2.0,           0.0,          0.0, 0.0, 0.0 } ,
  {    -8.0 / 5.0, 144.0 / 25.0,          -4.0,   16.0 / 25.0,          0.0, 0.0, 0.0 } ,
  { 361.0 / 320.0,  -18.0 / 5.0, 407.0 / 128.0,  -11.0 / 80.0, 55.0 / 128.0, 0.0, 0.0 } ,
  { -11.0 / 640.0,          0.0,  11.0 / 256.0, -11.0 / 160.0, 11.0 / 256.0, 0.0, 0.0 } ,
  {  93.0 / 640.0,  -18.0 / 5.0, 803.0 / 256.0, -11.0 / 160.0, 99.0 / 256.0, 0.0, 1.0 } ,	
 } ,

 {
      31.0 / 384.0,
               0.0,
   1125.0 / 2816.0,
        9.0 / 32.0,
     125.0 / 768.0,
        5.0 / 66.0,
               0.0,
               0.0,
 } ,

 {
      7.0 / 1408.0,
               0.0,
   1125.0 / 2816.0,
        9.0 / 32.0,
     125.0 / 768.0,
               0.0,
        5.0 / 66.0,
        5.0 / 66.0,
 }
 
} ;

#define A(i)   param.a[i]
#define B(i,j) param.b[i][j]
#define C(i)   param.c[i]
#define CHAT   param.chat[i]
#define DC(i) (param.c[i] - param.chat[i])

  int i ;
  double *ak2, *ak3, *ak4, *ak5, *ak6, *ak7, *ak8, *ytemp ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Allocate Workspace
 */
  
  ak2 = malloc ( n * sizeof ( double ) ) ;
  if ( ak2 == NULL ) {
    return 1 ;
  }
  ak3 = malloc ( n * sizeof ( double ) ) ;
  if ( ak3 == NULL ) {
    free ( ak2 ) ;
    return 1 ;
  }
  ak4 = malloc ( n * sizeof ( double ) ) ;
  if ( ak4 == NULL ) {
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak5 = malloc ( n * sizeof ( double ) ) ;
  if ( ak5 == NULL ) {
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak6 = malloc ( n * sizeof ( double ) ) ;
  if ( ak6 == NULL ) {
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak7 = malloc ( n * sizeof ( double ) ) ;
  if ( ak7 == NULL ) {
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak8 = malloc ( n * sizeof ( double ) ) ;
  if ( ak8 == NULL ) {
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
	
  ytemp = malloc ( n * sizeof ( double ) ) ;
  if ( ytemp == NULL ) {
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }

/*
 * Do it!
 */
 	
  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + B(1,0) * h * dydx[i] ;
  }
  ( *derivs ) ( x + A(1) * h, ytemp, ak2, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(2,0) * dydx[i] + B(2,1) * ak2[i]) ;
  }
  ( *derivs ) ( x + A(2) * h, ytemp, ak3, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(3,0) * dydx[i] + B(3,1) * ak2[i] + B(3,2) * ak3[i]) ;
  }
  ( *derivs ) ( x + A(3) * h, ytemp, ak4, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(4,0) * dydx[i] + B(4,1) * ak2[i] + B(4,2) * ak3[i] + B(4,3) * ak4[i]) ;
  }
  ( *derivs ) ( x + A(4) * h, ytemp, ak5, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(5,0) * dydx[i] + B(5,1) * ak2[i] + B(5,2) * ak3[i] + B(5,3) * ak4[i] + B(5,4) * ak5[i]) ;
  }
  ( *derivs ) ( x + A(5) * h, ytemp, ak6, derivsp ) ;

  for  ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(6,0) * dydx[i] + B(6,1) * ak2[i] + B(6,2) * ak3[i] + B(6,3) * ak4[i] + B(6,4) * ak5[i] + B(6,5) * ak6[i]) ;
  }
  ( *derivs ) ( x + A(6) * h, ytemp, ak7, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(7,0) * dydx[i] + B(7,1) * ak2[i] + B(7,2) * ak3[i] + B(7,3) * ak4[i] + B(7,4) * ak5[i] + B(7,5) * ak6[i] + B(7,6) * ak7[i]) ;
  }
  ( *derivs ) ( x + A(7) * h, ytemp, ak8, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    yout[i] = y[i] + h * (C(0) * dydx[i] + C(1) * ak2[i] + C(2) * ak3[i] + C(3) * ak4[i] + C(4) * ak5[i] + C(5) * ak6[i] + C(6) * ak7[i] + C(7) * ak8[i]) ;
  }
	
  for ( i = 0; i < n; i++ ) {
    yerr[i] = h * (DC(0) * dydx[i] + DC(1) * ak2[i] + DC(2) * ak3[i] + DC(3) * ak4[i] + DC(4) * ak5[i] + DC(5) * ak6[i] + DC(6) * ak7[i] + DC(7) * ak8[i]) ;
  }

/*
 * De-allocate Workspace Memory
 */
 	
  free ( ytemp ) ;
  free ( ak8 ) ;
  free ( ak7 ) ;
  free ( ak6 ) ; 
  free ( ak5 ) ;
  free ( ak4 ) ;
  free ( ak3 ) ;
  free ( ak2 ) ;
  
  return GAL_SUCCESS ;

/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
