/*  
 *  - - - - - - - - - - -
 *   g a l _ r k f s 7 8 
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine takes a Runge-Kutte-Fehlberg 7(8) step
 *  
 *  Status: 
 *  
 *     Math special function.  
 *  
 *  Given:  
 *
 *     y               d[]          dependent variable vector
 *     dydx            d[]          derivative of dependent variable vector
 *     n                i           Number of equations to integrate
 *     x                d           Independent variable value
 *     h                d           Stepsize
 *     *derivs         func         User defined function for calculating the right hand side derivatives
 *     *derivsp         i           Pointer to parameters structure for derivs routine
 *
 *  Returned:  
 *  
 *     yout            d[]          Ending y values
 *     yerr            d[]          Errors
 *     gal_rkfs45       i           Status
 *                                    0 = Success
 *                                    1 = Failed to allocate memory
 *  
 *  References:
 *
 *     NASA Technical Report TR R-352
 *     Some Experimental Results Concerning The Error Propagation in
 *     Runge-Kutte type integration formulas
 *     by Erwin Fehlberg
 *     October 1970
 *
 *  This revision:  
 * 
 *     2008 May 18 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdlib.h>
#include "gal_const.h"
#include "gal_rkfs78.h"

int gal_rkfs78
  (
    double y[],
    double dydx[],
    int n,
    double x,
    double h,
    double yout[],
    double yerr[],
    void ( *derivs ) ( double, double [], double [], int * ),
    int *derivsp
  ) 
  
{

struct {
  double a[13] ;
  double b[13][12] ;
  double c[13] ;
  double chat[13] ;
} param = {

 { 
          0.0,
   2.0 / 27.0,
    1.0 / 9.0,
    1.0 / 6.0,
   5.0 / 12.0,
    1.0 / 2.0,
    5.0 / 6.0,
    1.0 / 6.0,
    2.0 / 3.0,
    1.0 / 3.0,
          1.0,
          0.0,
          1.0,
 } ,

 {
   {              0.0,        0.0,          0.0,            0.0,             0.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {       2.0 / 27.0,        0.0,          0.0,            0.0,             0.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {       1.0 / 36.0, 1.0 / 12.0,          0.0,            0.0,             0.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {       1.0 / 24.0,        0.0,    1.0 / 8.0,            0.0,             0.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {       5.0 / 12.0,        0.0, -25.0 / 16.0,    25.0 / 16.0,             0.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {       1.0 / 20.0,        0.0,          0.0,      1.0 / 4.0,       1.0 / 5.0,           0.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {    -25.0 / 108.0,        0.0,          0.0,  125.0 / 108.0,    -65.0 / 27.0,  125.0 / 54.0,            0.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {     31.0 / 300.0,        0.0,          0.0,            0.0,    61.0 / 225.0,    -2.0 / 9.0,   13.0 / 900.0,          0.0,          0.0,         0.0, 0.0, 0.0 } ,
   {              2.0,        0.0,          0.0,    -53.0 / 6.0,    704.0 / 45.0,  -107.0 / 9.0,    67.0 / 90.0,          3.0,          0.0,         0.0, 0.0, 0.0 } ,
   {    -91.0 / 108.0,        0.0,          0.0,   23.0 / 108.0,  -976.0 / 135.0,  311.0 / 54.0,   -19.0 / 60.0,   17.0 / 6.0,  -1.0 / 12.0,         0.0, 0.0, 0.0 } ,
   {  2383.0 / 4100.0,        0.0,          0.0, -341.0 / 164.0, 4496.0 / 1025.0, -301.0 / 82.0,   2133.0 / 4100, 45.0 / 82.0, 45.0 / 164.0, 18.0 / 41.0, 0.0, 0.0 } ,
   {      3.0 / 205.0,        0.0,          0.0,            0.0,             0.0,   -6.0 / 41.0,    -3.0 / 205.0, -3.0 / 41.0,   3.0 / 41.0,  6.0 / 41.0, 0.0, 0.0 } ,
   { -1777.0 / 4100.0,        0.0,          0.0, -341.0 / 164.0, 4496.0 / 1025.0, -289.0 / 82.0, 2193.0 / 4100.0, 51.0 / 82.0, 33.0 / 164.0, 12.0 / 41.0, 0.0, 1.0 } ,
 } ,

 {
     41.0 / 840,
            0.0,
            0.0,
            0.0,
            0.0,
   34.0 / 105.0,
     9.0 / 35.0,
     9.0 / 35.0,
    9.0 / 280.0,
    9.0 / 280.0,
   41.0 / 840.0,
            0.0,
            0.0,
 } ,

 {
            0.0,
            0.0,
            0.0,
            0.0,
            0.0,
   34.0 / 105.0,
     9.0 / 35.0,
     9.0 / 35.0,
    9.0 / 280.0,
    9.0 / 280.0,
            0.0,
   41.0 / 840.0,
   41.0 / 840.0,
 }
 
} ;

#define A(i)   param.a[i]
#define B(i,j) param.b[i][j]
#define C(i)   param.c[i]
#define CHAT   param.chat[i]
#define DC(i) (param.c[i] - param.chat[i])

  int i ;
  double *ak2, *ak3, *ak4, *ak5, *ak6, *ak7, *ak8, *ak9, *ak10, *ak11, *ak12, *ak13, *ytemp ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Allocate Workspace
 */
  
  ak2 = malloc ( n * sizeof ( double ) ) ;
  if ( ak2 == NULL ) {
    return 1 ;
  }
  ak3 = malloc ( n * sizeof ( double ) ) ;
  if ( ak3 == NULL ) {
    free ( ak2 ) ;
    return 1 ;
  }
  ak4 = malloc ( n * sizeof ( double ) ) ;
  if ( ak4 == NULL ) {
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak5 = malloc ( n * sizeof ( double ) ) ;
  if ( ak5 == NULL ) {
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak6 = malloc ( n * sizeof ( double ) ) ;
  if ( ak6 == NULL ) {
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak7 = malloc ( n * sizeof ( double ) ) ;
  if ( ak7 == NULL ) {
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak8 = malloc ( n * sizeof ( double ) ) ;
  if ( ak8 == NULL ) {
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak9 = malloc ( n * sizeof ( double ) ) ;
  if ( ak9 == NULL ) {
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak10 = malloc ( n * sizeof ( double ) ) ;
  if ( ak10 == NULL ) {
    free ( ak9 ) ;
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak11 = malloc ( n * sizeof ( double ) ) ;
  if ( ak11 == NULL ) {
    free ( ak10 ) ;
    free ( ak9 ) ;
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak12 = malloc ( n * sizeof ( double ) ) ;
  if ( ak12 == NULL ) {
    free ( ak11 ) ;
    free ( ak10 ) ;
    free ( ak9 ) ;
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
  ak13 = malloc ( n * sizeof ( double ) ) ;
  if ( ak13 == NULL ) {
    free ( ak12 ) ;
    free ( ak11 ) ;
    free ( ak10 ) ;
    free ( ak9 ) ;
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }
	
  ytemp = malloc ( n * sizeof ( double ) ) ;
  if ( ytemp == NULL ) {
    free ( ak13 ) ;
    free ( ak12 ) ;
    free ( ak11 ) ;
    free ( ak10 ) ;
    free ( ak9 ) ;
    free ( ak8 ) ;
    free ( ak7 ) ;
    free ( ak6 ) ;
    free ( ak5 ) ;
    free ( ak4 ) ;
    free ( ak3 ) ;
    free ( ak2 ) ;
    return 1 ;
  }

/*
 * Do it!
 */
 	
  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + B(1,0) * h * dydx[i] ;
  }
  ( *derivs ) ( x + A(1) * h, ytemp, ak2, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(2,0) * dydx[i] + B(2,1) * ak2[i]) ;
  }
  ( *derivs ) ( x + A(2) * h, ytemp, ak3, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(3,0) * dydx[i] + B(3,1) * ak2[i] + B(3,2) * ak3[i]) ;
  }
  ( *derivs ) ( x + A(3) * h, ytemp, ak4, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(4,0) * dydx[i] + B(4,1) * ak2[i] + B(4,2) * ak3[i] + B(4,3) * ak4[i]) ;
  }
  ( *derivs ) ( x + A(4) * h, ytemp, ak5, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(5,0) * dydx[i] + B(5,1) * ak2[i] + B(5,2) * ak3[i] + B(5,3) * ak4[i] + B(5,4) * ak5[i]) ;
  }
  ( *derivs ) ( x + A(5) * h, ytemp, ak6, derivsp ) ;

  for  ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(6,0) * dydx[i] + B(6,1) * ak2[i] + B(6,2) * ak3[i] + B(6,3) * ak4[i] + B(6,4) * ak5[i] + B(6,5) * ak6[i]) ;
  }
  ( *derivs ) ( x + A(6) * h, ytemp, ak7, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(7,0) * dydx[i] + B(7,1) * ak2[i] + B(7,2) * ak3[i] + B(7,3) * ak4[i] + B(7,4) * ak5[i] + B(7,5) * ak6[i] + B(7,6) * ak7[i]) ;
  }
  ( *derivs ) ( x + A(7) * h, ytemp, ak8, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(8,0) * dydx[i] + B(8,1) * ak2[i] + B(8,2) * ak3[i] + B(8,3) * ak4[i] + B(8,4) * ak5[i] + B(8,5) * ak6[i] + B(8,6) * ak7[i] + B(8,7) * ak8[i]) ;
  }
  ( *derivs ) ( x + A(8) * h, ytemp, ak9, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(9,0) * dydx[i] + B(9,1) * ak2[i] + B(9,2) * ak3[i] + B(9,3) * ak4[i] + B(9,4) * ak5[i] + B(9,5) * ak6[i] + B(9,6) * ak7[i] + B(9,7) * ak8[i] + B(9,8) * ak9[i]) ;
  }
  ( *derivs ) ( x + A(9) * h, ytemp, ak10, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(10,0) * dydx[i] + B(10,1) * ak2[i] + B(10,2) * ak3[i] + B(10,3) * ak4[i] + B(10,4) * ak5[i] + B(10,5) * ak6[i] + B(10,6) * ak7[i] + B(10,7) * ak8[i] + B(10,8) * ak9[i] + B(10,9) * ak10[i]) ;
  }
  ( *derivs ) ( x + A(10) * h, ytemp, ak11, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(11,0) * dydx[i] + B(11,1) * ak2[i] + B(11,2) * ak3[i] + B(11,3) * ak4[i] + B(11,4) * ak5[i] + B(11,5) * ak6[i] + B(11,6) * ak7[i] + B(11,7) * ak8[i] + B(11,8) * ak9[i] + B(11,9) * ak10[i] + B(11,10) * ak11[i]) ;
  }
  ( *derivs ) ( x + A(11) * h, ytemp, ak12, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    ytemp[i] = y[i] + h * (B(12,0) * dydx[i] + B(12,1) * ak2[i] + B(12,2) * ak3[i] + B(12,3) * ak4[i] + B(12,4) * ak5[i] + B(12,5) * ak6[i] + B(12,6) * ak7[i] + B(12,7) * ak8[i] + B(12,8) * ak9[i] + B(12,9) * ak10[i] + B(12,10) * ak11[i] + B(12,11) * ak12[i]) ;
  }
  ( *derivs ) ( x + A(12) * h, ytemp, ak13, derivsp ) ;

  for ( i = 0; i < n; i++ ) {
    yout[i] = y[i] + h * (C(0) * dydx[i] + C(1) * ak2[i] + C(2) * ak3[i] + C(3) * ak4[i] + C(4) * ak5[i] + C(5) * ak6[i] + C(6) * ak7[i] + C(7) * ak8[i] + C(8) * ak9[i] + C(9) * ak10[i] + C(10) * ak11[i] + C(11) * ak12[i] + C(12) * ak13[i]) ;
  }
	
  for ( i = 0; i < n; i++ ) {
    yerr[i] = h * (DC(0) * dydx[i] + DC(1) * ak2[i] + DC(2) * ak3[i] + DC(3) * ak4[i] + DC(4) * ak5[i] + DC(5) * ak6[i] + DC(6) * ak7[i] + DC(7) * ak8[i] + DC(8) * ak9[i] + DC(9) * ak10[i] + DC(10) * ak11[i] + DC(11) * ak12[i] + DC(12) * ak13[i]) ;
  }

/*
 * De-allocate Workspace Memory
 */
 	
  free ( ytemp ) ;
  free ( ak13 ) ;
  free ( ak12 ) ;
  free ( ak11 ) ;
  free ( ak10 ) ;
  free ( ak9 ) ;
  free ( ak8 ) ;
  free ( ak7 ) ;
  free ( ak6 ) ; 
  free ( ak5 ) ;
  free ( ak4 ) ;
  free ( ak3 ) ;
  free ( ak2 ) ;
  
  return GAL_SUCCESS ;

/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

