/*  
 *  - - - - - - - - - -
 *   g a l _ b f 2 m e  
 *  - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine transforms a pv-vector from the planet body fixed
 *     reference frame to the planet mean equator reference frame.
 *
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     bfrf          d[2][3]        pv-vector in planet body fixed frame
 *     w                d           Prime meridian angle 
 *     omega            d           Planet rotation rate ( radians second^-1 )
 *  
 *  Returned:
 *  
 *     merf          d[2][3]        pv-vector in planet mean equator frame
 *
 *  Called:
 *
 *     gal_rz        Rotate r matrix about z axis
 *     gal_zp        Zero p-vector
 *     gal_pxp       p-vector cross product
 *     gal_pmp       Subtract p-vector from p-vector
 *     gal_ir        Form r identity matrix
 *     gal_rxpv      Multiply pv-vector by r-matrix
 *
 *  References:
 *
 *    Mars Pathfinder Project Planetary Constants and Models
 *    Jet Propulsion Laboratory, December 1995
 *    Chapters 2, 4, and 5
 *
 *  This revision:
 *  
 *     2008 December 30  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_bf2me.h"
#include "gal_rz.h"
#include "gal_zp.h"
#include "gal_pxp.h"
#include "gal_pmp.h"
#include "gal_ir.h"
#include "gal_rxpv.h"
 
void  
gal_bf2me  
 ( 
   double bfrf[2][3],
   double w,
   double omega,
   double merf[2][3]
 ) 
 
{

  double om[3], dv[3], wrot[3][3] ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Compute & apply velocity transform
 */
 
  gal_zp ( om ) ; 
  om[2] = omega ;
  gal_pxp ( om, bfrf[0], dv ) ;
  gal_ppp ( bfrf[1], dv, bfrf[1] ) ; 

/*
 * Apply rotation 
 */

  gal_ir ( wrot) ;
  gal_rz ( -w, wrot ) ;
  gal_rxpv ( wrot, bfrf, merf ) ;
            
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

