/*  
 *  - - - - - - - - -
 *   g a l _ c 2 m e  
 *  - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine transforms a pv-vector from the ICRF
 *     reference frame to the planet mean equator reference frame.
 *
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     icrf          d[2][3]        pv-vector in ICRF inertial frame
 *     alpha            d           Right ascension of planet spin axis ( radians )
 *     delta            d           Declination of planet spin axis ( radians )
 *  
 *  Returned:
 *  
 *     merf          d[2][3]        pv-vector in planet mean equator frame
 *
 *  Called:
 *
 *     gal_rz        Rotate r matrix about z axis
 *     gal_rx        Rotate r matrix about x axis
 *     gal_ir        Form r identity matrix
 *     gal_rxpv      Multiply pv-vector by r-matrix
 *
 *  References:
 *
 *    Mars Pathfinder Project Planetary Constants and Models
 *    Jet Propulsion Laboratory, December 1995
 *    Chapters 2, 4, and 5
 *
 *  This revision:
 *  
 *     2008 December 30  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_c2me.h"
#include "gal_ir.h"
#include "gal_rz.h"
#include "gal_rx.h"
#include "gal_rxpv.h"
#include "gal_const.h"
  
void  
gal_c2me  
 ( 
   double icrf[2][3],
   double alpha,
   double delta,
   double merf[2][3]
 ) 
 
{

  double rot[3][3] ;

  double pio2 = GAL_PI / 2.0 ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Build rotation matrix
 */
 
  gal_ir ( rot ) ;
  gal_rz ( ( pio2 + alpha ), rot ) ; 
  gal_rx ( ( pio2 - delta ), rot ) ; 

/*
 * Apply rotation matrix
 */

  gal_rxpv ( rot, icrf, merf ) ;

/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

