/*  
 *  - - - - - - - - - - - - 
 *   g a l _ c 2 r a d e c  
 *  - - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine converts a position and velocity vector in the GCRF reference
 *     frame to right ascension and declination.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     gcrf             d[2][3]     GCRF position and velocity vector ( m, m/s )
 *  
 *  Returned:
 *  
 *     *ra                 d        Right Ascension ( radians )
 *     *dec                d        Declination ( radians )
 *     *range              d        Range ( m )  
 *     *radot              d        Right Ascension dot ( radians/s )
 *     *decdot             d        Declination dot ( radians/s )
 *     *rangedot           d        Range dot ( m/s )  
 *  
 *  Called:
 *
 *     gal_anp             Normalize radians to range 0 to 2pi
 *     gal_pm              Modulus of a p vector
 *     gal_pdp             Dot product of two p vectors
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 248-250
 *
 *  This revision:
 *  
 *     2008 May 4   
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>  
#include "gal_c2radec.h"
#include "gal_pm.h"
#include "gal_const.h"
#include "gal_anp.h"
#include "gal_pdp.h"
  
void  
gal_c2radec  
 ( 
   double gcrf[2][3],
   double *ra,
   double *dec,
   double *range,
   double *radot,
   double *decdot,
   double *rangedot
 ) 
 
{

  double temp ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  *range = gal_pm ( &gcrf[0][0] ) ;
  
  *dec = asin ( gcrf[0][2] / *range ) ;
  
  temp = sqrt ( gcrf[0][0] * gcrf[0][0] + gcrf[0][1] * gcrf[0][1] ) ;
  
  if ( temp != 0.0 ) {
    *ra = asin ( gcrf[0][1] / temp ) ;
  }
  else {
    *ra = asin ( gcrf[1][1] / sqrt ( gcrf[1][0] * gcrf[1][0] + gcrf[1][1] * gcrf[1][1] ) ) ;
  }
  
  *ra = gal_anp ( *ra ) ;
  
  *rangedot = gal_pdp ( &gcrf[0][0], &gcrf[1][0] ) ;
  
  *rangedot /= *range ;
  
  *radot = (  gcrf[1][0] * gcrf[0][1] - gcrf[1][1] * gcrf[0][0] ) 
         / ( -gcrf[0][1] * gcrf[0][1] - gcrf[0][0] * gcrf[0][0] ) ;
         
  *decdot = ( gcrf[1][2] - *rangedot * gcrf[0][2] / (*range) ) / temp ;
  
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

