/*  
 *  - - - - - - - - - - - - -
 *   g a l _ l a t l o n 2 t  
 *  - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine creates a position vector in the ITRF reference
 *     frame from given geodetic latitude and longitude.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     lat                 d        Latitude ( radians )  
 *     lon                 d        Longitude ( radians ) 
 *     height              d        Height above the reference spheroid ( m ) 
 *     re                  d        Earth Equatorial Radius ( m )
 *     invf                d        Inverse flattening factor  
 *  
 *  Returned:
 *
 *     itrf               d[3]      ITRF position vector ( km )
 *  
 *  References:
 *
 *     Green, R.M., Spherical Astronomy, Cambridge University Press,
 *     (1985) Section 4.5, p96.
 *
 *     Explantory Supplement to the Astronomical Supplement
 *     Seidelmann P. Kenneth 1992
 *     Pages 202-207
 *
 *     The Astronomical Almanac 1997
 *     Pages K11-K12
 *
 *  This revision:
 *  
 *     2008 May 4   
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>
#include "gal_const.h"
#include "gal_latlon2t.h"
  
void  
gal_latlon2t  
 ( 
   double lat,
   double lon,
   double height,
   double re,
   double invf,
   double itrf[3]
 ) 
 
{

  double sinlat, coslat, r, w, d, as, ac ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  w = 1.0 - 1.0 / invf ;
  w *= w ;
  sinlat = sin ( lat ) ;
  coslat = cos ( lat ) ; 
  d = coslat * coslat + w * sinlat * sinlat ;
  ac = re / sqrt ( d ) ;
  as = w * ac ;
  r = ( ac + height ) * coslat ;
  itrf[0] = r * cos ( lon ) ;
  itrf[1] = r * sin ( lon ) ;
  itrf[2] = ( as + height ) * sinlat ;
    
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

