/*  
 *  - - - - - - - - - - - - - -
 *   g a l _ m o r o t e l 9 1  
 *  - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine returns the rotational elements
 *     for the Earth's Moon (IAU/IAG 1991).  
 *  
 *  Status:
 *  
 *     canonical routine.  
 *  
 *  Given:
 *  
 *     tdb1                d        TDB date part 1 ( see Note 1 )
 *     tdb2                d        TDB date part 2 ( see Note 1 )
 *  
 *  Returned:
 *  
 *     *alpha              d        Right ascension of planet spin axis( radians ) 
 *     *delta              d        Declination of planet spin axis ( radians ) 
 *     *w                  d        Prime meridian angle ( radians ) 
 *     *omega              d        Rotation Rate ( radians second^-1 )
 *  
 *  Notes:
 *
 *  1) The Julian Date is apportioned in any convenient way between  
 *     the arguments tdb1 and tdb2.  For example, JD=2450123.7 could  
 *     be expressed in any of these ways, among others:  
 *  
 *              tdb1         tdb2  
 *  
 *         2450123.7          0.0   (JD method)  
 *         2451545.0      -1421.3   (J2000 method)  
 *         2400000.5      50123.2   (MJD method)  
 *         2450123.5          0.2   (date & time method)  
 *
 *  2) Planetary coordinate systems are defined relative to their
 *     mean axis of rotation and various definitions of longitude
 *     depending on the body. The longitude systems of most of those
 *     bodies with observable rigid surfaces have been defined by
 *     references to a surface feature such as a crater. Approximate
 *     expressions for these rotational elements with respect to the
 *     J2000 inertial coordinate system have been derived. The
 *     International Celestial Reference Frame (ICRF) is the reference
 *     coordinate frame of epoch 2000 which is January 1.5 TDB. The 
 *     variable quantities are expressed in units of days or Julian
 *     centuries of 36525 days.
 *
 *     The north pole is that pole of rotation that lies on the north
 *     side of the invariable plane of the solar system. The direction
 *     of the north pole is specified by the value of its right
 *     ascension ra and declination dec, whereas the location of the prime
 *     meridian is specified by the angle that is measured along the 
 *     planet's equator in an easterly direction with respect to the
 *     planet's north pole from the node Q (located at right ascension
 *     90 degrees + ra) of the planet's equator on the standard equator to
 *     the point B where the prime meridian crosses the planet's equator.
 *     The right ascension of the point Q is 90 degrees + ra and the 
 *     inclination of the planet's equator to the standard equator is 
 *     90 degrees - dec. Because the prime meridian is assumed to rotate
 *     uniformly with the planet, W accordingly varies linearly with
 *     time. In addition, ra, dec, and W may vary with time due to a
 *     precession of the axis of rotation of the planet. If W increases
 *     with time, the planet has direct (or prograde) rotation and if W 
 *     decreases with time, the rotation is said to the retrograde.
 *
 *  References:
 *
 *     Satellite Orbits, Oliver Montenbruck & Eberhard Gill
 *     Springer 2005, Pages 167, 191
 *
 *     Explanatory Supplement to the Astronomical Almanac
 *     P. Kenneth Seidelmann Ed. Page 51
 *
 *     Report of the IAU/IAG Working Group on Cartographic
 *     Coordinates and Rotational Elements of the Planets
 *     and Satellites: 1991, M. E. Davies et al.
 *
 *  This revision:
 *  
 *     2008 December 30  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>
#include "gal_morotel91.h"
#include "gal_const.h"
 
void  
gal_morotel91  
 ( 
   double tdb1,
   double tdb2,
   double *alpha,
   double *delta,
   double *w,
   double *omega
 ) 
 
{

  double d, t, e1, e2, e3, e4, e5, se1, se2, se3, se4, se5, ce1, ce2, ce3, ce4 ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate time terms 
 */
 
  d = ( tdb1 - GAL_J2000 ) + tdb2 ;
  t = d / GAL_DJC ;

/*
 * Compute e terms
 */
 
  e1 = GAL_D2R * ( 125.045 -  0.052992 * d ) ;
  e2 = GAL_D2R * ( 250.090 -  0.105984 * d ) ;
  e3 = GAL_D2R * ( 260.008 + 13.012001 * d ) ;
  e4 = GAL_D2R * ( 176.625 + 13.340716 * d ) ;
  e5 = GAL_D2R * ( 357.529 +  0.985600 * d ) ;

/*
 * Compute interim values
 */
 
  se1 = sin ( e1 ) ;
  se2 = sin ( e2 ) ;
  se3 = sin ( e3 ) ;
  se4 = sin ( e4 ) ;
  se5 = sin ( e5 ) ;
  ce1 = cos ( e1 ) ;
  ce2 = cos ( e2 ) ;
  ce3 = cos ( e3 ) ;
  ce4 = cos ( e4 ) ;
  
/*
 * Compute right ascension of rotational axis
 */
 
  *alpha = GAL_D2R * ( 270.0 + 0.003 * t - 3.878 * se1 - 0.120 * se2 + 0.070 * se3 - 0.017 * se4 ) ;
  
/*
 * Compute declination of rotational axis
 */
 
  *delta = GAL_D2R * ( 66.541 + 0.013 * t + 1.543 * ce1 + 0.024 * ce2 - 0.028 * ce3 + 0.007 * ce4 ) ;
  
/*
 * Compute prime meridian angle
 */
 
  *w = GAL_D2R * ( 38.317 + 13.1763582 * d + 3.558 * se1 + 0.121 * se2 - 0.064 * se3 + 0.016 * se4 + 0.025 * se5 ) ;

/*
 * Calculate rotation rate ( estimate )
 */
 
  *omega = GAL_D2R * ( ( 360.0 / 27.32166) / GAL_D2S ) ;
  
/*  
 * Finished.  
 */

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

