/*  
 *  - - - - - - - - - - - - - -
 *   g a l _ p l r o t e l 0 0  
 *  - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine returns the rotational elements
 *     of a selected planet (IAU/IAG 2000).  
 *  
 *  Status:
 *  
 *     canonical routine.  
 *  
 *  Given:
 *  
 *     body                i        Planet identifier code ( see Note 2 )
 *     tdb1                d        TDB date part 1 ( see Note 1 )
 *     tdb2                d        TDB date part 2 ( see Note 1 )
 *  
 *  Returned:
 *  
 *     *alpha              d        Right ascension of planet spin axis ( radians ) 
 *     *delta              d        Declination of planet spin axis ( radians ) 
 *     *w                  d        Prime meridian angle ( radians ) 
 *     *omega              d        Rotation Rate ( radians second^-1 )
 *     gal_plrotel00       i        0 = success
 *                                  1 = invlaid planet identifier
 *  
 *  Notes:
 *
 *  1) The Julian Date is apportioned in any convenient way between  
 *     the arguments tdb1 and tdb2.  For example, JD=2450123.7 could  
 *     be expressed in any of these ways, among others:  
 *  
 *              tdb1         tdb2  
 *  
 *         2450123.7          0.0   (JD method)  
 *         2451545.0      -1421.3   (J2000 method)  
 *         2400000.5      50123.2   (MJD method)  
 *         2450123.5          0.2   (date & time method)  
 *
 *  2) The following constants defined in the header file gal_const.h 
 *     are valid for use with this routine:
 *
 *     GAL_SSB_SU = 0,   The Sun               
 *     GAL_SSB_ME = 1,   Mercury               
 *     GAL_SSB_VE = 2,   Venus                 
 *     GAL_SSB_EA = 3,   Earth                 
 *     GAL_SSB_MA = 4,   Mars                  
 *     GAL_SSB_JU = 5,   Jupiter               
 *     GAL_SSB_SA = 6,   Saturn                
 *     GAL_SSB_UR = 7,   Uranus                
 *     GAL_SSB_NE = 8,   Neptune               
 *     GAL_SSB_PL = 10,  Pluto                 
 *
 *  3) Planetary coordinate systems are defined relative to their
 *     mean axis of rotation and various definitions of longitude
 *     depending on the body. The longitude systems of most of those
 *     bodies with observable rigid surfaces have been defined by
 *     references to a surface feature such as a crater. Approximate
 *     expressions for these rotational elements with respect to the
 *     J2000 inertial coordinate system have been derived. The
 *     International Celestial Reference Frame (ICRF) is the reference
 *     coordinate frame of epoch 2000 which is January 1.5 TDB. The 
 *     variable quantities are expressed in units of days or Julian
 *     centuries of 36525 days.
 *
 *     The north pole is that pole of rotation that lies on the north
 *     side of the invariable plane of the solar system. The direction
 *     of the north pole is specified by the value of its right
 *     ascension ra and declination dec, whereas the location of the prime
 *     meridian is specified by the angle that is measured along the 
 *     planet's equator in an easterly direction with respect to the
 *     planet's north pole from the node Q (located at right ascension
 *     90 degrees + ra) of the planet's equator on the standard equator to
 *     the point B where the prime meridian crosses the planet's equator.
 *     The right ascension of the point Q is 90 degrees + ra and the 
 *     inclination of the planet's equator to the standard equator is 
 *     90 degrees - dec. Because the prime meridian is assumed to rotate
 *     uniformly with the planet, W accordingly varies linearly with
 *     time. In addition, ra, dec, and W may vary with time due to a
 *     precession of the axis of rotation of the planet. If W increases
 *     with time, the planet has direct (or prograde) rotation and if W 
 *     decreases with time, the rotation is said to the retrograde.
 *
 *  References:
 *
 *     Satellite Orbits, Oliver Montenbruck & Eberhard Gill
 *     Springer 2005, Pages 167, 191
 *
 *     Explanatory Supplement to the Astronomical Almanac
 *     P. Kenneth Seidelmann Ed. Page 51
 *
 *     Report of the IAU/IAG Working Group on Cartographic
 *     Coordinates and Rotational Elements of the Planets
 *     and Satellites: 2000, P. K. Seidelmann et al.
 *
 *  This revision:
 *  
 *     2008 December 29  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>  
#include "gal_plrotel00.h"
#include "gal_const.h"
#include "gal_anp.h"
#include "gal_anpm.h"
  
int  
gal_plrotel00  
 ( 
   int body,
   double tdb1,
   double tdb2,
   double *alpha,
   double *delta,
   double *w,
   double *omega
 ) 
 
{

  double d, t, n, sn ;
  
/*
 * ra1, ra2, dec1, dec2, W1, W2, W3, N1, N2
 */

  double rotel[10][9] = {
  
  { 286.13000,  0.0000,  63.87000,  0.0000,  84.100,   14.18440000,  0.00,   0.00,  0.000, } , /* Sun     */
  { 281.01000, -0.0330,  61.45000, -0.0050, 329.548,    6.13850250,  0.00,   0.00,  0.000, } , /* Mercury */
  { 272.76000,  0.0000,  67.16000,  0.0000, 160.200,   -1.48136880,  0.00,   0.00,  0.000, } , /* Venus   */
  {   0.00000, -0.6410,  90.00000, -0.5570, 190.147,  360.98562350,  0.00,   0.00,  0.000, } , /* Earth   */
  { 317.68143, -0.1061,  52.88650, -0.0609, 176.630,  350.89198226,  0.00,   0.00,  0.000, } , /* Mars    */
  { 268.05000, -0.0090,  64.49000,  0.0030, 284.950,  870.53664200,  0.00,   0.00,  0.000, } , /* Jupiter */
  {  40.58900, -0.0360,  83.53700, -0.0040,  38.900,  810.79390240,  0.00,   0.00,  0.000, } , /* Saturn  */
  { 257.31100,  0.0000, -15.17500,  0.0000, 203.810, -501.16009280,  0.00,   0.00,  0.000, } , /* Uranus  */
  { 299.36000,  0.7000,  43.46000, -0.5100, 253.180,  536.31284920, -0.48, 357.85, 52.316, } , /* Neptune */
  { 313.02000,  0.0000,   9.09000,  0.0000, 236.770,  -56.36231950,  0.00,   0.00,  0.000, } , /* Pluto   */
  
  } ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Check body identifier is valid
 */
 
  if ( body < GAL_SSB_SU || body > GAL_SSB_PL || body == GAL_SSB_EB ) {
    return GAL_FAILURE ;
  }
   
/*
 * Adjust index for Pluto
 */
 
  if ( body == GAL_SSB_PL ) {
    body-- ;
  }

/*
 * Calculate time terms 
 */
 
  d = ( tdb1 - GAL_J2000 ) + tdb2 ;
  t = d / GAL_DJC ;
  
/* 
 * Handle Neptune special case
 */
 
  if ( body == GAL_SSB_NE ) {
    n      = GAL_D2R * ( rotel[body][7] + rotel[body][8] * t ) ;
    sn     = sin ( n ) ;
    *alpha = GAL_D2R * ( rotel[body][0] + rotel[body][1] * sn ) ;
    *delta = GAL_D2R * ( rotel[body][2] + rotel[body][3] * cos ( n ) ) ;
    *w     = GAL_D2R * ( rotel[body][4] + rotel[body][5] * d + rotel[body][6] * sn ) ;
  }

/*
 * Handle regular cases
 */
  
  else {
    *alpha = GAL_D2R * ( rotel[body][0] + rotel[body][1] * t ) ;
    *delta = GAL_D2R * ( rotel[body][2] + rotel[body][3] * t ) ;
    *w     = GAL_D2R * ( rotel[body][4] + rotel[body][5] * d ) ;
  }

/*
 * Calculate rotation rate
 */
      
  *omega = GAL_D2R * ( rotel[body][5] / GAL_D2S ) ;

  return GAL_SUCCESS ;
  
/*  
 * Finished.  
 */
  
}
 
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

