/*  
 *  - - - - - - - - - - - - -
 *   g a l _ p q w 2 i j k m  
 *  - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine forms the PQW to IJK transformation matrix.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     raan             d           Longitude of the ascending mode (radians)
 *     argp             d           Argument of Pericentre (radians)
 *     inc              d           Inclination (radians)
 *  
 *  Returned:
 *  
 *     pqw2ijkm       d[3][3]       Transformation Matrix
 *  
 *  References:
 *
 *     Satellite Orbits
 *     Oliver Montenbruck, Eberhard Gill
 *     Springer 2005
 *     Chapter 2
 *
 *  This revision:
 *  
 *     2008 July 19   
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_pqw2ijkm.h"
#include <math.h>
  
void  
gal_pqw2ijkm  
 ( 
   double raan,
   double argp,
   double inc,
   double pqw2ijkm[3][3]
 ) 
 
{

  double cargp, sargp, craan, sraan, cinc, sinc ;
    
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate intermediates
 */
 
  cargp = cos ( argp ) ;
  sargp = sin ( argp ) ;
  craan = cos ( raan ) ;
  sraan = sin ( raan ) ;
  cinc  = cos ( inc  ) ;
  sinc  = sin ( inc  ) ;
  
/* P */

  pqw2ijkm[0][0] = +cargp * craan - sargp * cinc * sraan ;
  pqw2ijkm[1][0] = +cargp * sraan + sargp * cinc * craan ;
  pqw2ijkm[2][0] = +sargp * sinc ;

/* Q */

  pqw2ijkm[0][1] = -sargp * craan - cargp * cinc * sraan ;
  pqw2ijkm[1][1] = -sargp * sraan + cargp * cinc * craan ;
  pqw2ijkm[2][1] = +cargp * sinc ;

/* W */

  pqw2ijkm[0][2] = +sinc * sraan ;
  pqw2ijkm[1][2] = -sinc * craan ;
  pqw2ijkm[2][2] = +cinc ;

/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

