/*  
 *  - - - - - - - - - - -
 *   g a l _ t 2 a z e l  
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine converts a pv-vector in the ITRF reference
 *     frame to azimuth, elevation, range & range-rate  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     itrf              d[2][3]    ITRF position & velocity vector of target ( m / m s ^-1 )
 *     site              d[2][3]    ITRF position vector of observer ( m )
 *     lat                 d        Latitude of observer ( radians )
 *     lon                 d        Longitude of observer ( radians )
 *  
 *  Returned:
 *  
 *     *az                 d        Azimuth ( radians ) 
 *     *el                 d        Elevation ( radians ) 
 *     *range              d        Range ( m ) 
 *     *rdot               d        Range Rate ( m s^-2 )
 *  
 *  Called:
 *
 *     gal_pmp             p-vector minus p-vector
 *     gal_cp              copy p-vector
 *     gal_ir              Initialize r-matrix to identity
 *     gal_rz              Rotate r matrix about z axis
 *     gal_ry              Rotate r matrix about y axis
 *     gal_rxpv            Product of r-matrix and pv-vector
 *     gal_pdp             Dot product of two p-vectors
 *     gal_pm              Modulus of p-vector
 *
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 252-257
 *
 *  This revision:
 *  
 *     2008 May 24   
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>
#include "gal_const.h"
#include "gal_t2azel.h"
#include "gal_pmp.h"
#include "gal_cp.h"
#include "gal_ir.h"
#include "gal_rz.h"
#include "gal_ry.h"
#include "gal_rxpv.h"
#include "gal_pdp.h"
#include "gal_pm.h"
  
void  
gal_t2azel  
 ( 
   double itrf[2][3],
   double site[3],
   double lat,
   double lon,
   double *az,
   double *el,
   double *range,
   double *rdot
 ) 
 
{

  double srt[2][3], r[3][3], sez[2][3] ;
     
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Build Slant-Range Vectors
 */
 
 gal_pmp ( &itrf[0][0], site, &srt[0][0] ) ;

/*
 * Transform to SEZ 
 */
 
 gal_cp ( &itrf[1][0], &srt[1][0] ) ;
 
 gal_ir ( r ) ;
 gal_rz ( lon, r ) ;
 gal_ry ( GAL_PI / 2.0 - lat, r ) ;
 gal_rxpv ( r, srt, sez ) ;

/*
 * Calculate Range
 */
 
 *range = gal_pm ( &sez[0][0] ) ;
 
/*
 * Calculate Elevation
 */
 
 *el = asin ( sez[0][2] / *range ) ;

/*
 * Calculate Azimuth
 */

/*
 if ( *el != 90.0 ) {
   *az = asin ( sez[0][1] / sqrt ( sez[0][0] * sez[0][0] + sez[0][1] * sez[0][1] ) ) ;
 }
 else {
   *az = asin ( sez[1][1] / sqrt ( sez[1][0] * sez[1][0] + sez[1][1] * sez[1][1] ) ) ;
 }
*/ 
 
 *az = atan ( -sez[0][1] / sez[0][0] ) ; 
 if ( sez[0][0] > 0.0 ) {
   *az += GAL_PI ;
 } 
 if ( *az < 0.0 ) {
   *az += GAL_2PI ;
 } 
 
/*
 * Calculate Range Rate
 */
  
 *rdot = gal_pdp ( &sez[0][0], &sez[1][0] ) ;
 *rdot /= *range ;
 
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

