/*  
 *  - - - - - - - - - - - - -
 *   g a l _ t 2 l a t l o n  
 *  - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine converts a position vector in the ITRF reference
 *     frame to geodetic latitude and longitude.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     itrf               d[3]      ITRF position vector ( m )
 *     re                  d        Earth Equatorial Radius ( m )
 *     invf                d        Inverse flattening factor  
 *  
 *  Returned:
 *  
 *     *lat                d        Latitude ( radians )  
 *     *lon                d        Longitude ( radians ) 
 *     *height             d        Height above the reference spheroid ( m ) 
 *  
 *  Called:
 *
 *     gal_anpm                     Normalize angle into the range -pi <= a < +pi
 *
 *  Notes: 
 *
 *  1) The height refers to a height above the reference spheroid and differs from 
 *     the height above mean sea level (i.e. above ground) by the "undulation of the geoid"
 *     at that point.
 * 
 *  References:
 *
 *     Fundamentals of Astrodynamics and Applications
 *     Vallado, David A. Second Edition 2004
 *     Pages 177-178
 *
 *     Explantory Supplement to the Astronomical Supplement
 *     Seidelmann P. Kenneth 1992
 *     Pages 202-207
 *
 *     The Astronomical Almanac 1997
 *     Pages K11-K12
 *
 *  This revision:
 *  
 *     2008 May 4  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>  
#include "gal_t2latlon.h"
#include "gal_const.h"
#include "gal_anpm.h"
  
void  
gal_t2latlon  
 ( 
   double itrf[3],
   double re,
   double invf,
   double *lat,
   double *lon,
   double *height 
 ) 
 
{

  double f, e2, r, latold, sinlat, c ;

  const double TOL = 1e-16 ;
    
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate Longitude
 */

  *lon = gal_anpm ( atan2 ( itrf[1], itrf[0] ) ) ;

/*
 * Calculate Latitude
 */
   
  r = sqrt ( itrf[0] * itrf[0] + itrf[1] * itrf[1] ) ;
  f = 1.0 / invf ;
  e2 = f * ( 2.0 - f ) ;
  
  *lat = atan2 ( itrf[2], r ) ;
  do {
    latold = *lat ;
    sinlat = sin ( *lat ) ;
    c = re / sqrt ( 1.0 - e2 * sinlat * sinlat ) ;
    *lat = atan2 ( itrf[2] + c * e2 * sinlat, r ) ;
  } while ( fabs ( *lat - latold ) >= TOL ) ;
  
/*
 * Calculate Height ASL
 */
 
  *height = ( r / cos ( *lat ) - c ) ;

/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

