/*  
 *  - - - - - - - - - - - - - - - - - - -
 *   g a l _ t 2 l a t l o n _ w g s 7 2 
 *  - - - - - - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine converts a position vector in the ITRF reference
 *     frame to geodetic latitude and longitude using Fukushima's 1999 Method
 *     using the WGS72 reference ellipsoid.
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     itrf               d[3]      ITRF position vector ( m )
 *  
 *  Returned:
 *  
 *     *lat                d        Latitude ( radians )  
 *     *lon                d        Longitude ( radians ) 
 *     *height             d        Height above the reference spheroid ( m ) 
 *  
 *  Called:
 *
 *     gal_t2latlonf                Convert position vector in ITRF to Latitude & Longitude
 *     gal_emparams                 Get Ellipsoid parameters
 *
 *  Notes: 
 *
 *  1) The height refers to a height above the reference spheroid and differs from 
 *     the height above mean sea level (i.e. above ground) by the "undulation of the geoid"
 *     at that point.
 * 
 *  References:
 *
 *     Fukushima, T., "Fast transform from geocentric to geodetic
 *     coordinates", J.Geodesy (1999) 73: 603-610
 * 
 *  This revision:
 *  
 *     2008 May 4  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_t2latlon_wgs72.h"
#include "gal_ellipsoids.h"
  
void  
gal_t2latlon_wgs72  
 ( 
   double itrf[3],
   double *lat,
   double *lon,
   double *height
 ) 
 
{
  
  double sma, invf ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
  gal_emparams ( GAL_EMEA_WGS1972, &sma, &invf ) ;
  gal_t2latlonf ( itrf, sma, invf, lat, lon, height ) ;
    
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

