/*  
 *  - - - - - - - - - - - - - -
 *   g a l _ t 2 l a t l o n f 
 *  - - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *     This routine converts a position vector in the ITRF reference
 *     frame to geodetic latitude and longitude using Fukushima's 1999 Method.  
 *  
 *     This routine is an independent translation of a FORTRAN routine  
 *     that is part of IAU's SOFA software collection (unreleased code).  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     itrf               d[3]      ITRF position vector ( m )
 *     re                  d        Earth Equatorial Radius ( m )
 *     invf                d        Inverse flattening factor  
 *  
 *  Returned:
 *  
 *     *lat                d        Latitude ( radians )  
 *     *lon                d        Longitude ( radians ) 
 *     *height             d        Height above the reference spheroid ( m ) 
 *  
 *  Called:
 *
 *     gal_anpm                     Normalize angle into the range -pi <= a < +pi
 *
 *  Notes: 
 *
 *  1) The height refers to a height above the reference spheroid and differs from 
 *     the height above mean sea level (i.e. above ground) by the "undulation of the geoid"
 *     at that point.
 * 
 *  References:
 *
 *     Fukushima, T., "Fast transform from geocentric to geodetic
 *     coordinates", J.Geodesy (1999) 73: 603-610
 * 
 *  This revision:
 *  
 *     2003 September 1 ( c version 2008 May 4 )  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>
#include "gal_const.h"
#include "gal_t2latlonf.h"
  
void  
gal_t2latlonf  
 ( 
   double itrf[3],
   double re,
   double invf,
   double *lat,
   double *lon,
   double *height
 ) 
 
{

  const int ITMAX = 20 ;  /* Maximum iterations for Newton's method */

  double e2, ep, ainv, x, y, z, r, p, p2, p4,
         absz, zprime, u, u2, u3, v, t0, tm, fm,
         tau, tau2, tau3, tau4, dtau,
         tau21, tau21m, tauep2, fx, f ;
         
  int i ;
    
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/* -------------
 * Preliminaries
 * -------------
 */
 
/*
 * Functions of ellipsoid parameters.
 */

  f = 1.0 / invf ; 
  e2 = ( 2.0 - f ) * f ;
  ep = sqrt ( 1.0 - e2 ) ;
  ainv = 1.0 / re ;

/*
 * Cartesian components.
 */
 
  x = itrf[0] ;
  y = itrf[1] ;
  z = itrf[2] ;

/*
 * Distance from axis.
 */
 
  r = sqrt ( x * x + y * y ) ;

/*
 * Longitude.
 */
 
  if ( r != 0.0 ) {
    *lon = atan2 ( y, x ) ;
  }
  else {   
    *lon = 0.0 ;
  } 

/* ---------------------------------------------------
 * Compute coefficients of (modified) quartic equation
 * ---------------------------------------------------
 */
 
/*
 * (Coefficients are rescaled by dividing by 'A'.)
 */
 
  p = r * ainv ;
  p2 = 2.0 * p ;
  p4 = 4.0 * p ;
  absz = fabs( z ) * ainv ;
  zprime = ep * absz ;
  u = 2.0 * ( zprime - e2 ) ;
  u2 = 2.0 * u ;
  u3 = 3.0 * u ;
  v = 2.0 * ( zprime + e2 ) ;

/* ------------------------------
 * Select suitable starting value
 * ------------------------------
 */
 
  if ( p < 1e-16 ) {

/*
 * Case 0: p = 0.
 */
 
    t0 = 0.0 ;

  }
  else if ( u >= 0.0 ) {

/*
 * Case 1: u >=  0 (equivalent to t_M = (-u)/(2*P) <= 0).
 */
 
    t0 = ( p4 + u2 ) / ( p4 + u3 + v ) ;

  }
  else {

/*
 * Case u < 0.
 */
 
    if ( -u > p2 ) {

/*
 * Case 2: t_M > 1.
 */
 
      t0 = p / v ;

    }
    else {

/*
 * Case 3: 0 < t_M < =  1.
 */
 
      tm = -u / p2 ;
      fm = tm * ( tm * tm * ( tm * p + u ) + v ) - p ;
      if ( fm > 0.0 ) {

/*
 * Case 3a: f(t_M) > 0.
 */
 
        t0 = p / v ;

      }
      else {

/*
 * Case 3b: f(t_M) < =  0.
 */
 
        t0 = ( p4 + u2 ) / ( p4 + u3 + v ) ;

      }
    }
  }

/* --------------------------------
 * Solve quartic by Newton's method
 * --------------------------------
 */
 
  tau = t0 ;
  
  for ( i = 0; i < ITMAX; i++ ) {
  
/*
 * Powers of estimate.
 */
 
    tau2 = tau * tau ;
    tau3 = tau2 * tau ;
    tau4 = tau2 * tau2 ;

/*
 * Current residual.
 */

    fx = p * tau4 + u * tau3 + v * tau - p ;

/*
 * If small enough residual, break from loop.
 */
 
    if ( fabs ( fx ) < 1e-15 ) {
      break ;
    }

/*
 * Adjustment.
 */
 
    dtau = fx / ( p4 * tau3 + u3 * tau2 + v ) ;

/*
 * Improve the estimate.
 */
 
    tau -= dtau ;

/*
 * If small enough adjustment, update TAU2 and break from loop.
 */
 
    if ( fabs ( dtau ) < 1e-8 ) {
      tau2 = tau * tau ;
      break ;
    }

/*
 * Iterate.
 */
 
  }

/*
 * Functions of solution.
 */
 
  tau21 = 1.0 + tau2 ;
  tau21m = 1.0 - tau2 ;
  tauep2 = 2.0 * ep * tau ;

/* -------
 * Results
 * -------
 */
 
  *lat = GAL_SIGN ( atan2 ( tau21m, tauep2 ), z ) ;
  
  *height = re * ( p * tauep2 + absz * tau21m - ep * tau21 ) /
            sqrt ( tau21 * tau21 - 4.0 * e2 * tau2 ) ;
    
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  

