/*  
 *  - - - - - - - - - - 
 *   g a l _ d p p e r  
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine calculates the deep space long period periodic contributions
 *     to the mean elements.  By design, these periodics are zero at epoch.
 *     This used to be dscom which included initialization, but it's really a
 *     recurring function.
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     e3               d
 *     ee2              d
 *     peo              d
 *     pgho             d
 *     pho              d
 *     pinco            d
 *     plo              d
 *     se2              d
 *     se3              d
 *     sgh2             d
 *     sgh3             d
 *     sgh4             d
 *     sh2              d
 *     sh3              d
 *     si2              d
 *     si3              d
 *     sl2              d
 *     sl3              d
 *     sl4              d
 *     t                d
 *     xgh2             d
 *     xgh3             d
 *     xgh4             d
 *     xh2              d
 *     xh3              d
 *     xi2              d
 *     xi3              d
 *     xl2              d
 *     xl3              d
 *     xl4              d
 *     zmol             d
 *     zmos             d 
 *     inclo            d           Inclination - needed for lyddane modification
 *     init             c
 *     *ep              d           Eccentricity [0.0 - 1.0]
 *     *nodep           d           Right Ascension of Ascending Node
 *     *argpp           d           Argument of Perigee
 *     *mp              d           Mean Anomaly
 *  
 *  Returned:  
 *  
 *     *ep              d           Eccentricity [0.0 - 1.0]
 *     *inclp           d           Inclination
 *     *nodep           d           Right Ascension of Ascending Node
 *     *argpp           d           Argument of Perigee
 *     *mp              d           Mean Anomaly
 *  
 *  Notes:  
 *  
 *  1) This routine is a translation from c++ to c of David Vallado's SGP4UNIT.dpper 
 *     routine ( 2007 November 16 ).
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 April 6 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_const.h"
#include "gal_dpper.h"
  
void 
gal_dpper
  (
    double e3,     
    double ee2,    
    double peo,     
    double pgho,   
    double pho,
    double pinco,  
    double plo,    
    double se2,     
    double se3,    
    double sgh2,
    double sgh3,   
    double sgh4,   
    double sh2,     
    double sh3,    
    double si2,
    double si3,    
    double sl2,    
    double sl3,     
    double sl4,    
    double t,
    double xgh2,   
    double xgh3,   
    double xgh4,    
    double xh2,    
    double xh3,
    double xi2,    
    double xi3,    
    double xl2,     
    double xl3,    
    double xl4,
    double zmol,   
    double zmos,   
    double inclo,
    char init,
    double  *ep,    
    double *inclp, 
    double *nodep,  
    double *argpp, 
    double *mp
  ) 
  
{

  double alfdp, betdp, cosip, cosop, dalf, dbet, dls,
         f2,    f3,    pe,    pgh,   ph,   pinc, pl ,
         sel,   ses,   sghl,  sghs,  shll, shs,  sil,
         sinip, sinop, sinzf, sis,   sll,  sls,  xls,
         xnoh,  zf,    zm ;
         
  const double zns   = 1.19459e-5;
  const double zes   = 0.01675;
  const double znl   = 1.5835218e-4;
  const double zel   = 0.05490;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * Calculate time varying periodics 
 */
    
  zm    = zmos + zns * t;

/*
 * Be sure that the initial call has time set to zero
 */
 
  if ( init == 'y' )  {
    zm = zmos ;
  }
  zf    = zm + 2.0 * zes * sin ( zm ) ;
  sinzf = sin ( zf ) ;
  f2    =  0.5 * sinzf * sinzf - 0.25 ;
  f3    = -0.5 * sinzf * cos ( zf ) ;
  ses   = se2 * f2 + se3 * f3 ;
  sis   = si2 * f2 + si3 * f3 ;
  sls   = sl2 * f2 + sl3 * f3 + sl4 * sinzf ;
  sghs  = sgh2 * f2 + sgh3 * f3 + sgh4 * sinzf ;
  shs   = sh2 * f2 + sh3 * f3 ;
  zm    = zmol + znl * t ;
  if ( init == 'y' ) {
    zm = zmol ;
  }
  zf    = zm + 2.0 * zel * sin ( zm ) ;
  sinzf = sin ( zf ) ;
  f2    =  0.5 * sinzf * sinzf - 0.25 ;
  f3    = -0.5 * sinzf * cos ( zf ) ;
  sel   = ee2 * f2 + e3 * f3 ;
  sil   = xi2 * f2 + xi3 * f3 ;
  sll   = xl2 * f2 + xl3 * f3 + xl4 * sinzf ;
  sghl  = xgh2 * f2 + xgh3 * f3 + xgh4 * sinzf ;
  shll  = xh2 * f2 + xh3 * f3 ;
  pe    = ses + sel ;
  pinc  = sis + sil ;
  pl    = sls + sll ;
  pgh   = sghs + sghl ;
  ph    = shs + shll ;

  if ( init == 'n' ) {
    pe    -= peo ;
    pinc  -= pinco ;
    pl    -= plo ;
    pgh   -= pgho ;
    ph    -= pho ;
    *inclp += pinc;
    *ep    += pe;
    sinip = sin ( *inclp ) ;
    cosip = cos ( *inclp ) ;

/*
 * Apply periodics directly 
 * sgp4fix for lyddane choice
 * strn3 used original inclination - this is technically feasible
 * gsfc used perturbed inclination - also technically feasible
 * probably best to readjust the 0.2 limit value and limit discontinuity
 * 0.2 rad = 11.45916 deg
 * use next line for original strn3 approach and original inclination
 * if (inclo >= 0.2)
 * use next line for gsfc version and perturbed inclination
 */
 
    if ( *inclp >= 0.2 ) {
      ph     /= sinip ;
      pgh    -= cosip * ph;
      *argpp += pgh ;
      *nodep += ph ;
      *mp    += pl ;
    }
    else {

/* 
 * Apply periodics with lyddane modification 
 */
 
      sinop   = sin ( *nodep ) ;
      cosop   = cos ( *nodep ) ;
      alfdp   = sinip * sinop ;
      betdp   = sinip * cosop ;
      dalf    =  ph * cosop + pinc * cosip * sinop ;
      dbet    = -ph * sinop + pinc * cosip * cosop ;
      alfdp   = alfdp + dalf ;
      betdp   = betdp + dbet ;
      *nodep  = fmod ( *nodep, GAL_2PI ) ;
      
/*
 * sgp4fix for afspc written intrinsic functions
 * nodep used without a trigonometric function ahead
 */
 
      if ( *nodep < 0.0 ) {
        *nodep += GAL_2PI ;
      }
      xls     = *mp + *argpp + cosip * *nodep ;
      dls     = pl + pgh - pinc * *nodep * sinip ;
      xls     = xls + dls ;
      xnoh    = *nodep ;
      *nodep  = atan2 ( alfdp, betdp ) ;

/*
 * sgp4fix for afspc written intrinsic functions
 * nodep used without a trigonometric function ahead
 */
 
      if ( *nodep < 0.0 ) {
        *nodep += GAL_2PI ;
      }
      if ( fabs ( xnoh - *nodep ) > GAL_PI ) {
        if ( *nodep < xnoh ) {
          *nodep += GAL_2PI;
        }
        else {
          *nodep -= GAL_2PI;
        } 
      }
      *mp += pl;
      *argpp = xls - *mp - cosip * *nodep ;
    }
  }   /* if init == 'n' */
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
