/*  
 *  - - - - - - - - - - 
 *   g a l _ d s c o m  
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine provides deep space common items used by both the secular
 *     and periodics routines.  
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     epoch            d
 *     ep               d           Eccentricity
 *     argpp            d           Argument of Perigee
 *     tc               d
 *     inclp            d           Inclination
 *     nodep            d           Right Ascension of Ascending Node
 *     np               d           Mean Motion
 *  
 *  Returned:  
 *  
 *     *sinim           d           
 *     *cosim           d           
 *     *sinomm          d            
 *     *cosomm          d            
 *     *snodm           d            
 *     *cnodm           d           
 *     *day             d           
 *     *e3              d           
 *     *ee2             d           
 *     *em              d           Eccentricity
 *     *emsq            d           Eccentricity squared
 *     *gam             d           
 *     *peo             d           
 *     *pgho            d           
 *     *pho             d           
 *     *pinco           d           
 *     *plo             d           
 *     *rtemsq          d           
 *     *se2             d           
 *     *se3             d           
 *     *sgh2            d           
 *     *sgh3            d           
 *     *sgh4            d           
 *     *sh2             d           
 *     *sh3             d           
 *     *si2             d            
 *     *si3             d           
 *     *sl2             d           
 *     *sl3             d           
 *     *sl4             d           
 *     *s1              d            
 *     *s2              d           
 *     *s3              d           
 *     *s4              d           
 *     *s5              d           
 *     *s6              d           
 *     *s7              d           
 *     *ss1             d           
 *     *ss2             d           
 *     *ss3             d           
 *     *ss4             d           
 *     *ss5             d           
 *     *ss6             d           
 *     *ss7             d           
 *     *sz1             d           
 *     *sz2             d           
 *     *sz3             d           
 *     *sz11            d           
 *     *sz12            d           
 *     *sz13            d           
 *     *sz21            d           
 *     *sz22            d           
 *     *sz23            d           
 *     *sz31            d           
 *     *sz32            d           
 *     *sz33            d           
 *     *xgh2            d           
 *     *xgh3            d           
 *     *xgh4            d           
 *     *xh2             d           
 *     *xh3             d           
 *     *xi2             d           
 *     *xi3             d           
 *     *xl2             d           
 *     *xl3             d           
 *     *xl4             d           
 *     *nm              d           Mean Motion
 *     *z1              d           
 *     *z2              d           
 *     *z3              d           
 *     *z11             d           
 *     *z12             d           
 *     *z13             d           
 *     *z21             d           
 *     *z22             d           
 *     *z23             d           
 *     *z31             d           
 *     *z32             d           
 *     *z33             d           
 *     *zmol            d           
 *     *zmos            d           
 *  
 *  Notes:  
 *  
 *  1) This routine is a translation from c++ to c of David Vallado's SGP4UNIT.dscom 
 *     routine ( 2007 November 16 ). This routine used to be called dpper, but the 
 *     functions inside weren't well organized.
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 April 6 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_const.h"
#include "gal_dscom.h"
  
void 
gal_dscom
  (
    double epoch, 
    double ep,   
    double argpp, 
    double tc,  
    double inclp,
    double nodep, 
    double np,
    double *snodm,
    double *cnodm,
    double *sinim,
    double *cosim,
    double *sinomm,
    double *cosomm,
    double *day,  
    double *e3,   
    double *ee2,
    double *em,
    double *emsq, 
    double *gam, 
    double *peo,
    double *pgho, 
    double *pho,
    double *pinco,
    double *plo,  
    double *rtemsq,
    double *se2,  
    double *se3,
    double *sgh2, 
    double *sgh3,
    double *sgh4, 
    double *sh2, 
    double *sh3,
    double *si2, 
    double *si3, 
    double *sl2, 
    double *sl3, 
    double *sl4,
    double *s1,
    double *s2, 
    double *s3,  
    double *s4, 
    double *s5,
    double *s6,  
    double *s7,  
    double *ss1, 
    double *ss2, 
    double *ss3,
    double *ss4, 
    double *ss5, 
    double *ss6, 
    double *ss7, 
    double *sz1,
    double *sz2, 
    double *sz3,
    double *sz11,
    double *sz12, 
    double *sz13,
    double *sz21, 
    double *sz22,
    double *sz23, 
    double *sz31, 
    double *sz32,
    double *sz33, 
    double *xgh2, 
    double *xgh3,  
    double *xgh4, 
    double *xh2,
    double *xh3, 
    double *xi2, 
    double *xi3, 
    double *xl2, 
    double *xl3,
    double *xl4, 
    double *nm,  
    double *z1,  
    double *z2,  
    double *z3,
    double *z11, 
    double *z12, 
    double *z13,  
    double *z21,  
    double *z22,
    double *z23,  
    double *z31,
    double *z32,  
    double *z33, 
    double *zmol,
    double *zmos
  ) 
  
{

/*
 * Constants
 */
 
  const double zes    =  0.01675      ;
  const double zel    =  0.05490      ;
  const double c1ss   =  2.9864797e-6 ;
  const double c1l    =  4.7968065e-7 ;
  const double zsinis =  0.39785416   ;
  const double zcosis =  0.91744867   ;
  const double zcosgs =  0.1945905    ;
  const double zsings = -0.98088458   ;
  
/* 
 * Local Variables 
 */   
 
  int lsflg ;
  
  double a1    , a2    , a3    , a4    , a5    , a6    , a7    ,
         a8    , a9    , a10   , betasq, cc    , ctem  , stem  ,
         x1    , x2    , x3    , x4    , x5    , x6    , x7    ,
         x8    , xnodce, xnoi  , zcosg , zcosgl, zcosh , zcoshl,
         zcosi , zcosil, zsing , zsingl, zsinh , zsinhl, zsini ,
         zsinil, zx    , zy ;
         
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  *nm     = np ;
  *em     = ep ;
  *snodm  = sin ( nodep ) ;
  *cnodm  = cos ( nodep ) ;
  *sinomm = sin ( argpp ) ;
  *cosomm = cos ( argpp ) ;
  *sinim  = sin ( inclp ) ;
  *cosim  = cos ( inclp ) ;
  *emsq   = (*em) * (*em) ;
  betasq = 1.0 - *emsq ;
  *rtemsq = sqrt ( betasq ) ;

/*
 * Initialize lunar solar terms 
 */
 
  *peo    = 0.0 ;
  *pinco  = 0.0 ;
  *plo    = 0.0 ;
  *pgho   = 0.0 ;
  *pho    = 0.0 ;
  *day    = epoch + 18261.5 + tc / 1440.0 ;
  xnodce = fmod ( 4.5236020 - 9.2422029e-4 * (*day), GAL_2PI ) ;
  stem   = sin ( xnodce ) ;
  ctem   = cos ( xnodce ) ;
  zcosil = 0.91375164 - 0.03568096 * ctem ;
  zsinil = sqrt ( 1.0 - zcosil * zcosil ) ;
  zsinhl = 0.089683511 * stem / zsinil ;
  zcoshl = sqrt ( 1.0 - zsinhl * zsinhl ) ;
  *gam    = 5.8351514 + 0.0019443680 * (*day) ;
  zx     = 0.39785416 * stem / zsinil ;
  zy     = zcoshl * ctem + 0.91744867 * zsinhl * stem ;
  zx     = atan2 ( zx, zy ) ;
  zx     = *gam + zx - xnodce ;
  zcosgl = cos ( zx ) ;
  zsingl = sin ( zx ) ;

/* 
 * Do solar terms
 */
 
  zcosg = zcosgs ;
  zsing = zsings ;
  zcosi = zcosis ;
  zsini = zsinis ;
  zcosh = *cnodm ;
  zsinh = *snodm ;
  cc    = c1ss ;
  xnoi  = 1.0 / *nm ;

  for ( lsflg = 1; lsflg <= 2; lsflg++ ) {
    a1  =   zcosg * zcosh + zsing * zcosi * zsinh ;
    a3  =  -zsing * zcosh + zcosg * zcosi * zsinh ;
    a7  =  -zcosg * zsinh + zsing * zcosi * zcosh ;
    a8  =   zsing * zsini ;
    a9  =   zsing * zsinh + zcosg * zcosi * zcosh ;
    a10 =   zcosg * zsini ;
    a2  =   *cosim * a7 + *sinim * a8 ;
    a4  =   *cosim * a9 + *sinim * a10 ;
    a5  =  -*sinim * a7 + *cosim * a8 ;
    a6  =  -*sinim * a9 + *cosim * a10 ;

    x1  =  a1 * (*cosomm) + a2 * (*sinomm) ;
    x2  =  a3 * (*cosomm) + a4 * (*sinomm) ;
    x3  = -a1 * (*sinomm) + a2 * (*cosomm) ;
    x4  = -a3 * (*sinomm) + a4 * (*cosomm) ;
    x5  =  a5 * (*sinomm) ;
    x6  =  a6 * (*sinomm) ;
    x7  =  a5 * (*cosomm) ;
    x8  =  a6 * (*cosomm) ;

    *z31 = 12.0 * x1 * x1 - 3.0 * x3 * x3 ;
    *z32 = 24.0 * x1 * x2 - 6.0 * x3 * x4 ;
    *z33 = 12.0 * x2 * x2 - 3.0 * x4 * x4 ;
    *z1  =  3.0 *  ( a1 * a1 + a2 * a2 ) + *z31 * (*emsq) ;
    *z2  =  6.0 *  ( a1 * a3 + a2 * a4 ) + *z32 * (*emsq) ;
    *z3  =  3.0 *  ( a3 * a3 + a4 * a4 ) + *z33 * (*emsq) ;
    *z11 = -6.0 * a1 * a5 + (*emsq) *  ( -24.0 * x1 * x7-6.0 * x3 * x5 ) ;
    *z12 = -6.0 *  ( a1 * a6 + a3 * a5 ) + (*emsq) *
        ( -24.0 * (x2 * x7 + x1 * x8 ) - 6.0 * ( x3 * x6 + x4 * x5 ) ) ;
    *z13 = -6.0 * a3 * a6 + (*emsq) * ( -24.0 * x2 * x8 - 6.0 * x4 * x6 ) ;
    *z21 =  6.0 * a2 * a5 + (*emsq) * ( 24.0 * x1 * x5 - 6.0 * x3 * x7 ) ;
    *z22 =  6.0 *  ( a4 * a5 + a2 * a6) + (*emsq) *
          ( 24.0 *  ( x2 * x5 + x1 * x6 ) - 6.0 * ( x4 * x7 + x3 * x8 ) ) ;
    *z23 =  6.0 * a4 * a6 + (*emsq) * ( 24.0 * x2 * x6 - 6.0 * x4 * x8 ) ;
    *z1  = *z1 + *z1 + betasq * (*z31) ;
    *z2  = *z2 + *z2 + betasq * (*z32) ;
    *z3  = *z3 + *z3 + betasq * (*z33) ;
    *s3  = cc * xnoi ;
    *s2  = -0.5 * (*s3) / (*rtemsq) ;
    *s4  = (*s3) * (*rtemsq) ;
    *s1  = -15.0 * (*em) * (*s4) ;
    *s5  = x1 * x3 + x2 * x4 ;
    *s6  = x2 * x3 + x1 * x4 ;
    *s7  = x2 * x4 - x1 * x3 ;

/*
 * Do lunar terms -
 */
 
    if (lsflg == 1) {
      *ss1   = *s1 ;
      *ss2   = *s2 ;
      *ss3   = *s3 ;
      *ss4   = *s4 ;
      *ss5   = *s5 ;
      *ss6   = *s6 ;
      *ss7   = *s7 ;
      *sz1   = *z1 ;
      *sz2   = *z2 ;
      *sz3   = *z3 ;
      *sz11  = *z11 ;
      *sz12  = *z12 ;
      *sz13  = *z13 ;
      *sz21  = *z21 ;
      *sz22  = *z22 ;
      *sz23  = *z23 ;
      *sz31  = *z31 ;
      *sz32  = *z32 ;
      *sz33  = *z33 ;
      zcosg = zcosgl ;
      zsing = zsingl ;
      zcosi = zcosil ;
      zsini = zsinil ;
      zcosh = zcoshl * (*cnodm) + zsinhl * (*snodm) ;
      zsinh = (*snodm) * zcoshl - (*cnodm) * zsinhl ;
      cc    = c1l ;
    }
  }

  *zmol = fmod ( 4.7199672 + 0.22997150  * (*day) - (*gam), GAL_2PI ) ;
  *zmos = fmod ( 6.2565837 + 0.017201977 * (*day), GAL_2PI ) ;

/* 
 * Do solar terms 
 */
 
  *se2  =   2.0 * (*ss1) * (*ss6) ;
  *se3  =   2.0 * (*ss1) * (*ss7) ;
  *si2  =   2.0 * (*ss2) * (*sz12) ;
  *si3  =   2.0 * (*ss2) * ( *sz13 - *sz11 ) ;
  *sl2  =  -2.0 * (*ss3) * (*sz2) ;
  *sl3  =  -2.0 * (*ss3) * ( *sz3 - *sz1 ) ;
  *sl4  =  -2.0 * (*ss3) * ( -21.0 - 9.0 * (*emsq) ) * zes ;
  *sgh2 =   2.0 * (*ss4) * (*sz32) ;
  *sgh3 =   2.0 * (*ss4) * ( *sz33 - *sz31 ) ;
  *sgh4 = -18.0 * (*ss4) * zes ;
  *sh2  =  -2.0 * (*ss2) * (*sz22) ;
  *sh3  =  -2.0 * (*ss2) * ( *sz23 - *sz21 ) ;

/* 
 * Do lunar terms 
 */

   *ee2  =   2.0 * (*s1) * (*s6) ;
   *e3   =   2.0 * (*s1) * (*s7) ;
   *xi2  =   2.0 * (*s2) * (*z12) ;
   *xi3  =   2.0 * (*s2) * ( *z13 - *z11 ) ;
   *xl2  =  -2.0 * (*s3) * (*z2) ;
   *xl3  =  -2.0 * (*s3) * ( *z3 - *z1 ) ;
   *xl4  =  -2.0 * (*s3) * ( -21.0 - 9.0 * (*emsq) ) * zel ;
   *xgh2 =   2.0 * (*s4) * (*z32) ;
   *xgh3 =   2.0 * (*s4) * ( *z33 - *z31 ) ;
   *xgh4 = -18.0 * (*s4) * zel ;
   *xh2  =  -2.0 * (*s2) * (*z22) ;
   *xh3  =  -2.0 * (*s2) * ( *z23 - *z21 ) ;
   
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
