/*  
 *  - - - - - - - - - - -
 *   g a l _ d s p a c e  
 *  - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine provides deep space contributions to mean elements for
 *     perturbing third body. These effects have been averaged over one
 *     revolution of the sun and moon. For earth resonance effects, the
 *     effects have been averaged over no revolutions of the satellite.
 *     (mean motion)
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     d2201            d            
 *     d2211            d            
 *     d3210            d            
 *     d3222            d            
 *     d4410            d            
 *     d4422            d            
 *     d5220            d            
 *     d5232            d            
 *     d5421            d            
 *     d5433            d           
 *     dedt             d           
 *     del1             d            
 *     del2             d            
 *     del3             d           
 *     didt             d           
 *     dmdt             d           
 *     dnodt            d           
 *     domdt            d           
 *     irez             i           Flag for resonance           
 *                                  0 = none 
 *                                  1 = one day
 *                                  2 = half day
 *     argpo            d           Argument of Perigee
 *     argpdot          d           Argument of Perigee Dot (rate)
 *     t                d           Time
 *     tc               d           
 *     gsto             d           Greenwich Sidereal Time
 *     xfact            d           
 *     xlamo            d           
 *     no               d           Mean Motion
 *     *atime           d           
 *     *em              d           Eccentricity
 *     ft               d           
 *     *argpm           d           Argument of Perigee
 *     *inclm           d           Inclination
 *     *xli             d           
 *     *mm              d           Mean Anomaly
 *     *xni             d           Mean Motion
 *     *nodem           d           Right Ascension of Ascending Node
 *  
 *  Returned:  
 *  
 *     *atime           d           
 *     *em              d           Eccentricity
 *     *argpm           d           Argument of Perigee
 *     *inclm           d           Inclination
 *     *xli             d           
 *     *mm              d           Mean Anomaly
 *     *xni             d           Mean Motion
 *     *nodem           d           Right Ascension of Ascending Node
 *     *dndt            d           
 *     *nm              d           Mean Motion
 *
 *  Notes:  
 *  
 *  1) This routine is a translation from c++ to c of David Vallado's SGP4UNIT.dspace 
 *     routine ( 2007 November 16 ). 
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 April 6 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_const.h"
#include "gal_dspace.h"
  
void 
gal_dspace
  (
    int    irez,
    double d2201,  
    double d2211,  
    double d3210,   
    double d3222,  
    double d4410,
    double d4422,  
    double d5220,  
    double d5232,   
    double d5421,  
    double d5433,
    double dedt,   
    double del1,   
    double del2,    
    double del3,   
    double didt,
    double dmdt,   
    double dnodt,  
    double domdt,   
    double argpo,  
    double argpdot,
    double t,      
    double tc,     
    double gsto,    
    double xfact,  
    double xlamo,
    double no,
    double *atime, 
    double *em,    
    double *argpm,  
    double *inclm, 
    double *xli,
    double *mm,    
    double *xni,   
    double *nodem,  
    double *dndt,  
    double *nm
  ) 
  
{

  int iretn , iret;
  
  double delt,  ft,    theta, x2li,  x2omi, 
         xl,    xldot, xnddt, xndt,  xomi,  
         g22,   g32,   g44,   g52,   g54,   
         fasx2, fasx4, fasx6, rptim, 
         step2, stepn, stepp ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  ft    = 0.0                    ;
  fasx2 = 0.13130908             ;
  fasx4 = 2.8843198              ;
  fasx6 = 0.37448087             ;
  g22   = 5.7686396              ;
  g32   = 0.95240898             ;
  g44   = 1.8014998              ;
  g52   = 1.0508330              ;
  g54   = 4.4108898              ;
  rptim = 4.37526908801129966e-3 ; /* This equates to 7.29211514668855e-5 rad/sec */
  stepp =    720.0               ;
  stepn =   -720.0               ;
  step2 = 259200.0               ;

/* 
 * Calculate deep space resonance effects 
 */
 
  *dndt   = 0.0 ;
  theta   = fmod (gsto + tc * rptim, GAL_2PI ) ;
  *em     += dedt  * t ;
  *inclm  += didt  * t ;
  *argpm  += domdt * t ;
  *nodem  += dnodt * t ;
  *mm     += dmdt  * t ;

/*
 * sgp4fix for negative inclinations
 * the following if statement should be commented out
 * if ( inclm < 0.0 ) {
 *   inclm = -inclm;
 *   argpm -= GAL_PI ;
 *   nodem += GAL_PI ;
 * }
 *
 */
 
/*
 * Update resonances : numerical (euler-maclaurin) integration 
 * ---------------------- epoch restart ---------------------- 
 *
 * sgp4fix for propagator problems. The following integration works for negative 
 * time steps and periods. The specific changes are unknown because the original 
 * code was so convoluted
 */
 
  ft    = 0.0 ;
  *atime = 0.0 ;
  
  if ( irez != 0 ) {
  
    if ( *atime == 0.0 || ( t >= 0.0 && *atime < 0.0) || ( t < 0.0 && *atime >= 0.0) ) {
      if (t >= 0.0) {
        delt = stepp ;
      }  
      else {
        delt = stepn ;
      }
      *atime = 0.0 ;
      *xni   = no ;
      *xli   = xlamo ;
    }
    
    iretn = 381 ;  /* added for do loop */
    iret  =   0 ;  /* added for loop */
    
    while ( iretn == 381 ) {
    
      if ( fabs ( t ) < fabs ( *atime ) || iret == 351 ) {
        if ( t >= 0.0 ) {
          delt = stepn ;
        }
        else {
          delt = stepp ;
        }
        iret  = 351 ;
        iretn = 381 ; 
      }
      else {
        if ( t > 0.0 )  { /* error if prev if has atime:=0.0 and t:=0.0 (ge) */
          delt = stepp ;
        }
        else {
          delt = stepn ;
        }
        if ( fabs ( t - *atime ) >= stepp ) {
          iret  = 0 ;
          iretn = 381;
        }
        else {
          ft    = t - *atime ;
          iretn = 0 ;
        }
      }

/* 
 * Dot terms calculated 
 */

/* 
 * Near - synchronous resonance terms 
 */

      if ( irez != 2 ) {
      
        xndt  = del1 * sin ( *xli - fasx2 ) + del2 * sin ( 2.0 * ( *xli - fasx4 ) ) +
                del3 * sin ( 3.0 * ( *xli - fasx6 ) ) ;
                
        xldot = *xni + xfact ;
        
        xnddt = del1 * cos ( *xli - fasx2 ) +
                2.0 * del2 * cos ( 2.0 * ( *xli - fasx4 ) ) +
                3.0 * del3 * cos ( 3.0 * ( *xli - fasx6 ) ) ;
                
        xnddt *= xldot ;
        
      }
      else {

/* 
 * Near - half-day resonance terms 
 */
 
        xomi  = argpo + argpdot * (*atime) ;
        
        x2omi = xomi + xomi ;
        
        x2li  = *xli + *xli ;
        
        xndt  = d2201 * sin ( x2omi + *xli - g22 ) + d2211 * sin ( *xli   - g22       ) +
                d3210 * sin ( xomi  + *xli - g32 ) + d3222 * sin ( -xomi + *xli - g32 ) +
                d4410 * sin ( x2omi + x2li - g44 ) + d4422 * sin ( x2li  - g44        ) +
                d5220 * sin ( xomi  + *xli - g52 ) + d5232 * sin ( -xomi + *xli - g52 ) +
                d5421 * sin ( xomi  + x2li - g54 ) + d5433 * sin ( -xomi + x2li - g54 ) ;
                
        xldot = *xni + xfact ;
        
        xnddt = d2201 * cos ( x2omi + *xli  - g22 ) + d2211 * cos ( *xli   - g22       ) +
                d3210 * cos ( xomi  + *xli  - g32 ) + d3222 * cos ( -xomi + *xli - g32 ) +
                d5220 * cos ( xomi  + *xli  - g52 ) + d5232 * cos ( -xomi + *xli - g52 ) +
                                              2.0 * ( d4410 * cos ( x2omi + x2li - g44 ) +
                d4422 * cos ( x2li  - g44         ) + d5421 * cos ( xomi  + x2li - g54 ) +
                d5433 * cos ( -xomi + x2li - g54 ) ) ;
                
        xnddt *= xldot ;
        
      }

/* 
 * Integrator 
 */
 
      if ( iretn == 381 ) {
        *xli   += xldot * delt + xndt  * step2 ;
        *xni   += xndt  * delt + xnddt * step2 ;
        *atime += delt ;
      }
      
    }  /* while iretn = 381 */

    *nm = *xni + xndt  * ft + xnddt * ft * ft * 0.5 ;
    xl  = *xli + xldot * ft + xndt  * ft * ft * 0.5 ;
    
    if ( irez != 1 ) {
      *mm   = xl - 2.0 * *nodem + 2.0 * theta ;
      *dndt = *nm - no ;
    }
    else {
      *mm   = xl - *nodem - *argpm + theta ;
      *dndt = *nm - no ;
    }
    *nm = no + *dndt ;
  }
  
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
