/*  
 *  - - - - - - - - - - - - - -
 *   g a l _ i n i t s g p 4 p  
 *  - - - - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine initializes the spg4 propagator. All the initialization is
 *     consolidated here instead of having multiple loops inside other routines.
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     satn             d           Satellite NORAD Catalog Number
 *     *gm           gal_gm_t       Pointer to gravity model structure
 *     ecco             d           Eccentricity [0.0:1.0]
 *     epoch            d           Epoch time in days from Jan 0, 1950. 0 hr
 *     inclo            d           Inclination of satellite
 *     *no              d           Mean Motion of satellite
 *  
 *  Returned:  
 *  
 *     *no              d           Mean Motion of satellite
 *     *method          c           Flag for deep space 'd', 'n'
 *     *ainv            d           1.0 / a
 *     *ao              d           Semi Major Axis
 *     *con41           d           
 *     *con42           d           1.0 - 5.0 cos(i)
 *     *cosio           d           Cosine of Inclination
 *     *cosio2          d           cosio squared
 *     *eccsq           d           Eccentricity squared
 *     *omeosq          d           1.0 - ecco * ecco
 *     *posq            d           Semi-parameter squared
 *     *rp              d           Radius of Perigee
 *     *rteosq          d           Square root of (1.0 - ecco*ecco)
 *     *sinio           d           Sine of inclination
 *     *gsto            d           GST at time of observation (rad)
 *   
 *  Notes:  
 *  
 *  1) This routine is a translation from c++ to c of David Vallado's SGP4UNIT.initl 
 *     routine ( 2007 November 16 ).
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 April 6 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_const.h"
#include "gal_initsgp4p.h"
#include "gal_gmst82.h"
  
void 
gal_initsgp4p
  (
    int satn,      
    gal_gm_t *gm,
    double ecco,   
    double epoch,  
    double inclo,   
    double *no,
    char   *method,
    double *ainv,  
    double *ao,    
    double *con41,  
    double *con42, 
    double *cosio,
    double *cosio2,
    double *eccsq, 
    double *omeosq, 
    double *posq,
    double *rp,    
    double *rteosq,
    double *sinio , 
    double *gsto
  ) 
  
{

  double ak, d1, del, adel, po, x2o3, j2, xke,
         tumin, mu, radiusearthkm, j3, j4, j3oj2 ;
         
 
  double ts70, ds70, tfrac, c1, thgr70, fk5r, c1p2p ;  
  int ids70 ;

/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/* 
 * Earth constants 
 */

  gal_sgp4gm ( gm, &tumin, &mu, &radiusearthkm, &xke, &j2, &j3, &j4, &j3oj2 ) ;
  x2o3   = 2.0 / 3.0 ;

/* 
 *  Calculate auxillary epoch quantities 
 */

  *eccsq  = ecco * ecco ;
  *omeosq = 1.0 - *eccsq ;
  *rteosq = sqrt ( *omeosq ) ;
  *cosio  = cos ( inclo ) ;
  *cosio2 = (*cosio) * (*cosio) ;

/* 
 * Un-Kozai the Mean Motion 
 */
 
  ak    = pow ( xke / *no, x2o3 ) ;
  d1    = 0.75 * j2 * ( 3.0 * (*cosio2) - 1.0 ) / ( (*rteosq) * (*omeosq) ) ;
  del   = d1 / ( ak * ak ) ;
  adel  = ak * ( 1.0 - del * del - del * ( 1.0 / 3.0 + 134.0 * del * del / 81.0 ) ) ;
  del   = d1 / ( adel * adel ) ;
  *no   /= ( 1.0 + del ) ;

  *ao    = pow ( xke / *no, x2o3 ) ;
  *sinio = sin ( inclo ) ;
  po    = (*ao) * (*omeosq) ;
  *con42 = 1.0 - 5.0 * (*cosio2) ;
  *con41 = -(*con42) - (*cosio2) - (*cosio2) ;
  *ainv  = 1.0 / (*ao) ;
  *posq  = (po) * (po) ;
  *rp    = (*ao) * ( 1.0 - ecco ) ;
  *method = 'n' ;

/*
 * Find greenwich location at epoch
 */

/*
 * Old way of finding gst
 * count integer number of days from 0 jan 1970
 * This is what AFSPC still uses apparently
 */

  ts70   = epoch - 7305.0 ;
  ids70  = floor ( ts70 + 1.0e-8 ) ;
  ds70   = ids70 ;
  tfrac  = ts70 - ds70 ;
  c1     = 1.72027916940703639e-2 ;
  thgr70 = 1.7321343856509374 ;
  fk5r   = 5.07551419432269442e-15 ;
  c1p2p  = c1 + GAL_2PI ;
  *gsto  = fmod ( thgr70 + c1 * ds70 + c1p2p * tfrac + ts70 * ts70 * fk5r, GAL_2PI ) ;
  if ( *gsto < 0.0 ) {
    *gsto += GAL_2PI ;
  }

/*  *gsto = gal_gmst82 ( 2433281.5, epoch ) ; */

/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
