/*  
 *  - - - - - - - - -
 *   g a l _ s g p 4
 *  - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine is the SGP4 prediction model from Space Command. This is an
 *     updated and combined version of SGP4 and SDP4, which were originally
 *     published separately in Spacetrack Report #3. This version follows the
 *     methodology from the AIAA paper (2006) describing the history and
 *     development of the code.
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     *sgp4        gal_sgp4_t      Initialised structure from gal_sgp4init call.
 *     epoch1           d           UTC epoch part 1 (Note 1)  
 *     epoch2           d           UTC epoch part 2 (Note 1)  
 *  
 *  Returned:  
 *  
 *     *sgp4        gal_sgp4_t      Common values for subsequent calls
 *     pv            d[2][3]        Geocentric position/velocity (m, m/sec)  
 *     gal_sgp4         i           GAL_SUCCESS if OK
 *                                  1 = mean elements, ecc >= 1.0 or ecc < -0.001 or a < 0.95 er
 *                                  2 = mean motion less than 0.0
 *                                  3 = pert elements, ecc < 0.0  or  ecc > 1.0
 *                                  4 = semi-latus rectum < 0.0
 *                                  5 = epoch elements are sub-orbital
 *                                  6 = satellite has decayed
 *
 *  Notes:  
 *  
 *  1) This routine is a translation from c++ to c of David Vallado's SGP4UNIT.sgp4 
 *     routine ( 2007 November 16 ). 
 *
 *  2) The epoch epoch1+epoch2 is a Julian Date, apportioned in  
 *     any convenient way between the two arguments.  For example,  
 *     JD(TDB)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            epoch1       epoch2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *     However, the accuracy of the result is more likely to be  
 *     limited by the algorithm itself than the way the epoch has been  
 *     expressed. 
 * 
 *  Called:
 *
 *     gal_dpper        
 *     gal_dspace
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 May 4 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_const.h"
#include "gal_sgp4.h"
#include "gal_dpper.h"
#include "gal_dspace.h"
  
int 
gal_sgp4
  (
    gal_sgp4_t *sgp4,
    double epoch1,
    double epoch2,
    double pv[2][3]
  ) 
  
{

  double am    , axnl      , aynl  , betal , cosim  , cnod  ,
         cos2u , coseo1    , cosi  , cosip , cosisq , cossu , cosu ,
         delm  , delomg    , em    , emsq  , ecose  , el2   , eo1  ,
         ep    , esine     , argpm , argpp , argpdf , pl    , mrt = 0.0,
         mvt   , rdotl     , rl    , rvdot , rvdotl , sinim ,
         sin2u , sineo1    , sini  , sinip , sinsu  , sinu  ,
         snod  , su        , t2    , t3    , t4     , tem5  , temp ,
         temp1 , temp2     , tempa , tempe , templ  , u     , ux   ,
         uy    , uz        , vx    , vy    , vz     , inclm , mm   ,
         nm    , nodem     , xinc  , xincp , xl     , xlm   , mp   ,
         xmdf  , xmx       , xmy   , nodedf, xnode  , nodep , tc   , dndt ;
         
  
  int ktr ; 

/*
 * Set mathematical constants 
 */

  const double x2o3  = 2.0 / 3.0 ;

/*
 * sgp4fix divisor for divide by zero check on inclination
 */
 
  const double temp4 = 1.0 + cos ( GAL_PI - 1.0e-9 ) ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  sgp4->t     = ( ( epoch1 - sgp4->jdepoch1 ) + ( epoch2 - sgp4->jdepoch2 ) ) * GAL_D2M ;
  sgp4->error = GAL_SUCCESS ;

/* 
 * Update for secular gravity and atmospheric drag 
 */

  xmdf   = sgp4->mo + sgp4->mdot * sgp4->t ;
  argpdf = sgp4->argpo + sgp4->argpdot * sgp4->t ;
  nodedf = sgp4->nodeo + sgp4->nodedot * sgp4->t ;
  argpm  = argpdf ;
  mm     = xmdf ;
  t2     = sgp4->t * sgp4->t ;
  nodem  = nodedf + sgp4->nodecf * t2 ;
  tempa  = 1.0 - sgp4->cc1 * sgp4->t ;
  tempe  = sgp4->bstar * sgp4->cc4 * sgp4->t ;
  templ  = sgp4->t2cof * t2 ;

  if ( sgp4->isimp != 1 ) {
    delomg = sgp4->omgcof * sgp4->t;
    delm   = sgp4->xmcof * ( pow ( ( 1.0 + sgp4->eta * cos ( xmdf ) ), 3 ) - sgp4->delmo ) ;
    temp   = delomg + delm ;
    mm     = xmdf + temp ;
    argpm  = argpdf - temp ;
    t3     = t2 * sgp4->t ;
    t4     = t3 * sgp4->t ;
    tempa  = tempa - sgp4->d2 * t2 - sgp4->d3 * t3 - sgp4->d4 * t4 ;
    tempe  = tempe + sgp4->bstar * sgp4->cc5 * ( sin ( mm ) - sgp4->sinmao ) ;
    templ  = templ + sgp4->t3cof * t3 + t4 * ( sgp4->t4cof + sgp4->t * sgp4->t5cof ) ;
  }

  nm    = sgp4->no;
  em    = sgp4->ecco;
  inclm = sgp4->inclo;
  
  if ( sgp4->method == 'd' ) {
    tc = sgp4->t;
    gal_dspace
      (
        sgp4->irez,
        sgp4->d2201, 
        sgp4->d2211, 
        sgp4->d3210,
        sgp4->d3222, 
        sgp4->d4410, 
        sgp4->d4422,
        sgp4->d5220, 
        sgp4->d5232, 
        sgp4->d5421,
        sgp4->d5433, 
        sgp4->dedt,  
        sgp4->del1,
        sgp4->del2,  
        sgp4->del3,  
        sgp4->didt,
        sgp4->dmdt,  
        sgp4->dnodt, 
        sgp4->domdt,
        sgp4->argpo, 
        sgp4->argpdot, 
        sgp4->t, 
        tc,
        sgp4->gsto, 
        sgp4->xfact, 
        sgp4->xlamo,
        sgp4->no, 
        &sgp4->atime,
        &em, 
        &argpm, 
        &inclm, 
        &sgp4->xli, 
        &mm, 
        &sgp4->xni,
        &nodem, 
        &dndt, 
        &nm
      ) ;
  } /* if method = d */

  if ( nm <= 0.0 ) {
    sgp4->error = 2 ;
  }
  am = pow ( ( sgp4->xke / nm ), x2o3 ) * tempa * tempa ;
  nm = sgp4->xke / pow ( am, 1.5 ) ;
  em -= tempe ;

/*
 * Fix tolerance for error recognition
 */
 
  if ( em >= 1.0 || em < -0.001 || am < 0.95 ) {
    sgp4->error = 1 ;
  }
  if ( em < 0.0 ) {
    em  = 1.0e-6;
  }
  mm     += sgp4->no * templ ;
  xlm    = mm + argpm + nodem ;
  emsq   = em * em ;
  temp   = 1.0 - emsq ;

  nodem  = fmod ( nodem, GAL_2PI ) ;
  argpm  = fmod ( argpm, GAL_2PI ) ;
  xlm    = fmod ( xlm, GAL_2PI ) ;
  mm     = fmod ( xlm - argpm - nodem, GAL_2PI ) ;

/* 
 * Compute extra mean quantities 
 */

  sinim = sin ( inclm ) ;
  cosim = cos ( inclm ) ;

/* 
 * Add lunar-solar periodics 
 */

  ep     = em ;
  xincp  = inclm ;
  argpp  = argpm ;
  nodep  = nodem ;
  mp     = mm ;
  sinip  = sinim ;
  cosip  = cosim ;
     
  if (sgp4->method == 'd') {
     gal_dpper
       (
         sgp4->e3,   
         sgp4->ee2,  
         sgp4->peo,
         sgp4->pgho, 
         sgp4->pho,  
         sgp4->pinco,
         sgp4->plo,  
         sgp4->se2,  
         sgp4->se3,
         sgp4->sgh2, 
         sgp4->sgh3, 
         sgp4->sgh4,
         sgp4->sh2,  
         sgp4->sh3,  
         sgp4->si2,
         sgp4->si3,  
         sgp4->sl2,  
         sgp4->sl3,
         sgp4->sl4,  
         sgp4->t,    
         sgp4->xgh2,
         sgp4->xgh3, 
         sgp4->xgh4, 
         sgp4->xh2,
         sgp4->xh3,  
         sgp4->xi2,  
         sgp4->xi3,
         sgp4->xl2,  
         sgp4->xl3,  
         sgp4->xl4,
         sgp4->zmol, 
         sgp4->zmos, 
         sgp4->inclo,
         'n', 
         &ep, 
         &xincp, 
         &nodep, 
         &argpp, 
         &mp
       ) ;
    if ( xincp < 0.0 ) {
      xincp  = -xincp ;
      nodep = nodep + GAL_PI ;
      argpp  = argpp - GAL_PI ;
    }
    if ( ep < 0.0 || ep > 1.0 ) {
      sgp4->error = 3;
    }
  } /* if method = d */

/* 
 * Long period periodics 
 */
 
  if ( sgp4->method == 'd' ) {
    sinip =  sin ( xincp ) ;
    cosip =  cos ( xincp ) ;
    sgp4->aycof = -0.5 * sgp4->j3oj2 * sinip ;
/*
 * sgp4fix for divide by zero for xincp = 180 deg
 */
    if ( fabs ( cosip + 1.0 ) > 1.5e-12 ) {
      sgp4->xlcof = -0.25 * sgp4->j3oj2 * sinip * ( 3.0 + 5.0 * cosip ) / ( 1.0 + cosip ) ;
    }
    else {
      sgp4->xlcof = -0.25 * sgp4->j3oj2 * sinip * ( 3.0 + 5.0 * cosip ) / temp4 ;
    }
  }
  axnl = ep * cos ( argpp ) ;
  temp = 1.0 / ( am * ( 1.0 - ep * ep ) ) ;
  aynl = ep * sin ( argpp ) + temp * sgp4->aycof ;
  xl   = mp + argpp + nodep + temp * sgp4->xlcof * axnl ;

/* 
 * Solve kepler's equation 
 */
 
  u    = fmod ( xl - nodep, GAL_2PI ) ;
  eo1  = u ;
  tem5 = 9999.9 ;
  ktr = 1 ;
  
/*
 * sgp4fix for kepler iteration
 * the following iteration needs better limits on corrections
 */
 
  while ( fabs ( tem5 ) >= 1.0e-12 && ktr <= 10 ) {
    sineo1 = sin ( eo1 ) ;
    coseo1 = cos ( eo1 ) ;
    tem5   = 1.0 - coseo1 * axnl - sineo1 * aynl ;
    tem5   = ( u - aynl * coseo1 + axnl * sineo1 - eo1 ) / tem5 ;
    if ( fabs ( tem5 ) >= 0.95 ) {
      tem5 = tem5 > 0.0 ? 0.95 : -0.95 ;
    }
    eo1 = eo1 + tem5 ;
    ktr = ktr + 1 ;
  }

/* 
 * Short period preliminary quantities 
 */
 
  ecose = axnl *coseo1 + aynl * sineo1 ;
  esine = axnl *sineo1 - aynl * coseo1 ;
  el2   = axnl *axnl + aynl * aynl ;
  pl    = am * ( 1.0 - el2 ) ;
  if ( pl < 0.0 ) {
    sgp4->error = 4 ;
  }
  else {
    rl     = am * ( 1.0 - ecose ) ;
    rdotl  = sqrt ( am ) * esine / rl ;
    rvdotl = sqrt ( pl ) / rl ;
    betal  = sqrt ( 1.0 - el2 ) ;
    temp   = esine / ( 1.0 + betal ) ;
    sinu   = am / rl * ( sineo1 - aynl - axnl * temp ) ;
    cosu   = am / rl * ( coseo1 - axnl + aynl * temp ) ;
    su     = atan2 ( sinu, cosu ) ;
    sin2u  = ( cosu + cosu ) * sinu ;
    cos2u  = 1.0 - 2.0 * sinu * sinu ;
    temp   = 1.0 / pl ;
    temp1  = 0.5 * sgp4->j2 * temp ;
    temp2  = temp1 * temp ;

/* 
 * Update for short period periodics 
 */
 
    if ( sgp4->method == 'd' ) {
      cosisq       = cosip * cosip ;
      sgp4->con41  = 3.0*cosisq - 1.0 ;
      sgp4->x1mth2 = 1.0 - cosisq ;
      sgp4->x7thm1 = 7.0*cosisq - 1.0 ;
    }
    
    mrt   = rl * ( 1.0 - 1.5 * temp2 * betal * sgp4->con41 ) +
            0.5 * temp1 * sgp4->x1mth2 * cos2u ;
    su    = su - 0.25 * temp2 * sgp4->x7thm1 * sin2u ;
    xnode = nodep + 1.5 * temp2 * cosip * sin2u ;
    xinc  = xincp + 1.5 * temp2 * cosip * sinip * cos2u ;
    mvt   = rdotl - nm * temp1 * sgp4->x1mth2 * sin2u / sgp4->xke ;
    rvdot = rvdotl + nm * temp1 * ( sgp4->x1mth2 * cos2u + 1.5 * sgp4->con41 ) / sgp4->xke ;

/* 
 * Orientation vectors 
 */
  
    sinsu =  sin ( su    ) ;
    cossu =  cos ( su    ) ;
    snod  =  sin ( xnode ) ;
    cnod  =  cos ( xnode ) ;
    sini  =  sin ( xinc  ) ;
    cosi  =  cos ( xinc  ) ;
    xmx   = -snod * cosi   ;
    xmy   =  cnod * cosi   ;
    
    ux    =  xmx  * sinsu + cnod * cossu ;
    uy    =  xmy  * sinsu + snod * cossu ;
    uz    =  sini * sinsu ;
    
    vx    =  xmx  * cossu - cnod * sinsu ;
    vy    =  xmy  * cossu - snod * sinsu ;
    vz    =  sini * cossu ;

/* 
 * Position and velocity (in km and km/sec) 
 */
 
    pv[0][0] = ( mrt * ux )             * sgp4->radiusearthkm * GAL_KM2M ;
    pv[0][1] = ( mrt * uy )             * sgp4->radiusearthkm * GAL_KM2M ;
    pv[0][2] = ( mrt * uz )             * sgp4->radiusearthkm * GAL_KM2M ;
    pv[1][0] = ( mvt * ux + rvdot * vx) * sgp4->vkmpersec     * GAL_KM2M ;
    pv[1][1] = ( mvt * uy + rvdot * vy) * sgp4->vkmpersec     * GAL_KM2M ;
    pv[1][2] = ( mvt * uz + rvdot * vz) * sgp4->vkmpersec     * GAL_KM2M ;
    
  }  /* if pl > 0 */

/*
 * sgp4fix for decaying satellites
 */
 
  if ( mrt < 1.0 ) {
     sgp4->error = 6 ;
  }

  return sgp4->error;

/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
