/*  
 *  - - - - - - - - - - - 
 *   g a l _ s g p 4 g m  
 *  - - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine gets the gravity model parameters required by SGP4
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     *gm           gal_gm_t       Pointer to gravity model structure
 *  
 *  Returned:  
 *  
 *     *tumin           d           Minutes in one time unit
 *     *mu              d           Earth gravitational parameter
 *     *re              d           Radius of the earth in km
 *     *xke             d           Reciprocal of tumin
 *     *j2              d           Un-normalized second zonal harmonic value 
 *     *j3              d           Un-normalized third  zonal harmonic value 
 *     *j4              d           Un-normalized fourth zonal harmonic value
 *     *j3oj2           d           j3 divided by j2
 *  
 *  Called:  
 * 
 *     gal_gmuzh                    Calculate un-normalized zonal harmonic 
 *     gal_facexp_alloc             Allocate factorial exponent lookup table
 *     gal_facexp_free              Deallocate factorial exponent lookup table
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 May 30
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include "gal_sgp4gm.h"
#include "gal_gmuzh.h"
#include "gal_const.h"
#include "gal_facexp_alloc.h"
#include "gal_facexp_free.h"
  
void
gal_sgp4gm
  (
    gal_gm_t *gm,
    double *tumin,
    double *mu,
    double *re,
    double *xke,
    double *j2,
    double *j3,
    double *j4,
    double *j3oj2
  ) 
  
{

  gal_facexp_t *facexp ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

  facexp = gal_facexp_alloc ( 8 ) ;  /* 2 times max degree */
  
  *mu    = gm->gm / 1e9 ;            /* mult is now 1e14 -> adjust for WGS72 units */
  *re    = gm->sma / GAL_KM2M ;
  *xke   = 60.0 / sqrt ( (*re) * (*re) * (*re) / *mu ) ;
  *tumin = 1.0 / *xke ;
  *j2    = gal_gmuzh ( gm, facexp, 2 ) ;
  *j3    = gal_gmuzh ( gm, facexp, 3 ) ;
  *j4    = gal_gmuzh ( gm, facexp, 4 ) ;
  *j3oj2 = *j3 / *j2 ;
  
  gal_facexp_free ( facexp ) ;
  
/*  
 * Finished.  
 */  

}
  
/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
