/*  
 *  - - - - - - - - - - - - -
 *   g a l _ s g p 4 i n i t  
 *  - - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *
 *  Description:
 *
 *     This routine initializes the variables for gal_sgp4.
 *  
 *  Status: 
 *  
 *     SGP4 support routine.  
 *  
 *  Given:  
 *
 *     *gm           gal_gm_t       Gravity Model
 *     *tle          gal_tle_t      TLE parameters
 *  
 *  Returned:  
 *  
 *     *sgp4        gal_sgp4_t      Common values for subsequent calls
 *     gal_sgp4init     i           GAL_SUCCESS if OK
 *                                  1 = mean elements, ecc >= 1.0 or ecc < -0.001 or a < 0.95 er
 *                                  2 = mean motion less than 0.0
 *                                  3 = pert elements, ecc < 0.0  or  ecc > 1.0
 *                                  4 = semi-latus rectum < 0.0
 *                                  5 = epoch elements are sub-orbital
 *                                  6 = satellite has decayed
 *
 *  Notes:  
 *  
 *  1) This routine is based on a translation from c++ to c of David Vallado's SGP4UNIT.sgp4init 
 *     routine ( 2007 November 16 ). 
 *
 *  Called:
 *
 *     gal_sgp4gm       Get gravity model parameters for SGP4
 *     gal_initsgp4p    Initialize SGP4 propagator
 *     gal_dscom        
 *     gal_dpper        
 *     gal_dsinit    
 *     gal_sgp4         SGP4 orbit propagator
 *     gal_days2cal     Convert day of year to calendar date
 *     gal_cal2jd       Convert calendar date to Julian Date
 *
 *  References:
 *
 *     NORAD Spacetrack Report #3 1980
 *     Hoots, Roehrich 
 *
 *     NORAD Spacetrack Report #6 1986
 *     Hoots 
 *
 *     Hoots, Schumacher and Glover 2004
 *
 *     Revisting Spacetrack Report #3
 *     Vallado, David, Crawford, Paul, Hujsak, Richard, Kelso, T.S.
 *     AIAA 2006-6753
 *
 *  This revision:  
 * 
 *     2008 April 6 
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end. 
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>
#include <strings.h>
#include "gal_const.h"
#include "gal_sgp4init.h"
#include "gal_days2cal.h"
#include "gal_cal2jd.h"
  
int 
gal_sgp4init
  (
    gal_gm_t *gm,
    gal_tle_t *tle,
    gal_sgp4_t *sgp4
  ) 
  
{

  double ao,     ainv,   con42 , cosio, sinio, cosio2, eccsq,
         omeosq, posq,   rp,     rteosq,
         cnodm , snodm , cosim , sinim , cosomm, sinomm, cc1sq ,
         cc2   , cc3   , coef  , coef1 , cosio4, day   , dndt  ,
         em    , emsq  , eeta  , etasq , gam   , argpm , nodem ,
         inclm , mm    , nm    , perige, pinvsq, psisq , qzms24,
         rtemsq, s1    , s2    , s3    , s4    , s5    , s6    ,
         s7    , sfour , ss1   , ss2   , ss3   , ss4   , ss5   ,
         ss6   , ss7   , sz1   , sz2   , sz3   , sz11  , sz12  ,
         sz13  , sz21  , sz22  , sz23  , sz31  , sz32  , sz33  ,
         tc    , temp  , temp1 , temp2 , temp3 , tsi   , xpidot,
         xhdot1, z1    , z2    , z3    , z11   , z12   , z13   ,
         z21   , z22   , z23   , z31   , z32   , z33,
         qzms2t, ss, x2o3, pv[2][3] ;

  const double xpdotp = 1440.0 / GAL_2PI ;  
  
  int iy, im, id ;
  double fd ;
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/* 
 * Initialization 
 */

/*
 * sgp4fix divisor for divide by zero check on inclination
 */
 
  const double temp4    =   1.0 + cos ( GAL_PI - 1.0e-9 ) ;

/* 
 * Set all near earth variables to zero 
 */
 
  sgp4->isimp   = 0   ; sgp4->method = 'n' ; sgp4->aycof    = 0.0 ;
  sgp4->con41   = 0.0 ; sgp4->cc1    = 0.0 ; sgp4->cc4      = 0.0 ;
  sgp4->cc5     = 0.0 ; sgp4->d2     = 0.0 ; sgp4->d3       = 0.0 ;
  sgp4->d4      = 0.0 ; sgp4->delmo  = 0.0 ; sgp4->eta      = 0.0 ;
  sgp4->argpdot = 0.0 ; sgp4->omgcof = 0.0 ; sgp4->sinmao   = 0.0 ;
  sgp4->t       = 0.0 ; sgp4->t2cof  = 0.0 ; sgp4->t3cof    = 0.0 ;
  sgp4->t4cof   = 0.0 ; sgp4->t5cof  = 0.0 ; sgp4->x1mth2   = 0.0 ;
  sgp4->x7thm1  = 0.0 ; sgp4->mdot   = 0.0 ; sgp4->nodedot  = 0.0 ;
  sgp4->xlcof   = 0.0 ; sgp4->xmcof  = 0.0 ; sgp4->nodecf   = 0.0 ;

/* 
 * Set all deep space variables to zero 
 */
 
  sgp4->irez  = 0   ; sgp4->d2201 = 0.0 ; sgp4->d2211 = 0.0 ;
  sgp4->d3210 = 0.0 ; sgp4->d3222 = 0.0 ; sgp4->d4410 = 0.0 ;
  sgp4->d4422 = 0.0 ; sgp4->d5220 = 0.0 ; sgp4->d5232 = 0.0 ;
  sgp4->d5421 = 0.0 ; sgp4->d5433 = 0.0 ; sgp4->dedt  = 0.0 ;
  sgp4->del1  = 0.0 ; sgp4->del2  = 0.0 ; sgp4->del3  = 0.0 ;
  sgp4->didt  = 0.0 ; sgp4->dmdt  = 0.0 ; sgp4->dnodt = 0.0 ;
  sgp4->domdt = 0.0 ; sgp4->e3    = 0.0 ; sgp4->ee2   = 0.0 ;
  sgp4->peo   = 0.0 ; sgp4->pgho  = 0.0 ; sgp4->pho   = 0.0 ;
  sgp4->pinco = 0.0 ; sgp4->plo   = 0.0 ; sgp4->se2   = 0.0 ;
  sgp4->se3   = 0.0 ; sgp4->sgh2  = 0.0 ; sgp4->sgh3  = 0.0 ;
  sgp4->sgh4  = 0.0 ; sgp4->sh2   = 0.0 ; sgp4->sh3   = 0.0 ;
  sgp4->si2   = 0.0 ; sgp4->si3   = 0.0 ; sgp4->sl2   = 0.0 ;
  sgp4->sl3   = 0.0 ; sgp4->sl4   = 0.0 ; sgp4->gsto  = 0.0 ;
  sgp4->xfact = 0.0 ; sgp4->xgh2  = 0.0 ; sgp4->xgh3  = 0.0 ;
  sgp4->xgh4  = 0.0 ; sgp4->xh2   = 0.0 ; sgp4->xh3   = 0.0 ;
  sgp4->xi2   = 0.0 ; sgp4->xi3   = 0.0 ; sgp4->xl2   = 0.0 ;
  sgp4->xl3   = 0.0 ; sgp4->xl4   = 0.0 ; sgp4->xlamo = 0.0 ;
  sgp4->zmol  = 0.0 ; sgp4->zmos  = 0.0 ; sgp4->atime = 0.0 ;
  sgp4->xli   = 0.0 ; sgp4->xni   = 0.0 ;

/*
 * Earth constants 
 */

  gal_sgp4gm 
    ( 
      gm, 
      &sgp4->tumin, 
      &sgp4->mu, 
      &sgp4->radiusearthkm, 
      &sgp4->xke, 
      &sgp4->j2, 
      &sgp4->j3, 
      &sgp4->j4, 
      &sgp4->j3oj2 
    ) ;
    
  ss     = 78.0 / sgp4->radiusearthkm + 1.0 ;
  qzms2t = pow ( ( ( 120.0 - 78.0 ) / sgp4->radiusearthkm ), 4 ) ;
  x2o3   =  2.0 / 3.0 ;
  sgp4->vkmpersec = sgp4->radiusearthkm * sgp4->xke / 60.0 ;

/* 
 * Gain TLE Information
 */

  sgp4->satnum         = tle->satnum                                  ;
  sgp4->classification = tle->classification                          ;
  sgp4->ephtype        = tle->ephtype                                 ;
  sgp4->setnum         = tle->setnum                                  ;
  sgp4->revnum         = tle->revnum                                  ;
  sgp4->bstar          = tle->bstar                                   ;
  sgp4->ecco           = tle->ecco                                    ;
  sgp4->argpo          = tle->argpo * GAL_D2R                         ;
  sgp4->inclo          = tle->inclo * GAL_D2R                         ;
  sgp4->mo	           = tle->mo    * GAL_D2R                         ;
  sgp4->nodeo          = tle->nodeo * GAL_D2R                         ;
  sgp4->no	           = tle->no / xpdotp                             ;  /* rad/min */
  sgp4->a              = pow ( sgp4->no * sgp4->tumin, -x2o3 )        ;
  sgp4->alta           = sgp4->a * ( 1.0 + sgp4->ecco ) - 1.0         ;
  sgp4->altp           = sgp4->a * ( 1.0 - sgp4->ecco ) - 1.0         ;
  sgp4->ndot           = sgp4->ndot  / ( xpdotp * GAL_D2M )           ;  /* ? * minperday */
  sgp4->nddot          = sgp4->nddot / ( xpdotp * GAL_D2M * GAL_D2M ) ;
  sgp4->epochyr        = tle->epochyr                                 ;
  sgp4->epochdays      = tle->epochdays                               ; 
  
  strcpy(sgp4->intldesg, tle->intldesg) ;

  gal_days2cal ( sgp4->epochyr, sgp4->epochdays, &iy, &im, &id, &fd ) ;
  
  gal_cal2jd ( iy, im, id, &sgp4->jdepoch1, &sgp4->jdepoch2 ) ;
  
  sgp4->jdepoch2 += fd ;
  
  sgp4->epoch = ( sgp4->jdepoch1 - 2433281.5 ) + sgp4->jdepoch2 ;
   
  sgp4->init = 'y' ;
  sgp4->t	 = 0.0 ;

  gal_initsgp4p 
    (
      sgp4->satnum, 
      gm, 
      sgp4->ecco, 
      sgp4->epoch, 
      sgp4->inclo, 
      &sgp4->no, 
      &sgp4->method,
      &ainv, 
      &ao, 
      &sgp4->con41, 
      &con42, 
      &cosio, 
      &cosio2, 
      &eccsq, 
      &omeosq,
      &posq, 
      &rp, 
      &rteosq, 
      &sinio, 
      &sgp4->gsto
    ) ;
    
  sgp4->error = GAL_SUCCESS ;

  if (rp < 1.0) {
    sgp4->error = 5 ;
  }

  if ( omeosq >= 0.0 || sgp4->no >= 0.0 ) {
    sgp4->isimp = 0;
    if (rp < (220.0 / sgp4->radiusearthkm + 1.0)) {
      sgp4->isimp = 1 ;
    }
    sfour  = ss ;
    qzms24 = qzms2t ;
    perige = ( rp - 1.0 ) * sgp4->radiusearthkm ;

/*
 * For perigees below 156 km, s and qoms2t are altered 
 */
 
    if ( perige < 156.0 ) {
      sfour = perige - 78.0 ;
      if (perige < 98.0) {
        sfour = 20.0 ;
      }  
      qzms24 = pow ( ( ( 120.0 - sfour ) / sgp4->radiusearthkm ), 4.0 ) ;
      sfour  = sfour / sgp4->radiusearthkm + 1.0 ;
    }
    pinvsq = 1.0 / posq ;

    tsi  = 1.0 / ( ao - sfour ) ;
    sgp4->eta  = ao * sgp4->ecco * tsi ;
    etasq = sgp4->eta * sgp4->eta ;
    eeta  = sgp4->ecco * sgp4->eta ;
    psisq = fabs ( 1.0 - etasq ) ;
    coef  = qzms24 * pow ( tsi, 4.0 ) ;
    coef1 = coef / pow ( psisq, 3.5 ) ;
    cc2   = coef1 * sgp4->no * ( ao * ( 1.0 + 1.5 * etasq + eeta * ( 4.0 + etasq ) ) 
          + 0.375 * sgp4->j2 * tsi / psisq * sgp4->con41 * ( 8.0 + 3.0 * etasq * ( 8.0 + etasq ) ) ) ;
    sgp4->cc1   = sgp4->bstar * cc2 ;
    cc3   = 0.0 ;
    if (sgp4->ecco > 1.0e-4) {
      cc3 = -2.0 * coef * tsi * sgp4->j3oj2 * sgp4->no * sinio / sgp4->ecco ;
    }
    sgp4->x1mth2 = 1.0 - cosio2 ;
    sgp4->cc4    = 2.0 * sgp4->no * coef1 * ao * omeosq *
                           ( sgp4->eta * ( 2.0 + 0.5 * etasq ) + sgp4->ecco *
                           ( 0.5 + 2.0 * etasq ) - sgp4->j2 * tsi / ( ao * psisq ) *
                           ( -3.0 * sgp4->con41 * ( 1.0 - 2.0 * eeta + etasq *
                           ( 1.5 - 0.5 * eeta ) ) + 0.75 * sgp4->x1mth2 *
                           ( 2.0 * etasq - eeta * ( 1.0 + etasq ) ) * cos ( 2.0 * sgp4->argpo ) ) ) ;
    sgp4->cc5 = 2.0 * coef1 * ao * omeosq * ( 1.0 + 2.75 * ( etasq + eeta ) + eeta * etasq ) ;
    cosio4 = cosio2 * cosio2 ;
    temp1  = 1.5 * sgp4->j2 * pinvsq * sgp4->no ;
    temp2  = 0.5 * temp1 * sgp4->j2 * pinvsq ;
    temp3  = -0.46875 * sgp4->j4 * pinvsq * pinvsq * sgp4->no ;
    sgp4->mdot    = sgp4->no + 0.5 * temp1 * rteosq * sgp4->con41 + 0.0625 *
                     temp2 * rteosq * ( 13.0 - 78.0 * cosio2 + 137.0 * cosio4 ) ;
    sgp4->argpdot = -0.5 * temp1 * con42 + 0.0625 * temp2 *
                    ( 7.0 - 114.0 * cosio2 + 395.0 * cosio4 ) +
                      temp3 * (3.0 - 36.0 * cosio2 + 49.0 * cosio4 ) ;
    xhdot1        = -temp1 * cosio ;
    sgp4->nodedot = xhdot1 + ( 0.5 * temp2 * ( 4.0 - 19.0 * cosio2 ) +
                              2.0 * temp3 * ( 3.0 - 7.0 * cosio2 ) ) * cosio ;
    xpidot        = sgp4->argpdot + sgp4->nodedot ;
    sgp4->omgcof  = sgp4->bstar * cc3 * cos ( sgp4->argpo ) ;
    sgp4->xmcof   = 0.0 ;
    if (sgp4->ecco > 1.0e-4) {
      sgp4->xmcof = -x2o3 * coef * sgp4->bstar / eeta ;
    }
    sgp4->nodecf = 3.5 * omeosq * xhdot1 * sgp4->cc1 ;
    sgp4->t2cof  = 1.5 * sgp4->cc1 ;
    
/*
 * sgp4fix for divide by zero with xinco = 180 deg
 */
 
    if (fabs(cosio+1.0) > 1.5e-12) {
      sgp4->xlcof = -0.25 * sgp4->j3oj2 * sinio * ( 3.0 + 5.0 * cosio ) / ( 1.0 + cosio ) ;
    }
    else {
      sgp4->xlcof = -0.25 * sgp4->j3oj2 * sinio * ( 3.0 + 5.0 * cosio ) / temp4 ;
    }
    sgp4->aycof   = -0.5 * sgp4->j3oj2 * sinio;
    sgp4->delmo   = pow ( ( 1.0 + sgp4->eta * cos ( sgp4->mo ) ), 3 ) ;
    sgp4->sinmao  = sin ( sgp4->mo ) ;
    sgp4->x7thm1  = 7.0 * cosio2 - 1.0 ;

/* 
 * Deep space initialization 
 */
 
    if ( ( GAL_2PI / sgp4->no ) >= 225.0) {
    
      sgp4->method = 'd'         ;
      sgp4->isimp  = 1           ;
      tc           =  0.0        ;
      inclm        = sgp4->inclo ;

      gal_dscom
        (
          sgp4->epoch, 
          sgp4->ecco, 
          sgp4->argpo, 
          tc, 
          sgp4->inclo, 
          sgp4->nodeo,
          sgp4->no, 
          &snodm, 
          &cnodm,  
          &sinim, 
          &cosim,
          &sinomm,     
          &cosomm,
          &day, 
          &sgp4->e3, 
          &sgp4->ee2, 
          &em,         
          &emsq, 
          &gam,
          &sgp4->peo,  
          &sgp4->pgho,   
          &sgp4->pho, 
          &sgp4->pinco,
          &sgp4->plo,  
          &rtemsq,        
          &sgp4->se2, 
          &sgp4->se3,
          &sgp4->sgh2, 
          &sgp4->sgh3,   
          &sgp4->sgh4,
          &sgp4->sh2,  
          &sgp4->sh3,    
          &sgp4->si2, 
          &sgp4->si3,
          &sgp4->sl2,  
          &sgp4->sl3,    
          &sgp4->sl4, 
          &s1, 
          &s2, 
          &s3, 
          &s4, 
          &s5,
          &s6,   
          &s7,   
          &ss1,  
          &ss2,  
          &ss3,  
          &ss4,  
          &ss5,  
          &ss6,  
          &ss7, 
          &sz1, 
          &sz2, 
          &sz3,
          &sz11, 
          &sz12, 
          &sz13, 
          &sz21, 
          &sz22, 
          &sz23, 
          &sz31, 
          &sz32, 
          &sz33,
          &sgp4->xgh2, 
          &sgp4->xgh3,   
          &sgp4->xgh4, 
          &sgp4->xh2,
          &sgp4->xh3,  
          &sgp4->xi2,    
          &sgp4->xi3,  
          &sgp4->xl2,
          &sgp4->xl3,  
          &sgp4->xl4,    
          &nm, 
          &z1, 
          &z2, 
          &z3, 
          &z11,
          &z12, 
          &z13, 
          &z21, 
          &z22, 
          &z23, 
          &z31, 
          &z32, 
          &z33,
          &sgp4->zmol, 
          &sgp4->zmos
        ) ;
        
      gal_dpper
        (
          sgp4->e3, 
          sgp4->ee2, 
          sgp4->peo, 
          sgp4->pgho,
          sgp4->pho, 
          sgp4->pinco, 
          sgp4->plo, 
          sgp4->se2,
          sgp4->se3, 
          sgp4->sgh2, 
          sgp4->sgh3, 
          sgp4->sgh4,
          sgp4->sh2, 
          sgp4->sh3, 
          sgp4->si2,  
          sgp4->si3,
          sgp4->sl2, 
          sgp4->sl3, 
          sgp4->sl4,  
          sgp4->t,
          sgp4->xgh2,
          sgp4->xgh3,
          sgp4->xgh4, 
          sgp4->xh2,
          sgp4->xh3, 
          sgp4->xi2, 
          sgp4->xi3,  
          sgp4->xl2,
          sgp4->xl3, 
          sgp4->xl4, 
          sgp4->zmol, 
          sgp4->zmos, 
          inclm, 
          sgp4->init,
          &sgp4->ecco, 
          &sgp4->inclo, 
          &sgp4->nodeo, 
          &sgp4->argpo, 
          &sgp4->mo
        ) ;

      argpm  = 0.0 ;
      nodem  = 0.0 ;
      mm     = 0.0 ;

      gal_dsinit
        (
          gm,
          cosim, 
          emsq, 
          sgp4->argpo, 
          s1, 
          s2, 
          s3, 
          s4, 
          s5, 
          sinim, 
          ss1, 
          ss2, 
          ss3, 
          ss4,
          ss5, 
          sz1, 
          sz3, 
          sz11, 
          sz13, 
          sz21, 
          sz23, 
          sz31, 
          sz33, 
          sgp4->t, 
          tc,
          sgp4->gsto, 
          sgp4->mo, 
          sgp4->mdot, 
          sgp4->no, 
          sgp4->nodeo,
          sgp4->nodedot, 
          xpidot, 
          z1, 
          z3, 
          z11, 
          z13, 
          z21, 
          z23, 
          z31, 
          z33,
          sgp4->ecco, 
          eccsq, 
          &em, 
          &argpm, 
          &inclm, 
          &mm, 
          &nm, 
          &nodem,
          &sgp4->irez,  
          &sgp4->atime,
          &sgp4->d2201, 
          &sgp4->d2211, 
          &sgp4->d3210, 
          &sgp4->d3222,
          &sgp4->d4410, 
          &sgp4->d4422, 
          &sgp4->d5220, 
          &sgp4->d5232,
          &sgp4->d5421, 
          &sgp4->d5433, 
          &sgp4->dedt,  
          &sgp4->didt,
          &sgp4->dmdt,  
          &dndt,         
          &sgp4->dnodt, 
          &sgp4->domdt,
          &sgp4->del1,  
          &sgp4->del2,  
          &sgp4->del3,  
          &sgp4->xfact,
          &sgp4->xlamo, 
          &sgp4->xli,   
          &sgp4->xni
        ) ;
    }

/* 
 * Set variables if not deep space 
 */
 
    if (sgp4->isimp != 1) {
    
      cc1sq       = sgp4->cc1 * sgp4->cc1 ;
      
      sgp4->d2    = 4.0 * ao * tsi * cc1sq ;
      
      temp        = sgp4->d2 * tsi * sgp4->cc1 / 3.0 ;
      
      sgp4->d3    = ( 17.0 * ao + sfour ) * temp ;
      
      sgp4->d4    = 0.5 * temp * ao * tsi * ( 221.0 * ao + 31.0 * sfour ) * sgp4->cc1 ;
                    
      sgp4->t3cof = sgp4->d2 + 2.0 * cc1sq;
      
      sgp4->t4cof = 0.25 * ( 3.0 * sgp4->d3 + sgp4->cc1 * ( 12.0 * sgp4->d2 + 10.0 * cc1sq ) ) ;
                            
      sgp4->t5cof = 0.2 * (  3.0 * sgp4->d4 +
                            12.0 * sgp4->cc1 * sgp4->d3 +
                             6.0 * sgp4->d2  * sgp4->d2 +
                            15.0 * cc1sq * (2.0 * sgp4->d2 + cc1sq ) ) ;
    }

  } /* if omeosq = 0 ... */

/* 
 * Finally propogate to zero epoch to initialise all others. 
 */
 
  if ( sgp4->error == 0 ) {
    gal_sgp4 
      ( 
        sgp4, 
        sgp4->jdepoch1, 
        sgp4->jdepoch2, 
        pv 
      ) ;
  }
  
  sgp4->init = 'n';

  return sgp4->error ;
    
/*  
 * Finished.  
 */  

}

/*
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *    
 *  Contact:  
 *    
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
  
