/*  
 *  - - - - - - - - - -  
 *   g a l _ f k 5 2 h  
 *  - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Transform FK5 (J2000) star data into the Hipparcos system.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given (all FK5, equinox J2000, epoch J2000):  
 *     r5                  d        RA (radians)  
 *     d5                  d        Dec (radians)  
 *     dr5                 d        proper motion in RA (dRA/dt, rad/Jyear)  
 *     dd5                 d        proper motion in Dec (dDec/dt, rad/Jyear)  
 *     px5                 d        parallax (arcsec)  
 *     rv5                 d        radial velocity (positive = receding)  
 *  
 *  Returned (all Hipparcos, epoch J2000):  
 *     *rh                 d        RA (radians)  
 *     *dh                 d        Dec (radians)  
 *     *drh                d        proper motion in RA (dRA/dt, rad/Jyear)  
 *     *ddh                d        proper motion in Dec (dDec/dt, rad/Jyear)  
 *     *pxh                d        parallax (arcsec)  
 *     *rvh                d        radial velocity (positive = receding)  
 *  
 *  Notes:  
 *  
 *  1) This routine transforms FK5 star positions and proper motions  
 *     into the system of the Hipparcos catalogue.  
 *  
 *  2) The proper motions in RA are dRA/dt rather than  
 *     cos(Dec)*dRA/dt, and are per year rather than per century.  
 *  
 *  3) The FK5 to Hipparcos transformation is modeled as a pure  
 *     rotation and spin;  zonal errors in the FK5 catalogue are  
 *     not taken into account.  
 *  
 *  4) See also gal_h2fk5, gal_fk5hz, gal_hfk5z.  
 *  
 *  Called:
 *  
 *     gal_starpv         star catalog data to space motion pv-vector  
 *     gal_fk5hip         FK5 to Hipparcos rotation and spin  
 *     gal_rxp            product of r-matrix and p-vector  
 *     gal_pxp            vector product of two p-vectors  
 *     gal_ppp            p-vector plus p-vector  
 *     gal_pvstar         space motion pv-vector to star catalog data  
 *  
 *  References:  
 *  
 *     F.Mignard & M.Froeschle, Astron. Astrophys. 354, 732-739 (2000).  
 *  
 *  This revision:
 *  
 *     2007 April 18 ( c version 2008 February 4 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_fk52h.h"  
#include "gal_starpv.h"  
#include "gal_fk5hip.h"  
#include "gal_rxp.h"  
#include "gal_pxp.h"  
#include "gal_ppp.h"  
#include "gal_pvstar.h"  
  
void  
gal_fk52h  
 (  
    double r5,  
    double d5,  
    double dr5,  
    double dd5,  
    double px5,  
    double rv5,  
    double *rh,  
    double *dh,  
    double *drh,  
    double *ddh,  
    double *pxh,  
    double *rvh  
 )  
{  
  
    double s5h[3], wxp[3], vv[3], r5h[3][3], pv5[2][3], pvh[2][3] ;  
  
    int j ;  
  
/*  
 *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * FK5 barycentric position/velocity pv-vector (normalized).  
 */  
  
    j = gal_starpv ( r5, d5, dr5, dd5, px5, rv5, pv5 ) ;  
  
/*  
 * FK5 to Hipparcos orientation matrix and spin vector.  
 */  
  
    gal_fk5hip ( r5h, s5h ) ;  
  
/*  
 * Orient the FK5 position into the Hipparcos system.  
 */  
  
    gal_rxp ( r5h, &pv5[0][0], &pvh[0][0] ) ;  
  
/*  
 * Apply spin to the position giving an extra space motion component.  
 */  
  
    gal_pxp ( &pv5[0][0], s5h, wxp ) ;  
  
/*  
 * Add this component to the FK5 space motion.  
 */  
  
    gal_ppp ( wxp, &pv5[1][0], vv ) ;  
  
/*  
 * Orient the FK5 space motion into the Hipparcos system.  
 */  
  
    gal_rxp ( r5h, vv, &pvh[1][0] ) ;  
  
/*  
 * Hipparcos pv-vector to spherical.  
 */  
  
    j = gal_pvstar ( pvh, rh, dh, drh, ddh, pxh, rvh ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
