/*  
 *  - - - - - - - -  
 *   g a l _ c 2 s  
 *  - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  P-vector to spherical coordinates.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     p                  d[3]      p-vector  
 *  
 *  Returned:
 *  
 *     *theta              d        longitude angle (radians)  
 *     *phi                d        latitude angle (radians)  
 *  
 *  Notes:  
 *  
 *  1) p can have any magnitude; only its direction is used.  
 *  
 *  2) If p is null, zero theta and phi are returned.  
 *  
 *  3) At either pole, zero theta is returned.  
 *  
 *  This revision:
 *  
 *     2007 April 11 ( c version 2008 January 27 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_c2s.h"  
  
void  
gal_c2s  
 (  
    double p[3],  
    double *theta,  
    double *phi  
 )  
{  
  
    double x, y, z, d2 ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
    x = p[0] ;  
    y = p[1] ;  
    z = p[2] ;  
  
    d2 = x * x + y * y ;  
  
    if ( d2 == 0.0 ) {  
        *theta = 0.0 ;  
    }  
    else {  
        *theta = atan2 ( y, x ) ;  
    }  
  
    if ( x == 0.0 ) {  
        *phi = 0.0 ;  
    }  
    else {  
        *phi = atan2 ( z, sqrt ( d2 ) ) ;  
    }  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
