/*  
 *  - - - - - - - -  
 *   g a l _ p a p  
 *  - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Position-angle from two p-vectors.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     a                  d[3]      direction of reference point  
 *     b                  d[3]      direction of point whose PA is required  
 *  
 *  Returned:
 *  
 *     gal_pap             d        position angle of b with respect to b (radians)  
 *  
 *  Notes:  
 *  
 *  1) The result is the position angle, in radians, of direction b with  
 *     respect to direction a.  It is in the range -pi to +pi.  The sense  
 *     is such that if b is a small distance "north" of a the position  
 *     angle is approximately zero, and if b is a small distance "east" of  
 *     a the position angle is approximately +pi/2.  
 *  
 *  2) a and b need not be unit vectors.  
 *  
 *  3) Zero is returned if the two directions are the same or if either  
 *     vector is null.  
 *  
 *  4) If a is at a pole, the result is ill-defined.  
 *  
 *  Called:
 *  
 *     gal_pn             decompose p-vector into modulus and direction  
 *     gal_pm             modulus of p-vector  
 *     gal_pxp            vector product of two p-vectors  
 *     gal_pmp            p-vector minus p-vector  
 *     gal_pdp            scalar product of two p-vectors  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_pap.h"  
#include "gal_pn.h"  
#include "gal_pm.h" 
#include "gal_pxp.h"  
#include "gal_pmp.h"  
#include "gal_pdp.h"  
  
double  
gal_pap  
 (  
    double a[3],  
    double b[3]
 )  
{  
  
    double am, bm, st, ct, xa, ya, za , au[3], eta[3], xi[3], a2b[3], theta ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Modulus and direction of the A vector.  
 */  
  
    gal_pn ( a, &am, au ) ;  
  
/*  
 * Modulus of the B vector.  
 */  
  
    bm = gal_pm ( b ) ;  
  
/*  
 * Deal with the case of a null vector.  
 */  
  
    if ( am == 0.0 || bm == 0.0 ) {  
        st = 0.0 ;  
        ct = 1.0 ;  
    }  
    else {  
  
/*  
 *    The "north" axis tangential from A (arbitrary length).  
 */  
  
        xa = a[0] ;  
        ya = a[1] ;  
        za = a[2] ;  
  
        eta[0] = -xa * za ;  
        eta[1] = -ya * za ;  
        eta[2] =   xa * xa + ya * ya ;  
  
/*  
 * The "east" axis tangential from A (same length).  
 */  
  
        gal_pxp ( eta, au, xi ) ;  
  
/*  
 * The vector from A to B.  
 */  
  
        gal_pmp ( b, a, a2b ) ;  
  
/*  
 * Resolve into components along the north and east axes.  
 */  
  
        st = gal_pdp ( a2b, xi ) ;  
        ct = gal_pdp ( a2b, eta ) ;  
  
/*  
 * Deal with degenerate cases.  
 */  
  
        if ( st == 0.0 && ct == 0.0 ) {  
            ct = 1.0 ;  
        }  
  
    }  
  
/*  
 * Position angle.  
 */  
  
    theta = atan2 ( st, ct ) ;  
  
    return theta ;
    
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
