/*  
 *  - - - - - - -  
 *   g a l _ p n  
 *  - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Convert a p-vector into modulus and unit vector.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     p                  d[3]      p-vector  
 *  
 *  Returned:
 *  
 *     r                   d        modulus  
 *     u                  d[3]      unit vector  
 *  
 *  Notes:  
 *
 *  1) If p is null, the result is null.  Otherwise the result is  
 *     a unit vector.  
 *  
 *  Called:
 *  
 *     gal_pm             modulus of p-vector  
 *     gal_zp             zero p-vector  
 *     gal_sxp            multiply p-vector by scalar  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_pn.h"  
#include "gal_pm.h"  
#include "gal_zp.h"  
#include "gal_sxp.h"  
  
void  
gal_pn  
 (  
    double p[3],  
    double *r,  
    double u[3]  
 )  
{  
  
    double w ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Obtain the modulus and test for zero.  
 */  
  
    w = gal_pm ( p ) ;  
  
    if ( w == 0.0 ) {  
  
/*  
 * Null vector.  
 */  
  
        gal_zp ( u ) ;  
  
    }  
    else {  
  
/*  
 * Unit vector.  
 */  
  
        gal_sxp ( 1.0 / w, p, u ) ;  
    }  
  
/*  
 * Return the modulus.  
 */  
  
    *r = w ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
