/*  
 *  - - - - - - - - -  
 *   g a l _ p v 2 s  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Convert position/velocity from Cartesian to spherical coordinates.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     pv               d[2][3]     pv-vector  
 *  
 *  Returned:
 *  
 *     *theta              d        longitude angle (radians)  
 *     *phi                d        latitude angle (radians)  
 *     *r                  d        radial distance  
 *     *td                 d        rate of change of THETA  
 *     *pd                 d        rate of change of PHI  
 *     *rd                 d        rate of change of R  
 *  
 *  Notes:  
 *  
 *  1) If the position part of pv is null, theta, phi, td and pd  
 *     are indeterminate.  This is handled by extrapolating the  
 *     position through unit time by using the velocity part of  
 *     pv.  This moves the origin without changing the direction  
 *     of the velocity component.  If the position and velocity  
 *     components of pv are both null, zeroes are returned for all  
 *     six results.  
 *  
 *  2) If the position is a pole, theta, td and pd are indeterminate.  
 *     In such cases zeroes are returned for theta, td and pd.  
 *  
 *  This revision:
 *  
 *     2001 January 4 ( c version 2008 February 4 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_pv2s.h"  
  
void  
gal_pv2s  
 (  
    double pv[2][3],  
    double *theta,  
    double *phi,  
    double *r,  
    double *td,  
    double *pd,  
    double *rd  
 )  
{  
  
    double x, y, z, xd, yd, zd, rxy2, rxy, r2, rtrue, rw, xyp ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Components of position/velocity vector.  
 */  
  
    x  = pv[0][0] ;  
    y  = pv[0][1] ;  
    z  = pv[0][2] ;  
    xd = pv[1][0] ;  
    yd = pv[1][1] ;  
    zd = pv[1][2] ;  
  
/*  
 * Component of R in XY plane squared.  
 */  
  
    rxy2 = x * x + y * y ;  
  
/*  
 * Modulus squared.  
 */  
  
    r2 = rxy2 + z * z ;  
  
/*  
 * Modulus.  
 */  
  
    rtrue = sqrt ( r2 ) ;  
  
/*  
 * If null vector, move the origin along the direction of movement.  
 */  
  
    rw = rtrue ;  
    if ( rtrue == 0.0 ) {  
        x = xd ;  
        y = yd ;  
        z = zd ;  
        rxy2 = x * x + y * y ;  
        r2 = rxy2 + z * z ;  
        rw = sqrt ( r2 ) ;  
    }  
  
/*  
 * Position and velocity in spherical coordinates.  
 */  
  
    rxy = sqrt ( rxy2 ) ;  
    xyp = x * xd + y * yd ;  
    if ( rxy2 != 0.0 ) {  
        *theta = atan2 ( y, x ) ;  
        *phi = atan2 ( z, rxy ) ;  
        *td = ( x * yd - y * xd ) / rxy2 ;  
        *pd = ( zd * rxy2 - z * xyp ) / ( r2 * rxy ) ;  
    }  
    else {  
        *theta = 0.0 ;  
        if ( z != 0.0 ) {  
            *phi = atan2 ( z, rxy ) ;  
        }  
        else {  
            *phi = 0.0 ;  
        }  
        *td = 0.0 ;  
        *pd = 0.0 ;  
    }  
    *r = rtrue ;  
    if ( rw != 0.0 ) {  
        *rd = ( xyp + z * zd ) / rw ;  
    }  
    else {  
        *rd = 0.0 ;  
    }  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
