/*  
 *  - - - - - - - - - -  
 *   g a l _ p v x p v  
 *  - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Outer (=vector=cross) product of two pv-vectors.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     a                d[2][3]     first pv-vector  
 *     b                d[2][3]     second pv-vector  
 *  
 *  Returned:
 *  
 *     axb              d[2][3]     a x b  
 *  
 *  Notes:  
 *  
 *  1) If the position and velocity components of the two pv-vectors are  
 *     ( ap, av ) and ( bp, bv ), the result, a x b, is the pair of  
 *     vectors ( ap x bp, ap x bv + av x bp ).  The two vectors are the  
 *     cross-product of the two p-vectors and its derivative.  
 *  
 *  Called:
 *  
 *     gal_cpv            copy pv-vector  
 *     gal_pxp            vector product of two p-vectors  
 *     gal_ppp            p-vector plus p-vector  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 February 4 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include "gal_pvxpv.h"  
#include "gal_cpv.h"  
#include "gal_pxp.h"  
#include "gal_ppp.h"  
  
void  
gal_pvxpv  
 (  
    double a[2][3],  
    double b[2][3],  
    double axb[2][3]  
 )  
{  
  
    double wa[2][3], wb[2][3], axbd[3], adxb[3] ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Make copies of the inputs.  
 */  
  
    gal_cpv ( a, wa ) ;  
    gal_cpv ( b, wb ) ;  
  
/*  
 * a x b = position part of result.  
 */  
  
    gal_pxp ( &wa[0][0], &wb[0][0], &axb[0][0] ) ;  
  
/*  
 * a x bdot + adot x b = velocity part of result.  
 */  
  
    gal_pxp ( &wa[0][0], &wb[1][0], axbd ) ;  
    gal_pxp ( &wa[1][0], &wb[0][0], adxb ) ;  
    gal_ppp ( axbd, adxb, &axb[1][0] ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
