/*  
 *  - - - - - - - - -  
 *   g a l _ r m 2 v  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Express an r-matrix as an r-vector.  
 *  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     r                d[3][3]     rotation matrix  
 *  
 *  Returned:
 *  
 *     w                  d[3]      rotation vector (Note 1)  
 *  
 *  Notes:  
 *  
 *  1) A rotation matrix describes a rotation through some angle about  
 *     some arbitrary axis called the Euler axis.  The "rotation vector"  
 *     returned by this routine has the same direction as the Euler axis,  
 *     and its magnitude is the angle in radians.  (The magnitude and  
 *     direction can be separated by means of the routine gal_pn.)  
 *  
 *  2) If r is null, so is the result.  If r is not a rotation matrix  
 *     the result is undefined.  r must be proper (i.e. have a positive  
 *     determinant) and real orthogonal (inverse = transpose).  
 *  
 *  3) The reference frame rotates clockwise as seen looking along  
 *     the rotation vector from the origin.  
 *  
 *  This revision:
 *  
 *     2006 April 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_rm2v.h"  
  
void  
gal_rm2v  
 (  
    double r[3][3],  
    double w[3]  
 )  
{  
  
    double x, y, z, s2, c2, phi, f ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
    x = r[1][2] - r[2][1] ;  
    y = r[2][0] - r[0][2] ;  
    z = r[0][1] - r[1][0] ;  
    s2 = sqrt ( x * x + y * y + z * z ) ;  
    if ( s2 != 0.0 ) {  
        c2 = r[0][0] + r[1][1] + r[2][2] - 1.0 ;  
        phi = atan2 ( s2, c2 ) ;  
        f = phi / s2 ;  
        w[0] = x * f ;  
        w[1] = y * f ;  
        w[2] = z * f ;  
    }  
    else {  
        w[0] = 0.0 ;  
        w[1] = 0.0 ;  
        w[2] = 0.0 ;  
    }  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
