/*  
 *  - - - - - - - - -  
 *   g a l _ r v 2 m  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Form the r-matrix corresponding to a given r-vector.  
 *  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     w                  d[3]      rotation vector (Note 1)  
 *  
 *  Returned:
 *  
 *     r                d[3][3]     rotation matrix  
 *  
 *  Notes:  
 *  
 *  1) A rotation matrix describes a rotation through some angle about  
 *     some arbitrary axis called the Euler axis.  The "rotation vector"  
 *     supplied to this routine has the same direction as the Euler axis,  
 *     and its magnitude is the angle in radians.  
 *  
 *  2) If w is null, the unit matrix is returned.  
 *  
 *  3) The reference frame rotates clockwise as seen looking along  
 *     the rotation vector from the origin.  
 *  
 *  This revision:
 *  
 *     2006 April 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_rv2m.h"  
  
void  
gal_rv2m  
 (  
    double w[3],  
    double r[3][3]  
 )  
{  
  
    double x, y, z, phi, s, c, f ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Euler angle (magnitude of rotation vector) and functions.  
 */  
  
    x = w[0] ;  
    y = w[1] ;  
    z = w[2] ;  
    phi = sqrt ( x * x + y * y + z * z ) ;  
    s = sin ( phi ) ;  
    c = cos ( phi ) ;  
    f = 1.0 - c ;  
  
/*  
 * Euler axis (direction of rotation vector), perhaps null.  
 */  
  
    if ( phi != 0.0 ) {  
        x /= phi ;  
        y /= phi ;  
        z /= phi ;  
    }  
  
/*  
 * Form the rotation matrix.  
 */  
  
    r[0][0] = x * x * f + c     ;  
    r[0][1] = x * y * f + z * s ;  
    r[0][2] = x * z * f - y * s ;  
    r[1][0] = y * x * f - z * s ;  
    r[1][1] = y * y * f + c     ;  
    r[1][2] = y * z * f + x * s ;  
    r[2][0] = z * x * f + y * s ;  
    r[2][1] = z * y * f - x * s ;  
    r[2][2] = z * z * f + c     ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
