/*  
 *  - - - - - - -  
 *   g a l _ r y  
 *  - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Rotate an r-matrix about the y-axis.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     theta               d        angle (radians)  
 *  
 *  Given and returned:  
 *     r                d[3][3]     r-matrix  
 *  
 *  Notes:  
 *  
 *  1) Sign convention: The matrix can be used to rotate the  
 *     reference frame of a vector.  Calling this routine with  
 *     positive theta incorporates in the matrix an additional  
 *     rotation, about the y-axis, anticlockwise as seen looking  
 *     towards the origin from positive y.  
 *  
 *  Called:
 *  
 *     gal_ir             initialize r-matrix to identity  
 *     gal_rxr            product of two r-matrices  
 *     gal_cr             copy r-matrix  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_ry.h"  
#include "gal_ir.h"  
#include "gal_cr.h"  
#include "gal_rxr.h"  
  
void  
gal_ry  
 (  
    double theta,  
    double r[3][3]  
 )  
{  
    double s, c, a[3][3], w[3][3] ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Matrix representing new rotation.  
 */  
  
    s = sin ( theta ) ;  
    c = cos ( theta ) ;  
  
    gal_ir ( a ) ;  
  
    a[0][0] =  c ;  
    a[2][0] =  s ;  
    a[0][2] = -s ;  
    a[2][2] =  c ;  
  
/*  
 * Rotate.  
 */  
  
    gal_rxr ( a, r, w ) ;  
  
/*  
 * Return result.  
 */  
  
    gal_cr ( w, r ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
