/*  
 *  - - - - - - - - -  
 *   g a l _ s e p p  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Angular separation between two p-vectors.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     vector/matrix support routine.  
 *  
 *  Given:
 *  
 *     a                  d[3]      first p-vector (not necessarily unit length)  
 *     b                  d[3]      second p-vector (not necessarily unit length)  
 *  
 *  Returned:
 *  
 *     *s                  d        angular separation (radians, always positive)  
 *  
 *  Notes:  
 *  
 *  1) If either vector is null, a zero result is returned.  
 *  
 *  2) The angular separation is most simply formulated in terms of  
 *     scalar product.  However, this gives poor accuracy for angles  
 *     near zero and pi.  The present algorithm uses both cross product  
 *     and dot product, to deliver full accuracy whatever the size of  
 *     the angle.  
 *  
 *  Called:
 *  
 *     gal_pxp            vector product of two p-vectors  
 *     gal_pm             modulus of p-vector  
 *     gal_pdp            scalar product of two p-vectors  
 *  
 *  This revision:
 *  
 *     2006 November 13 ( c version 2008 June 8 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_sepp.h"  
#include "gal_pxp.h"  
#include "gal_pm.h"  
#include "gal_pdp.h"  
  
double  
gal_sepp  
 (  
    double a[3],  
    double b[3]
 )  
{  
  
    double ss, cs, axb[3], s ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Sine of the angle between the vectors, multiplied by the two moduli.  
 */  
  
    gal_pxp ( a, b, axb ) ;  
    ss = gal_pm ( axb ) ;  
  
/*  
 * Cosine of the angle, multiplied by the two moduli.  
 */  
  
    cs = gal_pdp ( a, b ) ;  
  
/*  
 * The angle.  
 */  
  
    if ( ss != 0.0 || cs != 0.0 ) {  
        s = atan2 ( ss, cs ) ;  
    }  
    else {  
        s = 0.0 ;  
    }
    
    return s ;
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
