/*  
 *  - - - - - - - - - - -  
 *   g a l _ n u t 0 0 a  
 *  - - - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Nutation, IAU 2000A model (MHB2000 luni-solar and planetary nutation  
 *  with free core nutation omitted).  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     *dpsi,*deps         d        nutation, luni-solar + planetary (Note 2)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The nutation components in longitude and obliquity are in radians  
 *     and with respect to the equinox and ecliptic of date.  The  
 *     obliquity at J2000 is assumed to be the Lieske et al. (1977) value  
 *     of 84381.448 arcsec.  
 *  
 *     Both the luni-solar and planetary nutations are included.  The  
 *     latter are due to direct planetary nutations and the perturbations  
 *     of the lunar and terrestrial orbits.  
 *  
 *  3) The routine computes the MHB2000 nutation series with the  
 *     associated corrections for planetary nutations.  It is an  
 *     implementation of the nutation part of the IAU 2000A precession-  
 *     nutation model, formally adopted by the IAU General Assembly in  
 *     2000, namely MHB2000 (Mathews et al. 2002), but with the free core  
 *     nutation (FCN - see Note 4) omitted.  
 *  
 *  4) The full MHB2000 model also contains contributions to the  
 *     nutations in longitude and obliquity due to the free-excitation of  
 *     the free-core-nutation during the period 1979-2000.  These FCN  
 *     terms, which are time-dependent and unpredictable, are NOT  
 *     included in the present routine and, if required, must be  
 *     independently computed.  With the FCN corrections included, the  
 *     present routine delivers a pole which is at current epochs  
 *     accurate to a few hundred microarcseconds.  The omission of FCN  
 *     introduces further errors of about that size.  
 *  
 *  5) The present routine provides classical nutation.  The MHB2000  
 *     algorithm, from which it is adapted, deals also with (i) the  
 *     offsets between the GCRS and mean poles and (ii) the adjustments  
 *     in longitude and obliquity due to the changed precession rates.  
 *     These additional functions, namely frame bias and precession  
 *     adjustments, are supported by the SOFA routines gal_bi00 and  
 *     gal_pr00.  
 *  
 *  6) The MHB2000 algorithm also provides "total" nutations, comprising  
 *     the arithmetic sum of the frame bias, precession adjustments,  
 *     luni-solar nutation and planetary nutation.  These total nutations  
 *     can be used in combination with an existing IAU 1976 precession  
 *     implementation, such as gal_pmat76, to deliver GCRS-to-true  
 *     predictions of sub-mas accuracy at current epochs.  However, there  
 *     are three shortcomings in the MHB2000 model that must be taken  
 *     into account if more accurate or definitive results are required  
 *     (see Wallace 2002):  
 *  
 *       (i) The MHB2000 total nutations are simply arithmetic sums,  
 *           yet in reality the various components are successive Euler  
 *           rotations.  This slight lack of rigor leads to cross terms  
 *           that exceed 1 mas after a century.  The rigorous procedure  
 *           is to form the GCRS-to-true rotation matrix by applying the  
 *           bias, precession and nutation in that order.  
 *  
 *      (ii) Although the precession adjustments are stated to be with  
 *           respect to Lieske et al. (1977), the MHB2000 model does  
 *           not specify which set of Euler angles are to be used and  
 *           how the adjustments are to be applied.  The most literal and  
 *           straightforward procedure is to adopt the 4-rotation  
 *           epsilon_0, psi_A, omega_A, xi_A option, and to add dpsipr to  
 *           psi_A and depspr to both omega_A and eps_A.  
 *  
 *     (iii) The MHB2000 model predates the determination by Chapront  
 *           et al. (2002) of a 14.6 mas displacement between the J2000  
 *           mean equinox and the origin of the ICRS frame.  It should,  
 *           however, be noted that neglecting this displacement when  
 *           calculating star coordinates does not lead to a 14.6 mas  
 *           change in right ascension, only a small second-order  
 *           distortion in the pattern of the precession-nutation effect.  
 *  
 *     For these reasons, the SOFA routines do not generate the "total  
 *     nutations" directly, though they can of course easily be generated  
 *     by calling gal_bi00, gal_pr00 and the present routine and adding  
 *     the results.  
 *  
 *  Called:
 *  
 *     gal_fal03          mean anomaly of the Moon  
 *     gal_faf03          mean argument of the latitude of the Moon  
 *     gal_faom03         mean longitude of the Moon's ascending node  
 *     gal_fame03         mean longitude of Mercury  
 *     gal_fave03         mean longitude of Venus  
 *     gal_fae03          mean longitude of Earth  
 *     gal_fama03         mean longitude of Mars  
 *     gal_faju03         mean longitude of Jupiter  
 *     gal_fasa03         mean longitude of Saturn  
 *     gal_faur03         mean longitude of Uranus  
 *     gal_fapa03         general accumulated precession in longitude  
 *  
 *  References:  
 *  
 *     Chapront, J., Chapront-Touze, M. & Francou, G. 2002,  
 *     Astron.Astrophys. 387, 700  
 *  
 *     Lieske, J.H., Lederle, T., Fricke, W. & Morando, B. 1977,  
 *     Astron.Astrophys. 58, 1-16  
 *  
 *     Mathews, P.M., Herring, T.A., Buffet, B.A. 2002, J.Geophys.Res.  
 *     107, B4.  The MHB_2000 code itself was obtained on 9th September  
 *     2002 from ftp//maia.usno.navy.mil/conv2000/chapter5/IAU2000A.  
 *  
 *     Simon, J.-L., Bretagnon, P., Chapront, J., Chapront-Touze, M.,  
 *     Francou, G., Laskar, J. 1994, Astron.Astrophys. 282, 663-683  
 *  
 *     Souchay, J., Loysel, B., Kinoshita, H., Folgueira, M. 1999,  
 *     Astron.Astrophys.Supp.Ser. 135, 111  
 *  
 *     Wallace, P.T., "Software for Implementing the IAU 2000  
 *     Resolutions", in IERS Workshop 5.1 (2002)  
 *  
 *  This revision:
 *  
 *     2007 February 9 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_nut00a.h"  
#include "gal_fal03.h"  
#include "gal_faf03.h"  
#include "gal_faom03.h"  
#include "gal_fame03.h"  
#include "gal_fave03.h"  
#include "gal_fae03.h"  
#include "gal_fama03.h"  
#include "gal_faju03.h"  
#include "gal_fasa03.h"  
#include "gal_faur03.h"  
#include "gal_fapa03.h"  
  
void  
gal_nut00a  
 (  
    double date1,  
    double date2,  
    double *dpsi,  
    double *deps  
 )  
{  
  
/*  
 * Miscellaneous  
 */  
  
    int i ;  
  
    double t, el, elp, f, d, om, arg, dp, de, sarg, carg,  
           al, alsu, af, ad, aom, alme, alve, alea, alma,  
           alju, alsa, alur, alne, apa, dpsils, depsls,  
           dpsipl, depspl ;  
  
/*  
 * ----------------------------------------  
 * Tables of argument and term coefficients  
 * ----------------------------------------  
 */  
  
/*  
 * -------------------------  
 * Luni-Solar nutation model  
 * -------------------------  
 */  
  
/*  
 * Number of terms in the luni-solar nutation model  
 */  
  
    #define NLS 678  
  
/*  
 * Coefficients for fundamental arguments  
 */  
  
    static const int NALS[ NLS ][5] = {  
  
/*  
 *  Luni-Solar argument multipliers  
 *      L     L'    F     D     Om  
 */  
  
     {  0,    0,    0,    0,    1, } ,  
     {  0,    0,    2,   -2,    2, } ,  
     {  0,    0,    2,    0,    2, } ,  
     {  0,    0,    0,    0,    2, } ,  
     {  0,    1,    0,    0,    0, } ,  
     {  0,    1,    2,   -2,    2, } ,  
     {  1,    0,    0,    0,    0, } ,  
     {  0,    0,    2,    0,    1, } ,  
     {  1,    0,    2,    0,    2, } ,  
     {  0,   -1,    2,   -2,    2, } ,  
     {  0,    0,    2,   -2,    1, } ,  
     { -1,    0,    2,    0,    2, } ,  
     { -1,    0,    0,    2,    0, } ,  
     {  1,    0,    0,    0,    1, } ,  
     { -1,    0,    0,    0,    1, } ,  
     { -1,    0,    2,    2,    2, } ,  
     {  1,    0,    2,    0,    1, } ,  
     { -2,    0,    2,    0,    1, } ,  
     {  0,    0,    0,    2,    0, } ,  
     {  0,    0,    2,    2,    2, } ,  
     {  0,   -2,    2,   -2,    2, } ,  
     { -2,    0,    0,    2,    0, } ,  
     {  2,    0,    2,    0,    2, } ,  
     {  1,    0,    2,   -2,    2, } ,  
     { -1,    0,    2,    0,    1, } ,  
     {  2,    0,    0,    0,    0, } ,  
     {  0,    0,    2,    0,    0, } ,  
     {  0,    1,    0,    0,    1, } ,  
     { -1,    0,    0,    2,    1, } ,  
     {  0,    2,    2,   -2,    2, } ,  
     {  0,    0,   -2,    2,    0, } ,  
     {  1,    0,    0,   -2,    1, } ,  
     {  0,   -1,    0,    0,    1, } ,  
     { -1,    0,    2,    2,    1, } ,  
     {  0,    2,    0,    0,    0, } ,  
     {  1,    0,    2,    2,    2, } ,  
     { -2,    0,    2,    0,    0, } ,  
     {  0,    1,    2,    0,    2, } ,  
     {  0,    0,    2,    2,    1, } ,  
     {  0,   -1,    2,    0,    2, } ,  
     {  0,    0,    0,    2,    1, } ,  
     {  1,    0,    2,   -2,    1, } ,  
     {  2,    0,    2,   -2,    2, } ,  
     { -2,    0,    0,    2,    1, } ,  
     {  2,    0,    2,    0,    1, } ,  
     {  0,   -1,    2,   -2,    1, } ,  
     {  0,    0,    0,   -2,    1, } ,  
     { -1,   -1,    0,    2,    0, } ,  
     {  2,    0,    0,   -2,    1, } ,  
     {  1,    0,    0,    2,    0, } ,  
     {  0,    1,    2,   -2,    1, } ,  
     {  1,   -1,    0,    0,    0, } ,  
     { -2,    0,    2,    0,    2, } ,  
     {  3,    0,    2,    0,    2, } ,  
     {  0,   -1,    0,    2,    0, } ,  
     {  1,   -1,    2,    0,    2, } ,  
     {  0,    0,    0,    1,    0, } ,  
     { -1,   -1,    2,    2,    2, } ,  
     { -1,    0,    2,    0,    0, } ,  
     {  0,   -1,    2,    2,    2, } ,  
     { -2,    0,    0,    0,    1, } ,  
     {  1,    1,    2,    0,    2, } ,  
     {  2,    0,    0,    0,    1, } ,  
     { -1,    1,    0,    1,    0, } ,  
     {  1,    1,    0,    0,    0, } ,  
     {  1,    0,    2,    0,    0, } ,  
     { -1,    0,    2,   -2,    1, } ,  
     {  1,    0,    0,    0,    2, } ,  
     { -1,    0,    0,    1,    0, } ,  
     {  0,    0,    2,    1,    2, } ,  
     { -1,    0,    2,    4,    2, } ,  
     { -1,    1,    0,    1,    1, } ,  
     {  0,   -2,    2,   -2,    1, } ,  
     {  1,    0,    2,    2,    1, } ,  
     { -2,    0,    2,    2,    2, } ,  
     { -1,    0,    0,    0,    2, } ,  
     {  1,    1,    2,   -2,    2, } ,  
     { -2,    0,    2,    4,    2, } ,  
     { -1,    0,    4,    0,    2, } ,  
     {  2,    0,    2,   -2,    1, } ,  
     {  2,    0,    2,    2,    2, } ,  
     {  1,    0,    0,    2,    1, } ,  
     {  3,    0,    0,    0,    0, } ,  
     {  3,    0,    2,   -2,    2, } ,  
     {  0,    0,    4,   -2,    2, } ,  
     {  0,    1,    2,    0,    1, } ,  
     {  0,    0,   -2,    2,    1, } ,  
     {  0,    0,    2,   -2,    3, } ,  
     { -1,    0,    0,    4,    0, } ,  
     {  2,    0,   -2,    0,    1, } ,  
     { -2,    0,    0,    4,    0, } ,  
     { -1,   -1,    0,    2,    1, } ,  
     { -1,    0,    0,    1,    1, } ,  
     {  0,    1,    0,    0,    2, } ,  
     {  0,    0,   -2,    0,    1, } ,  
     {  0,   -1,    2,    0,    1, } ,  
     {  0,    0,    2,   -1,    2, } ,  
     {  0,    0,    2,    4,    2, } ,  
     { -2,   -1,    0,    2,    0, } ,  
     {  1,    1,    0,   -2,    1, } ,  
     { -1,    1,    0,    2,    0, } ,  
     { -1,    1,    0,    1,    2, } ,  
     {  1,   -1,    0,    0,    1, } ,  
     {  1,   -1,    2,    2,    2, } ,  
     { -1,    1,    2,    2,    2, } ,  
     {  3,    0,    2,    0,    1, } ,  
     {  0,    1,   -2,    2,    0, } ,  
     { -1,    0,    0,   -2,    1, } ,  
     {  0,    1,    2,    2,    2, } ,  
     { -1,   -1,    2,    2,    1, } ,  
     {  0,   -1,    0,    0,    2, } ,  
     {  1,    0,    2,   -4,    1, } ,  
     { -1,    0,   -2,    2,    0, } ,  
     {  0,   -1,    2,    2,    1, } ,  
     {  2,   -1,    2,    0,    2, } ,  
     {  0,    0,    0,    2,    2, } ,  
     {  1,   -1,    2,    0,    1, } ,  
     { -1,    1,    2,    0,    2, } ,  
     {  0,    1,    0,    2,    0, } ,  
     {  0,   -1,   -2,    2,    0, } ,  
     {  0,    3,    2,   -2,    2, } ,  
     {  0,    0,    0,    1,    1, } ,  
     { -1,    0,    2,    2,    0, } ,  
     {  2,    1,    2,    0,    2, } ,  
     {  1,    1,    0,    0,    1, } ,  
     {  1,    1,    2,    0,    1, } ,  
     {  2,    0,    0,    2,    0, } ,  
     {  1,    0,   -2,    2,    0, } ,  
     { -1,    0,    0,    2,    2, } ,  
     {  0,    1,    0,    1,    0, } ,  
     {  0,    1,    0,   -2,    1, } ,  
     { -1,    0,    2,   -2,    2, } ,  
     {  0,    0,    0,   -1,    1, } ,  
     { -1,    1,    0,    0,    1, } ,  
     {  1,    0,    2,   -1,    2, } ,  
     {  1,   -1,    0,    2,    0, } ,  
     {  0,    0,    0,    4,    0, } ,  
     {  1,    0,    2,    1,    2, } ,  
     {  0,    0,    2,    1,    1, } ,  
     {  1,    0,    0,   -2,    2, } ,  
     { -1,    0,    2,    4,    1, } ,  
     {  1,    0,   -2,    0,    1, } ,  
     {  1,    1,    2,   -2,    1, } ,  
     {  0,    0,    2,    2,    0, } ,  
     { -1,    0,    2,   -1,    1, } ,  
     { -2,    0,    2,    2,    1, } ,  
     {  4,    0,    2,    0,    2, } ,  
     {  2,   -1,    0,    0,    0, } ,  
     {  2,    1,    2,   -2,    2, } ,  
     {  0,    1,    2,    1,    2, } ,  
     {  1,    0,    4,   -2,    2, } ,  
     { -1,   -1,    0,    0,    1, } ,  
     {  0,    1,    0,    2,    1, } ,  
     { -2,    0,    2,    4,    1, } ,  
     {  2,    0,    2,    0,    0, } ,  
     {  1,    0,    0,    1,    0, } ,  
     { -1,    0,    0,    4,    1, } ,  
     { -1,    0,    4,    0,    1, } ,  
     {  2,    0,    2,    2,    1, } ,  
     {  0,    0,    2,   -3,    2, } ,  
     { -1,   -2,    0,    2,    0, } ,  
     {  2,    1,    0,    0,    0, } ,  
     {  0,    0,    4,    0,    2, } ,  
     {  0,    0,    0,    0,    3, } ,  
     {  0,    3,    0,    0,    0, } ,  
     {  0,    0,    2,   -4,    1, } ,  
     {  0,   -1,    0,    2,    1, } ,  
     {  0,    0,    0,    4,    1, } ,  
     { -1,   -1,    2,    4,    2, } ,  
     {  1,    0,    2,    4,    2, } ,  
     { -2,    2,    0,    2,    0, } ,  
     { -2,   -1,    2,    0,    1, } ,  
     { -2,    0,    0,    2,    2, } ,  
     { -1,   -1,    2,    0,    2, } ,  
     {  0,    0,    4,   -2,    1, } ,  
     {  3,    0,    2,   -2,    1, } ,  
     { -2,   -1,    0,    2,    1, } ,  
     {  1,    0,    0,   -1,    1, } ,  
     {  0,   -2,    0,    2,    0, } ,  
     { -2,    0,    0,    4,    1, } ,  
     { -3,    0,    0,    0,    1, } ,  
     {  1,    1,    2,    2,    2, } ,  
     {  0,    0,    2,    4,    1, } ,  
     {  3,    0,    2,    2,    2, } ,  
     { -1,    1,    2,   -2,    1, } ,  
     {  2,    0,    0,   -4,    1, } ,  
     {  0,    0,    0,   -2,    2, } ,  
     {  2,    0,    2,   -4,    1, } ,  
     { -1,    1,    0,    2,    1, } ,  
     {  0,    0,    2,   -1,    1, } ,  
     {  0,   -2,    2,    2,    2, } ,  
     {  2,    0,    0,    2,    1, } ,  
     {  4,    0,    2,   -2,    2, } ,  
     {  2,    0,    0,   -2,    2, } ,  
     {  0,    2,    0,    0,    1, } ,  
     {  1,    0,    0,   -4,    1, } ,  
     {  0,    2,    2,   -2,    1, } ,  
     { -3,    0,    0,    4,    0, } ,  
     { -1,    1,    2,    0,    1, } ,  
     { -1,   -1,    0,    4,    0, } ,  
     { -1,   -2,    2,    2,    2, } ,  
     { -2,   -1,    2,    4,    2, } ,  
     {  1,   -1,    2,    2,    1, } ,  
     { -2,    1,    0,    2,    0, } ,  
     { -2,    1,    2,    0,    1, } ,  
     {  2,    1,    0,   -2,    1, } ,  
     { -3,    0,    2,    0,    1, } ,  
     { -2,    0,    2,   -2,    1, } ,  
     { -1,    1,    0,    2,    2, } ,  
     {  0,   -1,    2,   -1,    2, } ,  
     { -1,    0,    4,   -2,    2, } ,  
     {  0,   -2,    2,    0,    2, } ,  
     { -1,    0,    2,    1,    2, } ,  
     {  2,    0,    0,    0,    2, } ,  
     {  0,    0,    2,    0,    3, } ,  
     { -2,    0,    4,    0,    2, } ,  
     { -1,    0,   -2,    0,    1, } ,  
     { -1,    1,    2,    2,    1, } ,  
     {  3,    0,    0,    0,    1, } ,  
     { -1,    0,    2,    3,    2, } ,  
     {  2,   -1,    2,    0,    1, } ,  
     {  0,    1,    2,    2,    1, } ,  
     {  0,   -1,    2,    4,    2, } ,  
     {  2,   -1,    2,    2,    2, } ,  
     {  0,    2,   -2,    2,    0, } ,  
     { -1,   -1,    2,   -1,    1, } ,  
     {  0,   -2,    0,    0,    1, } ,  
     {  1,    0,    2,   -4,    2, } ,  
     {  1,   -1,    0,   -2,    1, } ,  
     { -1,   -1,    2,    0,    1, } ,  
     {  1,   -1,    2,   -2,    2, } ,  
     { -2,   -1,    0,    4,    0, } ,  
     { -1,    0,    0,    3,    0, } ,  
     { -2,   -1,    2,    2,    2, } ,  
     {  0,    2,    2,    0,    2, } ,  
     {  1,    1,    0,    2,    0, } ,  
     {  2,    0,    2,   -1,    2, } ,  
     {  1,    0,    2,    1,    1, } ,  
     {  4,    0,    0,    0,    0, } ,  
     {  2,    1,    2,    0,    1, } ,  
     {  3,   -1,    2,    0,    2, } ,  
     { -2,    2,    0,    2,    1, } ,  
     {  1,    0,    2,   -3,    1, } ,  
     {  1,    1,    2,   -4,    1, } ,  
     { -1,   -1,    2,   -2,    1, } ,  
     {  0,   -1,    0,   -1,    1, } ,  
     {  0,   -1,    0,   -2,    1, } ,  
     { -2,    0,    0,    0,    2, } ,  
     { -2,    0,   -2,    2,    0, } ,  
     { -1,    0,   -2,    4,    0, } ,  
     {  1,   -2,    0,    0,    0, } ,  
     {  0,    1,    0,    1,    1, } ,  
     { -1,    2,    0,    2,    0, } ,  
     {  1,   -1,    2,   -2,    1, } ,  
     {  1,    2,    2,   -2,    2, } ,  
     {  2,   -1,    2,   -2,    2, } ,  
     {  1,    0,    2,   -1,    1, } ,  
     {  2,    1,    2,   -2,    1, } ,  
     { -2,    0,    0,   -2,    1, } ,  
     {  1,   -2,    2,    0,    2, } ,  
     {  0,    1,    2,    1,    1, } ,  
     {  1,    0,    4,   -2,    1, } ,  
     { -2,    0,    4,    2,    2, } ,  
     {  1,    1,    2,    1,    2, } ,  
     {  1,    0,    0,    4,    0, } ,  
     {  1,    0,    2,    2,    0, } ,  
     {  2,    0,    2,    1,    2, } ,  
     {  3,    1,    2,    0,    2, } ,  
     {  4,    0,    2,    0,    1, } ,  
     { -2,   -1,    2,    0,    0, } ,  
     {  0,    1,   -2,    2,    1, } ,  
     {  1,    0,   -2,    1,    0, } ,  
     {  0,   -1,   -2,    2,    1, } ,  
     {  2,   -1,    0,   -2,    1, } ,  
     { -1,    0,    2,   -1,    2, } ,  
     {  1,    0,    2,   -3,    2, } ,  
     {  0,    1,    2,   -2,    3, } ,  
     {  0,    0,    2,   -3,    1, } ,  
     { -1,    0,   -2,    2,    1, } ,  
     {  0,    0,    2,   -4,    2, } ,  
     { -2,    1,    0,    0,    1, } ,  
     { -1,    0,    0,   -1,    1, } ,  
     {  2,    0,    2,   -4,    2, } ,  
     {  0,    0,    4,   -4,    4, } ,  
     {  0,    0,    4,   -4,    2, } ,  
     { -1,   -2,    0,    2,    1, } ,  
     { -2,    0,    0,    3,    0, } ,  
     {  1,    0,   -2,    2,    1, } ,  
     { -3,    0,    2,    2,    2, } ,  
     { -3,    0,    2,    2,    1, } ,  
     { -2,    0,    2,    2,    0, } ,  
     {  2,   -1,    0,    0,    1, } ,  
     { -2,    1,    2,    2,    2, } ,  
     {  1,    1,    0,    1,    0, } ,  
     {  0,    1,    4,   -2,    2, } ,  
     { -1,    1,    0,   -2,    1, } ,  
     {  0,    0,    0,   -4,    1, } ,  
     {  1,   -1,    0,    2,    1, } ,  
     {  1,    1,    0,    2,    1, } ,  
     { -1,    2,    2,    2,    2, } ,  
     {  3,    1,    2,   -2,    2, } ,  
     {  0,   -1,    0,    4,    0, } ,  
     {  2,   -1,    0,    2,    0, } ,  
     {  0,    0,    4,    0,    1, } ,  
     {  2,    0,    4,   -2,    2, } ,  
     { -1,   -1,    2,    4,    1, } ,  
     {  1,    0,    0,    4,    1, } ,  
     {  1,   -2,    2,    2,    2, } ,  
     {  0,    0,    2,    3,    2, } ,  
     { -1,    1,    2,    4,    2, } ,  
     {  3,    0,    0,    2,    0, } ,  
     { -1,    0,    4,    2,    2, } ,  
     {  1,    1,    2,    2,    1, } ,  
     { -2,    0,    2,    6,    2, } ,  
     {  2,    1,    2,    2,    2, } ,  
     { -1,    0,    2,    6,    2, } ,  
     {  1,    0,    2,    4,    1, } ,  
     {  2,    0,    2,    4,    2, } ,  
     {  1,    1,   -2,    1,    0, } ,  
     { -3,    1,    2,    1,    2, } ,  
     {  2,    0,   -2,    0,    2, } ,  
     { -1,    0,    0,    1,    2, } ,  
     { -4,    0,    2,    2,    1, } ,  
     { -1,   -1,    0,    1,    0, } ,  
     {  0,    0,   -2,    2,    2, } ,  
     {  1,    0,    0,   -1,    2, } ,  
     {  0,   -1,    2,   -2,    3, } ,  
     { -2,    1,    2,    0,    0, } ,  
     {  0,    0,    2,   -2,    4, } ,  
     { -2,   -2,    0,    2,    0, } ,  
     { -2,    0,   -2,    4,    0, } ,  
     {  0,   -2,   -2,    2,    0, } ,  
     {  1,    2,    0,   -2,    1, } ,  
     {  3,    0,    0,   -4,    1, } ,  
     { -1,    1,    2,   -2,    2, } ,  
     {  1,   -1,    2,   -4,    1, } ,  
     {  1,    1,    0,   -2,    2, } ,  
     { -3,    0,    2,    0,    0, } ,  
     { -3,    0,    2,    0,    2, } ,  
     { -2,    0,    0,    1,    0, } ,  
     {  0,    0,   -2,    1,    0, } ,  
     { -3,    0,    0,    2,    1, } ,  
     { -1,   -1,   -2,    2,    0, } ,  
     {  0,    1,    2,   -4,    1, } ,  
     {  2,    1,    0,   -4,    1, } ,  
     {  0,    2,    0,   -2,    1, } ,  
     {  1,    0,    0,   -3,    1, } ,  
     { -2,    0,    2,   -2,    2, } ,  
     { -2,   -1,    0,    0,    1, } ,  
     { -4,    0,    0,    2,    0, } ,  
     {  1,    1,    0,   -4,    1, } ,  
     { -1,    0,    2,   -4,    1, } ,  
     {  0,    0,    4,   -4,    1, } ,  
     {  0,    3,    2,   -2,    2, } ,  
     { -3,   -1,    0,    4,    0, } ,  
     { -3,    0,    0,    4,    1, } ,  
     {  1,   -1,   -2,    2,    0, } ,  
     { -1,   -1,    0,    2,    2, } ,  
     {  1,   -2,    0,    0,    1, } ,  
     {  1,   -1,    0,    0,    2, } ,  
     {  0,    0,    0,    1,    2, } ,  
     { -1,   -1,    2,    0,    0, } ,  
     {  1,   -2,    2,   -2,    2, } ,  
     {  0,   -1,    2,   -1,    1, } ,  
     { -1,    0,    2,    0,    3, } ,  
     {  1,    1,    0,    0,    2, } ,  
     { -1,    1,    2,    0,    0, } ,  
     {  1,    2,    0,    0,    0, } ,  
     { -1,    2,    2,    0,    2, } ,  
     { -1,    0,    4,   -2,    1, } ,  
     {  3,    0,    2,   -4,    2, } ,  
     {  1,    2,    2,   -2,    1, } ,  
     {  1,    0,    4,   -4,    2, } ,  
     { -2,   -1,    0,    4,    1, } ,  
     {  0,   -1,    0,    2,    2, } ,  
     { -2,    1,    0,    4,    0, } ,  
     { -2,   -1,    2,    2,    1, } ,  
     {  2,    0,   -2,    2,    0, } ,  
     {  1,    0,    0,    1,    1, } ,  
     {  0,    1,    0,    2,    2, } ,  
     {  1,   -1,    2,   -1,    2, } ,  
     { -2,    0,    4,    0,    1, } ,  
     {  2,    1,    0,    0,    1, } ,  
     {  0,    1,    2,    0,    0, } ,  
     {  0,   -1,    4,   -2,    2, } ,  
     {  0,    0,    4,   -2,    4, } ,  
     {  0,    2,    2,    0,    1, } ,  
     { -3,    0,    0,    6,    0, } ,  
     { -1,   -1,    0,    4,    1, } ,  
     {  1,   -2,    0,    2,    0, } ,  
     { -1,    0,    0,    4,    2, } ,  
     { -1,   -2,    2,    2,    1, } ,  
     { -1,    0,    0,   -2,    2, } ,  
     {  1,    0,   -2,   -2,    1, } ,  
     {  0,    0,   -2,   -2,    1, } ,  
     { -2,    0,   -2,    0,    1, } ,  
     {  0,    0,    0,    3,    1, } ,  
     {  0,    0,    0,    3,    0, } ,  
     { -1,    1,    0,    4,    0, } ,  
     { -1,   -1,    2,    2,    0, } ,  
     { -2,    0,    2,    3,    2, } ,  
     {  1,    0,    0,    2,    2, } ,  
     {  0,   -1,    2,    1,    2, } ,  
     {  3,   -1,    0,    0,    0, } ,  
     {  2,    0,    0,    1,    0, } ,  
     {  1,   -1,    2,    0,    0, } ,  
     {  0,    0,    2,    1,    0, } ,  
     {  1,    0,    2,    0,    3, } ,  
     {  3,    1,    0,    0,    0, } ,  
     {  3,   -1,    2,   -2,    2, } ,  
     {  2,    0,    2,   -1,    1, } ,  
     {  1,    1,    2,    0,    0, } ,  
     {  0,    0,    4,   -1,    2, } ,  
     {  1,    2,    2,    0,    2, } ,  
     { -2,    0,    0,    6,    0, } ,  
     {  0,   -1,    0,    4,    1, } ,  
     { -2,   -1,    2,    4,    1, } ,  
     {  0,   -2,    2,    2,    1, } ,  
     {  0,   -1,    2,    2,    0, } ,  
     { -1,    0,    2,    3,    1, } ,  
     { -2,    1,    2,    4,    2, } ,  
     {  2,    0,    0,    2,    2, } ,  
     {  2,   -2,    2,    0,    2, } ,  
     { -1,    1,    2,    3,    2, } ,  
     {  3,    0,    2,   -1,    2, } ,  
     {  4,    0,    2,   -2,    1, } ,  
     { -1,    0,    0,    6,    0, } ,  
     { -1,   -2,    2,    4,    2, } ,  
     { -3,    0,    2,    6,    2, } ,  
     { -1,    0,    2,    4,    0, } ,  
     {  3,    0,    0,    2,    1, } ,  
     {  3,   -1,    2,    0,    1, } ,  
     {  3,    0,    2,    0,    0, } ,  
     {  1,    0,    4,    0,    2, } ,  
     {  5,    0,    2,   -2,    2, } ,  
     {  0,   -1,    2,    4,    1, } ,  
     {  2,   -1,    2,    2,    1, } ,  
     {  0,    1,    2,    4,    2, } ,  
     {  1,   -1,    2,    4,    2, } ,  
     {  3,   -1,    2,    2,    2, } ,  
     {  3,    0,    2,    2,    1, } ,  
     {  5,    0,    2,    0,    2, } ,  
     {  0,    0,    2,    6,    2, } ,  
     {  4,    0,    2,    2,    2, } ,  
     {  0,   -1,    1,   -1,    1, } ,  
     { -1,    0,    1,    0,    3, } ,  
     {  0,   -2,    2,   -2,    3, } ,  
     {  1,    0,   -1,    0,    1, } ,  
     {  2,   -2,    0,   -2,    1, } ,  
     { -1,    0,    1,    0,    2, } ,  
     { -1,    0,    1,    0,    1, } ,  
     { -1,   -1,    2,   -1,    2, } ,  
     { -2,    2,    0,    2,    2, } ,  
     { -1,    0,    1,    0,    0, } ,  
     { -4,    1,    2,    2,    2, } ,  
     { -3,    0,    2,    1,    1, } ,  
     { -2,   -1,    2,    0,    2, } ,  
     {  1,    0,   -2,    1,    1, } ,  
     {  2,   -1,   -2,    0,    1, } ,  
     { -4,    0,    2,    2,    0, } ,  
     { -3,    1,    0,    3,    0, } ,  
     { -1,    0,   -1,    2,    0, } ,  
     {  0,   -2,    0,    0,    2, } ,  
     {  0,   -2,    0,    0,    2, } ,  
     { -3,    0,    0,    3,    0, } ,  
     { -2,   -1,    0,    2,    2, } ,  
     { -1,    0,   -2,    3,    0, } ,  
     { -4,    0,    0,    4,    0, } ,  
     {  2,    1,   -2,    0,    1, } ,  
     {  2,   -1,    0,   -2,    2, } ,  
     {  0,    0,    1,   -1,    0, } ,  
     { -1,    2,    0,    1,    0, } ,  
     { -2,    1,    2,    0,    2, } ,  
     {  1,    1,    0,   -1,    1, } ,  
     {  1,    0,    1,   -2,    1, } ,  
     {  0,    2,    0,    0,    2, } ,  
     {  1,   -1,    2,   -3,    1, } ,  
     { -1,    1,    2,   -1,    1, } ,  
     { -2,    0,    4,   -2,    2, } ,  
     { -2,    0,    4,   -2,    1, } ,  
     { -2,   -2,    0,    2,    1, } ,  
     { -2,    0,   -2,    4,    0, } ,  
     {  1,    2,    2,   -4,    1, } ,  
     {  1,    1,    2,   -4,    2, } ,  
     { -1,    2,    2,   -2,    1, } ,  
     {  2,    0,    0,   -3,    1, } ,  
     { -1,    2,    0,    0,    1, } ,  
     {  0,    0,    0,   -2,    0, } ,  
     { -1,   -1,    2,   -2,    2, } ,  
     { -1,    1,    0,    0,    2, } ,  
     {  0,    0,    0,   -1,    2, } ,  
     { -2,    1,    0,    1,    0, } ,  
     {  1,   -2,    0,   -2,    1, } ,  
     {  1,    0,   -2,    0,    2, } ,  
     { -3,    1,    0,    2,    0, } ,  
     { -1,    1,   -2,    2,    0, } ,  
     { -1,   -1,    0,    0,    2, } ,  
     { -3,    0,    0,    2,    0, } ,  
     { -3,   -1,    0,    2,    0, } ,  
     {  2,    0,    2,   -6,    1, } ,  
     {  0,    1,    2,   -4,    2, } ,  
     {  2,    0,    0,   -4,    2, } ,  
     { -2,    1,    2,   -2,    1, } ,  
     {  0,   -1,    2,   -4,    1, } ,  
     {  0,    1,    0,   -2,    2, } ,  
     { -1,    0,    0,   -2,    0, } ,  
     {  2,    0,   -2,   -2,    1, } ,  
     { -4,    0,    2,    0,    1, } ,  
     { -1,   -1,    0,   -1,    1, } ,  
     {  0,    0,   -2,    0,    2, } ,  
     { -3,    0,    0,    1,    0, } ,  
     { -1,    0,   -2,    1,    0, } ,  
     { -2,    0,   -2,    2,    1, } ,  
     {  0,    0,   -4,    2,    0, } ,  
     { -2,   -1,   -2,    2,    0, } ,  
     {  1,    0,    2,   -6,    1, } ,  
     { -1,    0,    2,   -4,    2, } ,  
     {  1,    0,    0,   -4,    2, } ,  
     {  2,    1,    2,   -4,    2, } ,  
     {  2,    1,    2,   -4,    1, } ,  
     {  0,    1,    4,   -4,    4, } ,  
     {  0,    1,    4,   -4,    2, } ,  
     { -1,   -1,   -2,    4,    0, } ,  
     { -1,   -3,    0,    2,    0, } ,  
     { -1,    0,   -2,    4,    1, } ,  
     { -2,   -1,    0,    3,    0, } ,  
     {  0,    0,   -2,    3,    0, } ,  
     { -2,    0,    0,    3,    1, } ,  
     {  0,   -1,    0,    1,    0, } ,  
     { -3,    0,    2,    2,    0, } ,  
     {  1,    1,   -2,    2,    0, } ,  
     { -1,    1,    0,    2,    2, } ,  
     {  1,   -2,    2,   -2,    1, } ,  
     {  0,    0,    1,    0,    2, } ,  
     {  0,    0,    1,    0,    1, } ,  
     {  0,    0,    1,    0,    0, } ,  
     { -1,    2,    0,    2,    1, } ,  
     {  0,    0,    2,    0,    2, } ,  
     { -2,    0,    2,    0,    2, } ,  
     {  2,    0,    0,   -1,    1, } ,  
     {  3,    0,    0,   -2,    1, } ,  
     {  1,    0,    2,   -2,    3, } ,  
     {  1,    2,    0,    0,    1, } ,  
     {  2,    0,    2,   -3,    2, } ,  
     { -1,    1,    4,   -2,    2, } ,  
     { -2,   -2,    0,    4,    0, } ,  
     {  0,   -3,    0,    2,    0, } ,  
     {  0,    0,   -2,    4,    0, } ,  
     { -1,   -1,    0,    3,    0, } ,  
     { -2,    0,    0,    4,    2, } ,  
     { -1,    0,    0,    3,    1, } ,  
     {  2,   -2,    0,    0,    0, } ,  
     {  1,   -1,    0,    1,    0, } ,  
     { -1,    0,    0,    2,    0, } ,  
     {  0,   -2,    2,    0,    1, } ,  
     { -1,    0,    1,    2,    1, } ,  
     { -1,    1,    0,    3,    0, } ,  
     { -1,   -1,    2,    1,    2, } ,  
     {  0,   -1,    2,    0,    0, } ,  
     { -2,    1,    2,    2,    1, } ,  
     {  2,   -2,    2,   -2,    2, } ,  
     {  1,    1,    0,    1,    1, } ,  
     {  1,    0,    1,    0,    1, } ,  
     {  1,    0,    1,    0,    0, } ,  
     {  0,    2,    0,    2,    0, } ,  
     {  2,   -1,    2,   -2,    1, } ,  
     {  0,   -1,    4,   -2,    1, } ,  
     {  0,    0,    4,   -2,    3, } ,  
     {  0,    1,    4,   -2,    1, } ,  
     {  4,    0,    2,   -4,    2, } ,  
     {  2,    2,    2,   -2,    2, } ,  
     {  2,    0,    4,   -4,    2, } ,  
     { -1,   -2,    0,    4,    0, } ,  
     { -1,   -3,    2,    2,    2, } ,  
     { -3,    0,    2,    4,    2, } ,  
     { -3,    0,    2,   -2,    1, } ,  
     { -1,   -1,    0,   -2,    1, } ,  
     { -3,    0,    0,    0,    2, } ,  
     { -3,    0,   -2,    2,    0, } ,  
     {  0,    1,    0,   -4,    1, } ,  
     { -2,    1,    0,   -2,    1, } ,  
     { -4,    0,    0,    0,    1, } ,  
     { -1,    0,    0,   -4,    1, } ,  
     { -3,    0,    0,   -2,    1, } ,  
     {  0,    0,    0,    3,    2, } ,  
     { -1,    1,    0,    4,    1, } ,  
     {  1,   -2,    2,    0,    1, } ,  
     {  0,    1,    0,    3,    0, } ,  
     { -1,    0,    2,    2,    3, } ,  
     {  0,    0,    2,    2,    2, } ,  
     { -2,    0,    2,    2,    2, } ,  
     { -1,    1,    2,    2,    0, } ,  
     {  3,    0,    0,    0,    2, } ,  
     {  2,    1,    0,    1,    0, } ,  
     {  2,   -1,    2,   -1,    2, } ,  
     {  0,    0,    2,    0,    1, } ,  
     {  0,    0,    3,    0,    3, } ,  
     {  0,    0,    3,    0,    2, } ,  
     { -1,    2,    2,    2,    1, } ,  
     { -1,    0,    4,    0,    0, } ,  
     {  1,    2,    2,    0,    1, } ,  
     {  3,    1,    2,   -2,    1, } ,  
     {  1,    1,    4,   -2,    2, } ,  
     { -2,   -1,    0,    6,    0, } ,  
     {  0,   -2,    0,    4,    0, } ,  
     { -2,    0,    0,    6,    1, } ,  
     { -2,   -2,    2,    4,    2, } ,  
     {  0,   -3,    2,    2,    2, } ,  
     {  0,    0,    0,    4,    2, } ,  
     { -1,   -1,    2,    3,    2, } ,  
     { -2,    0,    2,    4,    0, } ,  
     {  2,   -1,    0,    2,    1, } ,  
     {  1,    0,    0,    3,    0, } ,  
     {  0,    1,    0,    4,    1, } ,  
     {  0,    1,    0,    4,    0, } ,  
     {  1,   -1,    2,    1,    2, } ,  
     {  0,    0,    2,    2,    3, } ,  
     {  1,    0,    2,    2,    2, } ,  
     { -1,    0,    2,    2,    2, } ,  
     { -2,    0,    4,    2,    1, } ,  
     {  2,    1,    0,    2,    1, } ,  
     {  2,    1,    0,    2,    0, } ,  
     {  2,   -1,    2,    0,    0, } ,  
     {  1,    0,    2,    1,    0, } ,  
     {  0,    1,    2,    2,    0, } ,  
     {  2,    0,    2,    0,    3, } ,  
     {  3,    0,    2,    0,    2, } ,  
     {  1,    0,    2,    0,    2, } ,  
     {  1,    0,    3,    0,    3, } ,  
     {  1,    1,    2,    1,    1, } ,  
     {  0,    2,    2,    2,    2, } ,  
     {  2,    1,    2,    0,    0, } ,  
     {  2,    0,    4,   -2,    1, } ,  
     {  4,    1,    2,   -2,    2, } ,  
     { -1,   -1,    0,    6,    0, } ,  
     { -3,   -1,    2,    6,    2, } ,  
     { -1,    0,    0,    6,    1, } ,  
     { -3,    0,    2,    6,    1, } ,  
     {  1,   -1,    0,    4,    1, } ,  
     {  1,   -1,    0,    4,    0, } ,  
     { -2,    0,    2,    5,    2, } ,  
     {  1,   -2,    2,    2,    1, } ,  
     {  3,   -1,    0,    2,    0, } ,  
     {  1,   -1,    2,    2,    0, } ,  
     {  0,    0,    2,    3,    1, } ,  
     { -1,    1,    2,    4,    1, } ,  
     {  0,    1,    2,    3,    2, } ,  
     { -1,    0,    4,    2,    1, } ,  
     {  2,    0,    2,    1,    1, } ,  
     {  5,    0,    0,    0,    0, } ,  
     {  2,    1,    2,    1,    2, } ,  
     {  1,    0,    4,    0,    1, } ,  
     {  3,    1,    2,    0,    1, } ,  
     {  3,    0,    4,   -2,    2, } ,  
     { -2,   -1,    2,    6,    2, } ,  
     {  0,    0,    0,    6,    0, } ,  
     {  0,   -2,    2,    4,    2, } ,  
     { -2,    0,    2,    6,    1, } ,  
     {  2,    0,    0,    4,    1, } ,  
     {  2,    0,    0,    4,    0, } ,  
     {  2,   -2,    2,    2,    2, } ,  
     {  0,    0,    2,    4,    0, } ,  
     {  1,    0,    2,    3,    2, } ,  
     {  4,    0,    0,    2,    0, } ,  
     {  2,    0,    2,    2,    0, } ,  
     {  0,    0,    4,    2,    2, } ,  
     {  4,   -1,    2,    0,    2, } ,  
     {  3,    0,    2,    1,    2, } ,  
     {  2,    1,    2,    2,    1, } ,  
     {  4,    1,    2,    0,    2, } ,  
     { -1,   -1,    2,    6,    2, } ,  
     { -1,    0,    2,    6,    1, } ,  
     {  1,   -1,    2,    4,    1, } ,  
     {  1,    1,    2,    4,    2, } ,  
     {  3,    1,    2,    2,    2, } ,  
     {  5,    0,    2,    0,    1, } ,  
     {  2,   -1,    2,    4,    2, } ,  
     {  2,    0,    2,    4,    1, } ,  
  
    } ;  
  
/*  
 * Longitude and obliquity coefficients  
 */  
  
    static const double CLS[ NLS ][6] = {  
  
/*  
 *  Luni-Solar nutation coefficients, unit 1e-7 arcsec  
 * longitude (sin, t*sin, cos), obliquity (cos, t*cos, sin)  
 */  
  
     { -172064161e0, -174666e0, 33386e0, 92052331e0, 9086e0, 15377e0, } ,  
     {  -13170906e0,  -1675e0, -13696e0, 5730336e0, -3015e0, -4587e0, } ,  
     {   -2276413e0,   -234e0,  2796e0,  978459e0, -485e0, 1374e0, } ,  
     {    2074554e0,    207e0,  -698e0, -897492e0,  470e0, -291e0, } ,  
     {    1475877e0,  -3633e0, 11817e0,   73871e0, -184e0, -1924e0, } ,  
     {    -516821e0,   1226e0,  -524e0,  224386e0, -677e0, -174e0, } ,  
     {     711159e0,     73e0,  -872e0,   -6750e0,    0e0,  358e0, } ,  
     {    -387298e0,   -367e0,   380e0,  200728e0,   18e0,  318e0, } ,  
     {    -301461e0,    -36e0,   816e0,  129025e0,  -63e0,  367e0, } ,  
     {     215829e0,   -494e0,   111e0,  -95929e0,  299e0,  132e0, } ,  
     {     128227e0,    137e0,   181e0,  -68982e0,   -9e0,   39e0, } ,  
     {     123457e0,     11e0,    19e0,  -53311e0,   32e0,   -4e0, } ,  
     {     156994e0,     10e0,  -168e0,   -1235e0,    0e0,   82e0, } ,  
     {      63110e0,     63e0,    27e0,  -33228e0,    0e0,   -9e0, } ,  
     {     -57976e0,    -63e0,  -189e0,   31429e0,    0e0,  -75e0, } ,  
     {     -59641e0,    -11e0,   149e0,   25543e0,  -11e0,   66e0, } ,  
     {     -51613e0,    -42e0,   129e0,   26366e0,    0e0,   78e0, } ,  
     {      45893e0,     50e0,    31e0,  -24236e0,  -10e0,   20e0, } ,  
     {      63384e0,     11e0,  -150e0,   -1220e0,    0e0,   29e0, } ,  
     {     -38571e0,     -1e0,   158e0,   16452e0,  -11e0,   68e0, } ,  
     {      32481e0,      0e0,     0e0,  -13870e0,    0e0,    0e0, } ,  
     {     -47722e0,      0e0,   -18e0,     477e0,    0e0,  -25e0, } ,  
     {     -31046e0,     -1e0,   131e0,   13238e0,  -11e0,   59e0, } ,  
     {      28593e0,      0e0,    -1e0,  -12338e0,   10e0,   -3e0, } ,  
     {      20441e0,     21e0,    10e0,  -10758e0,    0e0,   -3e0, } ,  
     {      29243e0,      0e0,   -74e0,    -609e0,    0e0,   13e0, } ,  
     {      25887e0,      0e0,   -66e0,    -550e0,    0e0,   11e0, } ,  
     {     -14053e0,    -25e0,    79e0,    8551e0,   -2e0,  -45e0, } ,  
     {      15164e0,     10e0,    11e0,   -8001e0,    0e0,   -1e0, } ,  
     {     -15794e0,     72e0,   -16e0,    6850e0,  -42e0,   -5e0, } ,  
     {      21783e0,      0e0,    13e0,    -167e0,    0e0,   13e0, } ,  
     {     -12873e0,    -10e0,   -37e0,    6953e0,    0e0,  -14e0, } ,  
     {     -12654e0,     11e0,    63e0,    6415e0,    0e0,   26e0, } ,  
     {     -10204e0,      0e0,    25e0,    5222e0,    0e0,   15e0, } ,  
     {      16707e0,    -85e0,   -10e0,     168e0,   -1e0,   10e0, } ,  
     {      -7691e0,      0e0,    44e0,    3268e0,    0e0,   19e0, } ,  
     {     -11024e0,      0e0,   -14e0,     104e0,    0e0,    2e0, } ,  
     {       7566e0,    -21e0,   -11e0,   -3250e0,    0e0,   -5e0, } ,  
     {      -6637e0,    -11e0,    25e0,    3353e0,    0e0,   14e0, } ,  
     {      -7141e0,     21e0,     8e0,    3070e0,    0e0,    4e0, } ,  
     {      -6302e0,    -11e0,     2e0,    3272e0,    0e0,    4e0, } ,  
     {       5800e0,     10e0,     2e0,   -3045e0,    0e0,   -1e0, } ,  
     {       6443e0,      0e0,    -7e0,   -2768e0,    0e0,   -4e0, } ,  
     {      -5774e0,    -11e0,   -15e0,    3041e0,    0e0,   -5e0, } ,  
     {      -5350e0,      0e0,    21e0,    2695e0,    0e0,   12e0, } ,  
     {      -4752e0,    -11e0,    -3e0,    2719e0,    0e0,   -3e0, } ,  
     {      -4940e0,    -11e0,   -21e0,    2720e0,    0e0,   -9e0, } ,  
     {       7350e0,      0e0,    -8e0,     -51e0,    0e0,    4e0, } ,  
     {       4065e0,      0e0,     6e0,   -2206e0,    0e0,    1e0, } ,  
     {       6579e0,      0e0,   -24e0,    -199e0,    0e0,    2e0, } ,  
     {       3579e0,      0e0,     5e0,   -1900e0,    0e0,    1e0, } ,  
     {       4725e0,      0e0,    -6e0,     -41e0,    0e0,    3e0, } ,  
     {      -3075e0,      0e0,    -2e0,    1313e0,    0e0,   -1e0, } ,  
     {      -2904e0,      0e0,    15e0,    1233e0,    0e0,    7e0, } ,  
     {       4348e0,      0e0,   -10e0,     -81e0,    0e0,    2e0, } ,  
     {      -2878e0,      0e0,     8e0,    1232e0,    0e0,    4e0, } ,  
     {      -4230e0,      0e0,     5e0,     -20e0,    0e0,   -2e0, } ,  
     {      -2819e0,      0e0,     7e0,    1207e0,    0e0,    3e0, } ,  
     {      -4056e0,      0e0,     5e0,      40e0,    0e0,   -2e0, } ,  
     {      -2647e0,      0e0,    11e0,    1129e0,    0e0,    5e0, } ,  
     {      -2294e0,      0e0,   -10e0,    1266e0,    0e0,   -4e0, } ,  
     {       2481e0,      0e0,    -7e0,   -1062e0,    0e0,   -3e0, } ,  
     {       2179e0,      0e0,    -2e0,   -1129e0,    0e0,   -2e0, } ,  
     {       3276e0,      0e0,     1e0,      -9e0,    0e0,    0e0, } ,  
     {      -3389e0,      0e0,     5e0,      35e0,    0e0,   -2e0, } ,  
     {       3339e0,      0e0,   -13e0,    -107e0,    0e0,    1e0, } ,  
     {      -1987e0,      0e0,    -6e0,    1073e0,    0e0,   -2e0, } ,  
     {      -1981e0,      0e0,     0e0,     854e0,    0e0,    0e0, } ,  
     {       4026e0,      0e0,  -353e0,    -553e0,    0e0, -139e0, } ,  
     {       1660e0,      0e0,    -5e0,    -710e0,    0e0,   -2e0, } ,  
     {      -1521e0,      0e0,     9e0,     647e0,    0e0,    4e0, } ,  
     {       1314e0,      0e0,     0e0,    -700e0,    0e0,    0e0, } ,  
     {      -1283e0,      0e0,     0e0,     672e0,    0e0,    0e0, } ,  
     {      -1331e0,      0e0,     8e0,     663e0,    0e0,    4e0, } ,  
     {       1383e0,      0e0,    -2e0,    -594e0,    0e0,   -2e0, } ,  
     {       1405e0,      0e0,     4e0,    -610e0,    0e0,    2e0, } ,  
     {       1290e0,      0e0,     0e0,    -556e0,    0e0,    0e0, } ,  
     {      -1214e0,      0e0,     5e0,     518e0,    0e0,    2e0, } ,  
     {       1146e0,      0e0,    -3e0,    -490e0,    0e0,   -1e0, } ,  
     {       1019e0,      0e0,    -1e0,    -527e0,    0e0,   -1e0, } ,  
     {      -1100e0,      0e0,     9e0,     465e0,    0e0,    4e0, } ,  
     {       -970e0,      0e0,     2e0,     496e0,    0e0,    1e0, } ,  
     {       1575e0,      0e0,    -6e0,     -50e0,    0e0,    0e0, } ,  
     {        934e0,      0e0,    -3e0,    -399e0,    0e0,   -1e0, } ,  
     {        922e0,      0e0,    -1e0,    -395e0,    0e0,   -1e0, } ,  
     {        815e0,      0e0,    -1e0,    -422e0,    0e0,   -1e0, } ,  
     {        834e0,      0e0,     2e0,    -440e0,    0e0,    1e0, } ,  
     {       1248e0,      0e0,     0e0,    -170e0,    0e0,    1e0, } ,  
     {       1338e0,      0e0,    -5e0,     -39e0,    0e0,    0e0, } ,  
     {        716e0,      0e0,    -2e0,    -389e0,    0e0,   -1e0, } ,  
     {       1282e0,      0e0,    -3e0,     -23e0,    0e0,    1e0, } ,  
     {        742e0,      0e0,     1e0,    -391e0,    0e0,    0e0, } ,  
     {       1020e0,      0e0,   -25e0,    -495e0,    0e0,  -10e0, } ,  
     {        715e0,      0e0,    -4e0,    -326e0,    0e0,    2e0, } ,  
     {       -666e0,      0e0,    -3e0,     369e0,    0e0,   -1e0, } ,  
     {       -667e0,      0e0,     1e0,     346e0,    0e0,    1e0, } ,  
     {       -704e0,      0e0,     0e0,     304e0,    0e0,    0e0, } ,  
     {       -694e0,      0e0,     5e0,     294e0,    0e0,    2e0, } ,  
     {      -1014e0,      0e0,    -1e0,       4e0,    0e0,   -1e0, } ,  
     {       -585e0,      0e0,    -2e0,     316e0,    0e0,   -1e0, } ,  
     {       -949e0,      0e0,     1e0,       8e0,    0e0,   -1e0, } ,  
     {       -595e0,      0e0,     0e0,     258e0,    0e0,    0e0, } ,  
     {        528e0,      0e0,     0e0,    -279e0,    0e0,    0e0, } ,  
     {       -590e0,      0e0,     4e0,     252e0,    0e0,    2e0, } ,  
     {        570e0,      0e0,    -2e0,    -244e0,    0e0,   -1e0, } ,  
     {       -502e0,      0e0,     3e0,     250e0,    0e0,    2e0, } ,  
     {       -875e0,      0e0,     1e0,      29e0,    0e0,    0e0, } ,  
     {       -492e0,      0e0,    -3e0,     275e0,    0e0,   -1e0, } ,  
     {        535e0,      0e0,    -2e0,    -228e0,    0e0,   -1e0, } ,  
     {       -467e0,      0e0,     1e0,     240e0,    0e0,    1e0, } ,  
     {        591e0,      0e0,     0e0,    -253e0,    0e0,    0e0, } ,  
     {       -453e0,      0e0,    -1e0,     244e0,    0e0,   -1e0, } ,  
     {        766e0,      0e0,     1e0,       9e0,    0e0,    0e0, } ,  
     {       -446e0,      0e0,     2e0,     225e0,    0e0,    1e0, } ,  
     {       -488e0,      0e0,     2e0,     207e0,    0e0,    1e0, } ,  
     {       -468e0,      0e0,     0e0,     201e0,    0e0,    0e0, } ,  
     {       -421e0,      0e0,     1e0,     216e0,    0e0,    1e0, } ,  
     {        463e0,      0e0,     0e0,    -200e0,    0e0,    0e0, } ,  
     {       -673e0,      0e0,     2e0,      14e0,    0e0,    0e0, } ,  
     {        658e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {       -438e0,      0e0,     0e0,     188e0,    0e0,    0e0, } ,  
     {       -390e0,      0e0,     0e0,     205e0,    0e0,    0e0, } ,  
     {        639e0,    -11e0,    -2e0,     -19e0,    0e0,    0e0, } ,  
     {        412e0,      0e0,    -2e0,    -176e0,    0e0,   -1e0, } ,  
     {       -361e0,      0e0,     0e0,     189e0,    0e0,    0e0, } ,  
     {        360e0,      0e0,    -1e0,    -185e0,    0e0,   -1e0, } ,  
     {        588e0,      0e0,    -3e0,     -24e0,    0e0,    0e0, } ,  
     {       -578e0,      0e0,     1e0,       5e0,    0e0,    0e0, } ,  
     {       -396e0,      0e0,     0e0,     171e0,    0e0,    0e0, } ,  
     {        565e0,      0e0,    -1e0,      -6e0,    0e0,    0e0, } ,  
     {       -335e0,      0e0,    -1e0,     184e0,    0e0,   -1e0, } ,  
     {        357e0,      0e0,     1e0,    -154e0,    0e0,    0e0, } ,  
     {        321e0,      0e0,     1e0,    -174e0,    0e0,    0e0, } ,  
     {       -301e0,      0e0,    -1e0,     162e0,    0e0,    0e0, } ,  
     {       -334e0,      0e0,     0e0,     144e0,    0e0,    0e0, } ,  
     {        493e0,      0e0,    -2e0,     -15e0,    0e0,    0e0, } ,  
     {        494e0,      0e0,    -2e0,     -19e0,    0e0,    0e0, } ,  
     {        337e0,      0e0,    -1e0,    -143e0,    0e0,   -1e0, } ,  
     {        280e0,      0e0,    -1e0,    -144e0,    0e0,    0e0, } ,  
     {        309e0,      0e0,     1e0,    -134e0,    0e0,    0e0, } ,  
     {       -263e0,      0e0,     2e0,     131e0,    0e0,    1e0, } ,  
     {        253e0,      0e0,     1e0,    -138e0,    0e0,    0e0, } ,  
     {        245e0,      0e0,     0e0,    -128e0,    0e0,    0e0, } ,  
     {        416e0,      0e0,    -2e0,     -17e0,    0e0,    0e0, } ,  
     {       -229e0,      0e0,     0e0,     128e0,    0e0,    0e0, } ,  
     {        231e0,      0e0,     0e0,    -120e0,    0e0,    0e0, } ,  
     {       -259e0,      0e0,     2e0,     109e0,    0e0,    1e0, } ,  
     {        375e0,      0e0,    -1e0,      -8e0,    0e0,    0e0, } ,  
     {        252e0,      0e0,     0e0,    -108e0,    0e0,    0e0, } ,  
     {       -245e0,      0e0,     1e0,     104e0,    0e0,    0e0, } ,  
     {        243e0,      0e0,    -1e0,    -104e0,    0e0,    0e0, } ,  
     {        208e0,      0e0,     1e0,    -112e0,    0e0,    0e0, } ,  
     {        199e0,      0e0,     0e0,    -102e0,    0e0,    0e0, } ,  
     {       -208e0,      0e0,     1e0,     105e0,    0e0,    0e0, } ,  
     {        335e0,      0e0,    -2e0,     -14e0,    0e0,    0e0, } ,  
     {       -325e0,      0e0,     1e0,       7e0,    0e0,    0e0, } ,  
     {       -187e0,      0e0,     0e0,      96e0,    0e0,    0e0, } ,  
     {        197e0,      0e0,    -1e0,    -100e0,    0e0,    0e0, } ,  
     {       -192e0,      0e0,     2e0,      94e0,    0e0,    1e0, } ,  
     {       -188e0,      0e0,     0e0,      83e0,    0e0,    0e0, } ,  
     {        276e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {       -286e0,      0e0,     1e0,       6e0,    0e0,    0e0, } ,  
     {        186e0,      0e0,    -1e0,     -79e0,    0e0,    0e0, } ,  
     {       -219e0,      0e0,     0e0,      43e0,    0e0,    0e0, } ,  
     {        276e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {       -153e0,      0e0,    -1e0,      84e0,    0e0,    0e0, } ,  
     {       -156e0,      0e0,     0e0,      81e0,    0e0,    0e0, } ,  
     {       -154e0,      0e0,     1e0,      78e0,    0e0,    0e0, } ,  
     {       -174e0,      0e0,     1e0,      75e0,    0e0,    0e0, } ,  
     {       -163e0,      0e0,     2e0,      69e0,    0e0,    1e0, } ,  
     {       -228e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         91e0,      0e0,    -4e0,     -54e0,    0e0,   -2e0, } ,  
     {        175e0,      0e0,     0e0,     -75e0,    0e0,    0e0, } ,  
     {       -159e0,      0e0,     0e0,      69e0,    0e0,    0e0, } ,  
     {        141e0,      0e0,     0e0,     -72e0,    0e0,    0e0, } ,  
     {        147e0,      0e0,     0e0,     -75e0,    0e0,    0e0, } ,  
     {       -132e0,      0e0,     0e0,      69e0,    0e0,    0e0, } ,  
     {        159e0,      0e0,   -28e0,     -54e0,    0e0,   11e0, } ,  
     {        213e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {        123e0,      0e0,     0e0,     -64e0,    0e0,    0e0, } ,  
     {       -118e0,      0e0,    -1e0,      66e0,    0e0,    0e0, } ,  
     {        144e0,      0e0,    -1e0,     -61e0,    0e0,    0e0, } ,  
     {       -121e0,      0e0,     1e0,      60e0,    0e0,    0e0, } ,  
     {       -134e0,      0e0,     1e0,      56e0,    0e0,    1e0, } ,  
     {       -105e0,      0e0,     0e0,      57e0,    0e0,    0e0, } ,  
     {       -102e0,      0e0,     0e0,      56e0,    0e0,    0e0, } ,  
     {        120e0,      0e0,     0e0,     -52e0,    0e0,    0e0, } ,  
     {        101e0,      0e0,     0e0,     -54e0,    0e0,    0e0, } ,  
     {       -113e0,      0e0,     0e0,      59e0,    0e0,    0e0, } ,  
     {       -106e0,      0e0,     0e0,      61e0,    0e0,    0e0, } ,  
     {       -129e0,      0e0,     1e0,      55e0,    0e0,    0e0, } ,  
     {       -114e0,      0e0,     0e0,      57e0,    0e0,    0e0, } ,  
     {        113e0,      0e0,    -1e0,     -49e0,    0e0,    0e0, } ,  
     {       -102e0,      0e0,     0e0,      44e0,    0e0,    0e0, } ,  
     {        -94e0,      0e0,     0e0,      51e0,    0e0,    0e0, } ,  
     {       -100e0,      0e0,    -1e0,      56e0,    0e0,    0e0, } ,  
     {         87e0,      0e0,     0e0,     -47e0,    0e0,    0e0, } ,  
     {        161e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         96e0,      0e0,     0e0,     -50e0,    0e0,    0e0, } ,  
     {        151e0,      0e0,    -1e0,      -5e0,    0e0,    0e0, } ,  
     {       -104e0,      0e0,     0e0,      44e0,    0e0,    0e0, } ,  
     {       -110e0,      0e0,     0e0,      48e0,    0e0,    0e0, } ,  
     {       -100e0,      0e0,     1e0,      50e0,    0e0,    0e0, } ,  
     {         92e0,      0e0,    -5e0,      12e0,    0e0,   -2e0, } ,  
     {         82e0,      0e0,     0e0,     -45e0,    0e0,    0e0, } ,  
     {         82e0,      0e0,     0e0,     -45e0,    0e0,    0e0, } ,  
     {        -78e0,      0e0,     0e0,      41e0,    0e0,    0e0, } ,  
     {        -77e0,      0e0,     0e0,      43e0,    0e0,    0e0, } ,  
     {          2e0,      0e0,     0e0,      54e0,    0e0,    0e0, } ,  
     {         94e0,      0e0,     0e0,     -40e0,    0e0,    0e0, } ,  
     {        -93e0,      0e0,     0e0,      40e0,    0e0,    0e0, } ,  
     {        -83e0,      0e0,    10e0,      40e0,    0e0,   -2e0, } ,  
     {         83e0,      0e0,     0e0,     -36e0,    0e0,    0e0, } ,  
     {        -91e0,      0e0,     0e0,      39e0,    0e0,    0e0, } ,  
     {        128e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -79e0,      0e0,     0e0,      34e0,    0e0,    0e0, } ,  
     {        -83e0,      0e0,     0e0,      47e0,    0e0,    0e0, } ,  
     {         84e0,      0e0,     0e0,     -44e0,    0e0,    0e0, } ,  
     {         83e0,      0e0,     0e0,     -43e0,    0e0,    0e0, } ,  
     {         91e0,      0e0,     0e0,     -39e0,    0e0,    0e0, } ,  
     {        -77e0,      0e0,     0e0,      39e0,    0e0,    0e0, } ,  
     {         84e0,      0e0,     0e0,     -43e0,    0e0,    0e0, } ,  
     {        -92e0,      0e0,     1e0,      39e0,    0e0,    0e0, } ,  
     {        -92e0,      0e0,     1e0,      39e0,    0e0,    0e0, } ,  
     {        -94e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         68e0,      0e0,     0e0,     -36e0,    0e0,    0e0, } ,  
     {        -61e0,      0e0,     0e0,      32e0,    0e0,    0e0, } ,  
     {         71e0,      0e0,     0e0,     -31e0,    0e0,    0e0, } ,  
     {         62e0,      0e0,     0e0,     -34e0,    0e0,    0e0, } ,  
     {        -63e0,      0e0,     0e0,      33e0,    0e0,    0e0, } ,  
     {        -73e0,      0e0,     0e0,      32e0,    0e0,    0e0, } ,  
     {        115e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {       -103e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         63e0,      0e0,     0e0,     -28e0,    0e0,    0e0, } ,  
     {         74e0,      0e0,     0e0,     -32e0,    0e0,    0e0, } ,  
     {       -103e0,      0e0,    -3e0,       3e0,    0e0,   -1e0, } ,  
     {        -69e0,      0e0,     0e0,      30e0,    0e0,    0e0, } ,  
     {         57e0,      0e0,     0e0,     -29e0,    0e0,    0e0, } ,  
     {         94e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {         64e0,      0e0,     0e0,     -33e0,    0e0,    0e0, } ,  
     {        -63e0,      0e0,     0e0,      26e0,    0e0,    0e0, } ,  
     {        -38e0,      0e0,     0e0,      20e0,    0e0,    0e0, } ,  
     {        -43e0,      0e0,     0e0,      24e0,    0e0,    0e0, } ,  
     {        -45e0,      0e0,     0e0,      23e0,    0e0,    0e0, } ,  
     {         47e0,      0e0,     0e0,     -24e0,    0e0,    0e0, } ,  
     {        -48e0,      0e0,     0e0,      25e0,    0e0,    0e0, } ,  
     {         45e0,      0e0,     0e0,     -26e0,    0e0,    0e0, } ,  
     {         56e0,      0e0,     0e0,     -25e0,    0e0,    0e0, } ,  
     {         88e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {        -75e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         85e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         49e0,      0e0,     0e0,     -26e0,    0e0,    0e0, } ,  
     {        -74e0,      0e0,    -3e0,      -1e0,    0e0,   -1e0, } ,  
     {        -39e0,      0e0,     0e0,      21e0,    0e0,    0e0, } ,  
     {         45e0,      0e0,     0e0,     -20e0,    0e0,    0e0, } ,  
     {         51e0,      0e0,     0e0,     -22e0,    0e0,    0e0, } ,  
     {        -40e0,      0e0,     0e0,      21e0,    0e0,    0e0, } ,  
     {         41e0,      0e0,     0e0,     -21e0,    0e0,    0e0, } ,  
     {        -42e0,      0e0,     0e0,      24e0,    0e0,    0e0, } ,  
     {        -51e0,      0e0,     0e0,      22e0,    0e0,    0e0, } ,  
     {        -42e0,      0e0,     0e0,      22e0,    0e0,    0e0, } ,  
     {         39e0,      0e0,     0e0,     -21e0,    0e0,    0e0, } ,  
     {         46e0,      0e0,     0e0,     -18e0,    0e0,    0e0, } ,  
     {        -53e0,      0e0,     0e0,      22e0,    0e0,    0e0, } ,  
     {         82e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {         81e0,      0e0,    -1e0,      -4e0,    0e0,    0e0, } ,  
     {         47e0,      0e0,     0e0,     -19e0,    0e0,    0e0, } ,  
     {         53e0,      0e0,     0e0,     -23e0,    0e0,    0e0, } ,  
     {        -45e0,      0e0,     0e0,      22e0,    0e0,    0e0, } ,  
     {        -44e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {        -33e0,      0e0,     0e0,      16e0,    0e0,    0e0, } ,  
     {        -61e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         28e0,      0e0,     0e0,     -15e0,    0e0,    0e0, } ,  
     {        -38e0,      0e0,     0e0,      19e0,    0e0,    0e0, } ,  
     {        -33e0,      0e0,     0e0,      21e0,    0e0,    0e0, } ,  
     {        -60e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         48e0,      0e0,     0e0,     -10e0,    0e0,    0e0, } ,  
     {         27e0,      0e0,     0e0,     -14e0,    0e0,    0e0, } ,  
     {         38e0,      0e0,     0e0,     -20e0,    0e0,    0e0, } ,  
     {         31e0,      0e0,     0e0,     -13e0,    0e0,    0e0, } ,  
     {        -29e0,      0e0,     0e0,      15e0,    0e0,    0e0, } ,  
     {         28e0,      0e0,     0e0,     -15e0,    0e0,    0e0, } ,  
     {        -32e0,      0e0,     0e0,      15e0,    0e0,    0e0, } ,  
     {         45e0,      0e0,     0e0,      -8e0,    0e0,    0e0, } ,  
     {        -44e0,      0e0,     0e0,      19e0,    0e0,    0e0, } ,  
     {         28e0,      0e0,     0e0,     -15e0,    0e0,    0e0, } ,  
     {        -51e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -36e0,      0e0,     0e0,      20e0,    0e0,    0e0, } ,  
     {         44e0,      0e0,     0e0,     -19e0,    0e0,    0e0, } ,  
     {         26e0,      0e0,     0e0,     -14e0,    0e0,    0e0, } ,  
     {        -60e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         35e0,      0e0,     0e0,     -18e0,    0e0,    0e0, } ,  
     {        -27e0,      0e0,     0e0,      11e0,    0e0,    0e0, } ,  
     {         47e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         36e0,      0e0,     0e0,     -15e0,    0e0,    0e0, } ,  
     {        -36e0,      0e0,     0e0,      20e0,    0e0,    0e0, } ,  
     {        -35e0,      0e0,     0e0,      19e0,    0e0,    0e0, } ,  
     {        -37e0,      0e0,     0e0,      19e0,    0e0,    0e0, } ,  
     {         32e0,      0e0,     0e0,     -16e0,    0e0,    0e0, } ,  
     {         35e0,      0e0,     0e0,     -14e0,    0e0,    0e0, } ,  
     {         32e0,      0e0,     0e0,     -13e0,    0e0,    0e0, } ,  
     {         65e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         47e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         32e0,      0e0,     0e0,     -16e0,    0e0,    0e0, } ,  
     {         37e0,      0e0,     0e0,     -16e0,    0e0,    0e0, } ,  
     {        -30e0,      0e0,     0e0,      15e0,    0e0,    0e0, } ,  
     {        -32e0,      0e0,     0e0,      16e0,    0e0,    0e0, } ,  
     {        -31e0,      0e0,     0e0,      13e0,    0e0,    0e0, } ,  
     {         37e0,      0e0,     0e0,     -16e0,    0e0,    0e0, } ,  
     {         31e0,      0e0,     0e0,     -13e0,    0e0,    0e0, } ,  
     {         49e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         32e0,      0e0,     0e0,     -13e0,    0e0,    0e0, } ,  
     {         23e0,      0e0,     0e0,     -12e0,    0e0,    0e0, } ,  
     {        -43e0,      0e0,     0e0,      18e0,    0e0,    0e0, } ,  
     {         26e0,      0e0,     0e0,     -11e0,    0e0,    0e0, } ,  
     {        -32e0,      0e0,     0e0,      14e0,    0e0,    0e0, } ,  
     {        -29e0,      0e0,     0e0,      14e0,    0e0,    0e0, } ,  
     {        -27e0,      0e0,     0e0,      12e0,    0e0,    0e0, } ,  
     {         30e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -11e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -21e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {        -34e0,      0e0,     0e0,      15e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {        -36e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -9e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -21e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -29e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -15e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {        -20e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         28e0,      0e0,     0e0,       0e0,    0e0,   -2e0, } ,  
     {         17e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -22e0,      0e0,     0e0,      12e0,    0e0,    0e0, } ,  
     {        -14e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {         24e0,      0e0,     0e0,     -11e0,    0e0,    0e0, } ,  
     {         11e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {         14e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {         24e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         18e0,      0e0,     0e0,      -8e0,    0e0,    0e0, } ,  
     {        -38e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -31e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -16e0,      0e0,     0e0,       8e0,    0e0,    0e0, } ,  
     {         29e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -18e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -17e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {         16e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {         22e0,      0e0,     0e0,     -12e0,    0e0,    0e0, } ,  
     {         20e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -13e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {        -17e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {        -14e0,      0e0,     0e0,       8e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,     0e0,      -7e0,    0e0,    0e0, } ,  
     {         14e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         19e0,      0e0,     0e0,     -10e0,    0e0,    0e0, } ,  
     {        -34e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -20e0,      0e0,     0e0,       8e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {        -18e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {         13e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {         17e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {         15e0,      0e0,     0e0,      -8e0,    0e0,    0e0, } ,  
     {        -11e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         13e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {        -18e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -35e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {        -19e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {        -26e0,      0e0,     0e0,      11e0,    0e0,    0e0, } ,  
     {          8e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {        -21e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {        -15e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {        -29e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -19e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {         12e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {         22e0,      0e0,     0e0,      -9e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -20e0,      0e0,     0e0,      11e0,    0e0,    0e0, } ,  
     {        -20e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -17e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {         15e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          8e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {         14e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {         25e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -13e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {        -14e0,      0e0,     0e0,       8e0,    0e0,    0e0, } ,  
     {         13e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {        -17e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,     0e0,      -6e0,    0e0,    0e0, } ,  
     {        -15e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -22e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         28e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         15e0,      0e0,     0e0,      -7e0,    0e0,    0e0, } ,  
     {         23e0,      0e0,     0e0,     -10e0,    0e0,    0e0, } ,  
     {         12e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {         29e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -25e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         22e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -18e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         15e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {        -23e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         12e0,      0e0,     0e0,      -5e0,    0e0,    0e0, } ,  
     {         -8e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {        -19e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         21e0,      0e0,     0e0,      -9e0,    0e0,    0e0, } ,  
     {         23e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -16e0,      0e0,     0e0,       8e0,    0e0,    0e0, } ,  
     {        -19e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {        -22e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {         27e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         16e0,      0e0,     0e0,      -8e0,    0e0,    0e0, } ,  
     {         19e0,      0e0,     0e0,      -8e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {         -9e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         -9e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         -8e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         18e0,      0e0,     0e0,      -9e0,    0e0,    0e0, } ,  
     {         16e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -10e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {        -23e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {         16e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       6e0,    0e0,    0e0, } ,  
     {         -8e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         30e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         24e0,      0e0,     0e0,     -10e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {        -16e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {        -16e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {         17e0,      0e0,     0e0,      -7e0,    0e0,    0e0, } ,  
     {        -24e0,      0e0,     0e0,      10e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -24e0,      0e0,     0e0,      11e0,    0e0,    0e0, } ,  
     {        -23e0,      0e0,     0e0,       9e0,    0e0,    0e0, } ,  
     {        -13e0,      0e0,     0e0,       5e0,    0e0,    0e0, } ,  
     {        -15e0,      0e0,     0e0,       7e0,    0e0,    0e0, } ,  
     {          0e0,      0e0, -1988e0,       0e0,    0e0, -1679e0, } ,  
     {          0e0,      0e0,   -63e0,       0e0,    0e0,  -27e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,     5e0,       0e0,    0e0,    4e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,   364e0,       0e0,    0e0,  176e0, } ,  
     {          0e0,      0e0, -1044e0,       0e0,    0e0, -891e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,   330e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,     5e0,       0e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,   -12e0,       0e0,    0e0,  -10e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -8e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          8e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {        -13e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,    13e0,       6e0,    0e0,   -5e0, } ,  
     {          0e0,      0e0,    30e0,       0e0,    0e0,   14e0, } ,  
     {          0e0,      0e0,  -162e0,       0e0,    0e0, -138e0, } ,  
     {          0e0,      0e0,    75e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          9e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,    -3e0,       3e0,    0e0,    1e0, } ,  
     {          0e0,      0e0,    -3e0,       0e0,    0e0,   -2e0, } ,  
     {         11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -1e0,      0e0,     3e0,       3e0,    0e0,   -1e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,   -13e0,       0e0,    0e0,  -11e0, } ,  
     {          3e0,      0e0,     6e0,       0e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          8e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          8e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -8e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,   -26e0,       0e0,    0e0,  -11e0, } ,  
     {          0e0,      0e0,   -10e0,       0e0,    0e0,   -5e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {        -13e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -7e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         13e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {        -11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {        -12e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {          0e0,      0e0,    -5e0,       0e0,    0e0,   -2e0, } ,  
     {         -7e0,      0e0,     0e0,       4e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         12e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          6e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,      -4e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -5e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       3e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         10e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          7e0,      0e0,     0e0,      -3e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {         11e0,      0e0,     0e0,       0e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -6e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          5e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -4e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
     {          4e0,      0e0,     0e0,      -2e0,    0e0,    0e0, } ,  
     {          3e0,      0e0,     0e0,      -1e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       1e0,    0e0,    0e0, } ,  
     {         -3e0,      0e0,     0e0,       2e0,    0e0,    0e0, } ,  
    } ;  
  
/*  
 * ---------------  
 * Planetary terms  
 * ---------------  
 */  
  
/*  
 * Number of terms in the planetary nutation model  
 */  
  
    #define NPL 687  
  
/*  
 * Coefficients for fundamental arguments  
 */  
  
    static const int NAPL[ NPL ][14] = {  
  
/*  
 * Planetary argument multipliers  
 *      L  L'   F   D  Om  Me  Ve   E  Ma  Ju  Sa  Ur  Ne pre  
 */  
  
     {  0,  0,  0,  0,  0,  0,  0,  8,-16,  4,  5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -8, 16, -4, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  8,-16,  4,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  2,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  8, -1, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -8,  3,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0,  3, -8,  3,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0, 10, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0, -2,  6, -3,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -5,  8, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  8, -3,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -8,  1,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  6,  4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2, -5,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  2, -5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2, -5,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -2,  5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0, -2,  5,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0, -2,  5,  0,  0,  2, } ,  
     {  2,  0, -1, -1,  0,  0,  0,  3, -7,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0, 19,-21,  3,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  2, -4,  0, -3,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -1,  1,  0,  0, -1,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -4, 10,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  2,  0,  0, -5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -7,  4,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  1, -1,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0, 18,-16,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  2,  0,  0,  1,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  1, -1,  1,  0, 18,-17,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  1,  1,  0,  0,  2, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 13,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  2,  0, -8, 11,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 13,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -8, 12,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  8,-13,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  8,-14,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  8,-13,  0,  0,  0,  0,  0,  1, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  2,  0, -4,  5,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  2,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -3,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  3, -5,  0,  2,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -4,  3,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -1,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -2,  2,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  1,  0,  1,  0,  3, -5,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  1,  0,  0,  3, -4,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -2, -2,  0,  0,  0, } ,  
     { -2,  0,  2,  0,  2,  0,  0, -5,  9,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  0, -1,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  0,  0,  2,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  2, } ,  
     { -1,  0,  0,  1,  0,  0,  0,  3, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  0,  2,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0,  0,  2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -9, 17,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0, -3,  5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -1,  2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  1, -2,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0, 17,-16,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  1, -3,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  5, -6,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -2,  2,  0,  0,  0,  9,-13,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0, -2,  2,  0,  0,  5, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  1,  0,  5, -7,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  6, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  1, -3,  1,  0, -6,  7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0,  0,  0,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  1,  0,  0,  1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  0,  2,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -8, 15,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -8, 15,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -9, 15,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  8,-15,  0,  0,  0,  0,  0, } ,  
     {  1,  0, -1, -1,  0,  0,  0,  8,-15,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  0,  0,  2, -5,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -5,  5,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0,  0, -6,  8,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0,  0, -2,  0,  3,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  0,  0,  0,  1,  0, -3,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  1,  0,  0,  1,  0, -3,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  6, -8,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -1, -5,  0,  0,  0, } ,  
     { -1,  0,  0,  1,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     { -1,  0,  1,  1,  1,  0, -20,20,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0, 20,-21,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  8,-15,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0,-10, 15,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -2,  4,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0, -6,  8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -2,  2,  1,  0,  5, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0, -1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -9, 13,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  7,-13,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  5, -6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  9,-17,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -9, 17,  0,  0,  0,  0,  2, } ,  
     {  1,  0,  0, -1,  1,  0,  0, -3,  4,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -1,  1,  0, -3,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0,  0, -1,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  1,  0,  0,  0,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -2,  2,  0,  1,  0, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -5,  0,  2,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  2,  0, -3,  1,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  8,-13,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  8,-12,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0, -8, 11,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  1,  0,  0,  0,  2, -2,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  1,  0, 18,-16,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -1,  1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  3, -7,  4,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  1,  0,  0, -3,  7,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0, -2,  5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0, -2,  5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  1,  0,-10,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -2,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  1,  0, 10, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0,  2, -5,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  2, -5,  0,  0,  0, } ,  
     {  2,  0, -1, -1,  1,  0,  0,  3, -7,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0,  0, -5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -3,  7, -4,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  1,  0,-18, 16,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  1,  0,  0,  1,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0, -8, 12,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -8, 13,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1, -2,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0,  0, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -2,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  2,  0,  0,  0,  0,  1, } ,  
     { -1,  0,  0,  1,  1,  0,  3, -4,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  1,  1,  0,  0,  3, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0, -2,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  2, } ,  
     {  0,  0,  1, -1,  0,  0,  3, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -3,  5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0, -3,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -2,  4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0, -5,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  5, -7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  5, -8,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  6, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -8, 15,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  1,  0,  0,  6, -8,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -1,  1,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  3, -5,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  2, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0,  0, -1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0,  0, -1,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  0, -1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -7, 13,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7,-13,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0,  0, -5,  6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -8, 11,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1, -1,  0,  2,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  4, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2, -2,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  0,  3,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  2, } ,  
     { -2,  0,  0,  2,  0,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  2,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0,  0, -2,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0,  0, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  1, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  2, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0,  0, -2,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -2,  0,  0,  2,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0,  3, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  1,  0, -3,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  5,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  2,  0, -3,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  5,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2, -4,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0,  1, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  4,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -3,  4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  4,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  8,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  2,  0, -5,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  8,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  8,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -5,  7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  8,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0,  0, -1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  0,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -2,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6, 11,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,-11,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0, -1,  0,  4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  1,  0, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  0, -2,  1,  0, -3,  3,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  2,  0,  0, -2,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -7,  9,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  4, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  2,  0,  0, -2,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  1,  0,  3, -5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  3, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0, -3,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  2, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -4,  4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  2,  0, -5,  7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  6,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -4,  6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  6,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  6,  0,  0,  0,  0,  2, } ,  
     {  0,  0, -1,  1,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  2, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5,  9,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5,  9,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -9,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  0,  1,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -2,  0,  2,  0,  0,  0,  0, } ,  
     { -2,  0,  1,  1,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -2,  2,  0,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -6, 10,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -6, 10,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -2,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0, -1,  0,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -8,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  8,  0,  0,  0,  0,  2, } ,  
     {  0,  0, -2,  2,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  7,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -7,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -2,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -2,  0,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5, 10,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  1,  0, -1,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  5,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  5,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -5,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1, -2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  1, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  2,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7, 11,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7, 11,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0, -2,  2,  0,  0,  4, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2, -3,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0, -4,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  4, -5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1, -1,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  7,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -4,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  7,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -4,  5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -6,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0, } ,  
     {  0,  0, -1,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  5,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1, -3,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  3,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -7, 12,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  1,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -1,  0,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1, -1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  1, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  5,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  4,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -4,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -1,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6, 10,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6, 10,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -3,  0,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  7,  0,  0,  0,  0,  2, } ,  
     { -2,  0,  0,  2,  0,  0,  4, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5,  8,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -8,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  3,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -2,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  9,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  9,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -9,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  1,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0, -2,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -4,  6,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  3, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  9,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  2, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -1,  0,  2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0, -3,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  1, -5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  1,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -3,  5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -3,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0, -2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0, -1,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0, -2,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 14,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  2, -5,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -8,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  8, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0, -2,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 12,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 12,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  1, -2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  1,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  2,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  1,  0, -5,  5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  1,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  7,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  7,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -5,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0, -1,  0,  3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  6,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  1,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6,  9,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -9,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -2,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  0,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5,  7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -7,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0, -2,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -5,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0, -1,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7, 10,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7, 10,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -3,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  8,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  5,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  0,  5,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -9, 13,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  5,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  0,  4,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -4,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  5,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  8,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  8,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  1,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  9,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -6,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -6,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -2,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5, 10,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -4,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0, -3,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5, 13,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0, -2,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0, -2,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -2,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0, -1,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6, 15,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 15,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  9, -4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  2, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  8, -1, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -8,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  1, -1,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -6, 16, -4, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  8, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -2,  8, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -8,  1,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -2,  5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -5,  4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 11,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 11,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 11,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, 11,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  0,  1,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -3,  0,  2,  0,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  1,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  1, -1,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  1,  2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  7,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0,  2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  6,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7, -9,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2, -1,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -5,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -1,  4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7,  9,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -7,  9,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -3,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3, -1,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -4,  4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  1,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -3,  0,  5,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -9, 12,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -4,  0,  0,  0,  0, } ,  
     {  0,  0,  2, -2,  1,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7, -8,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -3,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -2,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  7,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -6,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -2,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0, -1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0,  0, -2,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4, -2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0,  0, -1,  0,  0,  2, } ,  
     {  0,  0,  2, -2,  1,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 16,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0,  2, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7, -8,  3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -5, 16, -4, -5,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0, -1,  8, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 10,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 10,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -8, 10,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  3,  0,  1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -3,  8,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -5,  5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -5,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -5,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -5,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  0,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7, -7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  7, -7,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -5,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  7, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -3,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  1,  2,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -9, 11,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -9, 11,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0, -4,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -6,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0, -2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6, -4,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  3, -1,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0, -1,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0,  0, -2,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5, -2,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  8, -9,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -4,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  1,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  2,  1,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -7,  7,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  7, -7,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -2,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -2,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  4, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5,  0, -4,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5,  0, -3,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  5,  0, -2,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -8,  8,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  8, -8,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -3,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  5, -3,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0, -9,  9,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -9,  9,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0, -9,  9,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  9, -9,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  6, -4,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  1, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  2, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2, } ,  
     {  1,  0,  0, -2,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  2,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0, -1,  1,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  0,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     { -2,  0,  0,  0,  0,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     { -1,  0,  1, -1,  1,  0,  0, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  1, -1,  1,  0,  0, -1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  0,  0,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  1,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  0,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  0,  2,  0,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0, -2,  1,  0,  0, -2,  0,  2,  0,  0,  0,  0, } ,  
     {  1,  0,  2, -2,  2,  0, -3,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  2, -2,  2,  0,  0, -2,  0,  2,  0,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  0,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  0, -2,  0,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0, -2,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0, -2,  2,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0, -1,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0, -2,  3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  0,  2,  0,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  1,  1,  2,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  0,  2,  0, 10, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  1,  1,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  0,  2,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     {  2,  0,  2, -2,  2,  0,  0, -2,  0,  3,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  1,  0,  0, -2,  0,  3,  0,  0,  0,  0, } ,  
     {  0,  0,  1,  1,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  0,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -2,  0,  2,  2,  2,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  2, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0,  1,  0, -1,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  2, -2,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  2,  2,  0,  0, -1,  0,  1,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0, -1,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  2,  2,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     {  2,  0,  2,  0,  2,  0,  0,  2,  0, -3,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0,  0, -4,  8, -3,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0,  0,  4, -8,  3,  0,  0,  0,  0, } ,  
     {  1,  0,  1,  1,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  0,  2,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     {  2,  0,  2,  0,  1,  0,  0,  1,  0,  0,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  2,  2,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
     { -1,  0,  2,  2,  2,  0,  3, -3,  0,  0,  0,  0,  0,  0, } ,  
     {  1,  0,  2,  0,  2,  0,  1, -1,  0,  0,  0,  0,  0,  0, } ,  
     {  0,  0,  2,  2,  2,  0,  0,  2,  0, -2,  0,  0,  0,  0, } ,  
    } ;  
  
/*  
 * Longitude and obliquity coefficients  
 */  
  
    static const int ICPL[ NPL ][4] = {  
  
/*  
 * Planetary nutation coefficients, unit 1e-7 arcsec  
 * longitude (sin, cos), obliquity (sin, cos)  
 */  
  
     {  1440,         0,         0,         0, } ,  
     {    56,      -117,       -42,       -40, } ,  
     {   125,       -43,         0,       -54, } ,  
     {     0,         5,         0,         0, } ,  
     {     3,        -7,        -3,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {  -114,         0,         0,        61, } ,  
     {  -219,        89,         0,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {  -462,      1604,         0,         0, } ,  
     {    99,         0,         0,       -53, } ,  
     {    -3,         0,         0,         2, } ,  
     {     0,         6,         2,         0, } ,  
     {     3,         0,         0,         0, } ,  
     {   -12,         0,         0,         0, } ,  
     {    14,      -218,       117,         8, } ,  
     {    31,      -481,      -257,       -17, } ,  
     {  -491,       128,         0,         0, } ,  
     { -3084,      5123,      2735,      1647, } ,  
     { -1444,      2409,     -1286,      -771, } ,  
     {    11,       -24,       -11,        -9, } ,  
     {    26,        -9,         0,         0, } ,  
     {   103,       -60,         0,         0, } ,  
     {     0,       -13,        -7,         0, } ,  
     {   -26,       -29,       -16,        14, } ,  
     {     9,       -27,       -14,        -5, } ,  
     {    12,         0,         0,        -6, } ,  
     {    -7,         0,         0,         0, } ,  
     {     0,        24,         0,         0, } ,  
     {   284,         0,         0,      -151, } ,  
     {   226,       101,         0,         0, } ,  
     {     0,        -8,        -2,         0, } ,  
     {     0,        -6,        -3,         0, } ,  
     {     5,         0,         0,        -3, } ,  
     {   -41,       175,        76,        17, } ,  
     {     0,        15,         6,         0, } ,  
     {   425,       212,      -133,       269, } ,  
     {  1200,       598,       319,      -641, } ,  
     {   235,       334,         0,         0, } ,  
     {    11,       -12,        -7,        -6, } ,  
     {     5,        -6,         3,         3, } ,  
     {    -5,         0,         0,         3, } ,  
     {     6,         0,         0,        -3, } ,  
     {    15,         0,         0,         0, } ,  
     {    13,         0,         0,        -7, } ,  
     {    -6,        -9,         0,         0, } ,  
     {   266,       -78,         0,         0, } ,  
     {  -460,      -435,      -232,       246, } ,  
     {     0,        15,         7,         0, } ,  
     {    -3,         0,         0,         2, } ,  
     {     0,       131,         0,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {     0,         4,         2,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {   -17,       -19,       -10,         9, } ,  
     {    -9,       -11,         6,        -5, } ,  
     {    -6,         0,         0,         3, } ,  
     {   -16,         8,         0,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {    11,        24,        11,        -5, } ,  
     {    -3,        -4,        -2,         1, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,        -8,        -4,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {     0,         5,         0,         0, } ,  
     {     0,         3,         2,         0, } ,  
     {    -6,         4,         2,         3, } ,  
     {    -3,        -5,         0,         0, } ,  
     {    -5,         0,         0,         2, } ,  
     {     4,        24,        13,        -2, } ,  
     {   -42,        20,         0,         0, } ,  
     {   -10,       233,         0,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {    78,       -18,         0,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {     0,        -3,        -1,         0, } ,  
     {     0,        -4,        -2,         1, } ,  
     {     0,        -8,        -4,        -1, } ,  
     {     0,        -5,         3,         0, } ,  
     {    -7,         0,         0,         3, } ,  
     {   -14,         8,         3,         6, } ,  
     {     0,         8,        -4,         0, } ,  
     {     0,        19,        10,         0, } ,  
     {    45,       -22,         0,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {     0,        -3,         0,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {     3,         5,         3,        -2, } ,  
     {    89,       -16,        -9,       -48, } ,  
     {     0,         3,         0,         0, } ,  
     {    -3,         7,         4,         2, } ,  
     {  -349,       -62,         0,         0, } ,  
     {   -15,        22,         0,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {   -53,         0,         0,         0, } ,  
     {     5,         0,         0,        -3, } ,  
     {     0,        -8,         0,         0, } ,  
     {    15,        -7,        -4,        -8, } ,  
     {    -3,         0,         0,         1, } ,  
     {   -21,       -78,         0,         0, } ,  
     {    20,       -70,       -37,       -11, } ,  
     {     0,         6,         3,         0, } ,  
     {     5,         3,         2,        -2, } ,  
     {   -17,        -4,        -2,         9, } ,  
     {     0,         6,         3,         0, } ,  
     {    32,        15,        -8,        17, } ,  
     {   174,        84,        45,       -93, } ,  
     {    11,        56,         0,         0, } ,  
     {   -66,       -12,        -6,        35, } ,  
     {    47,         8,         4,       -25, } ,  
     {     0,         8,         4,         0, } ,  
     {    10,       -22,       -12,        -5, } ,  
     {    -3,         0,         0,         2, } ,  
     {   -24,        12,         0,         0, } ,  
     {     5,        -6,         0,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {     4,         3,         1,        -2, } ,  
     {     0,        29,        15,         0, } ,  
     {    -5,        -4,        -2,         2, } ,  
     {     8,        -3,        -1,        -5, } ,  
     {     0,        -3,         0,         0, } ,  
     {    10,         0,         0,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {    -5,         0,         0,         3, } ,  
     {    46,        66,        35,       -25, } ,  
     {   -14,         7,         0,         0, } ,  
     {     0,         3,         2,         0, } ,  
     {    -5,         0,         0,         0, } ,  
     {   -68,       -34,       -18,        36, } ,  
     {     0,        14,         7,         0, } ,  
     {    10,        -6,        -3,        -5, } ,  
     {    -5,        -4,        -2,         3, } ,  
     {    -3,         5,         2,         1, } ,  
     {    76,        17,         9,       -41, } ,  
     {    84,       298,       159,       -45, } ,  
     {     3,         0,         0,        -1, } ,  
     {    -3,         0,         0,         2, } ,  
     {    -3,         0,         0,         1, } ,  
     {   -82,       292,       156,        44, } ,  
     {   -73,        17,         9,        39, } ,  
     {    -9,       -16,         0,         0, } ,  
     {     3,         0,        -1,        -2, } ,  
     {    -3,         0,         0,         0, } ,  
     {    -9,        -5,        -3,         5, } ,  
     {  -439,         0,         0,         0, } ,  
     {    57,       -28,       -15,       -30, } ,  
     {     0,        -6,        -3,         0, } ,  
     {    -4,         0,         0,         2, } ,  
     {   -40,        57,        30,        21, } ,  
     {    23,         7,         3,       -13, } ,  
     {   273,        80,        43,      -146, } ,  
     {  -449,       430,         0,         0, } ,  
     {    -8,       -47,       -25,         4, } ,  
     {     6,        47,        25,        -3, } ,  
     {     0,        23,        13,         0, } ,  
     {    -3,         0,         0,         2, } ,  
     {     3,        -4,        -2,        -2, } ,  
     {   -48,      -110,       -59,        26, } ,  
     {    51,       114,        61,       -27, } ,  
     {  -133,         0,         0,        57, } ,  
     {     0,         4,         0,         0, } ,  
     {   -21,        -6,        -3,        11, } ,  
     {     0,        -3,        -1,         0, } ,  
     {   -11,       -21,       -11,         6, } ,  
     {   -18,      -436,      -233,         9, } ,  
     {    35,        -7,         0,         0, } ,  
     {     0,         5,         3,         0, } ,  
     {    11,        -3,        -1,        -6, } ,  
     {    -5,        -3,        -1,         3, } ,  
     {   -53,        -9,        -5,        28, } ,  
     {     0,         3,         2,         1, } ,  
     {     4,         0,         0,        -2, } ,  
     {     0,        -4,         0,         0, } ,  
     {   -50,       194,       103,        27, } ,  
     {   -13,        52,        28,         7, } ,  
     {   -91,       248,         0,         0, } ,  
     {     6,        49,        26,        -3, } ,  
     {    -6,       -47,       -25,         3, } ,  
     {     0,         5,         3,         0, } ,  
     {    52,        23,        10,       -23, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,         5,         3,         0, } ,  
     {    -4,         0,         0,         0, } ,  
     {    -4,         8,         3,         2, } ,  
     {    10,         0,         0,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {     0,         8,         4,         0, } ,  
     {     0,         8,         4,         1, } ,  
     {    -4,         0,         0,         0, } ,  
     {    -4,         0,         0,         0, } ,  
     {    -8,         4,         2,         4, } ,  
     {     8,        -4,        -2,        -4, } ,  
     {     0,        15,         7,         0, } ,  
     {  -138,         0,         0,         0, } ,  
     {     0,        -7,        -3,         0, } ,  
     {     0,        -7,        -3,         0, } ,  
     {    54,         0,         0,       -29, } ,  
     {     0,        10,         4,         0, } ,  
     {    -7,         0,         0,         3, } ,  
     {   -37,        35,        19,        20, } ,  
     {     0,         4,         0,         0, } ,  
     {    -4,         9,         0,         0, } ,  
     {     8,         0,         0,        -4, } ,  
     {    -9,       -14,        -8,         5, } ,  
     {    -3,        -9,        -5,         3, } ,  
     {  -145,        47,         0,         0, } ,  
     {   -10,        40,        21,         5, } ,  
     {    11,       -49,       -26,        -7, } ,  
     { -2150,         0,         0,       932, } ,  
     {   -12,         0,         0,         5, } ,  
     {    85,         0,         0,       -37, } ,  
     {     4,         0,         0,        -2, } ,  
     {     3,         0,         0,        -2, } ,  
     {   -86,       153,         0,         0, } ,  
     {    -6,         9,         5,         3, } ,  
     {     9,       -13,        -7,        -5, } ,  
     {    -8,        12,         6,         4, } ,  
     {   -51,         0,         0,        22, } ,  
     {   -11,      -268,      -116,         5, } ,  
     {     0,        12,         5,         0, } ,  
     {     0,         7,         3,         0, } ,  
     {    31,         6,         3,       -17, } ,  
     {   140,        27,        14,       -75, } ,  
     {    57,        11,         6,       -30, } ,  
     {   -14,       -39,         0,         0, } ,  
     {     0,        -6,        -2,         0, } ,  
     {     4,        15,         8,        -2, } ,  
     {     0,         4,         0,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,        11,         5,         0, } ,  
     {     9,         6,         0,         0, } ,  
     {    -4,        10,         4,         2, } ,  
     {     5,         3,         0,         0, } ,  
     {    16,         0,         0,        -9, } ,  
     {    -3,         0,         0,         0, } ,  
     {     0,         3,         2,        -1, } ,  
     {     7,         0,         0,        -3, } ,  
     {   -25,        22,         0,         0, } ,  
     {    42,       223,       119,       -22, } ,  
     {   -27,      -143,       -77,        14, } ,  
     {     9,        49,        26,        -5, } ,  
     { -1166,         0,         0,       505, } ,  
     {    -5,         0,         0,         2, } ,  
     {    -6,         0,         0,         3, } ,  
     {    -8,         0,         1,         4, } ,  
     {     0,        -4,         0,         0, } ,  
     {   117,         0,         0,       -63, } ,  
     {    -4,         8,         4,         2, } ,  
     {     3,         0,         0,        -2, } ,  
     {    -5,         0,         0,         2, } ,  
     {     0,        31,         0,         0, } ,  
     {    -5,         0,         1,         3, } ,  
     {     4,         0,         0,        -2, } ,  
     {    -4,         0,         0,         2, } ,  
     {   -24,       -13,        -6,        10, } ,  
     {     3,         0,         0,         0, } ,  
     {     0,       -32,       -17,         0, } ,  
     {     8,        12,         5,        -3, } ,  
     {     3,         0,         0,        -1, } ,  
     {     7,        13,         0,         0, } ,  
     {    -3,        16,         0,         0, } ,  
     {    50,         0,         0,       -27, } ,  
     {     0,        -5,        -3,         0, } ,  
     {    13,         0,         0,         0, } ,  
     {     0,         5,         3,         1, } ,  
     {    24,         5,         2,       -11, } ,  
     {     5,       -11,        -5,        -2, } ,  
     {    30,        -3,        -2,       -16, } ,  
     {    18,         0,         0,        -9, } ,  
     {     8,       614,         0,         0, } ,  
     {     3,        -3,        -1,        -2, } ,  
     {     6,        17,         9,        -3, } ,  
     {    -3,        -9,        -5,         2, } ,  
     {     0,         6,         3,        -1, } ,  
     {  -127,        21,         9,        55, } ,  
     {     3,         5,         0,         0, } ,  
     {    -6,       -10,        -4,         3, } ,  
     {     5,         0,         0,         0, } ,  
     {    16,         9,         4,        -7, } ,  
     {     3,         0,         0,        -2, } ,  
     {     0,        22,         0,         0, } ,  
     {     0,        19,        10,         0, } ,  
     {     7,         0,         0,        -4, } ,  
     {     0,        -5,        -2,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {    -9,         3,         1,         4, } ,  
     {    17,         0,         0,        -7, } ,  
     {     0,        -3,        -2,        -1, } ,  
     {   -20,        34,         0,         0, } ,  
     {   -10,         0,         1,         5, } ,  
     {    -4,         0,         0,         2, } ,  
     {    22,       -87,         0,         0, } ,  
     {    -4,         0,         0,         2, } ,  
     {    -3,        -6,        -2,         1, } ,  
     {   -16,        -3,        -1,         7, } ,  
     {     0,        -3,        -2,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {   -68,        39,         0,         0, } ,  
     {    27,         0,         0,       -14, } ,  
     {     0,        -4,         0,         0, } ,  
     {   -25,         0,         0,         0, } ,  
     {   -12,        -3,        -2,         6, } ,  
     {     3,         0,         0,        -1, } ,  
     {     3,        66,        29,        -1, } ,  
     {   490,         0,         0,      -213, } ,  
     {   -22,        93,        49,        12, } ,  
     {    -7,        28,        15,         4, } ,  
     {    -3,        13,         7,         2, } ,  
     {   -46,        14,         0,         0, } ,  
     {    -5,         0,         0,         0, } ,  
     {     2,         1,         0,         0, } ,  
     {     0,        -3,         0,         0, } ,  
     {   -28,         0,         0,        15, } ,  
     {     5,         0,         0,        -2, } ,  
     {     0,         3,         0,         0, } ,  
     {   -11,         0,         0,         5, } ,  
     {     0,         3,         1,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {    25,       106,        57,       -13, } ,  
     {     5,        21,        11,        -3, } ,  
     {  1485,         0,         0,         0, } ,  
     {    -7,       -32,       -17,         4, } ,  
     {     0,         5,         3,         0, } ,  
     {    -6,        -3,        -2,         3, } ,  
     {    30,        -6,        -2,       -13, } ,  
     {    -4,         4,         0,         0, } ,  
     {   -19,         0,         0,        10, } ,  
     {     0,         4,         2,        -1, } ,  
     {     0,         3,         0,         0, } ,  
     {     4,         0,         0,        -2, } ,  
     {     0,        -3,        -1,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {     5,         3,         1,        -2, } ,  
     {     0,        11,         0,         0, } ,  
     {   118,         0,         0,       -52, } ,  
     {     0,        -5,        -3,         0, } ,  
     {   -28,        36,         0,         0, } ,  
     {     5,        -5,         0,         0, } ,  
     {    14,       -59,       -31,        -8, } ,  
     {     0,         9,         5,         1, } ,  
     {  -458,         0,         0,       198, } ,  
     {     0,       -45,       -20,         0, } ,  
     {     9,         0,         0,        -5, } ,  
     {     0,        -3,         0,         0, } ,  
     {     0,        -4,        -2,        -1, } ,  
     {    11,         0,         0,        -6, } ,  
     {     6,         0,         0,        -2, } ,  
     {   -16,        23,         0,         0, } ,  
     {     0,        -4,        -2,         0, } ,  
     {    -5,         0,         0,         2, } ,  
     {  -166,       269,         0,         0, } ,  
     {    15,         0,         0,        -8, } ,  
     {    10,         0,         0,        -4, } ,  
     {   -78,        45,         0,         0, } ,  
     {     0,        -5,        -2,         0, } ,  
     {     7,         0,         0,        -4, } ,  
     {    -5,       328,         0,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {     5,         0,         0,        -2, } ,  
     {     0,         3,         1,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {     0,        -4,        -2,         0, } ,  
     { -1223,       -26,         0,         0, } ,  
     {     0,         7,         3,         0, } ,  
     {     3,         0,         0,         0, } ,  
     {     0,         3,         2,         0, } ,  
     {    -6,        20,         0,         0, } ,  
     {  -368,         0,         0,         0, } ,  
     {   -75,         0,         0,         0, } ,  
     {    11,         0,         0,        -6, } ,  
     {     3,         0,         0,        -2, } ,  
     {    -3,         0,         0,         1, } ,  
     {   -13,       -30,         0,         0, } ,  
     {    21,         3,         0,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -4,         0,         0,         2, } ,  
     {     8,       -27,         0,         0, } ,  
     {   -19,       -11,         0,         0, } ,  
     {    -4,         0,         0,         2, } ,  
     {     0,         5,         2,         0, } ,  
     {    -6,         0,         0,         2, } ,  
     {    -8,         0,         0,         0, } ,  
     {    -1,         0,         0,         0, } ,  
     {   -14,         0,         0,         6, } ,  
     {     6,         0,         0,         0, } ,  
     {   -74,         0,         0,        32, } ,  
     {     0,        -3,        -1,         0, } ,  
     {     4,         0,         0,        -2, } ,  
     {     8,        11,         0,         0, } ,  
     {     0,         3,         2,         0, } ,  
     {  -262,         0,         0,       114, } ,  
     {     0,        -4,         0,         0, } ,  
     {    -7,         0,         0,         4, } ,  
     {     0,       -27,       -12,         0, } ,  
     {   -19,        -8,        -4,         8, } ,  
     {   202,         0,         0,       -87, } ,  
     {    -8,        35,        19,         5, } ,  
     {     0,         4,         2,         0, } ,  
     {    16,        -5,         0,         0, } ,  
     {     5,         0,         0,        -3, } ,  
     {     0,        -3,         0,         0, } ,  
     {     1,         0,         0,         0, } ,  
     {   -35,       -48,       -21,        15, } ,  
     {    -3,        -5,        -2,         1, } ,  
     {     6,         0,         0,        -3, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,        -5,         0,         0, } ,  
     {    12,        55,        29,        -6, } ,  
     {     0,         5,         3,         0, } ,  
     {  -598,         0,         0,         0, } ,  
     {    -3,       -13,        -7,         1, } ,  
     {    -5,        -7,        -3,         2, } ,  
     {     3,         0,         0,        -1, } ,  
     {     5,        -7,         0,         0, } ,  
     {     4,         0,         0,        -2, } ,  
     {    16,        -6,         0,         0, } ,  
     {     8,        -3,         0,         0, } ,  
     {     8,       -31,       -16,        -4, } ,  
     {     0,         3,         1,         0, } ,  
     {   113,         0,         0,       -49, } ,  
     {     0,       -24,       -10,         0, } ,  
     {     4,         0,         0,        -2, } ,  
     {    27,         0,         0,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,        -4,        -2,         0, } ,  
     {     5,         0,         0,        -2, } ,  
     {     0,        -3,         0,         0, } ,  
     {   -13,         0,         0,         6, } ,  
     {     5,         0,         0,        -2, } ,  
     {   -18,       -10,        -4,         8, } ,  
     {    -4,       -28,         0,         0, } ,  
     {    -5,         6,         3,         2, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -5,        -9,        -4,         2, } ,  
     {    17,         0,         0,        -7, } ,  
     {    11,         4,         0,         0, } ,  
     {     0,        -6,        -2,         0, } ,  
     {    83,        15,         0,         0, } ,  
     {    -4,         0,         0,         2, } ,  
     {     0,      -114,       -49,         0, } ,  
     {   117,         0,         0,       -51, } ,  
     {    -5,        19,        10,         2, } ,  
     {    -3,         0,         0,         0, } ,  
     {    -3,         0,         0,         2, } ,  
     {     0,        -3,        -1,         0, } ,  
     {     3,         0,         0,         0, } ,  
     {     0,        -6,        -2,         0, } ,  
     {   393,         3,         0,         0, } ,  
     {    -4,        21,        11,         2, } ,  
     {    -6,         0,        -1,         3, } ,  
     {    -3,         8,         4,         1, } ,  
     {     8,         0,         0,         0, } ,  
     {    18,       -29,       -13,        -8, } ,  
     {     8,        34,        18,        -4, } ,  
     {    89,         0,         0,         0, } ,  
     {     3,        12,         6,        -1, } ,  
     {    54,       -15,        -7,       -24, } ,  
     {     0,         3,         0,         0, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,        35,         0,         0, } ,  
     {  -154,       -30,       -13,        67, } ,  
     {    15,         0,         0,         0, } ,  
     {     0,         4,         2,         0, } ,  
     {     0,         9,         0,         0, } ,  
     {    80,       -71,       -31,       -35, } ,  
     {     0,       -20,        -9,         0, } ,  
     {    11,         5,         2,        -5, } ,  
     {    61,       -96,       -42,       -27, } ,  
     {    14,         9,         4,        -6, } ,  
     {   -11,        -6,        -3,         5, } ,  
     {     0,        -3,        -1,         0, } ,  
     {   123,      -415,      -180,       -53, } ,  
     {     0,         0,         0,       -35, } ,  
     {    -5,         0,         0,         0, } ,  
     {     7,       -32,       -17,        -4, } ,  
     {     0,        -9,        -5,         0, } ,  
     {     0,        -4,         2,         0, } ,  
     {   -89,         0,         0,        38, } ,  
     {     0,       -86,       -19,        -6, } ,  
     {     0,         0,       -19,         6, } ,  
     {  -123,      -416,      -180,        53, } ,  
     {     0,        -3,        -1,         0, } ,  
     {    12,        -6,        -3,        -5, } ,  
     {   -13,         9,         4,         6, } ,  
     {     0,       -15,        -7,         0, } ,  
     {     3,         0,         0,        -1, } ,  
     {   -62,       -97,       -42,        27, } ,  
     {   -11,         5,         2,         5, } ,  
     {     0,       -19,        -8,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,         4,         2,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {     0,         4,         2,         0, } ,  
     {   -85,       -70,       -31,        37, } ,  
     {   163,       -12,        -5,       -72, } ,  
     {   -63,       -16,        -7,        28, } ,  
     {   -21,       -32,       -14,         9, } ,  
     {     0,        -3,        -1,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {     0,         8,         0,         0, } ,  
     {     3,        10,         4,        -1, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,        -7,        -3,         0, } ,  
     {     0,        -4,        -2,         0, } ,  
     {     6,        19,         0,         0, } ,  
     {     5,      -173,       -75,        -2, } ,  
     {     0,        -7,        -3,         0, } ,  
     {     7,       -12,        -5,        -3, } ,  
     {    -3,         0,         0,         2, } ,  
     {     3,        -4,        -2,        -1, } ,  
     {    74,         0,         0,       -32, } ,  
     {    -3,        12,         6,         2, } ,  
     {    26,       -14,        -6,       -11, } ,  
     {    19,         0,         0,        -8, } ,  
     {     6,        24,        13,        -3, } ,  
     {    83,         0,         0,         0, } ,  
     {     0,       -10,        -5,         0, } ,  
     {    11,        -3,        -1,        -5, } ,  
     {     3,         0,         1,        -1, } ,  
     {     3,         0,         0,        -1, } ,  
     {    -4,         0,         0,         0, } ,  
     {     5,       -23,       -12,        -3, } ,  
     {  -339,         0,         0,       147, } ,  
     {     0,       -10,        -5,         0, } ,  
     {     5,         0,         0,         0, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,        -4,        -2,         0, } ,  
     {    18,        -3,         0,         0, } ,  
     {     9,       -11,        -5,        -4, } ,  
     {    -8,         0,         0,         4, } ,  
     {     3,         0,         0,        -1, } ,  
     {     0,         9,         0,         0, } ,  
     {     6,        -9,        -4,        -2, } ,  
     {    -4,       -12,         0,         0, } ,  
     {    67,       -91,       -39,       -29, } ,  
     {    30,       -18,        -8,       -13, } ,  
     {     0,         0,         0,         0, } ,  
     {     0,      -114,       -50,         0, } ,  
     {     0,         0,         0,        23, } ,  
     {   517,        16,         7,      -224, } ,  
     {     0,        -7,        -3,         0, } ,  
     {   143,        -3,        -1,       -62, } ,  
     {    29,         0,         0,       -13, } ,  
     {    -4,         0,         0,         2, } ,  
     {    -6,         0,         0,         3, } ,  
     {     5,        12,         5,        -2, } ,  
     {   -25,         0,         0,        11, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,         4,         2,         0, } ,  
     {   -22,        12,         5,        10, } ,  
     {    50,         0,         0,       -22, } ,  
     {     0,         7,         4,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {    -4,         4,         2,         2, } ,  
     {    -5,       -11,        -5,         2, } ,  
     {     0,         4,         2,         0, } ,  
     {     4,        17,         9,        -2, } ,  
     {    59,         0,         0,         0, } ,  
     {     0,        -4,        -2,         0, } ,  
     {    -8,         0,         0,         4, } ,  
     {    -3,         0,         0,         0, } ,  
     {     4,       -15,        -8,        -2, } ,  
     {   370,        -8,         0,      -160, } ,  
     {     0,         0,        -3,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {    -6,         3,         1,         3, } ,  
     {     0,         6,         0,         0, } ,  
     {   -10,         0,         0,         4, } ,  
     {     0,         9,         4,         0, } ,  
     {     4,        17,         7,        -2, } ,  
     {    34,         0,         0,       -15, } ,  
     {     0,         5,         3,         0, } ,  
     {    -5,         0,         0,         2, } ,  
     {   -37,        -7,        -3,        16, } ,  
     {     3,        13,         7,        -2, } ,  
     {    40,         0,         0,         0, } ,  
     {     0,        -3,        -2,         0, } ,  
     {  -184,        -3,        -1,        80, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -3,         0,         0,         0, } ,  
     {     0,       -10,        -6,        -1, } ,  
     {    31,        -6,         0,       -13, } ,  
     {    -3,       -32,       -14,         1, } ,  
     {    -7,         0,         0,         3, } ,  
     {     0,        -8,        -4,         0, } ,  
     {     3,        -4,         0,         0, } ,  
     {     0,         4,         0,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {    19,       -23,       -10,         2, } ,  
     {     0,         0,         0,       -10, } ,  
     {     0,         3,         2,         0, } ,  
     {     0,         9,         5,        -1, } ,  
     {    28,         0,         0,         0, } ,  
     {     0,        -7,        -4,         0, } ,  
     {     8,        -4,         0,        -4, } ,  
     {     0,         0,        -2,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -9,         0,         1,         4, } ,  
     {     3,        12,         5,        -1, } ,  
     {    17,        -3,        -1,         0, } ,  
     {     0,         7,         4,         0, } ,  
     {    19,         0,         0,         0, } ,  
     {     0,        -5,        -3,         0, } ,  
     {    14,        -3,         0,        -1, } ,  
     {     0,         0,        -1,         0, } ,  
     {     0,         0,         0,        -5, } ,  
     {     0,         5,         3,         0, } ,  
     {    13,         0,         0,         0, } ,  
     {     0,        -3,        -2,         0, } ,  
     {     2,         9,         4,         3, } ,  
     {     0,         0,         0,        -4, } ,  
     {     8,         0,         0,         0, } ,  
     {     0,         4,         2,         0, } ,  
     {     6,         0,         0,        -3, } ,  
     {     6,         0,         0,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {     5,         0,         0,        -2, } ,  
     {     3,         0,         0,        -1, } ,  
     {    -3,         0,         0,         0, } ,  
     {     6,         0,         0,         0, } ,  
     {     7,         0,         0,         0, } ,  
     {    -4,         0,         0,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {     6,         0,         0,         0, } ,  
     {     0,        -4,         0,         0, } ,  
     {     0,        -4,         0,         0, } ,  
     {     5,         0,         0,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {    -5,         0,         0,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {     0,         3,         0,         0, } ,  
     {    13,         0,         0,         0, } ,  
     {    21,        11,         0,         0, } ,  
     {     0,        -5,         0,         0, } ,  
     {     0,        -5,        -2,         0, } ,  
     {     0,         5,         3,         0, } ,  
     {     0,        -5,         0,         0, } ,  
     {    -3,         0,         0,         2, } ,  
     {    20,        10,         0,         0, } ,  
     {   -34,         0,         0,         0, } ,  
     {   -19,         0,         0,         0, } ,  
     {     3,         0,         0,        -2, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -6,         0,         0,         3, } ,  
     {    -4,         0,         0,         0, } ,  
     {     3,         0,         0,         0, } ,  
     {     3,         0,         0,         0, } ,  
     {     4,         0,         0,         0, } ,  
     {     3,         0,         0,        -1, } ,  
     {     6,         0,         0,        -3, } ,  
     {    -8,         0,         0,         3, } ,  
     {     0,         3,         1,         0, } ,  
     {    -3,         0,         0,         0, } ,  
     {     0,        -3,        -2,         0, } ,  
     {   126,       -63,       -27,       -55, } ,  
     {    -5,         0,         1,         2, } ,  
     {    -3,        28,        15,         2, } ,  
     {     5,         0,         1,        -2, } ,  
     {     0,         9,         4,         1, } ,  
     {     0,         9,         4,        -1, } ,  
     {  -126,       -63,       -27,        55, } ,  
     {     3,         0,         0,        -1, } ,  
     {    21,       -11,        -6,       -11, } ,  
     {     0,        -4,         0,         0, } ,  
     {   -21,       -11,        -6,        11, } ,  
     {    -3,         0,         0,         1, } ,  
     {     0,         3,         1,         0, } ,  
     {     8,         0,         0,        -4, } ,  
     {    -6,         0,         0,         3, } ,  
     {    -3,         0,         0,         1, } ,  
     {     3,         0,         0,        -1, } ,  
     {    -3,         0,         0,         1, } ,  
     {    -5,         0,         0,         2, } ,  
     {    24,       -12,        -5,       -11, } ,  
     {     0,         3,         1,         0, } ,  
     {     0,         3,         1,         0, } ,  
     {     0,         3,         2,         0, } ,  
     {   -24,       -12,        -5,        10, } ,  
     {     4,         0,        -1,        -2, } ,  
     {    13,         0,         0,        -6, } ,  
     {     7,         0,         0,        -3, } ,  
     {     3,         0,         0,        -1, } ,  
     {     3,         0,         0,        -1, } ,  
  
    } ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental date J2000.0 and given date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 * -------------------  
 * LUNI-SOLAR NUTATION  
 * -------------------  
 */  
  
/*  
 * Fundamental (Delaunay) arguments  
 */  
  
/*  
 * Mean anomaly of the Moon (IERS 2003).  
 */  
  
    el = gal_fal03 ( t ) ;  
  
/*  
 * Mean anomaly of the Sun (MHB2000).  
 */  
  
    elp = fmod (        1287104.79305 +  
                 t * ( 129596581.0481 +  
                 t * (       - 0.5532 +  
                 t * (       0.000136 +  
                 t * (   - 0.00001149 )))), GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean longitude of the Moon minus that of the ascending node  
 * (IERS 2003.  
 */  
  
    f = gal_faf03 ( t ) ;  
  
/*  
 * Mean elongation of the Moon from the Sun (MHB2000).  
 */  
  
    d = fmod (         1072260.70369 +  
               t * ( 1602961601.2090 +  
               t * (        - 6.3706 +  
               t * (        0.006593 +  
               t * (    - 0.00003169 )))), GAL_TURNAS ) * GAL_AS2R ;  
  
/*  
 * Mean longitude of the ascending node of the Moon (IERS 2003).  
 */  
  
    om = gal_faom03 ( t ) ;  
  
/*  
 * Initialize the nutation values.  
 */  
  
    dp = 0.0 ;  
    de = 0.0 ;  
  
/*  
 * Summation of luni-solar nutation series (in reverse order).  
 */  
  
    for ( i = NLS - 1 ; i >= 0; i-- ) {  
  
/*  
 * Argument and functions.  
 */  
  
        arg = fmod ( ( double ) NALS[i][0] * el  +  
                     ( double ) NALS[i][1] * elp +  
                     ( double ) NALS[i][2] * f   +  
                     ( double ) NALS[i][3] * d   +  
                     ( double ) NALS[i][4] * om, GAL_2PI ) ;  
  
        sarg = sin ( arg ) ;  
        carg = cos ( arg ) ;  
  
/*  
 * Term.  
 */  
  
        dp += ( CLS[i][0] + CLS[i][1] * t ) * sarg  
              + CLS[i][2]                   * carg ;  
  
        de += ( CLS[i][3] + CLS[i][4] * t ) * carg  
              + CLS[i][5]                   * sarg ;  
  
    }  
  
/*  
 * Convert from 0.1 microarcsec units to radians.  
 */  
  
    dpsils = dp * GAL_U2R ;  
    depsls = de * GAL_U2R ;  
  
/*  
 * ------------------  
 * PLANETARY NUTATION  
 * ------------------  
 */  
  
/*  
 *  n.b.  The MHB2000 code computes the luni-solar and planetary nutation  
 *        in different routines, using slightly different Delaunay  
 *        arguments in the two cases.  This behaviour is faithfully  
 *        reproduced here.  Use of the IERS 2003 expressions for both  
 *        cases leads to negligible changes, well below  
 *        0.1 microarcsecond.  
 */  
  
/*  
 * Mean anomaly of the Moon (MHB2000).  
 */  
  
    al = fmod ( 2.35555598 + 8328.6914269554 * t, GAL_2PI ) ;  
  
/*  
 * Mean anomaly of the Sun (MHB2000).  
 */  
  
    alsu = fmod ( 6.24006013 + 628.301955 * t, GAL_2PI ) ;  
  
/*  
 * Mean longitude of the Moon minus that of the ascending node  
 * (MHB2000).  
 */  
  
    af = fmod ( 1.627905234 + 8433.466158131 * t, GAL_2PI ) ;  
  
/*  
 * Mean elongation of the Moon from the Sun (MHB2000).  
 */  
  
    ad = fmod ( 5.198466741 + 7771.3771468121 * t, GAL_2PI ) ;  
  
/*  
 * Mean longitude of the ascending node of the Moon (MHB2000).  
 */  
  
    aom = fmod ( 2.18243920 - 33.757045 * t, GAL_2PI ) ;  
  
/*  
 * General accumulated precession in longitude (IERS 2003).  
 */  
  
    apa = gal_fapa03 ( t ) ;  
  
/*  
 * Planetary longitudes, Mercury through Uranus (IERS 2003).  
 */  
  
    alme = gal_fame03 ( t ) ;  
    alve = gal_fave03 ( t ) ;  
    alea = gal_fae03  ( t ) ;  
    alma = gal_fama03 ( t ) ;  
    alju = gal_faju03 ( t ) ;  
    alsa = gal_fasa03 ( t ) ;  
    alur = gal_faur03 ( t ) ;  
  
/*  
 * Neptune longitude (MHB2000).  
 */  
  
    alne = fmod ( 5.321159000 + 3.8127774000 * t, GAL_2PI ) ;  
  
/*  
 * Initialize the nutation values.  
 */  
  
    dp = 0.0 ;  
    de = 0.0 ;  
  
/*  
 * Summation of planetary nutation series (in reverse order).  
 */  
  
    for ( i = NPL - 1; i >= 0 ; i-- ) {  
  
/*  
 * Argument and functions.  
 */  
  
         arg = fmod( ( double ) NAPL[i][0]  * al   +  
                     ( double ) NAPL[i][1]  * alsu +  
                     ( double ) NAPL[i][2]  * af   +  
                     ( double ) NAPL[i][3]  * ad   +  
                     ( double ) NAPL[i][4]  * aom  +  
                     ( double ) NAPL[i][5]  * alme +  
                     ( double ) NAPL[i][6]  * alve +  
                     ( double ) NAPL[i][7]  * alea +  
                     ( double ) NAPL[i][8]  * alma +  
                     ( double ) NAPL[i][9]  * alju +  
                     ( double ) NAPL[i][10] * alsa +  
                     ( double ) NAPL[i][11] * alur +  
                     ( double ) NAPL[i][12] * alne +  
                     ( double ) NAPL[i][13] * apa, GAL_2PI ) ;  
  
         sarg = sin ( arg ) ;  
         carg = cos ( arg ) ;  
  
/*  
 * Term.  
 */  
  
         dp += ( double ) ICPL[i][0] * sarg + ( double ) ICPL[i][1] * carg ;  
         de += ( double ) ICPL[i][2] * sarg + ( double ) ICPL[i][3] * carg ;  
  
    }  
  
/*  
 * Convert from 0.1 microarcsec units to radians.  
 */  
  
    dpsipl = dp * GAL_U2R ;  
    depspl = de * GAL_U2R ;  
  
/*  
 * -------  
 * RESULTS  
 * -------  
 */  
  
/*  
 * Add luni-solar and planetary components.  
 */  
  
    *dpsi = dpsils + dpsipl ;  
    *deps = depsls + depspl ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
