/*  
 *  - - - - - - - - -  
 *   g a l _ x y 0 6  
 *  - - - - - - - - -  
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  X,Y coordinates of celestial intermediate pole from series based  
 *  on IAU 2006 precession and IAU 2000A nutation.  
 *  
 *  This routine is an independent translation of a FORTRAN routine  
 *  that is part of IAU's SOFA software collection.  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *  
 *     date1,date2         d        TT as a 2-part Julian Date (Note 1)  
 *  
 *  Returned:
 *  
 *     *x,*y               d        CIP X,Y coordinates (Note 2)  
 *  
 *  Notes:  
 *  
 *  1) The TT date date1+date2 is a Julian Date, apportioned in any  
 *     convenient way between the two arguments.  For example,  
 *     JD(TT)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            date1         date2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *  
 *  2) The x,y coordinates are those of the unit vector towards the  
 *     celestial intermediate pole.  They represent the combined effects  
 *     of frame bias, precession and nutation.  
 *  
 *  3) The fundamental arguments used are as adopted in IERS Conventions  
 *     (2003) and are from Simon et al. (1994) and Souchay et al. (1999).  
 *  
 *  4) This is an alternative to the angles-based method, via the SOFA  
 *     routine gal_fw2xy and as used in gal_xys06a for example.  The  
 *     two methods agree at the 1 microarcsecond level (at present),  
 *     a negligible amount compared with the intrinsic accuracy of the  
 *     models.  However, it would be unwise to mix the two methods  
 *     (angles-based and series-based) in a single application.  
 *  
 *  Called:
 *  
 *     gal_fal03          mean anomaly of the Moon  
 *     gal_falp03         mean anomaly of the Sun  
 *     gal_faf03          mean argument of the latitude of the Moon  
 *     gal_fad03          mean elongation of the Moon from the Sun  
 *     gal_faom03         mean longitude of the Moon's ascending node  
 *     gal_fame03         mean longitude of Mercury  
 *     gal_fave03         mean longitude of Venus  
 *     gal_fae03          mean longitude of Earth  
 *     gal_fama03         mean longitude of Mars  
 *     gal_faju03         mean longitude of Jupiter  
 *     gal_fasa03         mean longitude of Saturn  
 *     gal_faur03         mean longitude of Uranus  
 *     gal_fane03         mean longitude of Neptune  
 *     gal_fapa03         general accumulated precession in longitude  
 *  
 *  References:  
 *  
 *     Capitaine, N., Wallace, P.T. & Chapront, J., 2003,  
 *     Astron.Astrophys., 412, 567  
 *  
 *     Capitaine, N. & Wallace, P.T., 2006, Astron.Astrophys. 450, 855  
 *  
 *     McCarthy, D. D., Petit, G. (eds.), 2004, IERS Conventions (2003),  
 *     IERS Technical Note No. 32, BKG  
 *  
 *     Simon, J.L., Bretagnon, P., Chapront, J., Chapront-Touze, M.,  
 *     Francou, G. & Laskar, J., Astron.Astrophys., 1994, 282, 663  
 *  
 *     Souchay, J., Loysel, B., Kinoshita, H., Folgueira, M., 1999,  
 *     Astron.Astrophys.Supp.Ser. 135, 111  
 *  
 *     Wallace, P.T. & Capitaine, N., 2006, Astron.Astrophys. 459, 981  
 *  
 *  This revision:
 *  
 *     2007 June 15 ( c version 2008 January 19 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  
  
#include <math.h>  
#include "gal_const.h"  
#include "gal_xy06.h"  
#include "gal_fal03.h"  
#include "gal_falp03.h"  
#include "gal_faf03.h"  
#include "gal_fad03.h"  
#include "gal_faom03.h"  
#include "gal_fame03.h"  
#include "gal_fave03.h"  
#include "gal_fae03.h"  
#include "gal_fama03.h"  
#include "gal_faju03.h"  
#include "gal_fasa03.h"  
#include "gal_faur03.h"  
#include "gal_fane03.h"  
#include "gal_fapa03.h"  
  
void  
gal_xy06  
 (  
    double date1,  
    double date2,  
    double *x,  
    double *y  
 )  
{  
  
/*  
 * Maximum power of T in the polynomials for X and Y  
 */  
  
#define MAXPT 5  
  
/*  
 * Polynomial coefficients (arcsec).  
 */  
  
    static const double XYP[2][MAXPT+1] = {  
  
     {  
            -0.016617 ,  
          2004.191898 ,  
           -0.4297829 ,  
          -0.19861834 ,  
          0.000007578 ,  
         0.0000059285 ,  
     } ,  
     {  
            -0.006951 ,  
            -0.025896 ,  
          -22.4072747 ,  
           0.00190059 ,  
          0.001112526 ,  
         0.0000001358 ,  
      } ,  
  
    } ;  
  
/*  
 * Numbers of frequencies:  luni-solar, planetary, total  
 */  
  
#define NFLS 653  
#define NFPL 656  
#define NF   NFLS + NFPL  
  
/*  
 * Number of amplitude coefficients  
 */  
  
#define NA 4755  
  
/*  
 * Fundamental-argument multipliers:  luni-solar and planetary terms  
 */  
  
/*  
 * Fundamental-argument multipliers: luni-solar terms.  
 */  
  
    static const int MFALS[NFLS][5] = {  
  
     { 0,  0,  0,  0,  1, } ,  
     { 0,  0,  2, -2,  2, } ,  
     { 0,  0,  2,  0,  2, } ,  
     { 0,  0,  0,  0,  2, } ,  
     { 0,  1,  0,  0,  0, } ,  
     { 0,  1,  2, -2,  2, } ,  
     { 1,  0,  0,  0,  0, } ,  
     { 0,  0,  2,  0,  1, } ,  
     { 1,  0,  2,  0,  2, } ,  
     { 0,  1, -2,  2, -2, } ,  
     { 0,  0,  2, -2,  1, } ,  
     { 1,  0, -2,  0, -2, } ,  
     { 1,  0,  0, -2,  0, } ,  
     { 1,  0,  0,  0,  1, } ,  
     { 1,  0,  0,  0, -1, } ,  
     { 1,  0, -2, -2, -2, } ,  
     { 1,  0,  2,  0,  1, } ,  
     { 2,  0, -2,  0, -1, } ,  
     { 0,  0,  0,  2,  0, } ,  
     { 0,  0,  2,  2,  2, } ,  
     { 2,  0,  0, -2,  0, } ,  
     { 0,  2, -2,  2, -2, } ,  
     { 2,  0,  2,  0,  2, } ,  
     { 1,  0,  2, -2,  2, } ,  
     { 1,  0, -2,  0, -1, } ,  
     { 2,  0,  0,  0,  0, } ,  
     { 0,  0,  2,  0,  0, } ,  
     { 0,  1,  0,  0,  1, } ,  
     { 1,  0,  0, -2, -1, } ,  
     { 0,  2,  2, -2,  2, } ,  
     { 0,  0,  2, -2,  0, } ,  
     { 1,  0,  0, -2,  1, } ,  
     { 0,  1,  0,  0, -1, } ,  
     { 0,  2,  0,  0,  0, } ,  
     { 1,  0, -2, -2, -1, } ,  
     { 1,  0,  2,  2,  2, } ,  
     { 0,  1,  2,  0,  2, } ,  
     { 2,  0, -2,  0,  0, } ,  
     { 0,  0,  2,  2,  1, } ,  
     { 0,  1, -2,  0, -2, } ,  
     { 0,  0,  0,  2,  1, } ,  
     { 1,  0,  2, -2,  1, } ,  
     { 2,  0,  0, -2, -1, } ,  
     { 2,  0,  2, -2,  2, } ,  
     { 2,  0,  2,  0,  1, } ,  
     { 0,  0,  0,  2, -1, } ,  
     { 0,  1, -2,  2, -1, } ,  
     { 1,  1,  0, -2,  0, } ,  
     { 2,  0,  0, -2,  1, } ,  
     { 1,  0,  0,  2,  0, } ,  
     { 0,  1,  2, -2,  1, } ,  
     { 1, -1,  0,  0,  0, } ,  
     { 0,  1, -1,  1, -1, } ,  
     { 2,  0, -2,  0, -2, } ,  
     { 0,  1,  0, -2,  0, } ,  
     { 1,  0,  0, -1,  0, } ,  
     { 3,  0,  2,  0,  2, } ,  
     { 0,  0,  0,  1,  0, } ,  
     { 1, -1,  2,  0,  2, } ,  
     { 1,  1, -2, -2, -2, } ,  
     { 1,  0, -2,  0,  0, } ,  
     { 2,  0,  0,  0, -1, } ,  
     { 0,  1, -2, -2, -2, } ,  
     { 1,  1,  2,  0,  2, } ,  
     { 2,  0,  0,  0,  1, } ,  
     { 1,  1,  0,  0,  0, } ,  
     { 1,  0, -2,  2, -1, } ,  
     { 1,  0,  2,  0,  0, } ,  
     { 1, -1,  0, -1,  0, } ,  
     { 1,  0,  0,  0,  2, } ,  
     { 1,  0, -1,  0, -1, } ,  
     { 0,  0,  2,  1,  2, } ,  
     { 1,  0, -2, -4, -2, } ,  
     { 1, -1,  0, -1, -1, } ,  
     { 1,  0,  2,  2,  1, } ,  
     { 0,  2, -2,  2, -1, } ,  
     { 1,  0,  0,  0, -2, } ,  
     { 2,  0, -2, -2, -2, } ,  
     { 1,  1,  2, -2,  2, } ,  
     { 2,  0, -2, -4, -2, } ,  
     { 1,  0, -4,  0, -2, } ,  
     { 2,  0,  2, -2,  1, } ,  
     { 1,  0,  0, -1, -1, } ,  
     { 2,  0,  2,  2,  2, } ,  
     { 3,  0,  0,  0,  0, } ,  
     { 1,  0,  0,  2,  1, } ,  
     { 0,  0,  2, -2, -1, } ,  
     { 3,  0,  2, -2,  2, } ,  
     { 0,  0,  4, -2,  2, } ,  
     { 1,  0,  0, -4,  0, } ,  
     { 0,  1,  2,  0,  1, } ,  
     { 2,  0,  0, -4,  0, } ,  
     { 1,  1,  0, -2, -1, } ,  
     { 2,  0, -2,  0,  1, } ,  
     { 0,  0,  2,  0, -1, } ,  
     { 0,  1, -2,  0, -1, } ,  
     { 0,  1,  0,  0,  2, } ,  
     { 0,  0,  2, -1,  2, } ,  
     { 0,  0,  2,  4,  2, } ,  
     { 2,  1,  0, -2,  0, } ,  
     { 1,  1,  0, -2,  1, } ,  
     { 1, -1,  0, -2,  0, } ,  
     { 1, -1,  0, -1, -2, } ,  
     { 1, -1,  0,  0,  1, } ,  
     { 0,  1, -2,  2,  0, } ,  
     { 0,  1,  0,  0, -2, } ,  
     { 1, -1,  2,  2,  2, } ,  
     { 1,  0,  0,  2, -1, } ,  
     { 1, -1, -2, -2, -2, } ,  
     { 3,  0,  2,  0,  1, } ,  
     { 0,  1,  2,  2,  2, } ,  
     { 1,  0,  2, -2,  0, } ,  
     { 1,  1, -2, -2, -1, } ,  
     { 1,  0,  2, -4,  1, } ,  
     { 0,  1, -2, -2, -1, } ,  
     { 2, -1,  2,  0,  2, } ,  
     { 0,  0,  0,  2,  2, } ,  
     { 1, -1,  2,  0,  1, } ,  
     { 1, -1, -2,  0, -2, } ,  
     { 0,  1,  0,  2,  0, } ,  
     { 0,  1,  2, -2,  0, } ,  
     { 0,  0,  0,  1,  1, } ,  
     { 1,  0, -2, -2,  0, } ,  
     { 0,  3,  2, -2,  2, } ,  
     { 2,  1,  2,  0,  2, } ,  
     { 1,  1,  0,  0,  1, } ,  
     { 2,  0,  0,  2,  0, } ,  
     { 1,  1,  2,  0,  1, } ,  
     { 1,  0,  0, -2, -2, } ,  
     { 1,  0, -2,  2,  0, } ,  
     { 1,  0, -1,  0, -2, } ,  
     { 0,  1,  0, -2,  1, } ,  
     { 0,  1,  0,  1,  0, } ,  
     { 0,  0,  0,  1, -1, } ,  
     { 1,  0, -2,  2, -2, } ,  
     { 1, -1,  0,  0, -1, } ,  
     { 0,  0,  0,  4,  0, } ,  
     { 1, -1,  0,  2,  0, } ,  
     { 1,  0,  2,  1,  2, } ,  
     { 1,  0,  2, -1,  2, } ,  
     { 0,  0,  2,  1,  1, } ,  
     { 1,  0,  0, -2,  2, } ,  
     { 1,  0, -2,  0,  1, } ,  
     { 1,  0, -2, -4, -1, } ,  
     { 0,  0,  2,  2,  0, } ,  
     { 1,  1,  2, -2,  1, } ,  
     { 1,  0, -2,  1, -1, } ,  
     { 0,  0,  1,  0,  1, } ,  
     { 2,  0, -2, -2, -1, } ,  
     { 4,  0,  2,  0,  2, } ,  
     { 2, -1,  0,  0,  0, } ,  
     { 2,  1,  2, -2,  2, } ,  
     { 0,  1,  2,  1,  2, } ,  
     { 1,  0,  4, -2,  2, } ,  
     { 1,  1,  0,  0, -1, } ,  
     { 2,  0,  2,  0,  0, } ,  
     { 2,  0, -2, -4, -1, } ,  
     { 1,  0, -1,  0,  0, } ,  
     { 1,  0,  0,  1,  0, } ,  
     { 0,  1,  0,  2,  1, } ,  
     { 1,  0, -4,  0, -1, } ,  
     { 1,  0,  0, -4, -1, } ,  
     { 2,  0,  2,  2,  1, } ,  
     { 2,  1,  0,  0,  0, } ,  
     { 0,  0,  2, -3,  2, } ,  
     { 1,  2,  0, -2,  0, } ,  
     { 0,  3,  0,  0,  0, } ,  
     { 0,  0,  4,  0,  2, } ,  
     { 0,  0,  2, -4,  1, } ,  
     { 2,  0,  0, -2, -2, } ,  
     { 1,  1, -2, -4, -2, } ,  
     { 0,  1,  0, -2, -1, } ,  
     { 0,  0,  0,  4,  1, } ,  
     { 3,  0,  2, -2,  1, } ,  
     { 1,  0,  2,  4,  2, } ,  
     { 1,  1, -2,  0, -2, } ,  
     { 0,  0,  4, -2,  1, } ,  
     { 2, -2,  0, -2,  0, } ,  
     { 2,  1,  0, -2, -1, } ,  
     { 0,  2,  0, -2,  0, } ,  
     { 1,  0,  0, -1,  1, } ,  
     { 1,  1,  2,  2,  2, } ,  
     { 3,  0,  0,  0, -1, } ,  
     { 2,  0,  0, -4, -1, } ,  
     { 3,  0,  2,  2,  2, } ,  
     { 0,  0,  2,  4,  1, } ,  
     { 0,  2, -2, -2, -2, } ,  
     { 1, -1,  0, -2, -1, } ,  
     { 0,  0,  2, -1,  1, } ,  
     { 2,  0,  0,  2,  1, } ,  
     { 1, -1, -2,  2, -1, } ,  
     { 0,  0,  0,  2, -2, } ,  
     { 2,  0,  0, -4,  1, } ,  
     { 1,  0,  0, -4,  1, } ,  
     { 2,  0,  2, -4,  1, } ,  
     { 4,  0,  2, -2,  2, } ,  
     { 2,  1, -2,  0, -1, } ,  
     { 2,  1, -2, -4, -2, } ,  
     { 3,  0,  0, -4,  0, } ,  
     { 1, -1,  2,  2,  1, } ,  
     { 1, -1, -2,  0, -1, } ,  
     { 0,  2,  0,  0,  1, } ,  
     { 1,  2, -2, -2, -2, } ,  
     { 1,  1,  0, -4,  0, } ,  
     { 2,  0,  0, -2,  2, } ,  
     { 0,  2,  2, -2,  1, } ,  
     { 1,  0,  2,  0, -1, } ,  
     { 2,  1,  0, -2,  1, } ,  
     { 2, -1, -2,  0, -1, } ,  
     { 1, -1, -2, -2, -1, } ,  
     { 0,  1, -2,  1, -2, } ,  
     { 1,  0, -4,  2, -2, } ,  
     { 0,  1,  2,  2,  1, } ,  
     { 3,  0,  0,  0,  1, } ,  
     { 2, -1,  2,  2,  2, } ,  
     { 0,  1, -2, -4, -2, } ,  
     { 1,  0, -2, -3, -2, } ,  
     { 2,  0,  0,  0,  2, } ,  
     { 1, -1,  0, -2, -2, } ,  
     { 2,  0, -2,  2, -1, } ,  
     { 0,  2, -2,  0, -2, } ,  
     { 3,  0, -2,  0, -1, } ,  
     { 2, -1,  2,  0,  1, } ,  
     { 1,  0, -2, -1, -2, } ,  
     { 0,  0,  2,  0,  3, } ,  
     { 2,  0, -4,  0, -2, } ,  
     { 2,  1,  0, -4,  0, } ,  
     { 1,  1, -2,  1, -1, } ,  
     { 0,  2,  2,  0,  2, } ,  
     { 1, -1,  2, -2,  2, } ,  
     { 1, -1,  0, -2,  1, } ,  
     { 2,  1,  2,  0,  1, } ,  
     { 1,  0,  2, -4,  2, } ,  
     { 1,  1, -2,  0, -1, } ,  
     { 1,  1,  0,  2,  0, } ,  
     { 1,  0,  0, -3,  0, } ,  
     { 2,  0,  2, -1,  2, } ,  
     { 0,  2,  0,  0, -1, } ,  
     { 2, -1,  0, -2,  0, } ,  
     { 4,  0,  0,  0,  0, } ,  
     { 2,  1, -2, -2, -2, } ,  
     { 0,  2, -2,  2,  0, } ,  
     { 1,  0,  2,  1,  1, } ,  
     { 1,  0, -1,  0, -3, } ,  
     { 3, -1,  2,  0,  2, } ,  
     { 2,  0,  2, -2,  0, } ,  
     { 1, -2,  0,  0,  0, } ,  
     { 2,  0,  0,  0, -2, } ,  
     { 1,  0,  0,  4,  0, } ,  
     { 0,  1,  0,  1,  1, } ,  
     { 1,  0,  2,  2,  0, } ,  
     { 0,  1,  0,  2, -1, } ,  
     { 0,  1,  0,  1, -1, } ,  
     { 0,  0,  2, -2,  3, } ,  
     { 3,  1,  2,  0,  2, } ,  
     { 1,  1,  2,  1,  2, } ,  
     { 1,  1, -2,  2, -1, } ,  
     { 2, -1,  2, -2,  2, } ,  
     { 1, -2,  2,  0,  2, } ,  
     { 1,  0,  2, -4,  0, } ,  
     { 0,  0,  1,  0,  0, } ,  
     { 1,  0,  2, -3,  1, } ,  
     { 1, -2,  0, -2,  0, } ,  
     { 2,  0,  0,  2, -1, } ,  
     { 1,  1,  2, -4,  1, } ,  
     { 4,  0,  2,  0,  1, } ,  
     { 0,  1,  2,  1,  1, } ,  
     { 1,  2,  2, -2,  2, } ,  
     { 2,  0,  2,  1,  2, } ,  
     { 2,  1,  2, -2,  1, } ,  
     { 1,  0,  2, -1,  1, } ,  
     { 1,  0,  4, -2,  1, } ,  
     { 1, -1,  2, -2,  1, } ,  
     { 0,  1,  0, -4,  0, } ,  
     { 3,  0, -2, -2, -2, } ,  
     { 0,  0,  4, -4,  2, } ,  
     { 2,  0, -4, -2, -2, } ,  
     { 2, -2,  0, -2, -1, } ,  
     { 1,  0,  2, -2, -1, } ,  
     { 2,  0, -2, -6, -2, } ,  
     { 1,  0, -2,  1, -2, } ,  
     { 1,  0, -2,  2,  1, } ,  
     { 1, -1,  0,  2, -1, } ,  
     { 1,  0, -2,  1,  0, } ,  
     { 2, -1,  0, -2,  1, } ,  
     { 1, -1,  0,  2,  1, } ,  
     { 2,  0, -2, -2,  0, } ,  
     { 1,  0,  2, -3,  2, } ,  
     { 0,  0,  0,  4, -1, } ,  
     { 2, -1,  0,  0,  1, } ,  
     { 2,  0,  4, -2,  2, } ,  
     { 0,  0,  2,  3,  2, } ,  
     { 0,  1,  4, -2,  2, } ,  
     { 0,  1, -2,  2,  1, } ,  
     { 1,  1,  0,  2,  1, } ,  
     { 1,  0,  0,  4,  1, } ,  
     { 0,  0,  4,  0,  1, } ,  
     { 2,  0,  0, -3,  0, } ,  
     { 1,  0,  0, -1, -2, } ,  
     { 1, -2, -2, -2, -2, } ,  
     { 3,  0,  0,  2,  0, } ,  
     { 2,  0,  2, -4,  2, } ,  
     { 1,  1, -2, -4, -1, } ,  
     { 1,  0, -2, -6, -2, } ,  
     { 2, -1,  0,  0, -1, } ,  
     { 2, -1,  0,  2,  0, } ,  
     { 0,  1,  2, -2, -1, } ,  
     { 1,  1,  0,  1,  0, } ,  
     { 1,  2,  0, -2, -1, } ,  
     { 1,  0,  0,  1, -1, } ,  
     { 0,  0,  1,  0,  2, } ,  
     { 3,  1,  2, -2,  2, } ,  
     { 1,  0, -4, -2, -2, } ,  
     { 1,  0,  2,  4,  1, } ,  
     { 1, -2,  2,  2,  2, } ,  
     { 1, -1, -2, -4, -2, } ,  
     { 0,  0,  2, -4,  2, } ,  
     { 0,  0,  2, -3,  1, } ,  
     { 2,  1, -2,  0,  0, } ,  
     { 3,  0, -2, -2, -1, } ,  
     { 2,  0,  2,  4,  2, } ,  
     { 0,  0,  0,  0,  3, } ,  
     { 2, -1, -2, -2, -2, } ,  
     { 2,  0,  0, -1,  0, } ,  
     { 3,  0,  2, -4,  2, } ,  
     { 2,  1,  2,  2,  2, } ,  
     { 0,  0,  3,  0,  3, } ,  
     { 1,  1,  2,  2,  1, } ,  
     { 2,  1,  0,  0, -1, } ,  
     { 1,  2,  0, -2,  1, } ,  
     { 3,  0,  2,  2,  1, } ,  
     { 1, -1, -2,  2, -2, } ,  
     { 1,  1,  0, -1,  0, } ,  
     { 1,  2,  0,  0,  0, } ,  
     { 1,  0,  4,  0,  2, } ,  
     { 1, -1,  2,  4,  2, } ,  
     { 2,  1,  0,  0,  1, } ,  
     { 1,  0,  0,  2,  2, } ,  
     { 1, -1, -2,  2,  0, } ,  
     { 0,  2, -2, -2, -1, } ,  
     { 2,  0, -2,  0,  2, } ,  
     { 5,  0,  2,  0,  2, } ,  
     { 3,  0, -2, -6, -2, } ,  
     { 1, -1,  2, -1,  2, } ,  
     { 3,  0,  0, -4, -1, } ,  
     { 1,  0,  0,  1,  1, } ,  
     { 1,  0, -4,  2, -1, } ,  
     { 0,  1,  2, -4,  1, } ,  
     { 1,  2,  2,  0,  2, } ,  
     { 0,  1,  0, -2, -2, } ,  
     { 0,  0,  2, -1,  0, } ,  
     { 1,  0,  1,  0,  1, } ,  
     { 0,  2,  0, -2,  1, } ,  
     { 3,  0,  2,  0,  0, } ,  
     { 1,  1, -2,  1,  0, } ,  
     { 2,  1, -2, -4, -1, } ,  
     { 3, -1,  0,  0,  0, } ,  
     { 2, -1, -2,  0,  0, } ,  
     { 4,  0,  2, -2,  1, } ,  
     { 2,  0, -2,  2,  0, } ,  
     { 1,  1,  2, -2,  0, } ,  
     { 1,  0, -2,  4, -1, } ,  
     { 1,  0, -2, -2,  1, } ,  
     { 2,  0,  2, -4,  0, } ,  
     { 1,  1,  0, -2, -2, } ,  
     { 1,  1, -2, -2,  0, } ,  
     { 1,  0,  1, -2,  1, } ,  
     { 2, -1, -2, -4, -2, } ,  
     { 3,  0, -2,  0, -2, } ,  
     { 0,  1, -2, -2,  0, } ,  
     { 3,  0,  0, -2, -1, } ,  
     { 1,  0, -2, -3, -1, } ,  
     { 0,  1,  0, -4, -1, } ,  
     { 1, -2,  2, -2,  1, } ,  
     { 0,  1, -2,  1, -1, } ,  
     { 1, -1,  0,  0,  2, } ,  
     { 2,  0,  0,  1,  0, } ,  
     { 1, -2,  0,  2,  0, } ,  
     { 1,  2, -2, -2, -1, } ,  
     { 0,  0,  4, -4,  1, } ,  
     { 0,  1,  2,  4,  2, } ,  
     { 0,  1, -4,  2, -2, } ,  
     { 3,  0, -2,  0,  0, } ,  
     { 2, -1,  2,  2,  1, } ,  
     { 0,  1, -2, -4, -1, } ,  
     { 4,  0,  2,  2,  2, } ,  
     { 2,  0, -2, -3, -2, } ,  
     { 2,  0,  0, -6,  0, } ,  
     { 1,  0,  2,  0,  3, } ,  
     { 3,  1,  0,  0,  0, } ,  
     { 3,  0,  0, -4,  1, } ,  
     { 1, -1,  2,  0,  0, } ,  
     { 1, -1,  0, -4,  0, } ,  
     { 2,  0, -2,  2, -2, } ,  
     { 1,  1,  0, -2,  2, } ,  
     { 4,  0,  0, -2,  0, } ,  
     { 2,  2,  0, -2,  0, } ,  
     { 0,  1,  2,  0,  0, } ,  
     { 1,  1,  0, -4,  1, } ,  
     { 1,  0,  0, -4, -2, } ,  
     { 0,  0,  0,  1,  2, } ,  
     { 3,  0,  0,  2,  1, } ,  
     { 1,  1,  0, -4, -1, } ,  
     { 0,  0,  2,  2, -1, } ,  
     { 1,  1,  2,  0,  0, } ,  
     { 1, -1,  2, -4,  1, } ,  
     { 1,  1,  0,  0,  2, } ,  
     { 0,  0,  2,  6,  2, } ,  
     { 4,  0, -2, -2, -1, } ,  
     { 2,  1,  0, -4, -1, } ,  
     { 0,  0,  0,  3,  1, } ,  
     { 1, -1, -2,  0,  0, } ,  
     { 0,  0,  2,  1,  0, } ,  
     { 1,  0,  0,  2, -2, } ,  
     { 3, -1,  2,  2,  2, } ,  
     { 3, -1,  2, -2,  2, } ,  
     { 1,  0,  0, -1,  2, } ,  
     { 1, -2,  2, -2,  2, } ,  
     { 0,  1,  0,  2,  2, } ,  
     { 0,  1, -2, -1, -2, } ,  
     { 1,  1, -2,  0,  0, } ,  
     { 0,  2,  2, -2,  0, } ,  
     { 3, -1, -2, -1, -2, } ,  
     { 1,  0,  0, -6,  0, } ,  
     { 1,  0, -2, -4,  0, } ,  
     { 2,  1,  0, -4,  1, } ,  
     { 2,  0,  2,  0, -1, } ,  
     { 2,  0, -4,  0, -1, } ,  
     { 0,  0,  3,  0,  2, } ,  
     { 2,  1, -2, -2, -1, } ,  
     { 1, -2,  0,  0,  1, } ,  
     { 2, -1,  0, -4,  0, } ,  
     { 0,  0,  0,  3,  0, } ,  
     { 5,  0,  2, -2,  2, } ,  
     { 1,  2, -2, -4, -2, } ,  
     { 1,  0,  4, -4,  2, } ,  
     { 0,  0,  4, -1,  2, } ,  
     { 3,  1,  0, -4,  0, } ,  
     { 3,  0,  0, -6,  0, } ,  
     { 2,  0,  0,  2,  2, } ,  
     { 2, -2,  2,  0,  2, } ,  
     { 1,  0,  0, -3,  1, } ,  
     { 1, -2, -2,  0, -2, } ,  
     { 1, -1, -2, -3, -2, } ,  
     { 0,  0,  2, -2, -2, } ,  
     { 2,  0, -2, -4,  0, } ,  
     { 1,  0, -4,  0,  0, } ,  
     { 0,  1,  0, -1,  0, } ,  
     { 4,  0,  0,  0, -1, } ,  
     { 3,  0,  2, -1,  2, } ,  
     { 3, -1,  2,  0,  1, } ,  
     { 2,  0,  2, -1,  1, } ,  
     { 1,  2,  2, -2,  1, } ,  
     { 1,  1,  0,  2, -1, } ,  
     { 0,  2,  2,  0,  1, } ,  
     { 3,  1,  2,  0,  1, } ,  
     { 1,  1,  2,  1,  1, } ,  
     { 1,  1,  0, -1,  1, } ,  
     { 1, -2,  0, -2, -1, } ,  
     { 4,  0,  0, -4,  0, } ,  
     { 2,  1,  0,  2,  0, } ,  
     { 1, -1,  0,  4,  0, } ,  
     { 0,  1,  0, -2,  2, } ,  
     { 0,  0,  2,  0, -2, } ,  
     { 1,  0, -1,  0,  1, } ,  
     { 3,  0,  2, -2,  0, } ,  
     { 2,  0,  2,  2,  0, } ,  
     { 1,  2,  0, -4,  0, } ,  
     { 1, -1,  0, -3,  0, } ,  
     { 0,  1,  0,  4,  0, } ,  
     { 0,  1, -2,  0,  0, } ,  
     { 2,  2,  2, -2,  2, } ,  
     { 0,  0,  0,  1, -2, } ,  
     { 0,  2, -2,  0, -1, } ,  
     { 4,  0,  2, -4,  2, } ,  
     { 2,  0, -4,  2, -2, } ,  
     { 2, -1, -2,  0, -2, } ,  
     { 1,  1,  4, -2,  2, } ,  
     { 1,  1,  2, -4,  2, } ,  
     { 1,  0,  2,  3,  2, } ,  
     { 1,  0,  0,  4, -1, } ,  
     { 0,  0,  0,  4,  2, } ,  
     { 2,  0,  0,  4,  0, } ,  
     { 1,  1, -2,  2,  0, } ,  
     { 2,  1,  2,  1,  2, } ,  
     { 2,  1,  2, -4,  1, } ,  
     { 2,  0,  2,  1,  1, } ,  
     { 2,  0, -4, -2, -1, } ,  
     { 2,  0, -2, -6, -1, } ,  
     { 2, -1,  2, -1,  2, } ,  
     { 1, -2,  2,  0,  1, } ,  
     { 1, -2,  0, -2,  1, } ,  
     { 1, -1,  0, -4, -1, } ,  
     { 0,  2,  2,  2,  2, } ,  
     { 0,  2, -2, -4, -2, } ,  
     { 0,  1,  2,  3,  2, } ,  
     { 0,  1,  0, -4,  1, } ,  
     { 3,  0,  0, -2,  1, } ,  
     { 2,  1, -2,  0,  1, } ,  
     { 2,  0,  4, -2,  1, } ,  
     { 2,  0,  0, -3, -1, } ,  
     { 2, -2,  0, -2,  1, } ,  
     { 2, -1,  2, -2,  1, } ,  
     { 1,  0,  0, -6, -1, } ,  
     { 1, -2,  0,  0, -1, } ,  
     { 1, -2, -2, -2, -1, } ,  
     { 0,  1,  4, -2,  1, } ,  
     { 0,  0,  2,  3,  1, } ,  
     { 2, -1,  0, -1,  0, } ,  
     { 1,  3,  0, -2,  0, } ,  
     { 0,  3,  0, -2,  0, } ,  
     { 2, -2,  2, -2,  2, } ,  
     { 0,  0,  4, -2,  0, } ,  
     { 4, -1,  2,  0,  2, } ,  
     { 2,  2, -2, -4, -2, } ,  
     { 4,  1,  2,  0,  2, } ,  
     { 4, -1, -2, -2, -2, } ,  
     { 2,  1,  0, -2, -2, } ,  
     { 2,  1, -2, -6, -2, } ,  
     { 2,  0,  0, -1,  1, } ,  
     { 2, -1, -2,  2, -1, } ,  
     { 1,  1, -2,  2, -2, } ,  
     { 1,  1, -2, -3, -2, } ,  
     { 1,  0,  3,  0,  3, } ,  
     { 1,  0, -2,  1,  1, } ,  
     { 1,  0, -2,  0,  2, } ,  
     { 1, -1,  2,  1,  2, } ,  
     { 1, -1,  0,  0, -2, } ,  
     { 1, -1, -4,  2, -2, } ,  
     { 0,  3, -2, -2, -2, } ,  
     { 0,  1,  0,  4,  1, } ,  
     { 0,  0,  4,  2,  2, } ,  
     { 3,  0, -2, -2,  0, } ,  
     { 2, -2,  0,  0,  0, } ,  
     { 1,  1,  2, -4,  0, } ,  
     { 1,  1,  0, -3,  0, } ,  
     { 1,  0,  2, -3,  0, } ,  
     { 1, -1,  2, -2,  0, } ,  
     { 0,  2,  0,  2,  0, } ,  
     { 0,  0,  2,  4,  0, } ,  
     { 1,  0,  1,  0,  0, } ,  
     { 3,  1,  2, -2,  1, } ,  
     { 3,  0,  4, -2,  2, } ,  
     { 3,  0,  2,  1,  2, } ,  
     { 3,  0,  0,  2, -1, } ,  
     { 3,  0,  0,  0,  2, } ,  
     { 3,  0, -2,  2, -1, } ,  
     { 2,  0,  4, -4,  2, } ,  
     { 2,  0,  2, -3,  2, } ,  
     { 2,  0,  0,  4,  1, } ,  
     { 2,  0,  0, -3,  1, } ,  
     { 2,  0, -4,  2, -1, } ,  
     { 2,  0, -2, -2,  1, } ,  
     { 2, -2,  2,  2,  2, } ,  
     { 2, -2,  0, -2, -2, } ,  
     { 2, -1,  0,  2,  1, } ,  
     { 2, -1,  0,  2, -1, } ,  
     { 1,  1,  2,  4,  2, } ,  
     { 1,  1,  0,  1,  1, } ,  
     { 1,  1,  0,  1, -1, } ,  
     { 1,  1, -2, -6, -2, } ,  
     { 1,  0,  0, -3, -1, } ,  
     { 1,  0, -4, -2, -1, } ,  
     { 1,  0, -2, -6, -1, } ,  
     { 1, -2,  2,  2,  1, } ,  
     { 1, -2, -2,  2, -1, } ,  
     { 1, -1, -2, -4, -1, } ,  
     { 0,  2,  0,  0,  2, } ,  
     { 0,  1,  2, -4,  2, } ,  
     { 0,  1, -2,  4, -1, } ,  
     { 5,  0,  0,  0,  0, } ,  
     { 3,  0,  0, -3,  0, } ,  
     { 2,  2,  0, -4,  0, } ,  
     { 1, -1,  2,  2,  0, } ,  
     { 0,  1,  0,  3,  0, } ,  
     { 4,  0, -2,  0, -1, } ,  
     { 3,  0, -2, -6, -1, } ,  
     { 3,  0, -2, -1, -1, } ,  
     { 2,  1,  2,  2,  1, } ,  
     { 2,  1,  0,  2,  1, } ,  
     { 2,  0,  2,  4,  1, } ,  
     { 2,  0,  2, -6,  1, } ,  
     { 2,  0,  2, -2, -1, } ,  
     { 2,  0,  0, -6, -1, } ,  
     { 2, -1, -2, -2, -1, } ,  
     { 1,  2,  2,  0,  1, } ,  
     { 1,  2,  0,  0,  1, } ,  
     { 1,  0,  4,  0,  1, } ,  
     { 1,  0,  2, -6,  1, } ,  
     { 1,  0,  2, -4, -1, } ,  
     { 1,  0, -1, -2, -1, } ,  
     { 1, -1,  2,  4,  1, } ,  
     { 1, -1,  2, -3,  1, } ,  
     { 1, -1,  0,  4,  1, } ,  
     { 1, -1, -2,  1, -1, } ,  
     { 0,  1,  2, -2,  3, } ,  
     { 3,  0,  0, -2,  0, } ,  
     { 1,  0,  1, -2,  0, } ,  
     { 0,  2,  0, -4,  0, } ,  
     { 0,  0,  2, -4,  0, } ,  
     { 0,  0,  1, -1,  0, } ,  
     { 0,  0,  0,  6,  0, } ,  
     { 0,  2,  0,  0, -2, } ,  
     { 0,  1, -2,  2, -3, } ,  
     { 4,  0,  0,  2,  0, } ,  
     { 3,  0,  0, -1,  0, } ,  
     { 3, -1,  0,  2,  0, } ,  
     { 2,  1,  0,  1,  0, } ,  
     { 2,  1,  0, -6,  0, } ,  
     { 2, -1,  2,  0,  0, } ,  
     { 1,  0,  2, -1,  0, } ,  
     { 1, -1,  0,  1,  0, } ,  
     { 1, -1, -2, -2,  0, } ,  
     { 0,  1,  2,  2,  0, } ,  
     { 0,  0,  2, -3,  0, } ,  
     { 2,  2,  0, -2, -1, } ,  
     { 2, -1, -2,  0,  1, } ,  
     { 1,  2,  2, -4,  1, } ,  
     { 0,  1,  4, -4,  2, } ,  
     { 0,  0,  0,  3,  2, } ,  
     { 5,  0,  2,  0,  1, } ,  
     { 4,  1,  2, -2,  2, } ,  
     { 4,  0, -2, -2,  0, } ,  
     { 3,  1,  2,  2,  2, } ,  
     { 3,  1,  0, -2,  0, } ,  
     { 3,  1, -2, -6, -2, } ,  
     { 3,  0,  0,  0, -2, } ,  
     { 3,  0, -2, -4, -2, } ,  
     { 3, -1,  0, -3,  0, } ,  
     { 3, -1,  0, -2,  0, } ,  
     { 2,  1,  2,  0,  0, } ,  
     { 2,  1,  2, -4,  2, } ,  
     { 2,  1,  2, -2,  0, } ,  
     { 2,  1,  0, -3,  0, } ,  
     { 2,  1, -2,  0, -2, } ,  
     { 2,  0,  0, -4,  2, } ,  
     { 2,  0,  0, -4, -2, } ,  
     { 2,  0, -2, -5, -2, } ,  
     { 2, -1,  2,  4,  2, } ,  
     { 2, -1,  0, -2,  2, } ,  
     { 1,  3, -2, -2, -2, } ,  
     { 1,  1,  0,  0, -2, } ,  
     { 1,  1,  0, -6,  0, } ,  
     { 1,  1, -2,  1, -2, } ,  
     { 1,  1, -2, -1, -2, } ,  
     { 1,  0,  2,  1,  0, } ,  
     { 1,  0,  0,  3,  0, } ,  
     { 1,  0,  0, -4,  2, } ,  
     { 1,  0, -2,  4, -2, } ,  
     { 1, -2,  0, -1,  0, } ,  
     { 0,  1, -4,  2, -1, } ,  
     { 1,  0, -2,  0, -3, } ,  
     { 0,  0,  4, -4,  4, } ,  
  
    } ;  
  
/*  
 * Fundamental-argument multipliers{ planetary terms.  
 */  
  
    static const int MFAPL[NFPL][14] = {  
  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -2,   5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,  -8,  12,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -16,   4,   5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -1,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -13,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   2,  -5,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,  -5,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -6,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -1,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -8,   3,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -8,   3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,   0,  -2,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   1, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -13,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -8,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   1, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,  -1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -7,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,   0,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,  -2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -13,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -1,   0,   0,   0,   0,   0,   2, } ,  
     { 1,   0,   0,   0,   0,   0, -18,  16,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   1,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   1,   0,  -5,   7,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0, -10,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,   0,  -5,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -1,   0,   0,   0,   2, } ,  
     { 1,   0,   2,   0,   2,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -2,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   1, } ,  
     { 1,   0,  -2,   0,  -2,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   2,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -16,   4,   5,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,   3,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -11,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -16,   4,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -6,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -3,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -8,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -2,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -15,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -3,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -2,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -5,   8,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   2,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -2,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   1,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -8,   0,   0,   0,   0,   0,  -1, } ,  
     { 2,   0,   0,  -2,   0,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   8, -13,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,  -2,   5,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,   0,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   2, } ,  
     { 1,   0,   0,   0,  -1,   0, -18,  16,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,   2,  -5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,   1,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,  19, -21,   3,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -8,  13,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7,  -9,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   2, } ,  
     { 1,   0,   0,   0,   1,   0, -18,  16,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -16,   4,   5,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -7,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -7,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -2,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   1, } ,  
     { 2,   0,   0,  -2,   1,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   1,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   2, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -3,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,  -1,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,  -6,   8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -3,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,  -1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -10,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -9,   0,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,   0,  -1,   1,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -7,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -3,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   4,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,  -3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   1,   0,   2,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,   0,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -3,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -4,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   9, -11,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -3,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -15,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -4,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,  -1,   0,   0,   0,   2, } ,  
     { 1,   0,   0,  -1,   1,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,   1,   1,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -4,  10,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   0,  -1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -1,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -4,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,  -2, } ,  
     { 0,   0,   2,  -2,   1,   0,  -4,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,   0,  -1,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -3,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   0,   0,   2,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   1,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -9,  13,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   2,   0,   2,   0,   0,   2,   0,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,   0,   2,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,  -1,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   1, } ,  
     { 1,   0,   2,   0,   1,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 1,   0,  -2,   0,  -1,   0,   0,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -2,   4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,   1,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   2,   0,   2,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -8,   3,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   6, -10,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7,  -8,   3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   1,   0,  -3,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -1,   0,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,  -5,   7,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -2,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7, -10,   0,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   2,  -5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -8,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -9,  15,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -2,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -1,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -4,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,  -1,   0,   0,   2, } ,  
     { 2,   0,   0,  -2,   1,   0,  -6,   8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -5,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   3,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   8, -14,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   8, -15,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7,  -7,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   1,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -1,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   1,   0,   0,   2, } ,  
     { 2,   0,  -1,  -1,   0,   0,   0,   3,  -7,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -7,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -3,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -3,   4,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -6,   8,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -5,   6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   2,   1,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   2,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -9,   4,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -4,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   7, -11,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   4,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,  -1,   1,   0,   0,   0, } ,  
     { 2,   0,   0,   0,   0,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -15,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -6,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -1,   1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -2,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -7,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   2,  -4,   0,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   3,  -5,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -3,   0,   0,   0,   2, } ,  
     { 0,   0,   2,  -2,   2,   0,  -8,  11,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,  -2,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   1,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -9,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -5,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   7,  -9,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -7,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,  -2,  -2,  -2,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -2,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -3,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   2,  -5,   0,   0,   2, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -2,   0,   0,   5,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,  -6,   8,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8,  -8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,   2,  -5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   1,   0,   3,  -7,   4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,  -2,   5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -1,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -3,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,  11,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -15,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,   1,   0,   0,   0,   2, } ,  
     { 1,   0,   0,  -1,   0,   0,   0,  -3,   4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -3,   7,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,   0,  -2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   2,  -2,   2,   0,  -5,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   2,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -5,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -7,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -11,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -3,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   9, -12,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   1,  -1,   0,   0,  -8,  12,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -2,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7,  -7,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -6,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   1,   0,  -4,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -9,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,  -1,   0,   0,   0,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -5,   0,   0,   0,   0,  -2, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   3,  -1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,  -2,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -9,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   9,  -9,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   3,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   2,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -3,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   1, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -9,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -3,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   0,   0,   0,   2, } ,  
     { 0,   0,   2,   0,   2,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,   0,  -3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,  -1,   0,   0,  -1,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -3,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -2,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   5, -10,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -13,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   2,  -2,   1,  -1,   0,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   0,   2,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   3,  -5,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   9,  -9,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -8,  11,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -2,   0,   0,   2,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -1,   2,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -5,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -6,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   8, -15,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -2,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7, -13,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   3,   0,   0,   0,   2, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8,  -8,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -10,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -2,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -4,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -4,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -5,   6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   0,   0,  -2, } ,  
     { 2,   0,  -1,  -1,  -1,   0,   0,   3,  -7,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -8,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,  -1,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   4,  -3,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -11,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   1,   0,   0,  -6,   8,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   1,   5,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -5,   0,   0,   0,   0,   2, } ,  
     { 1,   0,  -2,  -2,  -2,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,   0,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   2,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   2,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -7,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,   0,  -2,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,   0,  -2,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -1,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -6,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   4,  -5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -5,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7, -13,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -2,   0,   0,   0,   2, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   0,   2,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -8,  15,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -2,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,  -1,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,   2,  -2,   2,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,   1,  -1,   0, -18,  17,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,  -1,   0,  -5,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   2,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -16,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   2, } ,  
     { 0,   0,   0,   0,   2,   0,   0,  -1,   2,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,  -2,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6, -10,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,  -2,   4,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   2,   0,   0,   0,   0,   2, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -2,   0,   4,  -5,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,  -1,   0,   0,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,  -1,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,  -1,  -1,   0,  20, -20,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   1,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -2,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   5,  -8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   0,   0,   0,  -1,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   9, -11,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -3,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -3,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -7,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,  -2,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,  -2,   5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -7,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -8,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -6,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,   0,  -2,   0,   0,  20, -21,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -12,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -12,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   9, -17,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   1,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -6,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -7,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,   0,  -1,   1,   0,   0,  -3,   4,   0,   0,   0,   0,   0, } ,  
     { 1,   0,  -2,   0,  -2,   0, -10,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -9,  17,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -4,   0,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,  -2,  -2,  -2,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,   1,  -1,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   2,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,   0,   1,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,  -5,   7,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   2,  -2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -5,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -4,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5, -10,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,  -4,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,  -5,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -5,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -2,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -2,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -3,   0,   0,   0,   0,   0,   1, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -7,   4,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   2,   0,   1,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,  -1,   0,   0,  -1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   1,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -10,   0,   0,   0,   0,  -2, } ,  
     { 1,   0,   0,  -1,   1,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -3,   0,   3,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,  -5,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   1,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -4,   6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   0,   0,  -1,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -5,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   3,  -4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7, -10,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -5,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -5,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -8,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   2,  -5,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7,  -9,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7,  -8,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -8,   3,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,  -2,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -4,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -1,   0,   0,   0,  -1, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -6,   8,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,   1,   0,   0,   3,  -7,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -7,   9,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   1,  -1,   2,   0,  -8,  12,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   2,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7,  -8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   1,   0,   0,  -5,   6,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,  -1,   0,   0,  -2,   0,   3,  -1,   0,   0,   0, } ,  
     { 1,   0,   1,   1,   1,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   1,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,  -1,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,  -1,   0,   0,  -3,   4,   0,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,   0,  -1,   0,  -3,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -4,   4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,  -8,  11,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,   0,   0,  -9,  13,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,   1,   2,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,   1,  -4,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,   0,  -1,   0,   1,  -3,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   7, -13,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,   2,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   1,   0,  -4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   7, -11,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -6,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -4,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -4,   0,   0,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -4,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   9, -17,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7,  -7,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   3,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   3,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -8,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,  -7,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -4,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,  -1,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,  17, -16,   0,  -2,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -1,   0,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -9,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -4,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,  -2,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   1,   0,   0,   0,   0,   2, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -4,   4,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   2,   2,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,   0,  -4,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,   1,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   3,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -2,   2,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   0,   1,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,  -4,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   2,   0,   0,   0,  -1,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   8,  -9,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   3,  -5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -2,   0,   0,   0, } ,  
     { 2,   0,  -2,  -2,  -2,   0,   0,  -2,   0,   2,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   1,   0, -10,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,  -1,   0, -10,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   2,  -3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,   2,  -2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,  -2,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   0,   2,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   2,   0,   0,   0,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,   0,  -1,   0,   2,   0,   0,   0,   0, } ,  
     { 2,   0,   2,  -2,   2,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 2,   0,   1,  -3,   1,   0,  -6,   7,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   2,  -5,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   5,  -5,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   1,   5,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   0,   5,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,   0,  -2,   0,   0,   2,   0,   0,   0, } ,  
     { 2,   0,   0,  -2,   0,   0,  -4,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -2,   0,  -2,   0,   0,   5,  -9,   0,   0,   0,   0,   0, } ,  
     { 2,   0,  -1,  -1,   0,   0,   0,  -1,   0,   3,   0,   0,   0,   0, } ,  
     { 1,   0,   2,   0,   2,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   2,   0,   2,   0,   0,   4,  -8,   3,   0,   0,   0,   0, } ,  
     { 1,   0,   2,   0,   2,   0,   0,  -4,   8,  -3,   0,   0,   0,   0, } ,  
     { 1,   0,   2,   0,   2,   0,  -1,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   2,  -2,   2,   0,  -3,   3,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 1,   0,   0,   0,   0,   0,   0,  -2,   0,   3,   0,   0,   0,   0, } ,  
     { 1,   0,   0,  -2,   0,   0,   0,   2,   0,  -2,   0,   0,   0,   0, } ,  
     { 1,   0,  -2,  -2,  -2,   0,   0,   1,   0,  -1,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,   1,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 1,   0,  -1,  -1,   0,   0,   0,   8, -15,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,   2,   2,   0,   0,   2,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   1,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0,  -2,   0,   1,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   1,   0,   0, -10,  15,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   2,  -2,   0,  -1,   0,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,   0,  -1,   0,   0,  -1,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   2,   0,  -3,   4,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -4,   6,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   1,   0,  -1,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,   0,  -1,   0,   0,  -2,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,   0,   0,  -1,   0,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   1,  -1,  -1,   0,  -5,   7,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   2,   0,   0,   0,   2,   0,  -2,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   2,   0,   0,  -2,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   2,   0,  -3,   5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   1,   0,  -1,   2,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   9, -13,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -14,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   8, -11,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -9,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -8,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   6,  -7,   0,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -6,  -4,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   5,  -4,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -8,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   4,  -5,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -3,   0,   2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   3,  -1,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   1,  -1,   0,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   7, -12,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -9,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -8,   1,   5,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6,  -4,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   6, -10,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,   0,  -4,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -9,   0,   0,   0,   0,  -1, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -8,   3,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -7,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5,  -6,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5, -16,   4,   5,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   5, -13,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,   0,  -5,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -9,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   3,  -7,   0,   0,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   2,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,   0,   0,  -3,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   2,  -8,   1,   5,   0,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   1,  -5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   2,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,   0,  -3,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,   0,  -3,   5,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   1,  -3,   0,   0,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   2,  -6,   3,   0,  -2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   1,  -2,   0,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2, } ,  
     { 0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0, } ,  
  
    } ;  
  
/*  
 * Pointers into amplitudes array, one pointer per frequency  
 */  
  
    static const int NC[NF] = {  
  
         1,   21,   37,   51,   65,   79,   91,  103,  115,  127,  
       139,  151,  163,  172,  184,  196,  207,  219,  231,  240,  
       252,  261,  273,  285,  297,  309,  318,  327,  339,  351,  
       363,  372,  384,  396,  405,  415,  423,  435,  444,  452,  
       460,  467,  474,  482,  490,  498,  506,  513,  521,  528,  
       536,  543,  551,  559,  566,  574,  582,  590,  597,  605,  
       613,  620,  628,  636,  644,  651,  658,  666,  674,  680,  
       687,  695,  702,  710,  717,  725,  732,  739,  746,  753,  
       760,  767,  774,  782,  790,  798,  805,  812,  819,  826,  
       833,  840,  846,  853,  860,  867,  874,  881,  888,  895,  
       901,  908,  914,  921,  928,  934,  941,  948,  955,  962,  
       969,  976,  982,  989,  996, 1003, 1010, 1017, 1024, 1031,  
      1037, 1043, 1050, 1057, 1064, 1071, 1078, 1084, 1091, 1098,  
      1104, 1112, 1118, 1124, 1131, 1138, 1145, 1151, 1157, 1164,  
      1171, 1178, 1185, 1192, 1199, 1205, 1212, 1218, 1226, 1232,  
      1239, 1245, 1252, 1259, 1266, 1272, 1278, 1284, 1292, 1298,  
      1304, 1310, 1316, 1323, 1329, 1335, 1341, 1347, 1353, 1359,  
      1365, 1371, 1377, 1383, 1389, 1396, 1402, 1408, 1414, 1420,  
      1426, 1434, 1440, 1446, 1452, 1459, 1465, 1471, 1477, 1482,  
      1488, 1493, 1499, 1504, 1509, 1514, 1520, 1527, 1532, 1538,  
      1543, 1548, 1553, 1558, 1564, 1569, 1574, 1579, 1584, 1589,  
      1594, 1596, 1598, 1600, 1602, 1605, 1608, 1610, 1612, 1617,  
      1619, 1623, 1625, 1627, 1629, 1632, 1634, 1640, 1642, 1644,  
      1646, 1648, 1650, 1652, 1654, 1658, 1660, 1662, 1664, 1668,  
      1670, 1672, 1673, 1675, 1679, 1681, 1683, 1684, 1686, 1688,  
      1690, 1693, 1695, 1697, 1701, 1703, 1705, 1707, 1709, 1711,  
      1712, 1715, 1717, 1721, 1723, 1725, 1727, 1729, 1731, 1733,  
      1735, 1737, 1739, 1741, 1743, 1745, 1747, 1749, 1751, 1753,  
      1755, 1757, 1759, 1761, 1762, 1764, 1766, 1768, 1769, 1771,  
      1773, 1775, 1777, 1779, 1781, 1783, 1785, 1787, 1788, 1790,  
      1792, 1794, 1796, 1798, 1800, 1802, 1804, 1806, 1807, 1809,  
      1811, 1815, 1817, 1819, 1821, 1823, 1825, 1827, 1829, 1831,  
      1833, 1835, 1837, 1839, 1840, 1842, 1844, 1848, 1850, 1852,  
      1854, 1856, 1858, 1859, 1860, 1862, 1864, 1866, 1868, 1869,  
      1871, 1873, 1875, 1877, 1879, 1881, 1883, 1885, 1887, 1889,  
      1891, 1892, 1896, 1898, 1900, 1901, 1903, 1905, 1907, 1909,  
      1910, 1911, 1913, 1915, 1919, 1921, 1923, 1927, 1929, 1931,  
      1933, 1935, 1937, 1939, 1943, 1945, 1947, 1948, 1949, 1951,  
      1953, 1955, 1957, 1958, 1960, 1962, 1964, 1966, 1968, 1970,  
      1971, 1973, 1974, 1975, 1977, 1979, 1980, 1981, 1982, 1984,  
      1986, 1988, 1990, 1992, 1994, 1995, 1997, 1999, 2001, 2003,  
      2005, 2007, 2008, 2009, 2011, 2013, 2015, 2017, 2019, 2021,  
      2023, 2024, 2025, 2027, 2029, 2031, 2033, 2035, 2037, 2041,  
      2043, 2045, 2046, 2047, 2049, 2051, 2053, 2055, 2056, 2057,  
      2059, 2061, 2063, 2065, 2067, 2069, 2070, 2071, 2072, 2074,  
      2076, 2078, 2080, 2082, 2084, 2086, 2088, 2090, 2092, 2094,  
      2095, 2096, 2097, 2099, 2101, 2105, 2106, 2107, 2108, 2109,  
      2110, 2111, 2113, 2115, 2119, 2121, 2123, 2125, 2127, 2129,  
      2131, 2133, 2135, 2136, 2137, 2139, 2141, 2143, 2145, 2147,  
      2149, 2151, 2153, 2155, 2157, 2159, 2161, 2163, 2165, 2167,  
      2169, 2171, 2173, 2175, 2177, 2179, 2181, 2183, 2185, 2186,  
      2187, 2188, 2192, 2193, 2195, 2197, 2199, 2201, 2203, 2205,  
      2207, 2209, 2211, 2213, 2217, 2219, 2221, 2223, 2225, 2227,  
      2229, 2231, 2233, 2234, 2235, 2236, 2237, 2238, 2239, 2240,  
      2241, 2244, 2246, 2248, 2250, 2252, 2254, 2256, 2258, 2260,  
      2262, 2264, 2266, 2268, 2270, 2272, 2274, 2276, 2278, 2280,  
      2282, 2284, 2286, 2288, 2290, 2292, 2294, 2296, 2298, 2300,  
      2302, 2303, 2304, 2305, 2306, 2307, 2309, 2311, 2313, 2315,  
      2317, 2319, 2321, 2323, 2325, 2327, 2329, 2331, 2333, 2335,  
      2337, 2341, 2343, 2345, 2347, 2349, 2351, 2352, 2355, 2356,  
      2357, 2358, 2359, 2361, 2363, 2364, 2365, 2366, 2367, 2368,  
      2369, 2370, 2371, 2372, 2373, 2374, 2376, 2378, 2380, 2382,  
      2384, 2385, 2386, 2387, 2388, 2389, 2390, 2391, 2392, 2393,  
      2394, 2395, 2396, 2397, 2398, 2399, 2400, 2401, 2402, 2403,  
      2404, 2405, 2406, 2407, 2408, 2409, 2410, 2411, 2412, 2413,  
      2414, 2415, 2417, 2418, 2430, 2438, 2445, 2453, 2460, 2468,  
      2474, 2480, 2488, 2496, 2504, 2512, 2520, 2527, 2535, 2543,  
      2550, 2558, 2566, 2574, 2580, 2588, 2596, 2604, 2612, 2619,  
      2627, 2634, 2642, 2648, 2656, 2664, 2671, 2679, 2685, 2693,  
      2701, 2709, 2717, 2725, 2733, 2739, 2747, 2753, 2761, 2769,  
      2777, 2785, 2793, 2801, 2809, 2817, 2825, 2833, 2841, 2848,  
      2856, 2864, 2872, 2878, 2884, 2892, 2898, 2906, 2914, 2922,  
      2930, 2938, 2944, 2952, 2958, 2966, 2974, 2982, 2988, 2996,  
      3001, 3009, 3017, 3025, 3032, 3039, 3045, 3052, 3059, 3067,  
      3069, 3076, 3083, 3090, 3098, 3105, 3109, 3111, 3113, 3120,  
      3124, 3128, 3132, 3136, 3140, 3144, 3146, 3150, 3158, 3161,  
      3165, 3166, 3168, 3172, 3176, 3180, 3182, 3185, 3189, 3193,  
      3194, 3197, 3200, 3204, 3208, 3212, 3216, 3219, 3221, 3222,  
      3226, 3230, 3234, 3238, 3242, 3243, 3247, 3251, 3254, 3258,  
      3262, 3266, 3270, 3274, 3275, 3279, 3283, 3287, 3289, 3293,  
      3296, 3300, 3303, 3307, 3311, 3315, 3319, 3321, 3324, 3327,  
      3330, 3334, 3338, 3340, 3342, 3346, 3350, 3354, 3358, 3361,  
      3365, 3369, 3373, 3377, 3381, 3385, 3389, 3393, 3394, 3398,  
      3402, 3406, 3410, 3413, 3417, 3421, 3425, 3429, 3433, 3435,  
      3439, 3443, 3446, 3450, 3453, 3457, 3458, 3461, 3464, 3468,  
      3472, 3476, 3478, 3481, 3485, 3489, 3493, 3497, 3501, 3505,  
      3507, 3511, 3514, 3517, 3521, 3524, 3525, 3527, 3529, 3533,  
      3536, 3540, 3541, 3545, 3548, 3551, 3555, 3559, 3563, 3567,  
      3569, 3570, 3574, 3576, 3578, 3582, 3586, 3590, 3593, 3596,  
      3600, 3604, 3608, 3612, 3616, 3620, 3623, 3626, 3630, 3632,  
      3636, 3640, 3643, 3646, 3648, 3652, 3656, 3660, 3664, 3667,  
      3669, 3671, 3675, 3679, 3683, 3687, 3689, 3693, 3694, 3695,  
      3699, 3703, 3705, 3707, 3710, 3713, 3717, 3721, 3725, 3729,  
      3733, 3736, 3740, 3744, 3748, 3752, 3754, 3757, 3759, 3763,  
      3767, 3770, 3773, 3777, 3779, 3783, 3786, 3790, 3794, 3798,  
      3801, 3805, 3809, 3813, 3817, 3821, 3825, 3827, 3831, 3835,  
      3836, 3837, 3840, 3844, 3848, 3852, 3856, 3859, 3863, 3867,  
      3869, 3871, 3875, 3879, 3883, 3887, 3890, 3894, 3898, 3901,  
      3905, 3909, 3913, 3917, 3921, 3922, 3923, 3924, 3926, 3930,  
      3932, 3936, 3938, 3940, 3944, 3948, 3952, 3956, 3959, 3963,  
      3965, 3969, 3973, 3977, 3979, 3981, 3982, 3986, 3989, 3993,  
      3997, 4001, 4004, 4006, 4009, 4012, 4016, 4020, 4024, 4026,  
      4028, 4032, 4036, 4040, 4044, 4046, 4050, 4054, 4058, 4060,  
      4062, 4063, 4064, 4068, 4071, 4075, 4077, 4081, 4083, 4087,  
      4089, 4091, 4095, 4099, 4101, 4103, 4105, 4107, 4111, 4115,  
      4119, 4123, 4127, 4129, 4131, 4135, 4139, 4141, 4143, 4145,  
      4149, 4153, 4157, 4161, 4165, 4169, 4173, 4177, 4180, 4183,  
      4187, 4191, 4195, 4198, 4201, 4205, 4209, 4212, 4213, 4216,  
      4217, 4221, 4223, 4226, 4230, 4234, 4236, 4240, 4244, 4248,  
      4252, 4256, 4258, 4262, 4264, 4266, 4268, 4270, 4272, 4276,  
      4279, 4283, 4285, 4287, 4289, 4293, 4295, 4299, 4300, 4301,  
      4305, 4309, 4313, 4317, 4319, 4323, 4325, 4329, 4331, 4333,  
      4335, 4337, 4341, 4345, 4349, 4351, 4353, 4357, 4361, 4365,  
      4367, 4369, 4373, 4377, 4381, 4383, 4387, 4389, 4391, 4395,  
      4399, 4403, 4407, 4411, 4413, 4414, 4415, 4418, 4419, 4421,  
      4423, 4427, 4429, 4431, 4433, 4435, 4437, 4439, 4443, 4446,  
      4450, 4452, 4456, 4458, 4460, 4462, 4466, 4469, 4473, 4477,  
      4481, 4483, 4487, 4489, 4491, 4493, 4497, 4499, 4501, 4504,  
      4506, 4510, 4513, 4514, 4515, 4518, 4521, 4522, 4525, 4526,  
      4527, 4530, 4533, 4534, 4537, 4541, 4542, 4543, 4544, 4545,  
      4546, 4547, 4550, 4553, 4554, 4555, 4558, 4561, 4564, 4567,  
      4568, 4571, 4574, 4575, 4578, 4581, 4582, 4585, 4586, 4588,  
      4590, 4592, 4596, 4598, 4602, 4604, 4608, 4612, 4613, 4616,  
      4619, 4622, 4623, 4624, 4625, 4626, 4629, 4632, 4633, 4636,  
      4639, 4640, 4641, 4642, 4643, 4644, 4645, 4648, 4649, 4650,  
      4651, 4652, 4653, 4656, 4657, 4660, 4661, 4664, 4667, 4670,  
      4671, 4674, 4675, 4676, 4677, 4678, 4681, 4682, 4683, 4684,  
      4687, 4688, 4689, 4692, 4693, 4696, 4697, 4700, 4701, 4702,  
      4703, 4704, 4707, 4708, 4711, 4712, 4715, 4716, 4717, 4718,  
      4719, 4720, 4721, 4722, 4723, 4726, 4729, 4730, 4733, 4736,  
      4737, 4740, 4741, 4742, 4745, 4746, 4749, 4752, 4753,  
  
    } ;  
  
/*  
 * Amplitude coefficients (microarcsec);  indexed using the NC array  
 */  
  
    static const double A[NA] = {  
  
-6844318.44e0, 9205236.26e0,    1328.67e0,    1538.18e0,  205833.11e0,  
  153041.79e0,   -3309.73e0,     853.32e0,    2037.98e0,   -2301.27e0,      81.46e0,  
     120.56e0,     -20.39e0,     -15.22e0,       1.73e0,      -1.61e0,       -0.1e0,       0.11e0,      -0.02e0,  
      -0.02e0, -523908.04e0,  573033.42e0,    -544.75e0,    -458.66e0,   12814.01e0,  
   11714.49e0,     198.97e0,    -290.91e0,     155.74e0,    -143.27e0,      -2.75e0,      -1.03e0,  
      -1.27e0,      -1.16e0,          0e0,      -0.01e0,  -90552.22e0,   97846.69e0,     111.23e0,  
     137.41e0,    2187.91e0,    2024.68e0,      41.44e0,     -51.26e0,      26.92e0,     -24.46e0,  
      -0.46e0,      -0.28e0,      -0.22e0,       -0.2e0,   82168.76e0,  -89618.24e0,     -27.64e0,  
     -29.05e0,   -2004.36e0,   -1837.32e0,     -36.07e0,         48e0,     -24.43e0,      22.41e0,  
       0.47e0,       0.24e0,        0.2e0,       0.18e0,   58707.02e0,    7387.02e0,     470.05e0,  
     -192.4e0,     164.33e0,   -1312.21e0,    -179.73e0,     -28.93e0,     -17.36e0,      -1.83e0,  
       -0.5e0,       3.57e0,          0e0,       0.13e0,  -20557.78e0,   22438.42e0,     -20.84e0,  
      -17.4e0,     501.82e0,     459.68e0,       59.2e0,      -67.3e0,       6.08e0,      -5.61e0,      -1.36e0,  
      -1.19e0,   28288.28e0,    -674.99e0,     -34.69e0,       35.8e0,     -15.07e0,    -632.54e0,  
     -11.19e0,       0.78e0,      -8.41e0,       0.17e0,       0.01e0,       0.07e0,  -15406.85e0,  
    20069.5e0,      15.12e0,       31.8e0,     448.76e0,      344.5e0,      -5.77e0,       1.41e0,       4.59e0,  
      -5.02e0,       0.17e0,       0.24e0,  -11991.74e0,   12902.66e0,      32.46e0,       36.7e0,  
     288.49e0,     268.14e0,        5.7e0,      -7.06e0,       3.57e0,      -3.23e0,      -0.06e0,      -0.04e0,  
   -8584.95e0,   -9592.72e0,       4.42e0,      -13.2e0,     -214.5e0,     192.06e0,      23.87e0,  
      29.83e0,       2.54e0,        2.4e0,        0.6e0,      -0.48e0,     5095.5e0,   -6918.22e0,  
       7.19e0,       3.92e0,    -154.91e0,    -113.94e0,       2.86e0,      -1.04e0,      -1.52e0,  
       1.73e0,      -0.07e0,       -0.1e0,   -4910.93e0,   -5331.13e0,       0.76e0,        0.4e0,  
    -119.21e0,     109.81e0,       2.16e0,        3.2e0,       1.46e0,       1.33e0,       0.04e0,      -0.02e0,  
   -6245.02e0,    -123.48e0,      -6.68e0,       -8.2e0,      -2.76e0,     139.64e0,       2.71e0,  
       0.15e0,       1.86e0,    2511.85e0,   -3323.89e0,       1.07e0,       -0.9e0,     -74.33e0,  
     -56.17e0,       1.16e0,      -0.01e0,      -0.75e0,       0.83e0,      -0.02e0,      -0.04e0,  
    2307.58e0,    3143.98e0,      -7.52e0,        7.5e0,      70.31e0,      -51.6e0,       1.46e0,       0.16e0,  
      -0.69e0,      -0.79e0,       0.02e0,      -0.05e0,    2372.58e0,    2554.51e0,       5.93e0,  
       -6.6e0,      57.12e0,     -53.05e0,      -0.96e0,      -1.24e0,      -0.71e0,      -0.64e0,      -0.01e0,  
   -2053.16e0,    2636.13e0,       5.13e0,        7.8e0,      58.94e0,      45.91e0,      -0.42e0,  
      -0.12e0,       0.61e0,      -0.66e0,       0.02e0,       0.03e0,   -1825.49e0,   -2423.59e0,  
       1.23e0,         -2e0,     -54.19e0,      40.82e0,      -1.07e0,      -1.02e0,       0.54e0,       0.61e0,  
      -0.04e0,       0.04e0,    2521.07e0,    -122.28e0,      -5.97e0,        2.9e0,      -2.73e0,  
     -56.37e0,      -0.82e0,       0.13e0,      -0.75e0,   -1534.09e0,    1645.01e0,       6.29e0,  
        6.8e0,      36.78e0,       34.3e0,       0.92e0,      -1.25e0,       0.46e0,      -0.41e0,      -0.02e0,  
      -0.01e0,    1898.27e0,       47.7e0,      -0.72e0,        2.5e0,       1.07e0,     -42.45e0,      -0.94e0,  
       0.02e0,      -0.56e0,   -1292.02e0,      -1387e0,          0e0,          0e0,     -31.01e0,      28.89e0,  
       0.68e0,          0e0,       0.38e0,       0.35e0,      -0.01e0,      -0.01e0,   -1234.96e0,  
    1323.81e0,       5.21e0,        5.9e0,       29.6e0,      27.61e0,       0.74e0,      -1.22e0,       0.37e0,  
      -0.33e0,      -0.02e0,      -0.01e0,    1137.48e0,   -1233.89e0,      -0.04e0,       -0.3e0,  
     -27.59e0,     -25.43e0,      -0.61e0,          1e0,      -0.34e0,       0.31e0,       0.01e0,       0.01e0,  
    -813.13e0,    -1075.6e0,        0.4e0,        0.3e0,     -24.05e0,      18.18e0,       -0.4e0,      -0.01e0,  
       0.24e0,       0.27e0,      -0.01e0,       0.01e0,    1163.22e0,      -60.9e0,      -2.94e0,        1.3e0,  
      -1.36e0,     -26.01e0,      -0.58e0,       0.07e0,      -0.35e0,     1029.7e0,     -55.55e0,  
      -2.63e0,        1.1e0,      -1.25e0,     -23.02e0,      -0.52e0,       0.06e0,      -0.31e0,  
    -556.26e0,     852.85e0,       3.16e0,      -4.48e0,      19.06e0,      12.44e0,      -0.81e0,  
      -0.27e0,       0.17e0,      -0.21e0,          0e0,       0.02e0,    -603.52e0,    -800.34e0,  
       0.44e0,        0.1e0,      -17.9e0,      13.49e0,      -0.08e0,      -0.01e0,       0.18e0,        0.2e0,  
      -0.01e0,       0.01e0,    -628.24e0,     684.99e0,      -0.64e0,       -0.5e0,      15.32e0,  
      14.05e0,       3.18e0,      -4.19e0,       0.19e0,      -0.17e0,      -0.09e0,      -0.07e0,  
    -866.48e0,     -16.26e0,       0.52e0,       -1.3e0,      -0.36e0,      19.37e0,       0.43e0,      -0.01e0,  
       0.26e0,    -512.37e0,     695.54e0,      -1.47e0,       -1.4e0,      15.55e0,      11.46e0,  
      -0.16e0,       0.03e0,       0.15e0,      -0.17e0,       0.01e0,       0.01e0,     506.65e0,  
     643.75e0,       2.54e0,      -2.62e0,       14.4e0,     -11.33e0,      -0.77e0,      -0.06e0,      -0.15e0,  
      -0.16e0,          0e0,       0.01e0,     664.57e0,      16.81e0,       -0.4e0,          1e0,       0.38e0,  
     -14.86e0,      -3.71e0,      -0.09e0,       -0.2e0,     405.91e0,     522.11e0,       0.99e0,       -1.5e0,  
      11.67e0,      -9.08e0,      -0.25e0,      -0.02e0,      -0.12e0,      -0.13e0,    -305.78e0,  
      326.6e0,       1.75e0,        1.9e0,        7.3e0,       6.84e0,        0.2e0,      -0.04e0,     300.99e0,  
    -325.03e0,      -0.44e0,       -0.5e0,      -7.27e0,      -6.73e0,      -1.01e0,       0.01e0,          0e0,  
       0.08e0,          0e0,       0.02e0,     438.51e0,      10.47e0,      -0.56e0,       -0.2e0,       0.24e0,  
      -9.81e0,      -0.24e0,       0.01e0,      -0.13e0,    -264.02e0,     335.24e0,       0.99e0,        1.4e0,  
       7.49e0,        5.9e0,      -0.27e0,      -0.02e0,     284.09e0,     307.03e0,       0.32e0,       -0.4e0,  
       6.87e0,      -6.35e0,      -0.99e0,      -0.01e0,    -250.54e0,     327.11e0,       0.08e0,        0.4e0,  
       7.31e0,        5.6e0,       -0.3e0,     230.72e0,    -304.46e0,       0.08e0,       -0.1e0,      -6.81e0,  
      -5.16e0,       0.27e0,     229.78e0,     304.17e0,       -0.6e0,        0.5e0,        6.8e0,      -5.14e0,  
       0.33e0,       0.01e0,      256.3e0,    -276.81e0,      -0.28e0,       -0.4e0,      -6.19e0,      -5.73e0,  
      -0.14e0,       0.01e0,    -212.82e0,     269.45e0,       0.84e0,        1.2e0,       6.02e0,       4.76e0,  
       0.14e0,      -0.02e0,     196.64e0,     272.05e0,      -0.84e0,        0.9e0,       6.08e0,       -4.4e0,  
       0.35e0,       0.02e0,     188.95e0,     272.22e0,      -0.12e0,        0.3e0,       6.09e0,      -4.22e0,  
       0.34e0,    -292.37e0,       -5.1e0,      -0.32e0,       -0.4e0,      -0.11e0,       6.54e0,       0.14e0,  
       0.01e0,     161.79e0,    -220.67e0,       0.24e0,        0.1e0,      -4.93e0,      -3.62e0,      -0.08e0,  
     261.54e0,     -19.94e0,      -0.95e0,        0.2e0,      -0.45e0,      -5.85e0,      -0.13e0,       0.02e0,  
     142.16e0,    -190.79e0,        0.2e0,        0.1e0,      -4.27e0,      -3.18e0,      -0.07e0,     187.95e0,  
      -4.11e0,      -0.24e0,        0.3e0,      -0.09e0,       -4.2e0,      -0.09e0,       0.01e0,          0e0,  
          0e0,     -79.08e0,      167.9e0,       0.04e0,          0e0,       3.75e0,       1.77e0,     121.98e0,  
     131.04e0,      -0.08e0,        0.1e0,       2.93e0,      -2.73e0,      -0.06e0,    -172.95e0,  
      -8.11e0,       -0.4e0,       -0.2e0,      -0.18e0,       3.87e0,       0.09e0,       0.01e0,  
    -160.15e0,      -55.3e0,     -14.04e0,       13.9e0,      -1.23e0,       3.58e0,        0.4e0,       0.31e0,  
     -115.4e0,      123.2e0,        0.6e0,        0.7e0,       2.75e0,       2.58e0,       0.08e0,      -0.01e0,  
    -168.26e0,         -2e0,        0.2e0,       -0.2e0,      -0.04e0,       3.76e0,       0.08e0,  
    -114.49e0,      123.2e0,       0.32e0,        0.4e0,       2.75e0,       2.56e0,       0.07e0,      -0.01e0,  
     112.14e0,      120.7e0,       0.28e0,       -0.3e0,        2.7e0,      -2.51e0,      -0.07e0,      -0.01e0,  
     161.34e0,       4.03e0,        0.2e0,        0.2e0,       0.09e0,      -3.61e0,      -0.08e0,      91.31e0,  
     126.64e0,       -0.4e0,        0.4e0,       2.83e0,      -2.04e0,      -0.04e0,       0.01e0,     105.29e0,  
      112.9e0,       0.44e0,       -0.5e0,       2.52e0,      -2.35e0,      -0.07e0,      -0.01e0,      98.69e0,  
     -106.2e0,      -0.28e0,       -0.3e0,      -2.37e0,      -2.21e0,      -0.06e0,       0.01e0,      86.74e0,  
    -112.94e0,      -0.08e0,       -0.2e0,      -2.53e0,      -1.94e0,      -0.05e0,    -134.81e0,  
       3.51e0,        0.2e0,       -0.2e0,       0.08e0,       3.01e0,       0.07e0,      79.03e0,     107.31e0,  
      -0.24e0,        0.2e0,        2.4e0,      -1.77e0,      -0.04e0,       0.01e0,     132.81e0,  
     -10.77e0,      -0.52e0,        0.1e0,      -0.24e0,      -2.97e0,      -0.07e0,       0.01e0,  
    -130.31e0,       -0.9e0,       0.04e0,          0e0,          0e0,       2.91e0,     -78.56e0,      85.32e0,  
          0e0,          0e0,       1.91e0,       1.76e0,       0.04e0,          0e0,          0e0,     -41.53e0,  
       89.1e0,       0.02e0,          0e0,       1.99e0,       0.93e0,      66.03e0,        -71e0,       -0.2e0,  
       -0.2e0,      -1.59e0,      -1.48e0,      -0.04e0,       60.5e0,       64.7e0,       0.36e0,       -0.4e0,  
       1.45e0,      -1.35e0,      -0.04e0,      -0.01e0,     -52.27e0,     -70.01e0,          0e0,          0e0,  
      -1.57e0,       1.17e0,       0.03e0,     -52.95e0,      66.29e0,       0.32e0,        0.4e0,       1.48e0,  
       1.18e0,       0.04e0,      -0.01e0,      51.02e0,      67.25e0,          0e0,          0e0,        1.5e0,  
      -1.14e0,      -0.03e0,     -55.66e0,     -60.92e0,       0.16e0,       -0.2e0,      -1.36e0,       1.24e0,  
       0.03e0,     -54.81e0,      -59.2e0,      -0.08e0,        0.2e0,      -1.32e0,       1.23e0,       0.03e0,  
      51.32e0,      -55.6e0,          0e0,          0e0,      -1.24e0,      -1.15e0,      -0.03e0,      48.29e0,  
       51.8e0,        0.2e0,       -0.2e0,       1.16e0,      -1.08e0,      -0.03e0,     -45.59e0,        -49e0,  
      -0.12e0,        0.1e0,       -1.1e0,       1.02e0,       0.03e0,      40.54e0,     -52.69e0,      -0.04e0,  
       -0.1e0,      -1.18e0,      -0.91e0,      -0.02e0,     -40.58e0,     -49.51e0,         -1e0,          1e0,  
      -1.11e0,       0.91e0,       0.04e0,       0.02e0,     -43.76e0,       46.5e0,       0.36e0,        0.4e0,  
       1.04e0,       0.98e0,       0.03e0,      -0.01e0,      62.65e0,         -5e0,      -0.24e0,          0e0,  
      -0.11e0,       -1.4e0,      -0.03e0,       0.01e0,     -38.57e0,      49.59e0,       0.08e0,        0.1e0,  
       1.11e0,       0.86e0,       0.02e0,     -33.22e0,     -44.04e0,       0.08e0,       -0.1e0,      -0.98e0,  
       0.74e0,       0.02e0,      37.15e0,      -39.9e0,      -0.12e0,       -0.1e0,      -0.89e0,      -0.83e0,  
      -0.02e0,      36.68e0,      -39.5e0,      -0.04e0,       -0.1e0,      -0.88e0,      -0.82e0,      -0.02e0,  
     -53.22e0,      -3.91e0,       -0.2e0,          0e0,      -0.09e0,       1.19e0,       0.03e0,      32.43e0,  
     -42.19e0,      -0.04e0,       -0.1e0,      -0.94e0,      -0.73e0,      -0.02e0,        -51e0,       -2.3e0,  
      -0.12e0,       -0.1e0,          0e0,       1.14e0,     -29.53e0,     -39.11e0,       0.04e0,          0e0,  
      -0.87e0,       0.66e0,       0.02e0,       28.5e0,     -38.92e0,      -0.08e0,       -0.1e0,      -0.87e0,  
      -0.64e0,      -0.02e0,      26.54e0,      36.95e0,      -0.12e0,        0.1e0,       0.83e0,      -0.59e0,  
      -0.01e0,      26.54e0,      34.59e0,       0.04e0,       -0.1e0,       0.77e0,      -0.59e0,      -0.02e0,  
      28.35e0,     -32.55e0,      -0.16e0,        0.2e0,      -0.73e0,      -0.63e0,      -0.01e0,        -28e0,  
       30.4e0,          0e0,          0e0,       0.68e0,       0.63e0,       0.01e0,     -27.61e0,       29.4e0,  
        0.2e0,        0.2e0,       0.66e0,       0.62e0,       0.02e0,      40.33e0,        0.4e0,      -0.04e0,  
        0.1e0,          0e0,       -0.9e0,      -23.28e0,     31.61e0,      -0.08e0,       -0.1e0,       0.71e0,  
       0.52e0,       0.01e0,      37.75e0,        0.8e0,       0.04e0,        0.1e0,          0e0,      -0.84e0,  
      23.66e0,       25.8e0,          0e0,          0e0,       0.58e0,      -0.53e0,      -0.01e0,      21.01e0,  
     -27.91e0,          0e0,          0e0,      -0.62e0,      -0.47e0,      -0.01e0,     -34.81e0,       2.89e0,  
       0.04e0,          0e0,          0e0,       0.78e0,     -23.49e0,     -25.31e0,          0e0,          0e0,  
      -0.57e0,       0.53e0,       0.01e0,     -23.47e0,       25.2e0,       0.16e0,        0.2e0,       0.56e0,  
       0.52e0,       0.02e0,      19.58e0,       27.5e0,      -0.12e0,        0.1e0,       0.62e0,      -0.44e0,  
      -0.01e0,     -22.67e0,      -24.4e0,      -0.08e0,        0.1e0,      -0.55e0,       0.51e0,       0.01e0,  
     -19.97e0,         25e0,       0.12e0,        0.2e0,       0.56e0,       0.45e0,       0.01e0,      21.28e0,  
      -22.8e0,      -0.08e0,       -0.1e0,      -0.51e0,      -0.48e0,      -0.01e0,     -30.47e0,       0.91e0,  
       0.04e0,          0e0,          0e0,       0.68e0,      18.58e0,         24e0,       0.04e0,       -0.1e0,  
       0.54e0,      -0.42e0,      -0.01e0,     -18.02e0,       24.4e0,      -0.04e0,       -0.1e0,       0.55e0,  
        0.4e0,       0.01e0,      17.74e0,       22.5e0,       0.08e0,       -0.1e0,        0.5e0,       -0.4e0,  
      -0.01e0,     -19.41e0,       20.7e0,       0.08e0,        0.1e0,       0.46e0,       0.43e0,       0.01e0,  
     -18.64e0,      20.11e0,          0e0,          0e0,       0.45e0,       0.42e0,       0.01e0,     -16.75e0,  
       21.6e0,       0.04e0,        0.1e0,       0.48e0,       0.37e0,       0.01e0,     -18.42e0,        -20e0,  
          0e0,          0e0,      -0.45e0,       0.41e0,       0.01e0,     -26.77e0,       1.41e0,       0.08e0,  
          0e0,          0e0,        0.6e0,     -26.17e0,      -0.19e0,          0e0,          0e0,          0e0,  
       0.59e0,     -15.52e0,      20.51e0,          0e0,          0e0,       0.46e0,       0.35e0,       0.01e0,  
     -25.42e0,      -1.91e0,      -0.08e0,          0e0,      -0.04e0,       0.57e0,       0.45e0,     -17.42e0,  
       18.1e0,          0e0,          0e0,        0.4e0,       0.39e0,       0.01e0,      16.39e0,      -17.6e0,  
      -0.08e0,       -0.1e0,      -0.39e0,      -0.37e0,      -0.01e0,      -14.37e0,     18.91e0,          0e0,  
          0e0,       0.42e0,       0.32e0,       0.01e0,      23.39e0,       -2.4e0,      -0.12e0,          0e0,  
          0e0,      -0.52e0,      14.32e0,      -18.5e0,      -0.04e0,       -0.1e0,      -0.41e0,      -0.32e0,  
      -0.01e0,      15.69e0,      17.08e0,          0e0,          0e0,       0.38e0,      -0.35e0,      -0.01e0,  
     -22.99e0,        0.5e0,       0.04e0,          0e0,          0e0,       0.51e0,          0e0,          0e0,  
      14.47e0,      -17.6e0,      -0.01e0,          0e0,      -0.39e0,      -0.32e0,     -13.33e0,       18.4e0,  
      -0.04e0,       -0.1e0,       0.41e0,        0.3e0,      22.47e0,       -0.6e0,      -0.04e0,          0e0,  
          0e0,       -0.5e0,     -12.78e0,     -17.41e0,       0.04e0,          0e0,      -0.39e0,       0.29e0,  
       0.01e0,      -14.1e0,     -15.31e0,       0.04e0,          0e0,      -0.34e0,       0.32e0,       0.01e0,  
      11.98e0,      16.21e0,      -0.04e0,          0e0,       0.36e0,      -0.27e0,      -0.01e0,      19.65e0,  
       -1.9e0,      -0.08e0,          0e0,          0e0,      -0.44e0,      19.61e0,       -1.5e0,      -0.08e0,  
          0e0,          0e0,      -0.44e0,      13.41e0,      -14.3e0,      -0.04e0,       -0.1e0,      -0.32e0,  
       -0.3e0,      -0.01e0,     -13.29e0,       14.4e0,          0e0,          0e0,       0.32e0,        0.3e0,  
       0.01e0,      11.14e0,      -14.4e0,      -0.04e0,          0e0,      -0.32e0,      -0.25e0,      -0.01e0,  
      12.24e0,     -13.38e0,       0.04e0,          0e0,       -0.3e0,      -0.27e0,      -0.01e0,      10.07e0,  
      -13.81e0,      0.04e0,          0e0,      -0.31e0,      -0.23e0,      -0.01e0,      10.46e0,       13.1e0,  
       0.08e0,       -0.1e0,       0.29e0,      -0.23e0,      -0.01e0,      16.55e0,      -1.71e0,      -0.08e0,  
          0e0,          0e0,      -0.37e0,       9.75e0,      -12.8e0,          0e0,          0e0,      -0.29e0,  
      -0.22e0,      -0.01e0,       9.11e0,       12.8e0,          0e0,          0e0,       0.29e0,       -0.2e0,  
          0e0,          0e0,      -6.44e0,      -13.8e0,          0e0,          0e0,      -0.31e0,       0.14e0,  
      -9.19e0,        -12e0,          0e0,          0e0,      -0.27e0,       0.21e0,      -10.3e0,       10.9e0,  
       0.08e0,        0.1e0,       0.24e0,       0.23e0,       0.01e0,      14.92e0,       -0.8e0,      -0.04e0,  
          0e0,          0e0,      -0.33e0,      10.02e0,      -10.8e0,          0e0,          0e0,      -0.24e0,  
      -0.22e0,      -0.01e0,      -9.75e0,       10.4e0,       0.04e0,          0e0,       0.23e0,       0.22e0,  
       0.01e0,       9.67e0,      -10.4e0,      -0.04e0,          0e0,      -0.23e0,      -0.22e0,      -0.01e0,  
      -8.28e0,      -11.2e0,       0.04e0,          0e0,      -0.25e0,       0.19e0,      13.32e0,      -1.41e0,  
      -0.08e0,          0e0,          0e0,       -0.3e0,       8.27e0,       10.5e0,       0.04e0,          0e0,  
       0.23e0,      -0.19e0,          0e0,          0e0,      13.13e0,          0e0,          0e0,          0e0,  
          0e0,      -0.29e0,      -12.93e0,       0.7e0,       0.04e0,          0e0,          0e0,       0.29e0,  
       7.91e0,      -10.2e0,          0e0,          0e0,      -0.23e0,      -0.18e0,      -7.84e0,        -10e0,  
      -0.04e0,          0e0,      -0.22e0,       0.18e0,       7.44e0,        9.6e0,          0e0,          0e0,  
       0.21e0,      -0.17e0,      -7.64e0,        9.4e0,       0.08e0,        0.1e0,       0.21e0,       0.17e0,  
       0.01e0,      -11.38e0,       0.6e0,       0.04e0,          0e0,          0e0,       0.25e0,      -7.48e0,  
        8.3e0,          0e0,          0e0,       0.19e0,       0.17e0,      -10.98e0,      -0.2e0,          0e0,  
          0e0,          0e0,       0.25e0,      10.98e0,        0.2e0,          0e0,          0e0,          0e0,  
      -0.25e0,        7.4e0,       -7.9e0,      -0.04e0,          0e0,      -0.18e0,      -0.17e0,      -6.09e0,  
        8.4e0,      -0.04e0,          0e0,       0.19e0,       0.14e0,      -6.94e0,      -7.49e0,          0e0,  
          0e0,      -0.17e0,       0.16e0,       6.92e0,        7.5e0,       0.04e0,          0e0,       0.17e0,  
      -0.15e0,        6.2e0,       8.09e0,          0e0,          0e0,       0.18e0,      -0.14e0,      -6.12e0,  
        7.8e0,       0.04e0,          0e0,       0.17e0,       0.14e0,       5.85e0,       -7.5e0,          0e0,  
          0e0,      -0.17e0,      -0.13e0,      -6.48e0,        6.9e0,       0.08e0,        0.1e0,       0.15e0,  
       0.14e0,       0.01e0,       6.32e0,        6.9e0,          0e0,          0e0,       0.15e0,      -0.14e0,  
       5.61e0,       -7.2e0,          0e0,          0e0,      -0.16e0,      -0.13e0,       9.07e0,          0e0,  
          0e0,          0e0,          0e0,       -0.2e0,       5.25e0,        6.9e0,          0e0,          0e0,  
       0.15e0,      -0.12e0,      -8.47e0,       -0.4e0,          0e0,          0e0,          0e0,       0.19e0,  
       6.32e0,      -5.39e0,      -1.11e0,        1.1e0,      -0.12e0,      -0.14e0,       0.02e0,       0.02e0,  
       5.73e0,       -6.1e0,      -0.04e0,          0e0,      -0.14e0,      -0.13e0,        4.7e0,        6.6e0,  
      -0.04e0,          0e0,       0.15e0,      -0.11e0,       -4.9e0,       -6.4e0,          0e0,          0e0,  
      -0.14e0,       0.11e0,      -5.33e0,        5.6e0,       0.04e0,        0.1e0,       0.13e0,       0.12e0,  
       0.01e0,      -4.81e0,          6e0,       0.04e0,          0e0,       0.13e0,       0.11e0,       5.13e0,  
        5.5e0,       0.04e0,          0e0,       0.12e0,      -0.11e0,        4.5e0,        5.9e0,          0e0,  
          0e0,       0.13e0,       -0.1e0,      -4.22e0,        6.1e0,          0e0,          0e0,       0.14e0,  
      -4.53e0,        5.7e0,          0e0,          0e0,       0.13e0,        0.1e0,       4.18e0,        5.7e0,  
          0e0,          0e0,       0.13e0,      -4.75e0,      -5.19e0,          0e0,          0e0,      -0.12e0,  
       0.11e0,      -4.06e0,        5.6e0,          0e0,          0e0,       0.13e0,      -3.98e0,        5.6e0,  
      -0.04e0,          0e0,       0.13e0,       4.02e0,       -5.4e0,          0e0,          0e0,      -0.12e0,  
       4.49e0,       -4.9e0,      -0.04e0,          0e0,      -0.11e0,       -0.1e0,      -3.62e0,       -5.4e0,  
      -0.16e0,        0.2e0,      -0.12e0,          0e0,       0.01e0,       4.38e0,        4.8e0,          0e0,  
          0e0,       0.11e0,       -6.4e0,       -0.1e0,          0e0,          0e0,          0e0,       0.14e0,  
      -3.98e0,          5e0,       0.04e0,          0e0,       0.11e0,      -3.82e0,         -5e0,          0e0,  
          0e0,      -0.11e0,      -3.71e0,       5.07e0,          0e0,          0e0,       0.11e0,       4.14e0,  
        4.4e0,          0e0,          0e0,        0.1e0,      -6.01e0,       -0.5e0,      -0.04e0,          0e0,  
          0e0,       0.13e0,      -4.04e0,       4.39e0,          0e0,          0e0,        0.1e0,       3.45e0,  
      -4.72e0,          0e0,          0e0,      -0.11e0,       3.31e0,       4.71e0,          0e0,          0e0,  
       0.11e0,       3.26e0,       -4.5e0,          0e0,          0e0,       -0.1e0,      -3.26e0,       -4.5e0,  
          0e0,          0e0,       -0.1e0,      -3.34e0,       -4.4e0,          0e0,          0e0,       -0.1e0,  
      -3.74e0,         -4e0,        3.7e0,          4e0,       3.34e0,       -4.3e0,        3.3e0,       -4.3e0,  
      -3.66e0,        3.9e0,       0.04e0,       3.66e0,        3.9e0,       0.04e0,      -3.62e0,       -3.9e0,  
      -3.61e0,        3.9e0,       -0.2e0,        5.3e0,          0e0,          0e0,       0.12e0,       3.06e0,  
        4.3e0,        3.3e0,          4e0,        0.4e0,        0.2e0,        3.1e0,        4.1e0,      -3.06e0,  
        3.9e0,       -3.3e0,       -3.6e0,       -3.3e0,       3.36e0,       0.01e0,       3.14e0,        3.4e0,  
      -4.57e0,       -0.2e0,          0e0,          0e0,          0e0,        0.1e0,       -2.7e0,       -3.6e0,  
       2.94e0,       -3.2e0,       -2.9e0,        3.2e0,       2.47e0,       -3.4e0,       2.55e0,       -3.3e0,  
        2.8e0,      -3.08e0,       2.51e0,        3.3e0,       -4.1e0,        0.3e0,      -0.12e0,       -0.1e0,  
        4.1e0,        0.2e0,      -2.74e0,          3e0,       2.46e0,       3.23e0,      -3.66e0,        1.2e0,  
       -0.2e0,        0.2e0,       3.74e0,       -0.4e0,      -2.51e0,       -2.8e0,      -3.74e0,       2.27e0,  
       -2.9e0,          0e0,          0e0,       -2.5e0,        2.7e0,      -2.51e0,        2.6e0,       -3.5e0,  
        0.2e0,       3.38e0,      -2.22e0,       -2.5e0,       3.26e0,       -0.4e0,       1.95e0,       -2.6e0,  
       3.22e0,       -0.4e0,      -0.04e0,      -1.79e0,       -2.6e0,       1.91e0,        2.5e0,       0.74e0,  
       3.05e0,      -0.04e0,       0.08e0,       2.11e0,       -2.3e0,      -2.11e0,        2.2e0,      -1.87e0,  
       -2.4e0,       2.03e0,       -2.2e0,      -2.03e0,        2.2e0,       2.98e0,          0e0,          0e0,  
       2.98e0,      -1.71e0,        2.4e0,       2.94e0,       -0.1e0,      -0.12e0,        0.1e0,       1.67e0,  
        2.4e0,      -1.79e0,        2.3e0,      -1.79e0,        2.2e0,      -1.67e0,        2.2e0,       1.79e0,  
         -2e0,       1.87e0,       -1.9e0,       1.63e0,       -2.1e0,      -1.59e0,        2.1e0,       1.55e0,  
       -2.1e0,      -1.55e0,        2.1e0,      -2.59e0,       -0.2e0,      -1.75e0,       -1.9e0,      -1.75e0,  
        1.9e0,      -1.83e0,       -1.8e0,       1.51e0,          2e0,      -1.51e0,         -2e0,       1.71e0,  
        1.8e0,       1.31e0,        2.1e0,      -1.43e0,          2e0,       1.43e0,          2e0,      -2.43e0,  
      -1.51e0,        1.9e0,      -1.47e0,        1.9e0,       2.39e0,        0.2e0,      -2.39e0,       1.39e0,  
        1.9e0,       1.39e0,       -1.8e0,       1.47e0,       -1.6e0,       1.47e0,       -1.6e0,       1.43e0,  
       -1.5e0,      -1.31e0,        1.6e0,       1.27e0,       -1.6e0,      -1.27e0,        1.6e0,       1.27e0,  
       -1.6e0,       2.03e0,       1.35e0,        1.5e0,      -1.39e0,       -1.4e0,       1.95e0,       -0.2e0,  
      -1.27e0,       1.49e0,       1.19e0,        1.5e0,       1.27e0,        1.4e0,       1.15e0,        1.5e0,  
       1.87e0,       -0.1e0,      -1.12e0,       -1.5e0,       1.87e0,      -1.11e0,       -1.5e0,      -1.11e0,  
       -1.5e0,          0e0,          0e0,       1.19e0,        1.4e0,       1.27e0,       -1.3e0,      -1.27e0,  
       -1.3e0,      -1.15e0,        1.4e0,      -1.23e0,        1.3e0,      -1.23e0,       -1.3e0,       1.22e0,  
      -1.29e0,       1.07e0,       -1.4e0,       1.75e0,       -0.2e0,      -1.03e0,       -1.4e0,      -1.07e0,  
        1.2e0,      -1.03e0,       1.15e0,       1.07e0,        1.1e0,       1.51e0,      -1.03e0,        1.1e0,  
       1.03e0,       -1.1e0,          0e0,          0e0,      -1.03e0,       -1.1e0,       0.91e0,       -1.2e0,  
      -0.88e0,       -1.2e0,      -0.88e0,        1.2e0,      -0.95e0,        1.1e0,      -0.95e0,       -1.1e0,  
       1.43e0,      -1.39e0,       0.95e0,         -1e0,      -0.95e0,          1e0,       -0.8e0,        1.1e0,  
       0.91e0,         -1e0,      -1.35e0,       0.88e0,          1e0,      -0.83e0,          1e0,      -0.91e0,  
        0.9e0,       0.91e0,        0.9e0,       0.88e0,       -0.9e0,      -0.76e0,         -1e0,      -0.76e0,  
          1e0,       0.76e0,          1e0,      -0.72e0,          1e0,       0.84e0,       -0.9e0,       0.84e0,  
        0.9e0,       1.23e0,          0e0,          0e0,      -0.52e0,       -1.1e0,      -0.68e0,          1e0,  
       1.19e0,       -0.2e0,       1.19e0,       0.76e0,        0.9e0,       1.15e0,       -0.1e0,       1.15e0,  
       -0.1e0,       0.72e0,       -0.9e0,      -1.15e0,      -1.15e0,       0.68e0,        0.9e0,      -0.68e0,  
        0.9e0,      -1.11e0,          0e0,          0e0,        0.2e0,       0.79e0,        0.8e0,      -1.11e0,  
       -0.1e0,          0e0,          0e0,      -0.48e0,         -1e0,      -0.76e0,       -0.8e0,      -0.72e0,  
       -0.8e0,      -1.07e0,       -0.1e0,       0.64e0,        0.8e0,      -0.64e0,       -0.8e0,       0.64e0,  
        0.8e0,        0.4e0,        0.6e0,       0.52e0,       -0.5e0,       -0.6e0,       -0.8e0,      -0.71e0,  
        0.7e0,      -0.99e0,       0.99e0,       0.56e0,        0.8e0,      -0.56e0,        0.8e0,       0.68e0,  
       -0.7e0,       0.68e0,        0.7e0,      -0.95e0,      -0.64e0,        0.7e0,       0.64e0,        0.7e0,  
       -0.6e0,        0.7e0,       -0.6e0,       -0.7e0,      -0.91e0,       -0.1e0,      -0.51e0,       0.76e0,  
      -0.91e0,      -0.56e0,        0.7e0,       0.88e0,       0.88e0,      -0.63e0,       -0.6e0,       0.55e0,  
       -0.6e0,       -0.8e0,        0.8e0,       -0.8e0,      -0.52e0,        0.6e0,       0.52e0,        0.6e0,  
       0.52e0,       -0.6e0,      -0.48e0,        0.6e0,       0.48e0,        0.6e0,       0.48e0,        0.6e0,  
      -0.76e0,       0.44e0,       -0.6e0,       0.52e0,       -0.5e0,      -0.52e0,        0.5e0,        0.4e0,  
        0.6e0,       -0.4e0,       -0.6e0,        0.4e0,       -0.6e0,       0.72e0,      -0.72e0,      -0.51e0,  
       -0.5e0,      -0.48e0,        0.5e0,       0.48e0,       -0.5e0,      -0.48e0,        0.5e0,      -0.48e0,  
        0.5e0,       0.48e0,       -0.5e0,      -0.48e0,       -0.5e0,      -0.68e0,      -0.68e0,       0.44e0,  
        0.5e0,      -0.64e0,       -0.1e0,      -0.64e0,       -0.1e0,       -0.4e0,        0.5e0,        0.4e0,  
        0.5e0,        0.4e0,        0.5e0,          0e0,          0e0,       -0.4e0,       -0.5e0,      -0.36e0,  
       -0.5e0,       0.36e0,       -0.5e0,        0.6e0,       -0.6e0,        0.4e0,       -0.4e0,        0.4e0,  
        0.4e0,       -0.4e0,        0.4e0,       -0.4e0,        0.4e0,      -0.56e0,      -0.56e0,       0.36e0,  
       -0.4e0,      -0.36e0,        0.4e0,       0.36e0,       -0.4e0,      -0.36e0,       -0.4e0,       0.36e0,  
        0.4e0,       0.36e0,        0.4e0,      -0.52e0,       0.52e0,       0.52e0,       0.32e0,        0.4e0,  
      -0.32e0,        0.4e0,      -0.32e0,        0.4e0,      -0.32e0,        0.4e0,       0.32e0,       -0.4e0,  
      -0.32e0,       -0.4e0,       0.32e0,       -0.4e0,       0.28e0,       -0.4e0,      -0.28e0,        0.4e0,  
       0.28e0,       -0.4e0,       0.28e0,        0.4e0,       0.48e0,      -0.48e0,       0.48e0,       0.36e0,  
       -0.3e0,      -0.36e0,       -0.3e0,          0e0,          0e0,        0.2e0,        0.4e0,      -0.44e0,  
       0.44e0,      -0.44e0,      -0.44e0,      -0.44e0,      -0.44e0,       0.32e0,       -0.3e0,       0.32e0,  
        0.3e0,       0.24e0,        0.3e0,      -0.12e0,       -0.1e0,      -0.28e0,        0.3e0,       0.28e0,  
        0.3e0,       0.28e0,        0.3e0,       0.28e0,       -0.3e0,       0.28e0,       -0.3e0,       0.28e0,  
       -0.3e0,       0.28e0,        0.3e0,      -0.28e0,        0.3e0,        0.4e0,        0.4e0,      -0.24e0,  
        0.3e0,       0.24e0,       -0.3e0,       0.24e0,       -0.3e0,      -0.24e0,       -0.3e0,       0.24e0,  
        0.3e0,       0.24e0,       -0.3e0,      -0.24e0,        0.3e0,       0.24e0,       -0.3e0,      -0.24e0,  
       -0.3e0,       0.24e0,       -0.3e0,       0.24e0,        0.3e0,      -0.24e0,        0.3e0,      -0.24e0,  
        0.3e0,        0.2e0,       -0.3e0,        0.2e0,       -0.3e0,        0.2e0,       -0.3e0,        0.2e0,  
        0.3e0,        0.2e0,       -0.3e0,        0.2e0,       -0.3e0,        0.2e0,        0.3e0,        0.2e0,  
        0.3e0,       -0.2e0,       -0.3e0,        0.2e0,       -0.3e0,        0.2e0,       -0.3e0,      -0.36e0,  
      -0.36e0,      -0.36e0,      -0.04e0,        0.3e0,       0.12e0,       -0.1e0,      -0.32e0,      -0.24e0,  
        0.2e0,       0.24e0,        0.2e0,        0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,  
       -0.2e0,        0.2e0,        0.2e0,        0.2e0,       -0.2e0,        0.2e0,        0.2e0,        0.2e0,  
        0.2e0,       -0.2e0,       -0.2e0,          0e0,          0e0,       -0.2e0,       -0.2e0,       -0.2e0,  
        0.2e0,       -0.2e0,        0.2e0,        0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,        0.2e0,  
        0.2e0,        0.2e0,        0.2e0,        0.2e0,       -0.2e0,        0.2e0,       -0.2e0,       0.28e0,  
       0.28e0,       0.28e0,       0.28e0,       0.28e0,       0.28e0,      -0.28e0,       0.28e0,       0.12e0,  
          0e0,       0.24e0,       0.16e0,       -0.2e0,       0.16e0,       -0.2e0,       0.16e0,       -0.2e0,  
       0.16e0,        0.2e0,      -0.16e0,        0.2e0,       0.16e0,        0.2e0,      -0.16e0,        0.2e0,  
      -0.16e0,        0.2e0,      -0.16e0,        0.2e0,       0.16e0,       -0.2e0,       0.16e0,        0.2e0,  
       0.16e0,       -0.2e0,      -0.16e0,        0.2e0,      -0.16e0,       -0.2e0,      -0.16e0,        0.2e0,  
       0.16e0,        0.2e0,       0.16e0,       -0.2e0,       0.16e0,       -0.2e0,       0.16e0,        0.2e0,  
       0.16e0,        0.2e0,       0.16e0,        0.2e0,      -0.16e0,       -0.2e0,       0.16e0,        0.2e0,  
      -0.16e0,        0.2e0,       0.16e0,        0.2e0,      -0.16e0,       -0.2e0,       0.16e0,       -0.2e0,  
       0.16e0,       -0.2e0,      -0.16e0,       -0.2e0,       0.24e0,      -0.24e0,      -0.24e0,       0.24e0,  
       0.24e0,       0.12e0,        0.2e0,       0.12e0,        0.2e0,      -0.12e0,       -0.2e0,       0.12e0,  
       -0.2e0,       0.12e0,       -0.2e0,      -0.12e0,        0.2e0,      -0.12e0,        0.2e0,      -0.12e0,  
       -0.2e0,       0.12e0,        0.2e0,       0.12e0,        0.2e0,       0.12e0,       -0.2e0,      -0.12e0,  
        0.2e0,       0.12e0,       -0.2e0,      -0.12e0,        0.2e0,       0.12e0,        0.2e0,          0e0,  
          0e0,      -0.12e0,        0.2e0,      -0.12e0,        0.2e0,       0.12e0,       -0.2e0,      -0.12e0,  
        0.2e0,       0.12e0,        0.2e0,          0e0,      -0.21e0,       -0.2e0,          0e0,          0e0,  
        0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,        0.2e0,      -0.16e0,       -0.1e0,          0e0,  
       0.17e0,       0.16e0,       0.16e0,       0.16e0,       0.16e0,      -0.16e0,       0.16e0,       0.16e0,  
      -0.16e0,       0.16e0,      -0.16e0,       0.16e0,       0.12e0,        0.1e0,       0.12e0,       -0.1e0,  
      -0.12e0,        0.1e0,      -0.12e0,        0.1e0,       0.12e0,       -0.1e0,      -0.12e0,       0.12e0,  
      -0.12e0,       0.12e0,      -0.12e0,       0.12e0,      -0.12e0,      -0.12e0,      -0.12e0,      -0.12e0,  
      -0.12e0,      -0.12e0,      -0.12e0,       0.12e0,       0.12e0,       0.12e0,       0.12e0,      -0.12e0,  
      -0.12e0,       0.12e0,       0.12e0,       0.12e0,      -0.12e0,       0.12e0,      -0.12e0,      -0.12e0,  
      -0.12e0,       0.12e0,      -0.12e0,      -0.12e0,       0.12e0,          0e0,       0.11e0,       0.11e0,  
    -122.67e0,      164.7e0,     203.78e0,      273.5e0,       3.58e0,       2.74e0,       6.18e0,      -4.56e0,  
          0e0,      -0.04e0,          0e0,      -0.07e0,      57.44e0,      -77.1e0,      95.82e0,      128.6e0,  
      -1.77e0,      -1.28e0,       2.85e0,      -2.14e0,      82.14e0,       89.5e0,          0e0,          0e0,  
          2e0,      -1.84e0,      -0.04e0,      47.73e0,      -64.1e0,      23.79e0,       31.9e0,      -1.45e0,  
      -1.07e0,       0.69e0,      -0.53e0,     -46.38e0,       50.5e0,          0e0,          0e0,       1.13e0,  
       1.04e0,       0.02e0,     -18.38e0,          0e0,       63.8e0,          0e0,          0e0,       0.41e0,  
          0e0,      -1.43e0,      59.07e0,          0e0,          0e0,          0e0,          0e0,      -1.32e0,  
      57.28e0,          0e0,          0e0,          0e0,          0e0,      -1.28e0,     -48.65e0,          0e0,  
      -1.15e0,          0e0,          0e0,       1.09e0,          0e0,       0.03e0,      -18.3e0,       24.6e0,  
      -17.3e0,      -23.2e0,       0.56e0,       0.41e0,      -0.51e0,       0.39e0,     -16.91e0,       26.9e0,  
       8.43e0,       13.3e0,        0.6e0,       0.38e0,       0.31e0,      -0.19e0,       1.23e0,       -1.7e0,  
     -19.13e0,      -25.7e0,      -0.03e0,      -0.03e0,      -0.58e0,       0.43e0,      -0.72e0,        0.9e0,  
     -17.34e0,      -23.3e0,       0.03e0,       0.02e0,      -0.52e0,       0.39e0,     -19.49e0,      -21.3e0,  
          0e0,          0e0,      -0.48e0,       0.44e0,       0.01e0,      20.57e0,      -20.1e0,       0.64e0,  
        0.7e0,      -0.45e0,      -0.46e0,          0e0,      -0.01e0,       4.89e0,        5.9e0,     -16.55e0,  
       19.9e0,       0.14e0,      -0.11e0,       0.44e0,       0.37e0,      18.22e0,       19.8e0,          0e0,  
          0e0,       0.44e0,      -0.41e0,      -0.01e0,       4.89e0,       -5.3e0,     -16.51e0,        -18e0,  
      -0.11e0,      -0.11e0,      -0.41e0,       0.37e0,     -17.86e0,          0e0,       17.1e0,          0e0,  
          0e0,        0.4e0,          0e0,      -0.38e0,       0.32e0,          0e0,      24.42e0,          0e0,  
          0e0,      -0.01e0,          0e0,      -0.55e0,     -23.79e0,          0e0,          0e0,          0e0,  
          0e0,       0.53e0,      14.72e0,        -16e0,      -0.32e0,          0e0,      -0.36e0,      -0.33e0,  
      -0.01e0,       0.01e0,       3.34e0,       -4.5e0,      11.86e0,       15.9e0,      -0.11e0,      -0.07e0,  
       0.35e0,      -0.27e0,      -3.26e0,        4.4e0,      11.62e0,       15.6e0,       0.09e0,       0.07e0,  
       0.35e0,      -0.26e0,     -19.53e0,          0e0,       5.09e0,          0e0,          0e0,       0.44e0,  
          0e0,      -0.11e0,     -13.48e0,       14.7e0,          0e0,          0e0,       0.33e0,        0.3e0,  
       0.01e0,      10.86e0,      -14.6e0,       3.18e0,        4.3e0,      -0.33e0,      -0.24e0,       0.09e0,  
      -0.07e0,      -11.3e0,      -15.1e0,          0e0,          0e0,      -0.34e0,       0.25e0,       0.01e0,  
       2.03e0,       -2.7e0,      10.82e0,       14.5e0,      -0.07e0,      -0.05e0,       0.32e0,      -0.24e0,  
      17.46e0,          0e0,          0e0,          0e0,          0e0,      -0.39e0,      16.43e0,          0e0,  
       0.52e0,          0e0,          0e0,      -0.37e0,          0e0,      -0.01e0,       9.35e0,          0e0,  
      13.29e0,          0e0,          0e0,      -0.21e0,          0e0,       -0.3e0,     -10.42e0,       11.4e0,  
          0e0,          0e0,       0.25e0,       0.23e0,       0.01e0,       0.44e0,        0.5e0,     -10.38e0,  
       11.3e0,       0.02e0,      -0.01e0,       0.25e0,       0.23e0,     -14.64e0,          0e0,          0e0,  
          0e0,          0e0,       0.33e0,       0.56e0,        0.8e0,      -8.67e0,       11.7e0,       0.02e0,  
      -0.01e0,       0.26e0,       0.19e0,      13.88e0,          0e0,      -2.47e0,          0e0,          0e0,  
      -0.31e0,          0e0,       0.06e0,      -1.99e0,        2.7e0,       7.72e0,       10.3e0,       0.06e0,  
       0.04e0,       0.23e0,      -0.17e0,       -0.2e0,          0e0,      13.05e0,          0e0,          0e0,  
          0e0,          0e0,      -0.29e0,       6.92e0,       -9.3e0,       3.34e0,        4.5e0,      -0.21e0,  
      -0.15e0,        0.1e0,      -0.07e0,       -6.6e0,          0e0,       10.7e0,          0e0,          0e0,  
       0.15e0,          0e0,      -0.24e0,      -8.04e0,       -8.7e0,          0e0,          0e0,      -0.19e0,  
       0.18e0,     -10.58e0,          0e0,       -3.1e0,          0e0,          0e0,       0.24e0,          0e0,  
       0.07e0,      -7.32e0,          8e0,      -0.12e0,       -0.1e0,       0.18e0,       0.16e0,       1.63e0,  
        1.7e0,       6.96e0,       -7.6e0,       0.03e0,      -0.04e0,      -0.17e0,      -0.16e0,      -3.62e0,  
          0e0,       9.86e0,          0e0,          0e0,       0.08e0,          0e0,      -0.22e0,        0.2e0,  
       -0.2e0,      -6.88e0,       -7.5e0,          0e0,          0e0,      -0.17e0,       0.15e0,      -8.99e0,  
          0e0,       4.02e0,          0e0,          0e0,        0.2e0,          0e0,      -0.09e0,      -1.07e0,  
        1.4e0,      -5.69e0,       -7.7e0,       0.03e0,       0.02e0,      -0.17e0,       0.13e0,       6.48e0,  
       -7.2e0,      -0.48e0,       -0.5e0,      -0.16e0,      -0.14e0,      -0.01e0,       0.01e0,       5.57e0,  
       -7.5e0,       1.07e0,        1.4e0,      -0.17e0,      -0.12e0,       0.03e0,      -0.02e0,       8.71e0,  
          0e0,       3.54e0,          0e0,          0e0,      -0.19e0,          0e0,      -0.08e0,        0.4e0,  
          0e0,       9.27e0,          0e0,          0e0,      -0.01e0,          0e0,      -0.21e0,      -6.13e0,  
        6.7e0,      -1.19e0,       -1.3e0,       0.15e0,       0.14e0,      -0.03e0,       0.03e0,       5.21e0,  
       -5.7e0,      -2.51e0,       -2.6e0,      -0.13e0,      -0.12e0,      -0.06e0,       0.06e0,       5.69e0,  
       -6.2e0,      -0.12e0,       -0.1e0,      -0.14e0,      -0.13e0,      -0.01e0,       2.03e0,       -2.7e0,  
       4.53e0,        6.1e0,      -0.06e0,      -0.05e0,       0.14e0,       -0.1e0,       5.01e0,        5.5e0,  
      -2.51e0,        2.7e0,       0.12e0,      -0.11e0,       0.06e0,       0.06e0,      -1.91e0,        2.6e0,  
      -4.38e0,       -5.9e0,       0.06e0,       0.04e0,      -0.13e0,        0.1e0,       4.65e0,       -6.3e0,  
          0e0,          0e0,      -0.14e0,       -0.1e0,      -5.29e0,        5.7e0,          0e0,          0e0,  
       0.13e0,       0.12e0,      -2.23e0,         -4e0,      -4.65e0,        4.2e0,      -0.09e0,       0.05e0,  
        0.1e0,        0.1e0,      -4.53e0,        6.1e0,          0e0,          0e0,       0.14e0,        0.1e0,  
       2.47e0,        2.7e0,      -4.46e0,        4.9e0,       0.06e0,      -0.06e0,       0.11e0,        0.1e0,  
      -5.05e0,        5.5e0,       0.84e0,        0.9e0,       0.12e0,       0.11e0,       0.02e0,      -0.02e0,  
       4.97e0,       -5.4e0,      -1.71e0,          0e0,      -0.12e0,      -0.11e0,          0e0,       0.04e0,  
      -0.99e0,       -1.3e0,       4.22e0,       -5.7e0,      -0.03e0,       0.02e0,      -0.13e0,      -0.09e0,  
       0.99e0,        1.4e0,       4.22e0,       -5.6e0,       0.03e0,      -0.02e0,      -0.13e0,      -0.09e0,  
      -4.69e0,       -5.2e0,          0e0,          0e0,      -0.12e0,        0.1e0,      -3.42e0,          0e0,  
       6.09e0,          0e0,          0e0,       0.08e0,          0e0,      -0.14e0,      -4.65e0,       -5.1e0,  
          0e0,          0e0,      -0.11e0,        0.1e0,          0e0,          0e0,      -4.53e0,         -5e0,  
          0e0,          0e0,      -0.11e0,        0.1e0,      -2.43e0,       -2.7e0,      -3.82e0,        4.2e0,  
      -0.06e0,       0.05e0,        0.1e0,       0.09e0,          0e0,          0e0,      -4.53e0,        4.9e0,  
          0e0,          0e0,       0.11e0,        0.1e0,      -4.49e0,       -4.9e0,          0e0,          0e0,  
      -0.11e0,        0.1e0,       2.67e0,       -2.9e0,      -3.62e0,       -3.9e0,      -0.06e0,      -0.06e0,  
      -0.09e0,       0.08e0,       3.94e0,       -5.3e0,          0e0,          0e0,      -0.12e0,      -3.38e0,  
        3.7e0,      -2.78e0,       -3.1e0,       0.08e0,       0.08e0,      -0.07e0,       0.06e0,       3.18e0,  
       -3.5e0,      -2.82e0,       -3.1e0,      -0.08e0,      -0.07e0,      -0.07e0,       0.06e0,      -5.77e0,  
          0e0,       1.87e0,          0e0,          0e0,       0.13e0,          0e0,      -0.04e0,       3.54e0,  
       -4.8e0,      -0.64e0,       -0.9e0,      -0.11e0,          0e0,      -0.02e0,       -3.5e0,       -4.7e0,  
       0.68e0,       -0.9e0,      -0.11e0,          0e0,      -0.02e0,       5.49e0,          0e0,          0e0,  
          0e0,          0e0,      -0.12e0,       1.83e0,       -2.5e0,       2.63e0,        3.5e0,      -0.06e0,  
          0e0,       0.08e0,       3.02e0,       -4.1e0,       0.68e0,        0.9e0,      -0.09e0,          0e0,  
       0.02e0,          0e0,          0e0,       5.21e0,          0e0,          0e0,          0e0,          0e0,  
      -0.12e0,      -3.54e0,        3.8e0,        2.7e0,        3.6e0,      -1.35e0,        1.8e0,       0.08e0,  
          0e0,       0.04e0,       -2.9e0,        3.9e0,       0.68e0,        0.9e0,       0.09e0,          0e0,  
       0.02e0,        0.8e0,       -1.1e0,      -2.78e0,       -3.7e0,      -0.02e0,          0e0,      -0.08e0,  
        4.1e0,          0e0,      -2.39e0,          0e0,          0e0,      -0.09e0,          0e0,       0.05e0,  
      -1.59e0,        2.1e0,       2.27e0,          3e0,       0.05e0,          0e0,       0.07e0,      -2.63e0,  
        3.5e0,      -0.48e0,       -0.6e0,      -2.94e0,       -3.2e0,      -2.94e0,        3.2e0,       2.27e0,  
         -3e0,      -1.11e0,       -1.5e0,      -0.07e0,          0e0,      -0.03e0,      -0.56e0,       -0.8e0,  
      -2.35e0,        3.1e0,          0e0,       -0.6e0,      -3.42e0,        1.9e0,      -0.12e0,       -0.1e0,  
       2.63e0,       -2.9e0,       2.51e0,        2.8e0,      -0.64e0,        0.7e0,      -0.48e0,       -0.6e0,  
       2.19e0,       -2.9e0,       0.24e0,       -0.3e0,       2.15e0,        2.9e0,       2.15e0,       -2.9e0,  
       0.52e0,        0.7e0,       2.07e0,       -2.8e0,       -3.1e0,          0e0,       1.79e0,          0e0,  
          0e0,       0.07e0,          0e0,      -0.04e0,       0.88e0,          0e0,      -3.46e0,       2.11e0,  
        2.8e0,      -0.36e0,        0.5e0,       3.54e0,       -0.2e0,       -3.5e0,      -1.39e0,        1.5e0,  
      -1.91e0,       -2.1e0,      -1.47e0,          2e0,       1.39e0,        1.9e0,       2.07e0,       -2.3e0,  
       0.91e0,          1e0,       1.99e0,       -2.7e0,        3.3e0,          0e0,        0.6e0,      -0.44e0,  
       -0.7e0,      -1.95e0,        2.6e0,       2.15e0,       -2.4e0,       -0.6e0,       -0.7e0,        3.3e0,  
       0.84e0,          0e0,       -3.1e0,       -3.1e0,          0e0,      -0.72e0,      -0.32e0,        0.4e0,  
      -1.87e0,       -2.5e0,       1.87e0,       -2.5e0,       0.32e0,        0.4e0,      -0.24e0,        0.3e0,  
      -1.87e0,       -2.5e0,      -0.24e0,       -0.3e0,       1.87e0,       -2.5e0,       -2.7e0,          0e0,  
       1.55e0,       2.03e0,        2.2e0,      -2.98e0,      -1.99e0,       -2.2e0,       0.12e0,       -0.1e0,  
       -0.4e0,        0.5e0,       1.59e0,        2.1e0,          0e0,          0e0,      -1.79e0,          2e0,  
      -1.03e0,        1.4e0,      -1.15e0,       -1.6e0,       0.32e0,        0.5e0,       1.39e0,       -1.9e0,  
       2.35e0,      -1.27e0,        1.7e0,        0.6e0,        0.8e0,      -0.32e0,       -0.4e0,       1.35e0,  
       -1.8e0,       0.44e0,          0e0,       2.23e0,      -0.84e0,        0.9e0,      -1.27e0,       -1.4e0,  
      -1.47e0,        1.6e0,      -0.28e0,       -0.3e0,      -0.28e0,        0.4e0,      -1.27e0,       -1.7e0,  
       0.28e0,       -0.4e0,      -1.43e0,       -1.5e0,          0e0,          0e0,      -1.27e0,       -1.7e0,  
       2.11e0,      -0.32e0,       -0.4e0,      -1.23e0,        1.6e0,       1.19e0,       -1.3e0,      -0.72e0,  
       -0.8e0,       0.72e0,       -0.8e0,      -1.15e0,       -1.3e0,      -1.35e0,       -1.5e0,      -1.19e0,  
       -1.6e0,      -0.12e0,        0.2e0,       1.79e0,          0e0,      -0.88e0,      -0.28e0,        0.4e0,  
       1.11e0,        1.5e0,      -1.83e0,          0e0,       0.56e0,      -0.12e0,        0.1e0,      -1.27e0,  
       -1.4e0,          0e0,          0e0,       1.15e0,        1.5e0,      -0.12e0,        0.2e0,       1.11e0,  
        1.5e0,       0.36e0,       -0.5e0,      -1.07e0,       -1.4e0,      -1.11e0,        1.5e0,       1.67e0,  
          0e0,        0.8e0,      -1.11e0,          0e0,       1.43e0,       1.23e0,       -1.3e0,      -0.24e0,  
      -1.19e0,       -1.3e0,      -0.24e0,        0.2e0,      -0.44e0,       -0.9e0,      -0.95e0,        1.1e0,  
       1.07e0,       -1.4e0,       1.15e0,       -1.3e0,       1.03e0,       -1.1e0,      -0.56e0,       -0.6e0,  
      -0.68e0,        0.9e0,      -0.76e0,         -1e0,      -0.24e0,       -0.3e0,       0.95e0,       -1.3e0,  
       0.56e0,        0.7e0,       0.84e0,       -1.1e0,      -0.56e0,          0e0,      -1.55e0,       0.91e0,  
       -1.3e0,       0.28e0,        0.3e0,       0.16e0,       -0.2e0,       0.95e0,        1.3e0,        0.4e0,  
       -0.5e0,      -0.88e0,       -1.2e0,       0.95e0,       -1.1e0,      -0.48e0,       -0.5e0,          0e0,  
          0e0,      -1.07e0,        1.2e0,       0.44e0,       -0.5e0,       0.95e0,        1.1e0,          0e0,  
          0e0,       0.92e0,       -1.3e0,       0.95e0,          1e0,      -0.52e0,        0.6e0,       1.59e0,  
       0.24e0,       -0.4e0,       0.91e0,        1.2e0,       0.84e0,       -1.1e0,      -0.44e0,       -0.6e0,  
       0.84e0,        1.1e0,      -0.44e0,        0.6e0,      -0.44e0,        0.6e0,      -0.84e0,       -1.1e0,  
       -0.8e0,          0e0,       1.35e0,       0.76e0,        0.2e0,      -0.91e0,         -1e0,        0.2e0,  
       -0.3e0,      -0.91e0,       -1.2e0,      -0.95e0,          1e0,      -0.48e0,       -0.5e0,       0.88e0,  
          1e0,       0.48e0,       -0.5e0,      -0.95e0,       -1.1e0,        0.2e0,       -0.2e0,      -0.99e0,  
        1.1e0,      -0.84e0,        1.1e0,      -0.24e0,       -0.3e0,        0.2e0,       -0.3e0,       0.84e0,  
        1.1e0,      -1.39e0,          0e0,      -0.28e0,      -0.16e0,        0.2e0,       0.84e0,        1.1e0,  
          0e0,          0e0,       1.39e0,          0e0,          0e0,      -0.95e0,          1e0,       1.35e0,  
      -0.99e0,          0e0,       0.88e0,      -0.52e0,          0e0,      -1.19e0,        0.2e0,        0.2e0,  
       0.76e0,         -1e0,          0e0,          0e0,       0.76e0,          1e0,          0e0,          0e0,  
       0.76e0,          1e0,      -0.76e0,          1e0,          0e0,          0e0,       1.23e0,       0.76e0,  
        0.8e0,      -0.32e0,        0.4e0,      -0.72e0,        0.8e0,       -0.4e0,       -0.4e0,          0e0,  
          0e0,       -0.8e0,       -0.9e0,      -0.68e0,        0.9e0,      -0.16e0,       -0.2e0,      -0.16e0,  
       -0.2e0,       0.68e0,       -0.9e0,      -0.36e0,        0.5e0,      -0.56e0,       -0.8e0,       0.72e0,  
       -0.9e0,       0.44e0,       -0.6e0,      -0.48e0,       -0.7e0,      -0.16e0,          0e0,      -1.11e0,  
       0.32e0,          0e0,      -1.07e0,        0.6e0,       -0.8e0,      -0.28e0,       -0.4e0,      -0.64e0,  
          0e0,       0.91e0,       1.11e0,       0.64e0,       -0.9e0,       0.76e0,       -0.8e0,          0e0,  
          0e0,      -0.76e0,       -0.8e0,       1.03e0,          0e0,      -0.36e0,      -0.64e0,       -0.7e0,  
       0.36e0,       -0.4e0,       1.07e0,       0.36e0,       -0.5e0,      -0.52e0,       -0.7e0,        0.6e0,  
          0e0,       0.88e0,       0.95e0,          0e0,       0.48e0,       0.16e0,       -0.2e0,        0.6e0,  
        0.8e0,       0.16e0,       -0.2e0,       -0.6e0,       -0.8e0,          0e0,         -1e0,       0.12e0,  
        0.2e0,       0.16e0,       -0.2e0,       0.68e0,        0.7e0,       0.59e0,       -0.8e0,      -0.99e0,  
      -0.56e0,       -0.6e0,       0.36e0,       -0.4e0,      -0.68e0,       -0.7e0,      -0.68e0,       -0.7e0,  
      -0.36e0,       -0.5e0,      -0.44e0,        0.6e0,       0.64e0,        0.7e0,      -0.12e0,        0.1e0,  
      -0.52e0,        0.6e0,       0.36e0,        0.4e0,          0e0,          0e0,       0.95e0,      -0.84e0,  
          0e0,       0.44e0,       0.56e0,        0.6e0,       0.32e0,       -0.3e0,          0e0,          0e0,  
        0.6e0,        0.7e0,          0e0,          0e0,        0.6e0,        0.7e0,      -0.12e0,       -0.2e0,  
       0.52e0,       -0.7e0,          0e0,          0e0,       0.56e0,        0.7e0,      -0.12e0,        0.1e0,  
      -0.52e0,       -0.7e0,          0e0,          0e0,       0.88e0,      -0.76e0,          0e0,      -0.44e0,  
          0e0,          0e0,      -0.52e0,       -0.7e0,       0.52e0,       -0.7e0,       0.36e0,       -0.4e0,  
      -0.44e0,       -0.5e0,          0e0,          0e0,        0.6e0,        0.6e0,       0.84e0,          0e0,  
       0.12e0,      -0.24e0,          0e0,        0.8e0,      -0.56e0,        0.6e0,      -0.32e0,       -0.3e0,  
       0.48e0,       -0.5e0,       0.28e0,       -0.3e0,      -0.48e0,       -0.5e0,       0.12e0,        0.2e0,  
       0.48e0,       -0.6e0,       0.48e0,        0.6e0,      -0.12e0,        0.2e0,       0.24e0,          0e0,  
       0.76e0,      -0.52e0,       -0.6e0,      -0.52e0,        0.6e0,       0.48e0,       -0.5e0,      -0.24e0,  
       -0.3e0,       0.12e0,       -0.1e0,       0.48e0,        0.6e0,       0.52e0,       -0.2e0,       0.36e0,  
        0.4e0,      -0.44e0,        0.5e0,      -0.24e0,       -0.3e0,      -0.48e0,       -0.6e0,      -0.44e0,  
       -0.6e0,      -0.12e0,        0.1e0,       0.76e0,       0.76e0,        0.2e0,       -0.2e0,       0.48e0,  
        0.5e0,        0.4e0,       -0.5e0,      -0.24e0,       -0.3e0,       0.44e0,       -0.6e0,       0.44e0,  
       -0.6e0,       0.36e0,          0e0,      -0.64e0,       0.72e0,          0e0,      -0.12e0,          0e0,  
       -0.1e0,       -0.4e0,       -0.6e0,       -0.2e0,       -0.2e0,      -0.44e0,        0.5e0,      -0.44e0,  
        0.5e0,        0.2e0,        0.2e0,      -0.44e0,       -0.5e0,        0.2e0,       -0.2e0,       -0.2e0,  
        0.2e0,      -0.44e0,       -0.5e0,       0.64e0,          0e0,       0.32e0,      -0.36e0,        0.5e0,  
       -0.2e0,       -0.3e0,       0.12e0,       -0.1e0,       0.48e0,        0.5e0,      -0.12e0,        0.3e0,  
      -0.36e0,       -0.5e0,          0e0,          0e0,       0.48e0,        0.5e0,      -0.48e0,        0.5e0,  
       0.68e0,          0e0,      -0.12e0,       0.56e0,       -0.4e0,       0.44e0,       -0.5e0,      -0.12e0,  
       -0.1e0,       0.24e0,        0.3e0,       -0.4e0,        0.4e0,       0.64e0,          0e0,      -0.24e0,  
       0.64e0,          0e0,       -0.2e0,          0e0,          0e0,       0.44e0,       -0.5e0,       0.44e0,  
        0.5e0,      -0.12e0,        0.2e0,      -0.36e0,       -0.5e0,       0.12e0,          0e0,       0.64e0,  
       -0.4e0,        0.5e0,          0e0,        0.1e0,          0e0,          0e0,       -0.4e0,        0.5e0,  
          0e0,          0e0,       -0.4e0,       -0.5e0,       0.56e0,          0e0,       0.28e0,          0e0,  
        0.1e0,       0.36e0,        0.5e0,          0e0,       -0.1e0,       0.36e0,       -0.5e0,       0.36e0,  
        0.5e0,          0e0,       -0.1e0,       0.24e0,       -0.2e0,      -0.36e0,       -0.4e0,       0.16e0,  
        0.2e0,        0.4e0,       -0.4e0,          0e0,          0e0,      -0.36e0,       -0.5e0,      -0.36e0,  
       -0.5e0,      -0.32e0,       -0.5e0,      -0.12e0,        0.1e0,        0.2e0,        0.2e0,      -0.36e0,  
        0.4e0,       -0.6e0,        0.6e0,       0.28e0,          0e0,       0.52e0,       0.12e0,       -0.1e0,  
        0.4e0,        0.4e0,          0e0,       -0.5e0,        0.2e0,       -0.2e0,      -0.32e0,        0.4e0,  
       0.16e0,        0.2e0,      -0.16e0,        0.2e0,       0.32e0,        0.4e0,       0.56e0,          0e0,  
      -0.12e0,       0.32e0,       -0.4e0,      -0.16e0,       -0.2e0,          0e0,          0e0,        0.4e0,  
        0.4e0,       -0.4e0,       -0.4e0,       -0.4e0,        0.4e0,      -0.36e0,        0.4e0,       0.12e0,  
        0.1e0,          0e0,        0.1e0,       0.36e0,        0.4e0,          0e0,       -0.1e0,       0.36e0,  
        0.4e0,      -0.36e0,        0.4e0,          0e0,        0.1e0,       0.32e0,          0e0,       0.44e0,  
       0.12e0,        0.2e0,       0.28e0,       -0.4e0,          0e0,          0e0,       0.36e0,        0.4e0,  
       0.32e0,       -0.4e0,      -0.16e0,       0.12e0,        0.1e0,       0.32e0,       -0.4e0,        0.2e0,  
        0.3e0,      -0.24e0,        0.3e0,          0e0,        0.1e0,       0.32e0,        0.4e0,          0e0,  
       -0.1e0,      -0.32e0,       -0.4e0,      -0.32e0,        0.4e0,          0e0,        0.1e0,      -0.52e0,  
      -0.52e0,       0.52e0,       0.32e0,       -0.4e0,          0e0,          0e0,       0.32e0,        0.4e0,  
       0.32e0,       -0.4e0,          0e0,          0e0,      -0.32e0,       -0.4e0,      -0.32e0,        0.4e0,  
       0.32e0,        0.4e0,          0e0,          0e0,       0.32e0,        0.4e0,          0e0,          0e0,  
      -0.32e0,       -0.4e0,          0e0,          0e0,       0.32e0,        0.4e0,       0.16e0,        0.2e0,  
       0.32e0,       -0.3e0,      -0.16e0,          0e0,      -0.48e0,       -0.2e0,        0.2e0,      -0.28e0,  
       -0.3e0,       0.28e0,       -0.4e0,          0e0,          0e0,       0.28e0,       -0.4e0,          0e0,  
          0e0,       0.28e0,       -0.4e0,          0e0,          0e0,      -0.28e0,       -0.4e0,       0.28e0,  
        0.4e0,      -0.28e0,       -0.4e0,      -0.48e0,       -0.2e0,        0.2e0,       0.24e0,        0.3e0,  
       0.44e0,          0e0,       0.16e0,       0.24e0,        0.3e0,       0.16e0,       -0.2e0,       0.24e0,  
        0.3e0,      -0.12e0,        0.2e0,        0.2e0,        0.3e0,      -0.16e0,        0.2e0,          0e0,  
          0e0,       0.44e0,      -0.32e0,        0.3e0,       0.24e0,          0e0,      -0.36e0,       0.36e0,  
          0e0,       0.24e0,       0.12e0,       -0.2e0,        0.2e0,        0.3e0,      -0.12e0,          0e0,  
      -0.28e0,        0.3e0,      -0.24e0,        0.3e0,       0.12e0,        0.1e0,      -0.28e0,       -0.3e0,  
      -0.28e0,        0.3e0,          0e0,          0e0,      -0.28e0,       -0.3e0,          0e0,          0e0,  
      -0.28e0,       -0.3e0,          0e0,          0e0,       0.28e0,        0.3e0,          0e0,          0e0,  
      -0.28e0,       -0.3e0,      -0.28e0,        0.3e0,          0e0,          0e0,      -0.28e0,       -0.3e0,  
          0e0,          0e0,       0.28e0,        0.3e0,          0e0,          0e0,      -0.28e0,        0.3e0,  
       0.28e0,       -0.3e0,      -0.28e0,        0.3e0,        0.4e0,        0.4e0,      -0.24e0,        0.3e0,  
          0e0,       -0.1e0,       0.16e0,          0e0,       0.36e0,       -0.2e0,        0.3e0,      -0.12e0,  
       -0.1e0,      -0.24e0,       -0.3e0,          0e0,          0e0,      -0.24e0,        0.3e0,      -0.24e0,  
        0.3e0,          0e0,          0e0,      -0.24e0,        0.3e0,      -0.24e0,        0.3e0,       0.24e0,  
       -0.3e0,          0e0,          0e0,       0.24e0,       -0.3e0,          0e0,          0e0,       0.24e0,  
        0.3e0,       0.24e0,       -0.3e0,       0.24e0,        0.3e0,      -0.24e0,        0.3e0,      -0.24e0,  
        0.3e0,       -0.2e0,        0.2e0,      -0.16e0,       -0.2e0,          0e0,          0e0,      -0.32e0,  
        0.2e0,          0e0,        0.1e0,        0.2e0,       -0.3e0,        0.2e0,       -0.2e0,       0.12e0,  
        0.2e0,      -0.16e0,        0.2e0,       0.16e0,        0.2e0,        0.2e0,        0.3e0,        0.2e0,  
        0.3e0,          0e0,          0e0,       -0.2e0,        0.3e0,          0e0,          0e0,        0.2e0,  
        0.3e0,       -0.2e0,       -0.3e0,       -0.2e0,       -0.3e0,        0.2e0,       -0.3e0,          0e0,  
          0e0,        0.2e0,        0.3e0,          0e0,          0e0,        0.2e0,        0.3e0,          0e0,  
          0e0,        0.2e0,        0.3e0,          0e0,          0e0,        0.2e0,        0.3e0,          0e0,  
          0e0,        0.2e0,       -0.3e0,          0e0,          0e0,       -0.2e0,       -0.3e0,          0e0,  
          0e0,       -0.2e0,        0.3e0,          0e0,          0e0,       -0.2e0,        0.3e0,          0e0,  
          0e0,       0.36e0,          0e0,          0e0,       0.36e0,       0.12e0,        0.1e0,      -0.24e0,  
        0.2e0,       0.12e0,       -0.2e0,      -0.16e0,       -0.2e0,      -0.13e0,        0.1e0,       0.22e0,  
       0.21e0,        0.2e0,          0e0,      -0.28e0,       0.32e0,          0e0,      -0.12e0,       -0.2e0,  
       -0.2e0,       0.12e0,       -0.1e0,       0.12e0,        0.1e0,       -0.2e0,        0.2e0,          0e0,  
          0e0,      -0.32e0,       0.32e0,          0e0,          0e0,       0.32e0,       0.32e0,          0e0,  
          0e0,      -0.24e0,       -0.2e0,       0.24e0,        0.2e0,        0.2e0,          0e0,      -0.24e0,  
          0e0,          0e0,      -0.24e0,       -0.2e0,          0e0,          0e0,       0.24e0,        0.2e0,  
      -0.24e0,       -0.2e0,          0e0,          0e0,      -0.24e0,        0.2e0,       0.16e0,       -0.2e0,  
       0.12e0,        0.1e0,        0.2e0,        0.2e0,          0e0,       -0.1e0,      -0.12e0,        0.1e0,  
      -0.16e0,       -0.2e0,      -0.12e0,       -0.1e0,      -0.16e0,        0.2e0,        0.2e0,        0.2e0,  
          0e0,          0e0,       -0.2e0,        0.2e0,       -0.2e0,        0.2e0,       -0.2e0,        0.2e0,  
       -0.2e0,        0.2e0,        0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,          0e0,          0e0,  
       -0.2e0,        0.2e0,        0.2e0,          0e0,       -0.2e0,          0e0,          0e0,       -0.2e0,  
        0.2e0,       -0.2e0,        0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,       -0.2e0,          0e0,  
          0e0,        0.2e0,        0.2e0,        0.2e0,        0.2e0,       0.12e0,       -0.2e0,      -0.12e0,  
       -0.1e0,       0.28e0,      -0.28e0,       0.16e0,       -0.2e0,          0e0,       -0.1e0,          0e0,  
        0.1e0,      -0.16e0,        0.2e0,          0e0,       -0.1e0,      -0.16e0,       -0.2e0,          0e0,  
       -0.1e0,       0.16e0,       -0.2e0,       0.16e0,       -0.2e0,          0e0,          0e0,       0.16e0,  
        0.2e0,      -0.16e0,        0.2e0,          0e0,          0e0,       0.16e0,        0.2e0,       0.16e0,  
       -0.2e0,       0.16e0,       -0.2e0,      -0.16e0,        0.2e0,       0.16e0,       -0.2e0,          0e0,  
          0e0,       0.16e0,        0.2e0,          0e0,          0e0,       0.16e0,        0.2e0,          0e0,  
          0e0,      -0.16e0,       -0.2e0,       0.16e0,       -0.2e0,      -0.16e0,       -0.2e0,          0e0,  
          0e0,      -0.16e0,       -0.2e0,          0e0,          0e0,      -0.16e0,        0.2e0,          0e0,  
          0e0,       0.16e0,       -0.2e0,       0.16e0,        0.2e0,       0.16e0,        0.2e0,          0e0,  
          0e0,      -0.16e0,       -0.2e0,          0e0,          0e0,      -0.16e0,       -0.2e0,          0e0,  
          0e0,       0.16e0,        0.2e0,       0.16e0,        0.2e0,          0e0,          0e0,       0.16e0,  
        0.2e0,       0.16e0,       -0.2e0,       0.16e0,        0.2e0,          0e0,          0e0,      -0.16e0,  
        0.2e0,          0e0,        0.1e0,       0.12e0,       -0.2e0,       0.12e0,       -0.2e0,          0e0,  
       -0.1e0,          0e0,       -0.1e0,       0.12e0,        0.2e0,          0e0,       -0.1e0,      -0.12e0,  
        0.2e0,      -0.15e0,        0.2e0,      -0.24e0,       0.24e0,          0e0,          0e0,       0.24e0,  
       0.24e0,       0.12e0,       -0.2e0,      -0.12e0,       -0.2e0,          0e0,          0e0,       0.12e0,  
        0.2e0,       0.12e0,       -0.2e0,       0.12e0,        0.2e0,       0.12e0,        0.2e0,       0.12e0,  
        0.2e0,       0.12e0,       -0.2e0,      -0.12e0,        0.2e0,          0e0,          0e0,       0.12e0,  
        0.2e0,       0.12e0,          0e0,       -0.2e0,          0e0,          0e0,      -0.12e0,       -0.2e0,  
       0.12e0,       -0.2e0,          0e0,          0e0,       0.12e0,        0.2e0,      -0.12e0,        0.2e0,  
      -0.12e0,        0.2e0,       0.12e0,       -0.2e0,          0e0,          0e0,       0.12e0,        0.2e0,  
        0.2e0,          0e0,       0.12e0,          0e0,          0e0,      -0.12e0,        0.2e0,          0e0,  
          0e0,      -0.12e0,       -0.2e0,          0e0,          0e0,      -0.12e0,       -0.2e0,      -0.12e0,  
       -0.2e0,          0e0,          0e0,       0.12e0,       -0.2e0,       0.12e0,       -0.2e0,       0.12e0,  
        0.2e0,      -0.12e0,       -0.2e0,          0e0,          0e0,       0.12e0,       -0.2e0,       0.12e0,  
       -0.2e0,       0.12e0,        0.2e0,       0.12e0,          0e0,        0.2e0,      -0.12e0,       -0.2e0,  
          0e0,          0e0,       0.12e0,        0.2e0,      -0.16e0,          0e0,       0.16e0,       -0.2e0,  
       0.2e0,          0e0,          0e0,       -0.2e0,          0e0,          0e0,       -0.2e0,        0.2e0,  
          0e0,          0e0,        0.2e0,        0.2e0,       -0.2e0,          0e0,          0e0,       -0.2e0,  
       0.12e0,          0e0,      -0.16e0,        0.2e0,          0e0,          0e0,        0.2e0,       0.12e0,  
       -0.1e0,          0e0,        0.1e0,       0.16e0,      -0.16e0,      -0.16e0,      -0.16e0,      -0.16e0,  
      -0.16e0,          0e0,          0e0,      -0.16e0,          0e0,          0e0,      -0.16e0,      -0.16e0,  
      -0.16e0,          0e0,          0e0,      -0.16e0,          0e0,          0e0,       0.16e0,          0e0,  
          0e0,       0.16e0,          0e0,          0e0,       0.16e0,       0.16e0,          0e0,          0e0,  
      -0.16e0,          0e0,          0e0,      -0.16e0,      -0.16e0,          0e0,          0e0,       0.16e0,  
          0e0,          0e0,      -0.16e0,      -0.16e0,          0e0,          0e0,      -0.16e0,      -0.16e0,  
       0.12e0,        0.1e0,       0.12e0,       -0.1e0,       0.12e0,        0.1e0,          0e0,          0e0,  
       0.12e0,        0.1e0,      -0.12e0,        0.1e0,          0e0,          0e0,       0.12e0,        0.1e0,  
       0.12e0,       -0.1e0,          0e0,          0e0,      -0.12e0,       -0.1e0,          0e0,          0e0,  
       0.12e0,        0.1e0,       0.12e0,          0e0,          0e0,       0.12e0,          0e0,          0e0,  
      -0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,       0.12e0,       0.12e0,       0.12e0,  
          0e0,          0e0,       0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,          0e0,  
          0e0,       0.12e0,          0e0,          0e0,       0.12e0,      -0.12e0,      -0.12e0,       0.12e0,  
       0.12e0,      -0.12e0,      -0.12e0,          0e0,          0e0,       0.12e0,      -0.12e0,       0.12e0,  
       0.12e0,      -0.12e0,      -0.12e0,          0e0,          0e0,      -0.12e0,      -0.12e0,          0e0,  
          0e0,      -0.12e0,       0.12e0,          0e0,          0e0,       0.12e0,          0e0,          0e0,  
       0.12e0,          0e0,          0e0,       0.12e0,      -0.12e0,          0e0,          0e0,      -0.12e0,  
       0.12e0,      -0.12e0,      -0.12e0,       0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,  
       0.12e0,      -0.12e0,          0e0,          0e0,      -0.12e0,      -0.12e0,      -0.12e0,          0e0,  
          0e0,      -0.12e0,      -0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,          0e0,  
          0e0,      -0.12e0,      -0.12e0,      -0.12e0,      -0.12e0,       0.12e0,          0e0,          0e0,  
       0.12e0,      -0.12e0,          0e0,          0e0,      -0.12e0,      -0.12e0,          0e0,          0e0,  
       0.12e0,      -0.12e0,      -0.12e0,      -0.12e0,      -0.12e0,       0.12e0,       0.12e0,      -0.12e0,  
      -0.12e0,          0e0,          0e0,      -0.12e0,          0e0,          0e0,      -0.12e0,       0.12e0,  
          0e0,          0e0,       0.12e0,          0e0,          0e0,      -0.12e0,      -0.12e0,          0e0,  
          0e0,      -0.12e0,      -0.12e0,       0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,  
          0e0,          0e0,       0.12e0,          0e0,          0e0,       0.12e0,       0.12e0,       0.08e0,  
          0e0,       0.04e0,  
  
    } ;  
  
/*  
 * Amplitude usage: X or Y, sin or cos, power of T.  
 */  
  
    static const int JAXY[MAXPT * 4] = {  
        0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1,  
    } ;  
  
    static const int JASC[MAXPT * 4] = {  
        0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0,  
    } ;  
  
    static const int JAPT[MAXPT * 4] = {  
        0, 0 ,0 ,0 ,1 ,1 ,1 ,1 ,2 ,2 ,2 ,2 ,3 ,3 ,3 ,3 ,4 ,4 ,4 ,4,  
    } ;  
  
/*  
 * Miscellaneous  
 */  
  
    double t, w, pt[MAXPT + 1], fa[14], xypr[2], xypl[2], xyls[2], arg, sc[2] ;  
  
    int jpt, i, j, jxy, ialast, ifreq, m, ia, jsc ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  
  
/*  
 * Interval between fundamental date J2000.0 and given date (JC).  
 */  
  
    t = ( ( date1 - GAL_J2000 ) + date2 ) / GAL_DJC ;  
  
/*  
 * Powers of T.  
 */  
  
    w = 1.0 ;  
    for ( jpt = 0; jpt <= MAXPT; jpt++ ) {  
        pt[jpt] = w ;  
        w *= t ;  
    }  
  
/*  
 * Luni-solar fundamental (Delaunay) arguments (IERS 2003)  
 */  
  
/*  
 * Mean anomaly of the Moon.  
 */  
  
    fa[0] = gal_fal03 ( t ) ;  
  
/*  
 * Mean anomaly of the Sun.  
 */  
  
    fa[1] = gal_falp03 ( t ) ;  
  
/*  
 * Mean argument of the latitude of the Moon.  
 */  
  
    fa[2] = gal_faf03 ( t ) ;  
  
/*  
 * Mean elongation of the Moon from the Sun.  
 */  
  
    fa[3] = gal_fad03 ( t ) ;  
  
/*  
 * Mean longitude of the ascending node of the Moon.  
 */  
  
    fa[4] = gal_faom03 ( t ) ;  
  
/*  
 * Planetary longitudes, Mercury through Neptune.  
 */  
  
    fa[5]  = gal_fame03 ( t ) ;  
    fa[6]  = gal_fave03 ( t ) ;  
    fa[7]  = gal_fae03  ( t ) ;  
    fa[8]  = gal_fama03 ( t ) ;  
    fa[9]  = gal_faju03 ( t ) ;  
    fa[10] = gal_fasa03 ( t ) ;  
    fa[11] = gal_faur03 ( t ) ;  
    fa[12] = gal_fane03 ( t ) ;  
  
/*  
 * General accumulated precession in longitude.  
 */  
  
    fa[13] = gal_fapa03 ( t ) ;  
  
/*  
 * --------------------------------------  
 * Polynomial part of precession-nutation  
 * --------------------------------------  
 */  
  
    for ( jxy = 0; jxy <= 1; jxy++ ) {  
        xypr[jxy] = 0.0 ;  
        for ( j = MAXPT; j >= 0 ; j-- ) {  
            xypr[jxy] += XYP[jxy][j] * pt[j] ;  
        }  
    }  
  
/*  
 * ----------------------------------  
 * Nutation periodic terms, planetary  
 * ----------------------------------  
 */  
  
/*  
 * Initialize totals in X and Y.  
 */  
  
    for ( jxy=0; jxy<=1; jxy++ ) {  
        xypl[jxy] = 0.0 ;  
    }  
  
/*  
 * Work backwards through the coefficients per frequency list.  
 */  
  
    ialast = NA ;  
  
    for ( ifreq = NFPL; ifreq > 0; ifreq-- ) {  
  
/*  
 * Obtain the argument functions.  
 */  
  
        arg = 0.0 ;  
        for ( i = 1; i <= 14; i++ ) {  
            m = MFAPL[ifreq-1][i-1] ;  
            if ( m != 0 ) {  
                arg += ( double )( m ) * fa[i-1] ;  
            }  
        }  
        sc[0] = sin ( arg ) ;  
        sc[1] = cos ( arg ) ;  
  
/*  
 * Work backwards through the amplitudes at this frequency.  
 */  
  
        ia = NC[ifreq + NFLS - 1] ;  
        for ( i = ialast; i >= ia; i--) {  
  
/*  
 * Coefficient number (0 = 1st).  
 */  
  
            j = i - ia ;  
  
/*  
 * X or Y.  
 */  
  
            jxy = JAXY[j] ;  
  
/*  
 * Sin or cos.  
 */  
  
            jsc = JASC[j] ;  
  
/*  
 * Power of T.  
 */  
  
            jpt = JAPT[j] ;  
  
/*  
 * Accumulate the component.  
 */  
  
            xypl[jxy] += A[i-1] * sc[jsc] * pt[jpt] ;  
  
        }  
  
         ialast = ia - 1 ;  
  
    }  
  
/*  
 * -----------------------------------  
 * Nutation periodic terms, luni-solar  
 * -----------------------------------  
 */  
  
/*  
 * Initialize totals in X and Y.  
 */  
  
    for ( jxy = 0; jxy <= 1; jxy++ ) {  
        xyls[jxy] = 0.0 ;  
    }  
  
/*  
 * Continue working backwards through the number of coefficients list.  
 */  
  
    for ( ifreq = NFLS; ifreq > 0; ifreq-- ) {  
  
/*  
 * Obtain the argument functions.  
 */  
  
        arg = 0.0 ;  
        for ( i = 1; i <= 5; i++) {  
            m = MFALS[ifreq-1][i-1] ;  
            if ( m != 0 ) {  
                arg += (double)( m ) * fa[i-1] ;  
            }  
        }  
        sc[0] = sin( arg ) ;  
        sc[1] = cos( arg ) ;  
  
/*  
 * Work backwards through the amplitudes at this frequency.  
 */  
  
        ia = NC[ifreq-1] ;  
  
        for ( i = ialast; i >= ia; i-- ) {  
  
/*  
 * Coefficient number (0 = 1st).  
 */  
  
            j = i - ia ;  
  
/*  
 * X or Y.  
 */  
  
            jxy = JAXY[j] ;  
  
/*  
 * Sin or cos.  
 */  
  
            jsc = JASC[j] ;  
  
/*  
 * Power of T.  
 */  
  
            jpt = JAPT[j] ;  
  
/*  
 * Accumulate the component.  
 */  
  
            xyls[jxy] += A[i-1] * sc[jsc] * pt[jpt] ;  
  
        }  
  
        ialast = ia - 1 ;  
  
    }  
  
/*  
 * -------  
 * Results  
 * -------  
 */  
  
/*  
 * CIP unit vector components.  
 */  
  
    *x = GAL_AS2R * ( xypr[0] + ( xyls[0] + xypl[0] ) / 1e6 ) ;  
    *y = GAL_AS2R * ( xypr[1] + ( xyls[1] + xypl[1] ) / 1e6 ) ;  
  
/*  
 * Finished.  
 */  
  
}  
  
/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
